--[[
		Recap - Hawksy (on Elune) (modified for WoW 2)
			  - original development by Gello (versions 3.32 and earlier)

		An AddOn for WoW to track and summarize all damage and healing done around the user.
]]

Recap_Version = 3.72
Recap_Sync_Version = 361 -- used to determine compatible versions of synchronization (integer for compact transmission) (only update if Recap sync format changes so as to be incompatible)


--[[ Saved Variables ]]

recap = {} -- options and current data (saved to SavedVariables)
recap_set = {} -- fight data sets (saved to SavedVariables)


--[[ Ace2 setup ]]

-- mixin AceComm for sync purposes
Recap = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceComm-2.0")

-- other Ace2 setup
Recap:SetCommPrefix("Recap")


--[[ local tables ]]

-- colors for the gauges
local gaugecolor = { ["DmgIn"] = { r=.66,g=.25,b=.25 }, ["DmgOut"] = { r=.25,g=.66,b=.35 }, ["Heal"] = { r=.25,g=.5,b=.85 },
					 ["Time"] = { r=.75, g=.75, b=.75 }, ["Kills"] = { r=.75, g=.75, b=.75 }, ["MaxHit"] = { r=.75, g=.75, b=.75 },
					 ["DPS"] = { r=.25, g=.66, b=.35 }, ["Over"] = { r=.25, g=.5, b=.85 } }

-- .Opt.x and its buttons
local recapbuttons = { ["Minimized"] = "RecapMinimizeButton", ["Pinned"] = "RecapPinButton",
					   ["Paused"] = "RecapPauseButton", ["SelfView"] = "RecapSelfViewButton" }


--[[ Callback functions ]]

function Recap:OnInitialize()
	-- slash commands
	SlashCmdList["RecapCOMMAND"] = Recap_SlashHandler
	SLASH_RecapCOMMAND1 = "/Recap"
end


function Recap:OnEnable()

	-- called on login and on reloadui
	-- recap_temp cleared, so Recap_Initialize will be called and its interior code will be executed

	if not recap_temp.Loaded then
		if not Recap_Initialize() then
			return false
		end
	end

	-- register some events
	Recap:RegisterEvent("PLAYER_ENTERING_WORLD")
	Recap:RegisterEvent("PLAYER_LEAVING_WORLD")
	Recap:RegisterEvent("WORLD_MAP_UPDATE")
	Recap:RegisterEvent("ZONE_CHANGED")
	Recap:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	Recap:RegisterEvent("ZONE_CHANGED_INDOORS")
	Recap:RegisterEvent("MINIMAP_ZONE_CHANGED")
	Recap:RegisterEvent("UNIT_NAME_UPDATE")
	Recap:RegisterEvent("PLAYER_REGEN_DISABLED")
	Recap:RegisterEvent("PLAYER_REGEN_ENABLED")
	Recap:RegisterEvent("PARTY_MEMBERS_CHANGED")
	Recap:RegisterEvent("RAID_ROSTER_UPDATE")
end

function Recap:OnDisable()
	-- nothing right now
end

function Recap:PLAYER_ENTERING_WORLD()

	-- called on login, on reloadui, on entering or leaving an instance, on respawning at a graveyard

	if not recap_temp.Loaded then
		if not Recap_Initialize() then
			return false
		end
	end
	recap_temp.EnteredWorld = 0
end

function Recap:PLAYER_LEAVING_WORLD()
	-- this event might indicate that we are entering void space, so delay checking for instance
	recap_temp.EnteredWorld = 0
end

function Recap:WORLD_MAP_UPDATE()
	-- this event might indicate that we are entering void space, so delay checking for instance
	recap_temp.EnteredWorld = 0
end

function Recap:ZONE_CHANGED()
	-- this event might indicate that we are entering void space, so delay checking for instance
	recap_temp.EnteredWorld = 0
end

function Recap:ZONE_CHANGED_NEW_AREA()
	-- this event might indicate that we are entering void space, so delay checking for instance
	recap_temp.EnteredWorld = 0
end

function Recap:ZONE_CHANGED_INDOORS()
	-- this event might indicate that we are entering void space, so delay checking for instance
	recap_temp.EnteredWorld = 0
end

function Recap:MINIMAP_ZONE_CHANGED()
	-- this event might indicate that we are entering void space, so delay checking for instance
	recap_temp.EnteredWorld = 0
end

function Recap:UNIT_NAME_UPDATE()
	if not recap_temp.Loaded then
		if not Recap_Initialize() then
			return false
		end
	end
end

function Recap:PLAYER_REGEN_DISABLED()
	if recap_temp.Loaded then -- protect from premature execution
		if recap.Opt.State.value=="Idle" then
			Recap_StartFight()
		end
	end
end

function Recap:PLAYER_REGEN_ENABLED()
	if recap_temp.Loaded then -- protect from premature execution
		-- player leaving combat, start the end of fight delay timer
		-- fight will not end until at least this amount of time has elapsed, longer if combat continues
		-- we leave the idle timer on, if it is on (and may the best timer win)
		recap_temp.EndFightDelayTimer = 0
	end
end

function Recap:PARTY_MEMBERS_CHANGED()
	if recap_temp.Loaded then -- protect from premature execution
		Recap_MakeFriends()
	end
end

function Recap:RAID_ROSTER_UPDATE()
	if recap_temp.Loaded then -- protect from premature execution
		Recap_MakeFriends()
	end
end

function Recap:CHAT_MSG_CHANNEL_NOTICE()

	if arg4 and string.find(arg4, "recaplog"..string.lower((UnitName("player"))), 1, true) and arg1=="YOU_JOINED" then
		-- we have successfully joined our private log channel

		local i, chatnum, chatname
		local logchat = "recaplog"..string.lower((UnitName("player")))

		chatnum = false
		for i=1,10 do
			j,chatname = GetChannelName(i)
			if chatname and string.lower(chatname)==logchat then
				chatnum=j
				break
			end
		end
		if chatnum then
			recap_temp.LogChatnum = chatnum
			-- begin logging to WOWChatLog.txt
			if LoggingChat() then
				-- already on
			else
				-- toggle on
				LoggingChat(true)
			end
			-- allow Report and Close buttons
			RecapOpenLogButton:SetText(RECAP_CHANNEL_JOINED)
			RecapOpenLogButton:Disable()
			RecapReportToLogButton:Enable()
			RecapCloseLogButton:Enable()
		else
			-- oops, didn't work after all
			DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.NoChannelNumber)
			RecapOpenLogButton:SetText(RECAP_OPEN_WOWCHATLOG_TXT)
			RecapOpenLogButton:Enable()
		end
		RecapFrame:UnregisterEvent("CHAT_MSG_CHANNEL_NOTICE")
	end
end

function Recap_OnUpdate(arg1)

	-- this function is called continually except when Recap is "Stopped"

	-- set lock (some of this update processing contains indefinite delays)
	RecapUpdateFrame:Hide() -- disable OnUpdate

	if not recap_temp.Loaded then
		return
	end

	-- paranoia
	if recap.Opt.State.value == "Stopped" then
		return
	end

	-- we're using OnUpdate for instant response to mouse-over, so we might as well use it for all other timed events
	-- as far as I can see there would be no benefit from using AceEvent

	-- generic half-second timer
	recap_temp.UpdateTimer = recap_temp.UpdateTimer + arg1
	if recap_temp.UpdateTimer > 0.5 then
		recap_temp.UpdateTimer = 0

		-- if fighting and a plugin loaded or we're minimized, update min dps
		if recap.Opt.State.value=="Active" and ((IB_Recap_Update or TitanPanelRecap_Update) or (RecapFrame:IsShown() and recap.Opt.Minimized.value)) then
			Recap_UpdateMinimizedDPS()
		end

		-- if fighting, check for friends and get their maximum hit points and current hit points (all three can change during combat)
		-- ( may have the side benefit of tracking shadowfiend ownership and mage water elemental ownership better )
		if recap.Opt.State.value=="Active" then
			Recap_MakeFriends()
		end

		-- possibly blink the sync status light (0 set to default value, 1 turn off, 2 turn on)
		if RecapFrame:IsVisible() then
			if recap_temp.SyncStatusLightBlink == 0 then
				if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
					RecapMinSyncStatusTexture:SetVertexColor(0,0,1)
				else
					RecapMinSyncStatusTexture:SetVertexColor(.5,.5,.5)
				end
			elseif recap_temp.SyncStatusLightBlink == 1 then
				recap_temp.SyncStatusLightBlink = 2
				RecapMinSyncStatusTexture:SetVertexColor(.5,.5,.5)
			elseif recap_temp.SyncStatusLightBlink == 2 then
				recap_temp.SyncStatusLightBlink = 0
				RecapMinSyncStatusTexture:SetVertexColor(0,0,1)
			end

			-- update main panel title bar (for the sync member count)
			if not recap.Opt.Minimized.value then
				Recap_TitleBar()
			end
		end

		-- if sync tab is visible and we're in a synchronization, update details
		if RecapOptSubFrame5:IsVisible() and ((recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader")) then
			Recap_SetSyncStateDetail()
		end

		-- timer to end idle fights if option checked
		if recap_temp.IdleTimer~=-1 and recap.Opt.IdleFight.value then
			recap_temp.IdleTimer = recap_temp.IdleTimer + 0.5
			if recap_temp.IdleTimer > recap.Opt.IdleFightTimer.value then
				recap_temp.IdleTimer = -1
				Recap_EndFight(false)
				Recap_CheckJoinSync() -- could contain an indefinite pause
				Recap_CheckSendSummary()
				Recap_CheckProcessSyncData()
			end
		end

		-- hidden timer to end idle fights after 10 minutes
		if recap_temp.HiddenIdleTimer~=-1 then
			recap_temp.HiddenIdleTimer = recap_temp.HiddenIdleTimer + 0.5
			if recap_temp.HiddenIdleTimer > 600 then
				recap_temp.HiddenIdleTimer = -1
				Recap_EndFight(false)
				Recap_CheckJoinSync() -- could contain an indefinite pause
				Recap_CheckSendSummary()
				Recap_CheckProcessSyncData()
			end
		end

		-- timer to wait after an end-of-fight is triggered before actually ending the fight
		-- prevents a fight being ended prematurely when it's just a short gap
		if recap_temp.EndFightDelayTimer~=-1 then
			recap_temp.EndFightDelayTimer = recap_temp.EndFightDelayTimer + 0.5
			if recap_temp.EndFightDelayTimer > recap.Opt.EndFightDelay.value then
				recap_temp.EndFightDelayTimer = -1
				Recap_EndFight(false)
				Recap_CheckJoinSync() -- could contain an indefinite pause
				Recap_CheckSendSummary()
				Recap_CheckProcessSyncData()
			end
		end

		-- check group status
		Recap_JoinLeaveGroup() -- could contain an indefinite pause

		if recap_temp.EnteredWorld then
			-- wait 3 seconds after entering world before checking instance status (this avoids most void space answer problems)
			if recap_temp.EnteredWorld < 6 then
				recap_temp.EnteredWorld = recap_temp.EnteredWorld + 1
			else
				-- check instance status
				Recap_EnterLeaveInstance() -- could contain an indefinite pause
			end
		end
	end

	-- update fade timer if option checked
	if RecapFrame:IsShown() and recap.Opt.AutoFade.value and not recap.Opt.Minimized.value and MouseIsOver(RecapFrame) and recap_temp.FadeTimer<recap.Opt.AutoFadeTimer.value then
		recap_temp.FadeTimer = 0
	elseif recap_temp.FadeTimer~=-1 and recap.Opt.AutoFade.value and not recap.Opt.Minimized.value then
		recap_temp.FadeTimer = recap_temp.FadeTimer + arg1
		if recap_temp.FadeTimer > recap.Opt.AutoFadeTimer.value then
			RecapFrame:SetAlpha(max(1-min(2*(recap_temp.FadeTimer-recap.Opt.AutoFadeTimer.value),1),.1))
			if recap_temp.FadeTimer > (recap.Opt.AutoFadeTimer.value+.5) then
				RecapFrame_Hide()
			end
		end
	end

	-- check every update since we want instant response to the mouse-over
	if RecapFrame:IsShown() and recap.Opt.AutoMinimize.value and recap_temp.Selected==0 then
		if not recap.Opt.Minimized.value and not MouseIsOver(RecapFrame) and not IsShiftKeyDown() then
			Recap_Minimize()
		elseif recap.Opt.Minimized.value and MouseIsOver(RecapFrame) and not IsShiftKeyDown() then
			Recap_Maximize()
		end
	end

	-- free lock (some of this update processing contains indefinite delays)
	RecapUpdateFrame:Show() -- enable OnUpdate

end

function Recap_JoinLeaveGroup()
	-- If option is on, remind player on joining and leaving a group
	-- GetNumPartyMembers returns number of *other* people, not counting you
	-- GetNumRaidMembers returns number of people, including you
	local count = math.max(1+GetNumPartyMembers(),GetNumRaidMembers())
	if (recap.Opt.NumPartyMembers.value == 1) and (count > 1) and recap.Opt.RemindGroupStatus.value then
		StaticPopupDialogs["recap_temp.REMINDGROUP"] = {
			text = TEXT(recap_temp.Localize.RemindJoinGroup),
			button1 = TEXT(OKAY),
			OnAccept = function()
			end,
			timeout = 0,
			showAlert = 1,
			whileDead = 1
		}
		StaticPopup_Show("recap_temp.REMINDGROUP")
	elseif (recap.Opt.NumPartyMembers.value > 1) and (count == 1) and recap.Opt.RemindGroupStatus.value then
		StaticPopupDialogs["recap_temp.REMINDGROUP"] = {
			text = TEXT(recap_temp.Localize.RemindLeaveGroup),
			button1 = TEXT(OKAY),
			OnAccept = function()
			end,
			timeout = 0,
			showAlert = 1,
			whileDead = 1
		}
		StaticPopup_Show("recap_temp.REMINDGROUP")
	end
	recap.Opt.NumPartyMembers.value = count
end

function Recap_EnterLeaveInstance()
	-- If option is on, remind player on entering and leaving an instance
	-- GetPlayerMapPosition returns exacty zero for both values when in an instance
	local newX, newY, newInside
	newX, newY = GetPlayerMapPosition("player")
	if (newX == 0) and (newY == 0) then
		newInside = true
	else
		newInside = false
	end
	if (recap.Opt.InsideInstance.value == false) and (newInside == true) and recap.Opt.RemindInstanceStatus.value then
		StaticPopupDialogs["recap_temp.REMINDINSTANCE"] = {
			text = TEXT(recap_temp.Localize.RemindEnterInstance),
			button1 = TEXT(OKAY),
			OnAccept = function()
			end,
			timeout = 0,
			showAlert = 1,
			whileDead = 1
		}
		StaticPopup_Show("recap_temp.REMINDINSTANCE")
	elseif (recap.Opt.InsideInstance.value == true) and (newInside == false) and recap.Opt.RemindInstanceStatus.value then
		StaticPopupDialogs["recap_temp.REMINDINSTANCE"] = {
			text = TEXT(recap_temp.Localize.RemindLeaveInstance),
			button1 = TEXT(OKAY),
			OnAccept = function()
			end,
			timeout = 0,
			showAlert = 1,
			whileDead = 1
		}
		StaticPopup_Show("recap_temp.REMINDINSTANCE")
	end
	recap.Opt.InsideInstance.value = newInside
end

function Recap_SlashHandler(arg1)

	local i

	if not Recap_Initialize() then
		-- TODO: these puppies aren't localized
		DEFAULT_CHAT_FRAME:AddMessage("Recap is not initialized, try re-installing.", 1.0, 0.45, 0.0)
		return false
	end

	if arg1 and string.find(arg1, "cent", 1, true) then  -- short for centre or center
		RecapFrame:ClearAllPoints()
		RecapFrame:SetPoint("CENTER","UIParent","CENTER")
		RecapOptFrame:ClearAllPoints()
		RecapOptFrame:SetPoint("CENTER","UIParent","CENTER")
		recap.Opt.Pinned.value = false
		Recap_SetView()
		RecapFrame_Show()
		Recap_Maximize()
		RecapFrame:SetAlpha(1)
		recap_temp.FadeTimer = -1

	elseif arg1=="debug" then
		if recap.debug then
			-- TODO: these puppies aren't localized
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug is ON. To turn it off: /recap debug off")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug is OFF. To turn it on: /recap debug on")
		end
	elseif arg1=="debug on" then
		if recap.debug then
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug was on, and is still ON.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug was off, and is now ON.")
		end
		recap.debug = true
	elseif arg1=="debug off" then
		if recap.debug then
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug was on, and is now OFF.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug was off, and is still OFF.")
		end
		recap.debug = false

	-- Hawksy: debug setting, to automatically screenshot
	elseif arg1=="snap" then
		if recap_temp.snap then
			DEFAULT_CHAT_FRAME:AddMessage("recap_temp.snap is ON. To turn it off: /recap snap off")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap_temp.snap is OFF. To turn it on: /recap snap on")
		end
	elseif arg1=="snap on" then
		if recap_temp.snap then
			DEFAULT_CHAT_FRAME:AddMessage("recap_temp.snap was on, and is still ON.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap_temp.snap was off, and is now ON.")
		end
		recap_temp.snap = true
	elseif arg1=="snap off" then
		if recap_temp.snap then
			DEFAULT_CHAT_FRAME:AddMessage("recap_temp.snap was on, and is now OFF.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap_temp.snap was off, and is still OFF.")
		end
		recap_temp.snap = false

	-- Hawksy: lightly documented setting, to calculate gross minima for Self rather than net minima
	elseif arg1=="gmin" then
		if recap.gmin then
			DEFAULT_CHAT_FRAME:AddMessage("recap.gmin is ON. To turn it off: /recap gmin off")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.gmin is OFF. To turn it on: /recap gmin on")
		end
	elseif arg1=="gmin on" then
		if recap.gmin then
			DEFAULT_CHAT_FRAME:AddMessage("recap.gmin was on, and is still ON.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.gmin was off, and is now ON.")
		end
		recap.gmin = true
	elseif arg1=="gmin off" then
		if recap.gmin then
			DEFAULT_CHAT_FRAME:AddMessage("recap.gmin was on, and is now OFF.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.gmin was off, and is still OFF.")
		end
		recap.gmin = false

	-- Hawksy: debug setting, to dump ParserLib events to WOWChatLog.txt
	elseif arg1=="dump" then
		if recap.dump then
			DEFAULT_CHAT_FRAME:AddMessage("recap.dump is ON. To turn it off: /recap dump off")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.dump is OFF. To turn it on: /recap dump on")
		end
	elseif arg1=="dump on" then
		if recap_temp.LogChatnum then
			-- have a log channel, WOWChatLog should be open with logging turned on
			if recap.dump then
				DEFAULT_CHAT_FRAME:AddMessage("recap.dump was on, and is still ON.")
			else
				DEFAULT_CHAT_FRAME:AddMessage("recap.dump was off, and is now ON.")
			end
			ChatThrottleLib:SendChatMessage("BULK", "Recap", string.format(recap_temp.Localize.LogHeader,(UnitName("player")),recap_temp.LastAll[recap.Opt.View.value],date(),GetRealZoneText()), "CHANNEL", nil, recap_temp.LogChatnum)
			recap.dump = true
			recap.dumplines = 0
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.dump logging not on, please go to Options / Reports and Open the WOWChatLog.txt file.")
		end
	elseif arg1=="dump off" then
		if recap.dump then
			DEFAULT_CHAT_FRAME:AddMessage("recap.dump was on, and is now OFF.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.dump was off, and is still OFF.")
		end
		recap.dump = false

	elseif arg1 and string.find(arg1, "opt", 1, true) then  -- short for option
		if RecapOptFrame:IsShown() then
			RecapOptFrame:Hide()
		else
			RecapOptFrame:Show()
			Recap_SetSyncDisplay()
		end

	elseif RecapFrame:IsShown() then
		RecapFrame_Hide()
		RecapOptFrame:Hide()
		-- TODO: these puppies aren't localized
		DEFAULT_CHAT_FRAME:AddMessage("Recap has been hidden.")

	else
		RecapFrame_Show()
		Recap_CheckWindowBounds()
		if recap.Opt.State.value == "Stopped" then
			Recap_ClearAsIfEndOfFight()
			Recap_SetState("Idle")
			recap.Opt.Paused.value = false
			Recap_SetButtons()
		end
		-- TODO: these puppies aren't localized
		DEFAULT_CHAT_FRAME:AddMessage("If Recap is still not visible, try '/recap centre'.")
	end
end

--[[ Window appearance functions ]]

function Recap_SetState(arg1)
	if arg1 then
		recap.Opt.State.value = arg1
	end
	RecapUpdateFrame:Hide() -- disable OnUpdate
	if recap.Opt.State.value=="Stopped" then
		RecapStatusTexture:SetVertexColor(1,0,0)
		Recap_Unregister_CombatEvents()
	elseif recap.Opt.State.value=="Active" then
		RecapStatusTexture:SetVertexColor(0,1,0)
		Recap_Register_CombatEvents()
	elseif recap.Opt.State.value=="Idle" then
		RecapStatusTexture:SetVertexColor(.5,.5,.5)
		Recap_Register_CombatEvents()
	end
	if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
		RecapMinSyncStatusTexture:SetVertexColor(0,0,1)
	else
		RecapMinSyncStatusTexture:SetVertexColor(.5,.5,.5)
	end
	Recap_SetButtons()
	Recap_UpdatePlugins()
	if recap.Opt.State.value~="Stopped" then
		RecapUpdateFrame:Show() -- enable OnUpdate
	end
end

function RecapFrame_Hide()

	if not recap_temp.Loaded then
		if not Recap_Initialize() then
			return false
		end
	end
	recap_temp.FadeTimer = -1
	RecapFrame:SetAlpha(1)
	recap.Opt.Visible.value = false
	RecapFrame:Hide()
end

function RecapFrame_Show()

	if not recap_temp.Loaded then
		if not Recap_Initialize() then
			return false
		end
	end
	recap.Opt.Visible.value = true
	RecapFrame:Show()
end

-- Hawksy: catch Escape when maximized, and do a minimize instead; and genuinely hide otherwise
function RecapFrame_HideOrMinimize()

	if not recap_temp.Loaded then
		if not Recap_Initialize() then
			return false
		end
	end
	if recap.Opt.Visible.value then
		-- other code wants it visible, so this must be an Escape, so minimize and show
		-- Minimize will disable the special status of RecapFrame so that it lets the Escape through
		-- Maximize will enable the special status of RecapFrame so that it captures the Escape
		if recap.Opt.MinimizeOnEscape.value then
			-- but only minimize if the option is set
			Recap_Minimize()
		end
		RecapFrame:Show()
	else
		-- other code wants it hidden, so let it happen (called from OnHide, so don't recurse)
	end
end

function RecapOptFrame_Hide()
	RecapOptFrame:Hide()
end

function RecapFrame_Toggle()
	if RecapFrame:IsShown() then
		RecapFrame_Hide()
		RecapOptFrame:Hide()
	else
		RecapFrame_Show()
	end
end

function Recap_Shutdown()

	recap.Opt.Paused.value = true
	recap_temp.HoldFight = "Normal"
	Recap_EndFight(true)
	Recap_SetState("Stopped")
	Recap_ClearAsIfEndOfFight()
	Recap_SetButtons()
	RecapOptFrame:Hide()
	RecapFrame_Hide()

	-- Hawksy: cleanup for 3.59 and later
	local i, iCombatant
	for i in pairs(recap.Combatant) do
		if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
			iCombatant = recap.Combatant[i]
			if iCombatant.Incoming then
				iCombatant.Incoming = nil -- keep as nil
			end
			if iCombatant.Detail then
				iCombatant.Detail = nil -- keep as nil
			end
		end
	end
end

function Recap_ToggleView()

	if recap.Opt.View.value=="All" then
		Recap_SetView("Last")
	else
		Recap_SetView("All")
	end
	Recap_SetColumns()
end

-- changes button textures depending on their state
function Recap_SetButtons()

	local i, iButton
	local _G = getfenv(0)

	for i in pairs(recapbuttons) do
		iButton = _G[recapbuttons[i]]
		if recap.Opt[i].value then
			iButton:SetNormalTexture("Interface\\AddOns\\Recap\\Images\\Recap-Buttons-Down")
			iButton:SetPushedTexture("Interface\\AddOns\\Recap\\Images\\Recap-Buttons-Up")
		else
			iButton:SetNormalTexture("Interface\\AddOns\\Recap\\Images\\Recap-Buttons-Up")
			iButton:SetPushedTexture("Interface\\AddOns\\Recap\\Images\\Recap-Buttons-Down")
		end
	end

	if recap.Opt.View.value=="Last" then
		RecapShowAllButton:SetNormalTexture("Interface\\AddOns\\Recap\\Images\\Recap-Buttons-Up")
		RecapShowAllButton:SetPushedTexture("Interface\\AddOns\\Recap\\Images\\Recap-Buttons-Down")
		if recap_temp.HoldFight == "HoldNext" then
			RecapHoldButton:SetText(RECAP_WAITING)
			RecapHoldButton:Disable()
		elseif recap_temp.HoldFight == "Holding" then
			RecapHoldButton:SetText(RECAP_RESUME)
			RecapHoldButton:Enable()
		else
			recap_temp.HoldFight = "Normal"
			RecapHoldButton:SetText(RECAP_HOLD)
			RecapHoldButton:Enable()
		end
		if recap.Opt.Minimized.value then
			RecapHoldButton:Hide()
		else
			RecapHoldButton:Show()
		end
	else
		RecapShowAllButton:SetNormalTexture("Interface\\AddOns\\Recap\\Images\\Recap-Buttons-Down")
		RecapShowAllButton:SetPushedTexture("Interface\\AddOns\\Recap\\Images\\Recap-Buttons-Up")
		RecapHoldButton:Hide()
	end

	if recap.Opt.SelfView.value then
		RecapHoldButton:Hide()
	end
end

-- this resizes the window width to the columns defined in options
function Recap_SetColumns()

	local i, j, cx, item, subitem, iOpt
	local _G = getfenv(0)

	-- Hawksy: name width was 120 changed to 144 to allow for longer names, especially
	--   under Spells and Abilities on the "Personal Details" panel.
	-- Matching changes made in Recap.xml for parent_EName, parent_Name, RecapHeader_Name, and
	--   RecapHeader_EName.
	cx = recap_temp.ScrollAndEdgeWidth + recap_temp.NameWidth

	if recap.Opt.SelfView.value then
		-- personal view
		for i in pairs(recap.Opt) do
			iOpt = recap_temp.DefaultOpt[i]
			if iOpt.ewidth then
				if recap.Opt[i].value then
					_G["RecapHeader_"..i]:SetWidth(iOpt.ewidth)
					_G["RecapHeader_"..i]:Show()
					for j=1,15 do
						_G["RecapSelf"..j.."_"..i]:SetWidth(iOpt.ewidth)
						_G["RecapSelf"..j.."_"..i]:Show()
					end
					cx = cx + iOpt.ewidth
				else
					_G["RecapHeader_"..i]:SetWidth(1)
					_G["RecapHeader_"..i]:Hide()
					for j=1,15 do
						_G["RecapSelf"..j.."_"..i]:SetWidth(1)
						_G["RecapSelf"..j.."_"..i]:Hide()
					end
					cx = cx + 1
				end
			elseif iOpt.width then
				_G["RecapHeader_"..i]:Hide()
			end
		end

		RecapTotals:Hide()
		RecapHeader_Name:Hide()
		RecapHeader_EName:Show()

		for i=1,15 do
			_G["RecapList"..i]:Hide()
			_G["RecapSelf"..i]:Show()
			_G["RecapSelf"..i]:SetWidth(cx - recap_temp.ScrollAndEdgeWidth)
		end

	else
		-- general view
		for i in pairs(recap.Opt) do
			iOpt = recap_temp.DefaultOpt[i]
			if iOpt and iOpt.width then
				if recap.Opt[i].value then
					-- column will appear
					item = _G["RecapHeader_"..i]
					item:SetWidth(iOpt.width)
					item:Show()
					item = _G["RecapTotals_"..i]
					item:SetWidth(iOpt.width)
					item:Show()
					-- Hawksy: Faction and Level always show and hide together
					if i == "Faction" then
						-- Hawksy: split the width (hardcoded numbers)
						item:SetWidth(14)
						subitem = _G["RecapTotals_Level"]
						subitem:SetWidth(18)
						subitem:Show()
					end
					for j=1,15 do
						item = _G["RecapList"..j.."_"..i]
						item:SetWidth(iOpt.width)
						item:Show()
						-- Hawksy: Faction and Level always show and hide together
						if i == "Faction" then
							-- Hawksy: split the width (hardcoded numbers)
							item:SetWidth(14)
							subitem = _G["RecapList"..j.."_Level"]
							subitem:SetWidth(18)
							subitem:Show()
						end
					end
					cx = cx + iOpt.width
				else
					-- column will not appear
					item = _G["RecapHeader_"..i]
					item:SetWidth(1)
					item:Hide()
					-- Hawksy: Ranks, Faction, Class, and Level column fix (see below)
					if (i == "Ranks") or (i == "Faction") or (i == "Class") then
						item:Show()
					end
					-- there is no header for Level
					item = _G["RecapTotals_"..i]
					item:SetWidth(1)
					item:Hide()
					-- Hawksy: Ranks, Faction, Class, and Level column fix (see below)
					if (i == "Ranks") or (i == "Faction") or (i == "Class") then
						item:Show()
					end
					-- Hawksy: Faction and Level always show together
					if i == "Faction" then
						subitem = _G["RecapTotals_Level"]
						subitem:SetWidth(1)
						subitem:Show()
						cx = cx + 1 -- two columns here, not just one
					end
					for j=1,15 do
						item = _G["RecapList"..j.."_"..i]
						item:SetWidth(1)
						item:Hide()
						-- Hawksy: Ranks, Faction, Class, and Level column fix
						--   To fix a display bug these columns always appear, even if
						--   only blank (or transparent black) and one column wide.
						--   See also RecapScrollBar_Update and Recap_ConstructList.
						if (i == "Ranks") or (i == "Faction") or (i == "Class") then
							item:Show()
						end
						-- Hawksy: Faction and Level always show together
						if i == "Faction" then
							subitem = _G["RecapList"..j.."_Level"]
							subitem:SetWidth(1)
							subitem:Show()
						end
					end
					cx = cx + 1
				end
			elseif iOpt and iOpt.ewidth then
				_G["RecapHeader_"..i]:Hide()
			end
		end

		for i=1,15 do
			_G["RecapSelf"..i]:Hide()
			_G["RecapList"..i]:Show()
			_G["RecapList"..i]:SetWidth(cx - recap_temp.ScrollAndEdgeWidth)
		end

		RecapTotals:Show()
		RecapHeader_Name:Show()
		RecapHeader_EName:Hide()

	end

	-- Hawksy: the name slot on the Total line is much narrower
	-- see also the offset in Recap.xml
	RecapTotals_Name:SetWidth(recap_temp.NameWidth-142)
	RecapTotals:SetWidth(cx-142)

	if recap.Opt.GrowLeftwards.value and RecapFrame then
		i = RecapFrame:GetRight()
		j = RecapFrame:GetTop()
		if i and j then
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("TOPLEFT","UIParent","BOTTOMLEFT",i-cx,j) -- *** i not defined sometimes :( **
		else
			-- default behaviour if any undefined
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("CENTER","UIParent","CENTER")
		end
	end

	RecapTopBar:SetWidth(cx-8)
	RecapBottomBar:SetWidth(cx-16)
	RecapFrame:SetWidth(cx)
	RecapScrollBar:SetWidth(cx-16)

	recap_temp.GaugeWidth = cx - recap_temp.ScrollAndEdgeWidth - 3 -
							( (not recap.Opt.SelfView.value) and
							   ((((recap.Opt.Faction.value and recap.Opt.Faction.width) or 1) +
								 ((recap.Opt.Class.value and recap.Opt.Class.width) or 1) +
								 ((recap.Opt.Ranks.value and recap.Opt.Ranks.width) or 1)) or 0) or 0)

	Recap_CheckWindowBounds()
	Recap_SetView()
	if not recap.Opt.Minimized.value then
		RecapScrollBar_Update()
	end

end

-- changes view mode
function Recap_SetView(arg1)

	if arg1 then
		recap.Opt.View.value = arg1
	end

	local wasMinimized = recap.Opt.Minimized.value
	if wasMinimized then
		-- big hammer hack to fix a display anomaly that occurs when switching between All Fights and Last Fights view while minimised
		Recap_Maximize()
	end
	Recap_SetButtons()
	Recap_ConstructList()
	Recap_TitleBar()
	RecapHeader_Name:SetText((recap_temp.ListSize-1).." "..RECAP_COMBATANTS)
	Recap_ChangeBack()
	if wasMinimized then
		Recap_Minimize()
	end
end

function Recap_TitleBar()

	local cx, text, index

	cx = RecapFrame:GetWidth()
	if cx and cx>300 and recap.Opt.SelfView.value then
		text = recap_temp.Player..": "..recap_temp.Localize.Details
	elseif cx and cx>230 and recap.Opt.SelfView.value then
		text = recap_temp.Player
	elseif cx and cx<230 or recap.Opt.Minimized.value then
		text = ""
		if cx>180 and not recap.Opt.SelfView.value then
			text = recap.Opt.View.value
		end
	elseif cx and cx<260 then
		text = recap_temp.LastAll[recap.Opt.View.value]
	else
		text = recap_temp.Recap.." "..recap_temp.Localize.Of.." "..recap_temp.LastAll[recap.Opt.View.value]
	end

	-- add sync member count
	if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
		index = 0
		for i in pairs(recap_temp.SyncMembers) do
			index = index + 1
		end
		text = text.."   ("..tostring(index).." "..recap_temp.Localize.InSync..")"
	end

	RecapTitle:SetText(text)
end

function Recap_SetColors()

	local i
	local _G = getfenv(0)

	if recap.Opt.UseColor.value then
		recap_temp.ColorDmgIn = recap_temp.ColorRed
		recap_temp.ColorDmgOut = recap_temp.ColorGreen
		recap_temp.ColorHeal = recap_temp.ColorBlue
		recap_temp.ColorDmgInPale = recap_temp.ColorRedPale
		recap_temp.ColorDmgOutPale = recap_temp.ColorGreenPale
		recap_temp.ColorHealPale = recap_temp.ColorBluePale
		recap_temp.ColorGlances = recap_temp.ColorLightCyan
		recap_temp.ColorHits = recap_temp.ColorCyan
		recap_temp.ColorTicks = recap_temp.ColorYellow
		recap_temp.ColorCrits = recap_temp.ColorLime
		recap_temp.ColorMiss = recap_temp.ColorPink
	else
		recap_temp.ColorDmgIn = recap_temp.ColorWhite
		recap_temp.ColorDmgOut = recap_temp.ColorWhite
		recap_temp.ColorHeal = recap_temp.ColorWhite
		recap_temp.ColorDmgInPale = recap_temp.ColorWhite
		recap_temp.ColorDmgOutPale = recap_temp.ColorWhite
		recap_temp.ColorHealPale = recap_temp.ColorWhite
		recap_temp.ColorGlances = recap_temp.ColorWhite
		recap_temp.ColorHits = recap_temp.ColorWhite
		recap_temp.ColorTicks = recap_temp.ColorWhite
		recap_temp.ColorCrits = recap_temp.ColorWhite
		recap_temp.ColorMiss = recap_temp.ColorWhite
	end

	RecapTotals_DmgIn:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
	RecapTotals_DmgOut:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
	RecapTotals_HPS:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
	RecapTotals_DPSIn:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
	RecapTotals_DPS:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
	RecapTotals_DPSvsAll:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
	RecapTotals_Heal:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
	RecapTotals_Over:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
	RecapMinDPSIn_Text:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
	RecapMinDPSOut_Text:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
	for i=1,15 do
		_G["RecapList"..i.."_HealP"]:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
		_G["RecapList"..i.."_Over"]:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
		_G["RecapList"..i.."_DmgInP"]:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
		_G["RecapList"..i.."_DmgOutP"]:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
		_G["RecapSelf"..i.."_EGlances"]:SetTextColor(recap_temp.ColorGlances.r,recap_temp.ColorGlances.g,recap_temp.ColorGlances.b)
		_G["RecapSelf"..i.."_EGlancesMin"]:SetTextColor(recap_temp.ColorGlances.r,recap_temp.ColorGlances.g,recap_temp.ColorGlances.b)
		_G["RecapSelf"..i.."_EGlancesAvg"]:SetTextColor(recap_temp.ColorGlances.r,recap_temp.ColorGlances.g,recap_temp.ColorGlances.b)
		_G["RecapSelf"..i.."_EGlancesMax"]:SetTextColor(recap_temp.ColorGlances.r,recap_temp.ColorGlances.g,recap_temp.ColorGlances.b)
		_G["RecapSelf"..i.."_EHits"]:SetTextColor(recap_temp.ColorHits.r,recap_temp.ColorHits.g,recap_temp.ColorHits.b)
		_G["RecapSelf"..i.."_EHitsMin"]:SetTextColor(recap_temp.ColorHits.r,recap_temp.ColorHits.g,recap_temp.ColorHits.b)
		_G["RecapSelf"..i.."_EHitsAvg"]:SetTextColor(recap_temp.ColorHits.r,recap_temp.ColorHits.g,recap_temp.ColorHits.b)
		_G["RecapSelf"..i.."_EHitsMax"]:SetTextColor(recap_temp.ColorHits.r,recap_temp.ColorHits.g,recap_temp.ColorHits.b)
		_G["RecapSelf"..i.."_ETicks"]:SetTextColor(recap_temp.ColorTicks.r,recap_temp.ColorTicks.g,recap_temp.ColorTicks.b)
		_G["RecapSelf"..i.."_ETicksMin"]:SetTextColor(recap_temp.ColorTicks.r,recap_temp.ColorTicks.g,recap_temp.ColorTicks.b)
		_G["RecapSelf"..i.."_ETicksAvg"]:SetTextColor(recap_temp.ColorTicks.r,recap_temp.ColorTicks.g,recap_temp.ColorTicks.b)
		_G["RecapSelf"..i.."_ETicksMax"]:SetTextColor(recap_temp.ColorTicks.r,recap_temp.ColorTicks.g,recap_temp.ColorTicks.b)
		_G["RecapSelf"..i.."_ECrits"]:SetTextColor(recap_temp.ColorCrits.r,recap_temp.ColorCrits.g,recap_temp.ColorCrits.b)
		_G["RecapSelf"..i.."_ECritsMin"]:SetTextColor(recap_temp.ColorCrits.r,recap_temp.ColorCrits.g,recap_temp.ColorCrits.b)
		_G["RecapSelf"..i.."_ECritsAvg"]:SetTextColor(recap_temp.ColorCrits.r,recap_temp.ColorCrits.g,recap_temp.ColorCrits.b)
		_G["RecapSelf"..i.."_ECritsMax"]:SetTextColor(recap_temp.ColorCrits.r,recap_temp.ColorCrits.g,recap_temp.ColorCrits.b)
		_G["RecapSelf"..i.."_ECritsP"]:SetTextColor(recap_temp.ColorCrits.r,recap_temp.ColorCrits.g,recap_temp.ColorCrits.b)
		_G["RecapSelf"..i.."_EMiss"]:SetTextColor(recap_temp.ColorMiss.r,recap_temp.ColorMiss.g,recap_temp.ColorMiss.b)
		_G["RecapSelf"..i.."_EMissP"]:SetTextColor(recap_temp.ColorMiss.r,recap_temp.ColorMiss.g,recap_temp.ColorMiss.b)
	end
	Recap_ChangeBack()
end

function Recap_ChangeBack()

	if recap.Opt.UseColor.value and (recap.Opt.MinBack.value or not recap.Opt.Minimized.value) then

		if recap.Opt.SelfView.value and not recap.Opt.Minimized.value then
			RecapTitleBack:SetVertexColor(0,0,.8)
		elseif recap.Opt.View.value=="Last" then
			RecapTitleBack:SetVertexColor(0,.66,0)
		else
			RecapTitleBack:SetVertexColor(.66,0,0)
		end
		RecapTitleBack:Show()
	else
		RecapTitleBack:Hide()
	end
end


--[[ Initialization ]]

-- Can be called often, will immediately dump out if it ran through successfully
function Recap_Initialize()

	local i, x, iPlayer, iCombatant
	local init_time

	if not recap_temp.Loaded then

		if not Recap_IntegrityCheck() then
			return false
		end

		if (UnitName("player")) and (UnitName("player"))~=UNKNOWNOBJECT and RecapFrame and RecapOptFrame then

			init_time = GetTime()

			-- take control of lua error messages (WoW 2.1 defaults off)
			recap_Original_ErrorHandler = geterrorhandler()
			seterrorhandler(Recap_TrapError)

			recap_temp.Player = (UnitName("player"))
			recap_temp.Server = GetCVar("realmName")

			-- recap index for this player
			recap_temp.p = recap_temp.Player..recap_temp.Server -- use for stuff that can be made global
			recap_temp.s = recap_temp.Player..recap_temp.Server -- use for stuff that should never be global

			if recap and recap.UseOneSettings and recap.User then
				recap_temp.p = recap.User
			end

			-- Hawksy: force the dump flags to false, otherwise we could wake up spamming a channel other than our proper log channel
			recap.dump = false
			recap_temp.LogChatnum = false
			recap.debug = false

			-- Ensure that this state variable starts properly
			recap_temp.HoldFight = "Normal"

			-- Hawksy: force snap on for me every time
			if (recap_temp.p == "HawksyElune") or (recap_temp.p == "GalassaElune") or (recap_temp.p == "MetrognomeElune") then
				recap_temp.snap = true
			end

			Recap_InitializeData()

			if recap.Opt.RecentData.value then
				Recap_CreateRecent()
			else
				recap_temp.Recent = {}
			end

			iPlayer = recap[recap_temp.p]
			if not iPlayer.WindowTop and RecapFrame then
				RecapFrame:ClearAllPoints()
				RecapFrame:SetPoint("CENTER","UIParent","CENTER")
				RecapOptFrame:ClearAllPoints()
				RecapOptFrame:SetPoint("CENTER", "UIParent", "CENTER")
			elseif iPlayer.WindowTop and RecapFrame then
				RecapFrame:ClearAllPoints()
				RecapFrame:SetPoint("TOPLEFT","UIParent","BOTTOMLEFT",iPlayer.WindowLeft,iPlayer.WindowTop)
				RecapFrame:SetWidth(iPlayer.WindowWidth)
				RecapFrame:SetHeight(iPlayer.WindowHeight)
			end

			-- Hawksy: silent repair
			if not recap.Combatant then
				recap.Combatant = {}
			end
			for i in pairs(recap.Combatant) do
				if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
					iCombatant = recap.Combatant[i]
					iCombatant.WasInCurrent = false
					iCombatant.WasInLast = false
				end
			end
			iPlayer.LastDuration = 0
			iPlayer.LastDurationIn = 0
			iPlayer.LastDurationHeal = 0
			-- guard code for upgrade for 3.59 and later
			if not iPlayer.TotalDurationIn then
				iPlayer.TotalDurationIn = iPlayer.TotalDuration
			end
			if not iPlayer.TotalDurationHeal then
				iPlayer.TotalDurationHeal = iPlayer.TotalDuration
			end
			-- Hawksy: silent repair
			if not recap.Self[recap_temp.s] then
				recap.Self[recap_temp.s] = {}
			end
			-- not at the moment being done for the player's pets

			Recap_SetColors()

			-- if we loaded in any state except 'Stopped' (i.e. perhaps logged out in mid-fight), force into idle state
			if recap.Opt.State.value ~= "Stopped" then
				recap.Opt.State.value = "Idle"
			end
			Recap_ClearAsIfEndOfFight()
			Recap_SetState(false)

			-- Hawksy: was 82 now 144
			RecapTotals_Name:SetWidth(recap_temp.NameWidth)

			Recap_InitDataSets()

			if recap.Opt.LightData.value then
				recap.Opt.SelfView.value = false
				RecapSelfViewButton:Hide()
			end

			if recap.Opt.Minimized.value then
				Recap_SetButtons()
				Recap_ConstructList()
				Recap_Minimize()
			else
				Recap_Maximize()
			end

			Recap_Register_CombatEvents()

			recap_temp.Loaded = true

			Recap_InitializeOptions()
			Recap_MoveMinimize()
			Recap_SetRecapFrameBackdrop()

			-- initialize panel
			recap_temp.Selected = 0
			if recap.Opt.LightData.value then
				recap.Opt.PanelView.value = 4
				RecapPanelTab1:SetAlpha(.5)
				RecapPanelTab2:SetAlpha(.5)
				RecapPanelTab3:SetAlpha(.5)
			end
			RecapPanel_SwitchPanels(recap.Opt.PanelView.value)
			if recap.Opt.PanelAnchor.value then
				RecapPanel:ClearAllPoints()
				RecapPanel:SetPoint(recap.Opt.PanelAnchor.Panel,"RecapFrame",recap.Opt.PanelAnchor.Main,Recap_PanelOffset("x"),Recap_PanelOffset("y"))
			end

			-- clear any Sync values dependent on time, since we can't get times that are good across reboots
			if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
				recap.Opt.SyncUpdateWhen.value = math.floor(1000*GetTime()) -- integer thousandths
				recap_temp.SyncData = {}
				recap_temp.SyncInviteTimestamp = false
				recap_temp.SyncInviteLeader = false
				recap_temp.SyncInviteMergePets = false
				recap_temp.SyncInviteWhen = false
				recap_temp.SyncInviteNewOngoing = false
				recap_temp.AskingJoinSync = false
				recap_temp.OnCommReceive = false
			end

			-- Hawksy: removed the Alpha reduction for Faction icons

			if recap.Opt.Visible.value then
				RecapFrame_Show()
			else
				RecapFrame_Hide()
			end

			-- set frames that need to see the Escape key
			table.insert(UISpecialFrames,"RecapOptFrame")
			table.insert(UISpecialFrames,"RecapPanel")
			-- Hawksy: added main frame so that we can trap Escape and minimize instead
			if recap.Opt.Minimized.value or not recap.Opt.MinimizeOnEscape.value then
				-- a dummy if we are minimized or have Minimize on Escape turned off, so that we don't capture Escape when minimized
				table.insert(UISpecialFrames,"RecapFrameDummy")
			else
				table.insert(UISpecialFrames,"RecapFrame")
			end

			if recap.debug then
				DEFAULT_CHAT_FRAME:AddMessage("(recap.debug) Time to initialize: "..string.format("%.4f",GetTime()-init_time).." seconds.")
				if (recap_temp.p == "HawksyElune") or (recap_temp.p == "GalassaElune") or (recap_temp.p == "MetrognomeElune") then
					-- for me only, reset CPU profiling numbers
					ResetCPUUsage()
				end
			end

			-- warn if user has a lot of data (over 500 combatants)
			x = 0
			for i in pairs(recap.Combatant) do
				if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
					x = x + 1
				end
			end
			if x>500 and recap.Opt.WarnData.value then
				StaticPopupDialogs["recap_temp.Localize.LotsOfDataWarning"] = {
					text = TEXT(string.format(recap_temp.Localize.LotsOfDataWarning,x)),
					button1 = TEXT(OKAY),
					timeout = 0,
					showAlert = 1,
					whileDead = 1
				}
				StaticPopup_Show("recap_temp.Localize.LotsOfDataWarning")
			end
		end
	end

	return true
end

function Recap_IntegrityCheck()
	-- Hawksy: this statement, and the corresponding declarations, need to be updated for every version
	--   (an attempt at a simple package integrity check, we'll see if I can live with it)
	if not Recap_lua_372 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (Recap.lua)", 1.0, 0.45, 0.0)
		Recap_Shutdown()
		return false
	end
	if not RecapCombat_lua_372 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (RecapCombat.lua)", 1.0, 0.45, 0.0)
		Recap_Shutdown()
		return false
	end
	if not RecapSync_lua_372 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (RecapSync.lua)", 1.0, 0.45, 0.0)
		Recap_Shutdown()
		return false
	end
	if not RecapAux_lua_372 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (RecapAux.lua)", 1.0, 0.45, 0.0)
		Recap_Shutdown()
		return false
	end
	if not RecapOptions_lua_372 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (RecapOptions.lua)", 1.0, 0.45, 0.0)
		Recap_Shutdown()
		return false
	end
	if not RecapPanel_lua_372 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (RecapPanel.lua)", 1.0, 0.45, 0.0)
		Recap_Shutdown()
		return false
	end
	if not Recap_xml_372 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (Recap.xml)", 1.0, 0.45, 0.0)
		Recap_Shutdown()
		return false
	end
	if not RecapOptions_xml_372 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (RecapOptions.xml)", 1.0, 0.45, 0.0)
		Recap_Shutdown()
		return false
	end
	if not RecapPanel_xml_372 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (RecapPanel.xml)", 1.0, 0.45, 0.0)
		Recap_Shutdown()
		return false
	end
	if not localization_lua_372 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (localization.lua)", 1.0, 0.45, 0.0)
		Recap_Shutdown()
		return false
	end
	if not localization_de_lua_372 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (localization.de.lua)", 1.0, 0.45, 0.0)
		Recap_Shutdown()
		return false
	end
	if not localization_cn_lua_372 then
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.VersionCheck.." (localization.cn.lua)", 1.0, 0.45, 0.0)
		Recap_Shutdown()
		return false
	end
	return true
end

-- arg1==1 to hide buttons, 16 to show
function Recap_ShowButtons(arg1)

	RecapCloseButton:SetWidth(arg1)
	RecapPinButton:SetWidth(arg1)
	RecapPauseButton:SetWidth(arg1)
	RecapOptionsButton:SetWidth(arg1)
	RecapShowAllButton:SetWidth(arg1)

	if arg1==16 then
		RecapCloseButton:Show()
		RecapPinButton:Show()
		RecapPauseButton:Show()
		RecapOptionsButton:Show()
		RecapShowAllButton:Show()
	else
		RecapCloseButton:Hide()
		RecapPinButton:Hide()
		RecapPauseButton:Hide()
		RecapOptionsButton:Hide()
		RecapShowAllButton:Hide()
	end
end

function Recap_Minimize()

	local i, j, iOpt
	local _G = getfenv(0)
	local cx = 20 -- 8(edge)+4(space right of status)+8

	recap.Opt.Minimized.value = true

	-- Hawksy: to support the desired Escape behaviour, obscure the special status of RecapFrame so that it doesn't capture Escape when minimized
	local k, m
	for k,m in ipairs(UISpecialFrames) do
		if m == "RecapFrame" then
			UISpecialFrames[k] = "RecapFrameDummy"
		end
	end

	for i in pairs(recap.Opt) do
		iOpt = recap_temp.DefaultOpt[i]
		if iOpt.minwidth then
			if recap.Opt[i].value then
				item = _G["Recap"..i]
				item:SetWidth(iOpt.minwidth)
				item:Show()
				cx = cx + iOpt.minwidth
			else
				item = _G["Recap"..i]
				item:SetWidth(1)
				item:Hide()
				cx = cx + 1
			end
		elseif iOpt.width or iOpt.ewidth then
			_G["RecapHeader_"..i]:Hide()
		end
	end

	if recap.Opt.MinButtons.value then
		cx = cx + 108
		Recap_ShowButtons(16)
	else
		Recap_ShowButtons(1)
		cx = cx + 23 -- 5 + 18
	end

	RecapTitle:SetText(recap.Opt.View.value)

	RecapTopBar:Hide()
	RecapBottomBar:Hide()
	RecapResetButton:Hide()
	RecapHoldButton:Hide()

	for i=1,15 do
		_G["RecapList"..i]:Hide()
		_G["RecapSelf"..i]:Hide()
	end
	RecapTotals:Hide()
	RecapHeader_Name:Hide()
	RecapHeader_EName:Hide()
	RecapScrollBar:Hide()

	if recap.Opt.AutoMinimize.value then
		cx = cx-15
	end

	if recap.Opt.GrowLeftwards.value and RecapFrame then
		i = RecapFrame:GetRight()
		j = RecapFrame:GetTop()
		if i and j then
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("TOPLEFT","UIParent","BOTTOMLEFT",i-cx,j)
		else
			-- default behaviour if any undefined
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("CENTER","UIParent","CENTER")
		end
	end
	RecapFrame:SetWidth(cx)
	Recap_SetHeight(28)

	if recap.Opt.MinBack.value then
		RecapFrame:SetBackdropColor(1,1,1,1)
		RecapFrame:SetBackdropBorderColor(.5,.5,.5,1)
	else
		RecapFrame:SetBackdropColor(0,0,0,0)
		RecapFrame:SetBackdropBorderColor(0,0,0,0)
	end

	RecapSelfViewButton:Hide()
	RecapPanel_Hide(1)
	Recap_MoveMinimize()
	Recap_ChangeBack()

	RecapMinView:GetLeft()

end

function Recap_Maximize()

	local i, iOpt
	local _G = getfenv(0)

	recap.Opt.Minimized.value = false

	Recap_SetColumns()

	for i in pairs(recap.Opt) do
		iOpt = recap_temp.DefaultOpt[i]
		if iOpt.minwidth then
			item = _G["Recap"..i]
			item:SetWidth(iOpt.minwidth)
			item:Hide()
		end
	end
	RecapMinStatus:Show()
	RecapMinSyncStatus:Show()
	RecapMinView:Show()
	Recap_ShowButtons(16)

	RecapTopBar:Show()
	RecapBottomBar:Show()
	RecapResetButton:Show()
	if recap.Opt.View.value=="Last" then
		if recap_temp.HoldFight == "HoldNext" then
			RecapHoldButton:SetText(RECAP_WAITING)
			RecapHoldButton:Disable()
		elseif recap_temp.HoldFight == "Holding" then
			RecapHoldButton:SetText(RECAP_RESUME)
			RecapHoldButton:Enable()
		else
			recap_temp.HoldFight = "Normal"
			RecapHoldButton:SetText(RECAP_HOLD)
			RecapHoldButton:Enable()
		end
		RecapHoldButton:Show()
	else
		RecapHoldButton:Hide()
	end

	RecapScrollBar:Show()

	if recap.Opt.SelfView.value then
		RecapTotals:Hide()
		RecapHeader_Name:Hide()
		RecapHeader_EName:Show()
	else
		RecapTotals:Show()
		RecapHeader_Name:Show()
		RecapHeader_EName:Hide()
	end

	RecapFrame:SetBackdropColor(1,1,1,1)
	RecapFrame:SetBackdropBorderColor(.5,.5,.5,1)
	if not recap.Opt.LightData.value then
		RecapSelfViewButton:Show()
	end
	Recap_MoveMinimize()

	-- Hawksy: to support the desired Escape behaviour, make RecapFrame special so that it captures Escape when maximized (but only when we have Minimize on Escape turned on)
	if recap.Opt.MinimizeOnEscape.value then
		local k, m
		for k,m in ipairs(UISpecialFrames) do
			if m == "RecapFrameDummy" then
				UISpecialFrames[k] = "RecapFrame"
			end
		end
	end

end

function Recap_MoveMinimize()

	RecapCloseButton:ClearAllPoints()
	RecapCloseButton:SetPoint("TOPRIGHT", "RecapFrame", "TOPRIGHT", -6, -6)
	RecapMinStatus:ClearAllPoints()
	RecapMinStatus:SetPoint("TOPLEFT", "RecapFrame", "TOPLEFT", 6, -7)
	RecapResetButton:ClearAllPoints()
	RecapResetButton:SetPoint("BOTTOMLEFT", "RecapFrame", "BOTTOMLEFT", 5, 4)
	RecapMinimizeButton:ClearAllPoints()

	if recap.Opt.GrowLeftwards.value and not recap.Opt.GrowUpwards.value then -- topright
		RecapMinimizeButton:SetPoint("TOPRIGHT", "RecapFrame", "TOPRIGHT", -6, -6)
		RecapCloseButton:SetPoint("TOPRIGHT", "RecapMinimizeButton", "TOPLEFT", -2, 0)
	elseif recap.Opt.GrowLeftwards.value and recap.Opt.GrowUpwards.value then -- bottomright
		RecapMinimizeButton:SetPoint("BOTTOMRIGHT", "RecapFrame", "BOTTOMRIGHT", -6, 6)
		if recap.Opt.Minimized.value then
			RecapCloseButton:SetPoint("TOPRIGHT", "RecapMinimizeButton", "TOPLEFT", -2, 0)
		end
	elseif not recap.Opt.GrowLeftwards.value and not recap.Opt.GrowUpwards.value then -- topleft
		RecapMinimizeButton:SetPoint("TOPLEFT", "RecapFrame", "TOPLEFT", 6, -6)
		RecapMinStatus:SetPoint("TOPLEFT", "RecapMinimizeButton", "TOPRIGHT", 2, -1)
	else -- bottomleft
		RecapMinimizeButton:SetPoint("BOTTOMLEFT", "RecapFrame", "BOTTOMLEFT", 6, 6)
		RecapResetButton:SetPoint("BOTTOMLEFT", "RecapFrame", "BOTTOMLEFT", 28, 4)
	end

	if recap.Opt.Minimized.value and not recap.Opt.GrowLeftwards.value then
		RecapMinStatus:SetPoint("TOPLEFT", "RecapMinimizeButton", "TOPRIGHT", 2, -1)
	end

	if recap.Opt.AutoMinimize.value then
		RecapMinimizeButton:SetWidth(1)
		RecapMinimizeButton:Hide()
	else
		RecapMinimizeButton:SetWidth(16)
		RecapMinimizeButton:Show()
	end

end

function Recap_SetHeight(newcy)

	local i,j

	if recap.Opt.GrowUpwards.value and RecapFrame then
		i = RecapFrame:GetBottom()
		j = RecapFrame:GetLeft()
		if i and j then
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("TOPLEFT","UIParent","BOTTOMLEFT",j,i+newcy)
		else
			-- default behaviour if any undefined
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("CENTER","UIParent","CENTER")
		end
	end

	RecapFrame:SetHeight(newcy)
	Recap_CheckWindowBounds()
end

function RecapScrollBar_Update()

	-- only called when not minimised

	local i, j, index, item
	local listsize = recap.Opt.SelfView.value and recap_temp.SelfListSize or recap_temp.ListSize
	local thisCombatant, iCombatant, iList
	local _G = getfenv(0)

	i = 72 + ( math.max( math.min(listsize-1,recap.Opt.MaxRows.value),1 )*14 )
	Recap_SetHeight(i)
	RecapScrollBar:SetHeight(i-63)
	FauxScrollFrame_Update(RecapScrollBar,listsize-1,recap.Opt.MaxRows.value,14)

	for i=1,recap.Opt.MaxRows.value do
		index = i + FauxScrollFrame_GetOffset(RecapScrollBar)

		if not recap.Opt.SelfView.value then
			-- general view

			if index < listsize then
				-- Hawksy: guard code
				iList = recap_temp.List[index]
				thisCombatant = iList.Name
				if thisCombatant then
					iCombatant = recap.Combatant[thisCombatant]
					if iCombatant then

						-- Hawksy: there is a display problem when switching from Personal Details
						--   to All Fights or Last Fight if the fights display begins with hidden
						--   columns.  The workaround appears to be to force the first four columns
						--   (Ranks, Faction, Class, and Level) to always appear (though perhaps
						--   with blank content).
						--   See more code in Recap_SetColumns and Recap_ConstructList,
						--   and a texture addition to RecapHeader_Faction.
						if recap.Opt.Ranks.value then
							-- Ranks will appear
							_G["RecapList"..i.."_Ranks"]:SetText(index..".")
						else
							-- Ranks needs to be set to blank
							_G["RecapList"..i.."_Ranks"]:SetText("")
						end
						if recap.Opt.Faction.value then
							-- Faction and Level will appear
							Recap_SetFactionIcon(i,iCombatant.Faction)
							if iCombatant.Level==-1 then
								_G["RecapList"..i.."_Level"]:SetText("??")
							elseif iCombatant.Level and tonumber(iCombatant.Level)>0 then
								_G["RecapList"..i.."_Level"]:SetText(iCombatant.Level)
							end
						else
							-- Faction icon needs to be set to black, and Level to blank
							Recap_SetFactionIcon(i,nil)
							_G["RecapList"..i.."_Level"]:SetText("")
						end
						if recap.Opt.Class.value then
							-- Class will appear
							Recap_SetClassIcon(i,iCombatant.Class)
						else
							-- Class icon needs to be set to black
							Recap_SetClassIcon(i,nil)
						end

						item = _G["RecapList"..i.."_Name"]
						item:SetText(thisCombatant)
						if iCombatant.Friend then
							item:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						item = _G["RecapList"..i.."_Gauge"]
						if recap.Opt.ShowGauges.value and recap_temp.GaugeMax>0 and iCombatant.Friend and recap_temp.GaugeBy then
							item:SetWidth(1+recap_temp.GaugeWidth*iList[recap_temp.GaugeBy]/recap_temp.GaugeMax)
							item:Show()
						else
							item:Hide()
						end

						_G["RecapList"..i.."_Kills"]:SetText(iList.Kills)

						_G["RecapList"..i.."_Time"]:SetText(Recap_FormatTime(iList.Time))
						_G["RecapList"..i.."_TimeIn"]:SetText(Recap_FormatTime(iList.TimeIn))
						_G["RecapList"..i.."_TimeHeal"]:SetText(Recap_FormatTime(iList.TimeHeal))

						item = _G["RecapList"..i.."_Heal"]
						item:SetText(iList.RawHeal - iList.OverHeal)
						if iCombatant.Friend then
							item:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						_G["RecapList"..i.."_HealP"]:SetText(iCombatant.Friend and iList.HealP.."%" or "")
						_G["RecapList"..i.."_Over"]:SetText(iCombatant.Friend and iList.Over.."%" or "")

						_G["RecapList"..i.."_MaxHit"]:SetText(iList.MaxHit)

						item = _G["RecapList"..i.."_DmgIn"]
						item:SetText(iList.DmgIn)
						if iCombatant.Friend then
							item:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						_G["RecapList"..i.."_DmgInP"]:SetText(iCombatant.Friend and iList.DmgInP.."%" or "")

						item = _G["RecapList"..i.."_DmgOut"]
						item:SetText(iList.DmgOut)
						if iCombatant.Friend then
							item:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						_G["RecapList"..i.."_DmgOutP"]:SetText(iCombatant.Friend and iList.DmgOutP.."%" or "")

						item = _G["RecapList"..i.."_HPS"]
						if iList.HPS<1000 then
							item:SetText(string.format("%.1f",iList.HPS))
						else
							item:SetText(string.format("%d",Recap_Round1(iList.HPS)))
						end
						if iCombatant.Friend then
							item:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end
						item = _G["RecapList"..i.."_DPSIn"]
						if iList.DPSIn<1000 then
							item:SetText(string.format("%.1f",iList.DPSIn))
						else
							item:SetText(string.format("%d",Recap_Round1(iList.DPSIn)))
						end
						if iCombatant.Friend then
							item:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end
						item = _G["RecapList"..i.."_DPS"]
						if iList.DPS<1000 then
							item:SetText(string.format("%.1f",iList.DPS))
						else
							item:SetText(string.format("%d",Recap_Round1(iList.DPS)))
						end
						if iCombatant.Friend then
							item:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						item = _G["RecapList"..i.."_DPSvsAll"]
						if iList.DPSvsAll<1000 then
							item:SetText(string.format("%.1f",iList.DPSvsAll))
						else
							item:SetText(string.format("%d",Recap_Round1(iList.DPSvsAll)))
						end
						if iCombatant.Friend then
							item:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						item = _G["RecapList"..i]
						item:Show()
						if recap_temp.Selected == index then
							item:LockHighlight()
						else
							item:UnlockHighlight()
						end
					end
				end
			else
				_G["RecapList"..i]:Hide()
				_G["RecapList"..i]:UnlockHighlight()
			end

			for i=recap.Opt.MaxRows.value+1,15 do
				_G["RecapList"..i]:Hide()
			end

		elseif recap.Opt.SelfView.value then
			-- personal view

			if index < listsize then
				iList = recap_temp.SelfList[index]

				_G["RecapSelf"..i.."_EName"]:SetText(string.sub(iList.EName,2))
				_G["RecapSelf"..i.."_EElement"]:SetText(iList.EElement)
				_G["RecapSelf"..i.."_ETotal"]:SetText(iList.ETotal)
				_G["RecapSelf"..i.."_EGlances"]:SetText(iList.EGlances)
				_G["RecapSelf"..i.."_EGlancesMin"]:SetText(iList.EGlancesMin)
				_G["RecapSelf"..i.."_EGlancesAvg"]:SetText(iList.EGlancesAvg)
				_G["RecapSelf"..i.."_EGlancesMax"]:SetText(iList.EGlancesMax)
				_G["RecapSelf"..i.."_EHits"]:SetText(iList.EHits)
				_G["RecapSelf"..i.."_EHitsMin"]:SetText(iList.EHitsMin)
				_G["RecapSelf"..i.."_EHitsAvg"]:SetText(iList.EHitsAvg)
				_G["RecapSelf"..i.."_EHitsMax"]:SetText(iList.EHitsMax)
				_G["RecapSelf"..i.."_ETicks"]:SetText(iList.ETicks)
				_G["RecapSelf"..i.."_ETicksMin"]:SetText(iList.ETicksMin)
				_G["RecapSelf"..i.."_ETicksAvg"]:SetText(iList.ETicksAvg)
				_G["RecapSelf"..i.."_ETicksMax"]:SetText(iList.ETicksMax)
				_G["RecapSelf"..i.."_ECrits"]:SetText(iList.ECrits)
				_G["RecapSelf"..i.."_ECritsMin"]:SetText(iList.ECritsMin)
				_G["RecapSelf"..i.."_ECritsAvg"]:SetText(iList.ECritsAvg)
				_G["RecapSelf"..i.."_ECritsMax"]:SetText(iList.ECritsMax)
				_G["RecapSelf"..i.."_ECritsP"]:SetText(iList.ECritsP)
				_G["RecapSelf"..i.."_EMiss"]:SetText(iList.EMiss)
				_G["RecapSelf"..i.."_EMissP"]:SetText(iList.EMissP)
				_G["RecapSelf"..i.."_EMaxAll"]:SetText(iList.EMaxAll)
				_G["RecapSelf"..i.."_ETotalP"]:SetText(iList.ETotalP)
				if (string.sub(iList.EName,1,1) == "1") or (string.sub(iList.EName,1,1) == "2") then
					_G["RecapSelf"..i.."_EName"]:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
					_G["RecapSelf"..i.."_EElement"]:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
					_G["RecapSelf"..i.."_ETotal"]:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
					_G["RecapSelf"..i.."_ETotalP"]:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
				else
					_G["RecapSelf"..i.."_EName"]:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
					_G["RecapSelf"..i.."_EElement"]:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
					_G["RecapSelf"..i.."_ETotal"]:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
					_G["RecapSelf"..i.."_ETotalP"]:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
				end
				_G["RecapSelf"..i]:Show()
			else
				_G["RecapSelf"..i]:Hide()
			end

			for i=recap.Opt.MaxRows.value+1,15 do
				_G["RecapSelf"..i]:Hide()
			end
		end
	end
end


-- returns the current DPS of combatant if it exists.
function Recap_GetCurrentDPS(combatant, iLast)

	local duration, damage

	if iLast and iLast.DmgOut>0 then
		if iLast.StartOut > 0 and (iLast.EndOut > iLast.StartOut) then
			duration = iLast.EndOut - iLast.StartOut
		else
			duration = 0
		end
		damage = iLast.DmgOut
		if recap.Opt.View.value=="All" then
			local iTotal = recap.Combatant[combatant]
			duration = duration + iTotal.TotalTime
			damage = damage + iTotal.TotalDmgOut
		end

		if duration > recap_temp.MinTime then
			return (damage / duration)
		end
	end

	return 0
end
function Recap_GetCurrentDPSMergePets(owner, iLastOwner, pet, iLastPet)

	local duration, damage, startTime, endTime

	if iLastOwner and (iLastOwner.DmgOut>0) and iLastPet and (iLastPet.DmgOut>0) then
		-- both doing damage; merge times and add damages
		startTime = math.min(iLastOwner.StartOut,iLastPet.StartOut)
		endTime = math.max(iLastOwner.EndOut,iLastPet.EndOut)
		if startTime > 0 and (endTime > startTime) then
			duration = endTime - startTime
		else
			duration = 0
		end
		damage = iLastOwner.DmgOut +  iLastPet.DmgOut
		if recap.Opt.View.value=="All" then
			-- for All Fights the TotalTime and TotalDmgOut are already properly merged under the owner
			local iTotal = recap.Combatant[owner]
			duration = duration + iTotal.TotalTime
			damage = damage + iTotal.TotalDmgOut
		end

		if duration > recap_temp.MinTime then
			return (damage / duration)
		end
	else
		-- only one of owner and pet doing damage in this fight so far, use regular code
		return Recap_GetCurrentDPS(owner, iLastOwner) + Recap_GetCurrentDPS(pet, iLastPet)
	end

	return 0
end

function Recap_UpdateMinimizedDPS()

	local i, pet, iLastOwner, iLastPet

	if recap.Opt.State.value=="Active" then

		iLastOwner = recap_temp.Last[recap_temp.Player]
		pet = (UnitName("pet"))
		if pet and pet~=UNKNOWNOBJECT then
			iLastPet = recap_temp.Last[pet]
		else
			iLastPet = nil
		end

		if (iLastOwner and iLastOwner.DmgOut>0) or (iLastPet and iLastPet.DmgOut>0) then
			if Recap_MergePets() then
				i = Recap_GetCurrentDPSMergePets(recap_temp.Player, iLastOwner, pet, iLastPet)
			else
				i = Recap_GetCurrentDPS(recap_temp.Player, iLastOwner) + Recap_GetCurrentDPS(pet, iLastPet)
			end
			RecapMinYourDPS_Text:SetText(string.format("%.1f",Recap_Round1(i)))
		end

		-- calculate running total DPS In and Out
		recap_temp.UpdateDuration = 0
		recap_temp.UpdateDurationIn = 0
		if recap_temp.FightStart>0 and (recap_temp.FightEnd > recap_temp.FightStart) then
			recap_temp.UpdateDuration = recap_temp.FightEnd - recap_temp.FightStart
		end
		if recap_temp.FightStartIn>0 and (recap_temp.FightEndIn > recap_temp.FightStartIn) then
			recap_temp.UpdateDurationIn = recap_temp.FightEndIn - recap_temp.FightStartIn
		end
		if recap.Opt.View.value=="All" then
			local iTotal = recap[recap_temp.p]
			recap_temp.UpdateDuration = recap_temp.UpdateDuration + iTotal.TotalDuration
			recap_temp.UpdateDurationIn = recap_temp.UpdateDurationIn + iTotal.TotalDurationIn
		end

		if recap_temp.UpdateDuration>recap_temp.MinTime or recap_temp.UpdateDurationIn>recap_temp.MinTime then

			recap_temp.UpdateDmgIn = 0
			recap_temp.UpdateDmgOut = 0
			for i in pairs(recap_temp.Last) do
				if recap.Combatant[i].Friend then
					local iLast = recap_temp.Last[i]
					recap_temp.UpdateDmgIn = recap_temp.UpdateDmgIn + iLast.DmgIn
					recap_temp.UpdateDmgOut = recap_temp.UpdateDmgOut + iLast.DmgOut
				end
			end

			if recap.Opt.View.value=="All" then
				for i in pairs(recap.Combatant) do
					if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
						local iTotal = recap.Combatant[i]
						if iTotal.Friend then
							recap_temp.UpdateDmgIn = recap_temp.UpdateDmgIn + iTotal.TotalDmgIn
							recap_temp.UpdateDmgOut = recap_temp.UpdateDmgOut + iTotal.TotalDmgOut
						end
					end
				end
			end

			if recap_temp.UpdateDuration>recap_temp.MinTime then
				RecapMinDPSOut_Text:SetText(string.format("%.1f",Recap_Div1(recap_temp.UpdateDmgOut,recap_temp.UpdateDuration)))
			else
				RecapMinDPSOut_Text:SetText("--")
			end
			if recap_temp.UpdateDurationIn>recap_temp.MinTime then
				RecapMinDPSIn_Text:SetText(string.format("%.1f",Recap_Div1(recap_temp.UpdateDmgIn,recap_temp.UpdateDurationIn)))
			else
				RecapMinDPSIn_Text:SetText("--")
			end

		end

		Recap_UpdatePlugins()
	end
end

function Recap_ConstructSelf()

	local damage, heal, miss, i, j, iSelf, iType

	-- Hawksy: clear the 'SelfList' table entirely
	recap_temp.SelfListSize = 1
	recap_temp.SelfList = {} -- breaks table.sort if we try to re-use the table by setting individual values to nil

	-- Add player, and all of player's pets
	recap_temp.SelfListSize, damage, heal, miss = Recap_AddSelfListPlayerAndPets()

	if recap_temp.SelfListSize>1 then
		for i=1,recap_temp.SelfListSize-1 do
			iSelf = recap_temp.SelfList[i]
			iType = string.sub(iSelf.EName,1,1)
			if ((iType == "1") or (iType == "2")) and damage>0 then
				iSelf.ETotalP = string.format("%d%%",Recap_Div0(100*iSelf.ETotal,damage))
			elseif ((iType == "3") or (iType == "4")) and heal>0 then
				iSelf.ETotalP = string.format("%d%%",Recap_Div0(100*iSelf.ETotal,heal))
			else
				iSelf.ETotalP = "--"
			end
		end
	end

	-- Hawksy: now sort the personal details by any column
	Recap_SortSelfList()

end

function Recap_AddSelfListPlayerAndPets()

	local i, j, k, iSelf, iEffect, iList
	local found, petName
	local idx, damage, heal, miss
	idx = 1
	damage = 0
	heal = 0
	miss = 0

	for i in pairs(recap.Self) do
		if string.find(i, recap_temp.s, 1, true) then
			-- this is the player or one of the player's pets
			iSelf = recap.Self[i]
			for j in pairs(iSelf) do
				iEffect = iSelf[j]
				if not recap_temp.SelfList[idx] then
					recap_temp.SelfList[idx] = {}
				end
				iList = recap_temp.SelfList[idx]
				-- Hawksy: skip effects that have zero damage
				-- Hawksy: note that Self does not (yet) separate out glancing damage
				local total
				total = (iEffect.HitsDmg or 0) + (iEffect.CritsDmg or 0) + (iEffect.TicksDmg or 0)
				if total == 0 then
					iEffect = nil -- keep as nil
				else
					found,_,petName = string.find(i, "^.+:(.+)$")
					if found then
						-- one of the pets
						if string.sub(j,1,1)=="1" then
							iList.EName = "2"..petName..": "..string.sub(j,2)
						else
							iList.EName = "4"..petName..": "..string.sub(j,2)
						end
					else
						-- the player
						iList.EName = j
					end
					iList.EElement = iEffect.Element or "?"
					iList.ETotal = total
					iList.EGlances = iEffect.Glances or 0
					iList.EHits = iEffect.Hits or 0
					iList.ECrits = iEffect.Crits or 0
					iList.ETicks = iEffect.Ticks or 0
					iList.EGlancesMin = iEffect.GlancesMin or 0
					iList.EHitsMin = iEffect.HitsMin or 0
					iList.ETicksMin = iEffect.TicksMin or 0
					iList.ECritsMin = iEffect.CritsMin or 0
					iList.EGlancesMax = iEffect.GlancesMax or 0
					iList.EHitsMax = iEffect.HitsMax or 0
					iList.ETicksMax = iEffect.TicksMax or 0
					iList.ECritsMax = iEffect.CritsMax or 0
					-- Hawksy: make the averages into numbers (not strings) to allow for sorting
					iList.EGlancesAvg = tonumber(string.format("%d",((iList.EGlances>0) and Recap_Div0((iEffect.GlancesDmg or 0),iList.EGlances) or 0)))
					iList.EHitsAvg = tonumber(string.format("%d",((iList.EHits>0) and Recap_Div0((iEffect.HitsDmg or 0),iList.EHits) or 0)))
					iList.ETicksAvg = tonumber(string.format("%d",((iList.ETicks>0) and Recap_Div0((iEffect.TicksDmg or 0),iList.ETicks) or 0)))
					iList.ECritsAvg = tonumber(string.format("%d",((iList.ECrits>0) and Recap_Div0((iEffect.CritsDmg or 0),iList.ECrits) or 0)))
					iList.EMaxAll = math.max( iList.EHitsMax, math.max(iList.ETicksMax,iList.ECritsMax) )
					if (tonumber(string.sub(j,1,1)) == 1) or (tonumber(string.sub(j,1,1)) == 2) then -- damage event
						miss = (iEffect.Missed or 0) + (iEffect.Dodged or 0) + (iEffect.Parried or 0) + (iEffect.Blocked or 0) +
								(iEffect.Deflected or 0) + (iEffect.Resisted or 0) + (iEffect.Reflected or 0) + (iEffect.Absorbed or 0) +
								(iEffect.Immune or 0) + (iEffect.Evaded or 0)
						iList.EMiss = miss
						k = miss + iList.EGlances + iList.EHits + iList.ECrits
						iList.ECritsP = string.format("%.1f%%",(k>0) and Recap_Div1(100*iList.ECrits,k) or 0)
						iList.EMissP = string.format("%.1f%%",(k>0) and Recap_Div1(100*iList.EMiss,k) or 0)
						damage = damage + iList.ETotal
					else
						k = iList.EGlances + iList.EHits + iList.ECrits -- yeah I know, heals probably don't glance, but it doesn't hurt, and if they ever do Recap will be ready
						iList.ECritsP = string.format("%.1f%%",(k>0) and Recap_Div1(100*iList.ECrits,k) or 0)
						iList.EMiss = "--"
						iList.EMissP = "--"
						heal = heal + iList.ETotal
					end

					if iList.ETicks==0 then
						iList.ETicks = "--"
						iList.ETicksMin = "--"
						iList.ETicksAvg = "--"
						iList.ETicksMax = "--"
					end
					if not iEffect.CritsEvents then
						iList.ECrits = "--"
						iList.ECritsMin = "--"
						iList.ECritsAvg = "--"
						iList.ECritsMax = "--"
						iList.ECritsP = "--"
						iList.EHits = "--"
						iList.EHitsMin = "--"
						iList.EHitsAvg = "--"
						iList.EHitsMax = "--"
						iList.EGlances = "--"
						iList.EGlancesMin = "--"
						iList.EGlancesAvg = "--"
						iList.EGlancesMax = "--"
					end

					idx = idx + 1
				end
			end
		end
	end

	return idx, damage, heal, miss
end

function Recap_ResetSelfPlayerAndPets()

	local i

	-- reset owner and pets
	for i in pairs(recap.Self) do
		if string.find(i, recap_temp.s, 1, true) then
			recap.Self[i] = {}
		end
	end
end


function Recap_ConstructList()

	local i, j, dpsin, dpsout, entry, overallDuration, tempname, thisCombatant, iCombatant, iList, iTotal
	local maxhit = 0
	local dmgin = 0
	local dmgout = 0
	local kills = 0
	local rawheal = 0
	local overheal = 0
	local duration = 0
	local durationIn = 0
	local durationHeal = 0
	local overallDPS = 0

	if recap.Opt.SelfView.value then
		Recap_ConstructSelf()
	end

	if (recap_temp.Selected ~= 0) and (recap_temp.Selected ~= recap_temp.FriendlyIndex) and (recap_temp.Selected ~= recap_temp.NonFriendlyIndex) then
		tempname = recap_temp.List[recap_temp.Selected].Name
	else
		tempname = nil
	end

	if recap.Opt.View.value=="Last" then
		overallDuration = recap[recap_temp.p].LastDuration
	else
		overallDuration = recap[recap_temp.p].TotalDuration
	end

	-- Hawksy: clear the 'List' table entirely
	recap_temp.ListSize = 1
	recap_temp.List = {} -- breaks table.sort if we try to re-use the table by setting individual values to nil

	for i in pairs(recap.Combatant) do
		if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
			if not recap_temp.List[recap_temp.ListSize] then
				recap_temp.List[recap_temp.ListSize] = {}
			end
			entry = false

			iCombatant = recap.Combatant[i]
			iList = recap_temp.List[recap_temp.ListSize]

			-- Hawksy: added heals, so that healers show up from the beginning, rather than
			--   only after they have received or given damage.
			if (iCombatant.LastDmgIn==0 and iCombatant.LastDmgOut==0 and
				iCombatant.TotalDmgOut==0 and iCombatant.TotalDmgIn==0 and
				iCombatant.LastRawHeal==0 and iCombatant.TotalRawHeal==0) or
			   (recap.Opt.SelfData.value and not Recap_CheckForSelf(i, iCombatant.OwnedBy)) or
			   (recap.Opt.PartyData.value and not Recap_CheckForParty(i, iCombatant.OwnedBy)) or
			   (recap.Opt.HideFoe.value and not iCombatant.Friend) or
			   (recap.Opt.HideYardTrash.value and (not iCombatant.Friend and iCombatant.TotalKills>1)) or
			   (iCombatant.Ignore) then
				entry = false -- strip out unwanted combatants

			elseif recap.Opt.View.value=="Last" and iCombatant.WasInLast then
				-- Hawksy: relax time restriction on "Last Fight" stats to TotalTime rather than LastTime
				-- Hawksy: relax further, for TotalTimeIn or TotalTimeHeal also
				if not recap.Opt.HideZero.value or ((iCombatant.TotalTime>recap_temp.MinTime) or (iCombatant.TotalTimeIn>recap_temp.MinTime) or (iCombatant.TotalTimeHeal>recap_temp.MinTime)) then
					iList.Name = i
					iList.Time = iCombatant.LastTime
					iList.TimeIn = iCombatant.LastTimeIn
					iList.TimeHeal = iCombatant.LastTimeHeal
					iList.MaxHit = iCombatant.LastMaxHit
					iList.DmgIn = iCombatant.LastDmgIn
					iList.DmgOut = iCombatant.LastDmgOut
					iList.HPS = iCombatant.LastHPS
					iList.DPSIn = iCombatant.LastDPSIn
					iList.DPS = iCombatant.LastDPS
					iList.DPSvsAll = (overallDuration>recap_temp.MinTime and Recap_Div1(iCombatant.LastDmgOut,overallDuration)) or 0
					iList.Kills = iCombatant.LastKills
					-- two hidden variables
					iList.RawHeal = iCombatant.LastRawHeal
					iList.OverHeal = iCombatant.LastOverHeal
					-- calculated rather than assigned
					iList.Heal = iCombatant.LastRawHeal - iCombatant.LastOverHeal
					if iList.RawHeal > 0 then
						iList.Over = Recap_Div0(100*iList.OverHeal,iList.RawHeal)
					else
						iList.Over = 0
					end
					entry = true
				end

			elseif recap.Opt.View.value=="All" then
				-- Hawksy: relax further, for TotalTimeIn or TotalTimeHeal also
				if not recap.Opt.HideZero.value or ((iCombatant.TotalTime>recap_temp.MinTime) or (iCombatant.TotalTimeIn>recap_temp.MinTime) or (iCombatant.TotalTimeHeal>recap_temp.MinTime)) then
					iList.Name = i
					iList.Time = iCombatant.TotalTime
					iList.TimeIn = iCombatant.TotalTimeIn
					iList.TimeHeal = iCombatant.TotalTimeHeal
					iList.MaxHit = iCombatant.TotalMaxHit
					iList.DmgIn = iCombatant.TotalDmgIn
					iList.DmgOut = iCombatant.TotalDmgOut
					iList.HPS = iCombatant.TotalHPS
					iList.DPSIn = iCombatant.TotalDPSIn
					iList.DPS = iCombatant.TotalDPS
					iList.DPSvsAll = (overallDuration>recap_temp.MinTime and Recap_Div1(iCombatant.TotalDmgOut,overallDuration)) or 0
					iList.Kills = iCombatant.TotalKills
					-- two hidden variables
					iList.RawHeal = iCombatant.TotalRawHeal
					iList.OverHeal = iCombatant.TotalOverHeal
					-- calculated rather than assigned
					iList.Heal = iCombatant.TotalRawHeal - iCombatant.TotalOverHeal
					if iList.RawHeal > 0 then
						iList.Over = Recap_Div0(100*iList.OverHeal,iList.RawHeal)
					else
						iList.Over = 0
					end
					entry = true
				end
			end

			if entry then
				-- Hawksy: guard code
				thisCombatant = iList.Name
				if thisCombatant and recap.Combatant[thisCombatant] and recap.Combatant[thisCombatant].Friend then
					duration = duration + iList.Time
					durationIn = durationIn + iList.TimeIn
					durationHeal = durationHeal + iList.TimeHeal
					if iList.MaxHit > maxhit then
						maxhit = iList.MaxHit
					end
					dmgin = dmgin + iList.DmgIn
					dmgout = dmgout + iList.DmgOut
					kills = kills + iList.Kills
					-- two hidden variables
					rawheal = rawheal + iList.RawHeal
					overheal = overheal + iList.OverHeal
					overallDPS = overallDPS + iList.DPS
				end
				recap_temp.ListSize = recap_temp.ListSize + 1
			end
		end
	end

	if not recap_temp.List[recap_temp.ListSize] then
		recap_temp.List[recap_temp.ListSize] = {}
	end

	-- the final entry is the totals
	iTotal = recap_temp.List[recap_temp.ListSize]
	iTotal.Name = recap_temp.Localize.Totals
	iTotal.Time = duration
	RecapTotals_Time:SetText(Recap_FormatTime(duration))
	iTotal.TimeIn = durationIn
	RecapTotals_TimeIn:SetText(Recap_FormatTime(durationIn))
	iTotal.TimeHeal = durationHeal
	RecapTotals_TimeHeal:SetText(Recap_FormatTime(durationHeal))
	iTotal.MaxHit = maxhit
	RecapTotals_MaxHit:SetText(maxhit)
	iTotal.DmgIn = dmgin
	RecapTotals_DmgIn:SetText(dmgin)
	iTotal.DmgOut = dmgout
	RecapTotals_DmgOut:SetText(dmgout)
	iTotal.Kills = kills
	RecapTotals_Kills:SetText(kills)
	-- two hidden variables
	iTotal.RawHeal = rawheal
	iTotal.OverHeal = overheal
	-- calculated rather than assigned
	iTotal.Heal = rawheal - overheal
	RecapTotals_Heal:SetText(rawheal - overheal)
	if (iTotal.RawHeal > 0) then
		iTotal.Over = Recap_Div0(100*iTotal.OverHeal,iTotal.RawHeal)
	else
		iTotal.Over = 0
	end
	RecapTotals_Over:SetText(string.format("%d%%",Recap_Round0(iTotal.Over)))

	-- three calculations of total friendly DPS, DPS In, and HPS
	if duration > recap_temp.MinTime then
		iTotal.TotDPSOut = Recap_Div1(dmgout,duration)
	else
		iTotal.TotDPSOut = 0
	end
	if durationIn > recap_temp.MinTime then
		iTotal.TotDPSIn = Recap_Div1(dmgin,durationIn)
	else
		iTotal.TotDPSIn = 0
	end
	if durationHeal > recap_temp.MinTime then
		iTotal.TotHPS = Recap_Div1((rawheal - overheal),durationHeal)
	else
		iTotal.TotHPS = 0
	end

	if iTotal.TotHPS<1000 then
		RecapTotals_HPS:SetText(string.format("%.1f",iTotal.TotHPS))
	else
		RecapTotals_HPS:SetText(string.format("%d",Recap_Round0(iTotal.TotHPS)))
	end
	if iTotal.TotDPSIn<1000 then
		RecapTotals_DPSIn:SetText(string.format("%.1f",iTotal.TotDPSIn))
	else
		RecapTotals_DPSIn:SetText(string.format("%d",Recap_Round0(iTotal.TotDPSIn)))
	end
	if iTotal.TotDPSOut<1000 then
		RecapTotals_DPS:SetText(string.format("%.1f",iTotal.TotDPSOut))
	else
		RecapTotals_DPS:SetText(string.format("%d",Recap_Round0(iTotal.TotDPSOut)))
	end
	if iTotal.TotDPSOut<1000 then
		RecapTotals_DPSvsAll:SetText(string.format("%.1f",iTotal.TotDPSOut))
	else
		RecapTotals_DPSvsAll:SetText(string.format("%d",Recap_Round0(iTotal.TotDPSOut)))
	end

	-- Hawksy: Class column fix set to black, see also RecapScrollBar_Update and Recap_SetColumns.
	RecapTotals_Class:SetTexCoord(.9,1,.9,1)

	-- strings for when minimized
	RecapMinDPSIn_Text:SetText(string.format("%.1f",iTotal.TotDPSIn))
	RecapMinDPSOut_Text:SetText(string.format("%.1f",iTotal.TotDPSOut))
	if recap.Combatant[recap_temp.Player] and recap.Opt.View.value=="Last" then
		i = recap.Combatant[recap_temp.Player].LastDPS
		if Recap_MergePets() then
			-- owner and pet already merged
			RecapMinYourDPS_Text:SetText(string.format("%.1f",i))
		else
			-- need to add the pet in to get the merged value that is needed for the minimum DPS
			local pet = (UnitName("pet"))
			if pet and pet~=UNKNOWNOBJECT and recap.Combatant[pet] then
				i = i + recap.Combatant[pet].LastDPS
			end
			RecapMinYourDPS_Text:SetText(string.format("%.1f",i))
		end
	elseif recap.Combatant[recap_temp.Player] and recap.Opt.View.value=="All" then
		i = recap.Combatant[recap_temp.Player].TotalDPS
		if Recap_MergePets() then
			-- owner and pet already merged
			RecapMinYourDPS_Text:SetText(string.format("%.1f",i))
		else
			-- need to add the pet in to get the merged value that is needed for the minimum DPS
			local pet = (UnitName("pet"))
			if pet and pet~=UNKNOWNOBJECT and recap.Combatant[pet] then
				i = i + recap.Combatant[pet].TotalDPS
			end
			RecapMinYourDPS_Text:SetText(string.format("%.1f",i))
		end
	else
		RecapMinYourDPS_Text:SetText("--")
	end

	-- calculate percentages
	for i=1,recap_temp.ListSize-1 do
		iList = recap_temp.List[i]
		iList.HealP = 0
		iList.DmgInP = 0
		iList.DmgOutP = 0
		-- Hawksy: guard code
		thisCombatant = iList.Name
		if thisCombatant and recap.Combatant[thisCombatant] and recap.Combatant[thisCombatant].Friend then
			if iTotal.Heal>0 then
				iList.HealP = Recap_Div0(100*iList.Heal,iTotal.Heal)
			end
			if iTotal.DmgIn>0 then
				iList.DmgInP = Recap_Div0(100*iList.DmgIn,iTotal.DmgIn)
			end
			if iTotal.DmgOut>0 then
				iList.DmgOutP = Recap_Div0(100*iList.DmgOut,iTotal.DmgOut)
			end
		end
	end

	-- two hidden variables for total line only
	iTotal.OverallDuration = overallDuration
	iTotal.OverallDPS = overallDPS

	-- restore a valid value for Selected if possible
	if tempname then -- follow name to pre-sort selected
		recap_temp.Selected = Recap_GetSelectedByName(tempname)
		if recap_temp.Selected~=0 then
			RecapPanel_Show(tempname)
		else
			RecapPanel_Hide()
		end
	end

	Recap_SortList()
	if not recap.Opt.Minimized.value then
		RecapScrollBar_Update()
	end
	Recap_UpdatePlugins()

end

function Recap_DefineGauges()

	local i, thisCombatant
	local _G = getfenv(0)

	recap_temp.GaugeMax = 0

	if recap_temp.ListSize>1 then
		recap_temp.GaugeBy = string.gsub(recap.Opt.SortBy.value,"P$","")
		recap_temp.GaugeBy = recap_temp.GaugeBy=="DPSvsAll" and "DmgOut" or recap_temp.GaugeBy
		if not gaugecolor[recap_temp.GaugeBy] then
			recap_temp.GaugeBy = false
		else
			for i=1,recap_temp.ListSize-1 do
				-- Hawksy: guard code
				thisCombatant = recap_temp.List[i].Name
				if thisCombatant and recap.Combatant[thisCombatant] and recap_temp.GaugeBy then
					recap_temp.GaugeMax = math.max(recap.Combatant[thisCombatant].Friend and recap_temp.List[i][recap_temp.GaugeBy] or 0,recap_temp.GaugeMax)
				end
			end
			for i=1,15 do
				_G["RecapList"..i.."_Gauge"]:SetVertexColor(gaugecolor[recap_temp.GaugeBy].r,gaugecolor[recap_temp.GaugeBy].g,gaugecolor[recap_temp.GaugeBy].b)
			end
		end
	end
end


function Recap_GetSelectedByName(arg1)

	local sel, i = 0
	local thisCombatant

	if arg1 and recap_temp.ListSize>1 then
		for i=1,recap_temp.ListSize-1 do
			-- Hawksy: guard code
			thisCombatant = recap_temp.List[i].Name
			if thisCombatant and (thisCombatant==arg1) then
				sel = i
			end
		end
	end

	return sel
end


--[[ Sorting functions ]]

-- initial sort by field done with table.sort for speed, then
-- friends shifted to top with an insertion sort for "stable" list
function Recap_SortList()

	local temp, tempname

	if (recap_temp.Selected ~= 0) and (recap_temp.Selected ~= recap_temp.FriendlyIndex) and (recap_temp.Selected ~= recap_temp.NonFriendlyIndex) then
		tempname = recap_temp.List[recap_temp.Selected].Name
	else
		tempname = nil
	end

	-- remember the last ('Total') element
	temp = recap_temp.List[recap_temp.ListSize]

	-- Hawksy: remove the last element from the 'List' table in case it could confuse 'table.sort'
	table.remove(recap_temp.List) -- without second parameter this defaults to last element

	if recap.Opt.SortBy.value then
		if recap.Opt.SortDir.value then
			table.sort(recap_temp.List,Recap_SortDown)
		else
			table.sort(recap_temp.List,Recap_SortUp)
		end
		Recap_SortFriends()
	end

	-- restore the last ('Total') element
	recap_temp.List[recap_temp.ListSize] = temp

	Recap_DefineGauges()
	recap_temp.Selected = Recap_GetSelectedByName(tempname)

end

function Recap_SortDown(e1,e2)

	-- Hawksy: guard code
	if e1[recap.Opt.SortBy.value] and e2[recap.Opt.SortBy.value] then
		-- bypass a problem with mixed strings and numbers (originally, people running too advanced a ParserLib)
		if type(e1[recap.Opt.SortBy.value]) == type(e2[recap.Opt.SortBy.value]) then
			-- types match, normal compare
			if ( e1[recap.Opt.SortBy.value] < e2[recap.Opt.SortBy.value] ) then
				return true
			else
				return false
			end
		else
			-- types don't match, force string compare
			-- this may be due to a damaged saved variables file, but bypass it anyway
			if ( tostring(e1[recap.Opt.SortBy.value]) < tostring(e2[recap.Opt.SortBy.value]) ) then
				return true
			else
				return false
			end
		end
	else
		return false
	end

end

function Recap_SortUp(e1,e2)

	-- Hawksy: guard code
	if e1[recap.Opt.SortBy.value] and e2[recap.Opt.SortBy.value] then
		-- bypass a problem with mixed strings and numbers (originally, people running too advanced a ParserLib)
		if type(e1[recap.Opt.SortBy.value]) == type(e2[recap.Opt.SortBy.value]) then
			-- types match, normal compare
			if ( e1[recap.Opt.SortBy.value] > e2[recap.Opt.SortBy.value] ) then
				return true
			else
				return false
			end
		else
			-- types don't match, force string compare
			-- this may be due to a damaged saved variables file, but bypass it anyway
			if ( tostring(e1[recap.Opt.SortBy.value]) > tostring(e2[recap.Opt.SortBy.value]) ) then
				return true
			else
				return false
			end
		end
	else
		return false
	end

end

-- perform stable insertion sort on the list
function Recap_SortFriends()

	local i, j, iList, other
	local changed = true

	if recap_temp.ListSize>2 then
		for i=2,(recap_temp.ListSize-1) do
			iList = recap_temp.List[i]
			-- Hawksy: guard code
			if (iList) and (iList.Name) and (recap.Combatant[iList.Name]) and (recap.Combatant[iList.Name].Friend) then
				j=i
				while (j>1) do
					other = recap_temp.List[j-1]
					-- Hawksy: guard code
					if (other) and (other.Name) and (recap.Combatant[other.Name]) and (not recap.Combatant[other.Name].Friend) then
						recap_temp.List[j] = other
						j = j - 1
					else
						break
					end
				end
				recap_temp.List[j] = iList
			end
		end
	end

end

-- Hawksy: three new functions allowing the Personal Details panel to be sorted
--   by column (damage effects always appear first, healing effects second) (function
--   Recap_SelfDamageSort, which was a fixed sort, has been removed)
function Recap_SortSelfList()

	if recap.Opt.ESortBy.value then
		if recap.Opt.ESortDir.value then
			table.sort(recap_temp.SelfList,Recap_SortSelfDown)
		else
			table.sort(recap_temp.SelfList,Recap_SortSelfUp)
		end
	end

end

function Recap_SortSelfDown(e1,e2)

	local effect1, effect2, value1, value2

	if e1.EName and e2.EName then
		effect1 = string.sub(e1.EName,1,1)
		effect2 = string.sub(e2.EName,1,1)
		if e1[recap.Opt.ESortBy.value] and e2[recap.Opt.ESortBy.value] then
			value1 = e1[recap.Opt.ESortBy.value]
			if value1 == "--" then
				value1 = -1
			end
			-- turn a percentage string into a number
			if (type(value1) == "string") and (string.sub(value1, -1) == "%") then
				value1 = tonumber(string.sub(value1, 1, string.len(value1)-1))
			end
			value2 = e2[recap.Opt.ESortBy.value]
			if value2 == "--" then
				value2 = -1
			end
			-- turn a percentage string into a number
			if (type(value2) == "string") and (string.sub(value2, -1) == "%") then
				value2 = tonumber(string.sub(value2, 1, string.len(value2)-1))
			end
			if (((value1 < value2) and (effect1==effect2)) or (effect1<effect2)) then
				return true
			else
				return false
			end
		else
			return false
		end
	else
		return false
	end

end

function Recap_SortSelfUp(e1,e2)

	local effect1, effect2, value1, value2

	if e1.EName and e2.EName then
		effect1 = string.sub(e1.EName,1,1)
		effect2 = string.sub(e2.EName,1,1)
		if e1[recap.Opt.ESortBy.value] and e2[recap.Opt.ESortBy.value] then
			value1 = e1[recap.Opt.ESortBy.value]
			if value1 == "--" then
				value1 = -1
			end
			-- turn a percentage string into a number
			if (type(value1) == "string") and (string.sub(value1, -1) == "%") then
				value1 = tonumber(string.sub(value1, 1, string.len(value1)-1))
			end
			value2 = e2[recap.Opt.ESortBy.value]
			if value2 == "--" then
				value2 = -1
			end
			-- turn a percentage string into a number
			if (type(value2) == "string") and (string.sub(value2, -1) == "%") then
				value2 = tonumber(string.sub(value2, 1, string.len(value2)-1))
			end
			if (((value1 > value2) and (effect1==effect2)) or (effect1<effect2)) then
				return true
			else
				return false
			end
		else
			return false
		end
	else
		return false
	end

end

--[[ Window movement functions ]]

function Recap_OnMouseDown(arg1)

	if recap_temp.Loaded and arg1=="LeftButton" and not recap.Opt.Pinned.value then
		RecapFrame:StartMoving()
	end
end

function Recap_OnMouseUp(arg1)

	if recap_temp.Loaded and arg1=="LeftButton" and not recap.Opt.Pinned.value then
		RecapFrame:StopMovingOrSizing()
		Recap_CheckWindowBounds()

	elseif arg1=="RightButton" and recap.Opt.Minimized.value then
		RecapCreateMenu(recap_temp.Localize.MinMenu, nil)
	end

end

-- repositions RecapFrame if any part of it goes off the screen
function Recap_CheckWindowBounds()

	local x1, y1, x2, y2, cx, cy
	local needs_moved = false

	if recap_temp.Loaded and RecapFrame then

		x1 = RecapFrame:GetLeft()
		y1 = RecapFrame:GetTop()
		cx = RecapFrame:GetWidth()
		cy = RecapFrame:GetHeight()

		if x1 and y1 and cx and cy then

			x2 = RecapFrame:GetRight()
			y2 = RecapFrame:GetBottom()
			uix2 = UIParent:GetRight()
			uix1 = UIParent:GetLeft()
			uiy1 = UIParent:GetTop()
			uiy2 = UIParent:GetBottom()

			if x2 and uix2 and x2 > uix2 then
				x1 = x1 + uix2 - x2
				needs_moved = true
			elseif uix1 and uix1 > x1 then
				x1 = x1 + uix1 - x1
				needs_moved = true
			end

			if y1 and y1 > uiy1 then
				y1 = y1 + uiy1 - y1
				needs_moved = true
			elseif uiy2 and uiy2 > y2 then
				y1 = y1 + uiy2 - y2
				needs_moved = true
			end

			if needs_moved then
				RecapFrame:ClearAllPoints()
				RecapFrame:SetPoint("TOPLEFT","UIParent","BOTTOMLEFT",x1,y1)
			end

			recap[recap_temp.p].WindowTop = RecapFrame:GetTop()
			recap[recap_temp.p].WindowLeft = RecapFrame:GetLeft()
			recap[recap_temp.p].WindowHeight = RecapFrame:GetHeight()
			recap[recap_temp.p].WindowWidth = RecapFrame:GetWidth()

		else
			-- default behaviour if any undefined
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("CENTER","UIParent","CENTER")
		end
	end
end

--[[ Dialog control functions ]]

function Recap_OnClick(arg1) -- clicks on buttons

	local i

	if not recap_temp.Loaded then
		return
	end

	local fight_count = 0
	local total_fight_count = 0
	local filen = RecapSetEditBox:GetText()
	local set_friend, set_dmgin, set_dmgout, set_maxhit, set_kills, set_heal, set_time

	PlaySound("GAMEGENERICBUTTONPRESS")

	if arg1=="Close" then
		RecapFrame_Hide()

	elseif arg1=="Minimize" then

		if recap.Opt.Minimized.value then
			Recap_Maximize()
		else
			Recap_Minimize()
		end
		RecapFrame:SetAlpha(1)
		recap_temp.FadeTimer = -1
		Recap_SetButtons()
		Recap_OnTooltip("Minimize")

	elseif arg1=="Pin" then
		recap.Opt.Pinned.value = not recap.Opt.Pinned.value
		Recap_SetButtons()
		Recap_OnTooltip("Pin")

	elseif arg1=="Pause" then
		if recap.Opt.Paused.value then
			-- resuming
			recap.Opt.Paused.value = false
			Recap_ClearAsIfEndOfFight()
			Recap_SetState("Idle")
		else
			-- pausing
			if ((recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader")) then
				-- for a member, ignore if in synchronization
				PlaySound("igQuestFailed")
				Recap_WarnNotAllowedDuringSync()
			else
				recap.Opt.Paused.value = true
				recap_temp.HoldFight = "Normal"
				Recap_EndFight(true)
				Recap_SetState("Stopped")
				Recap_ClearAsIfEndOfFight()
			end
		end
		Recap_SetButtons()
		Recap_OnTooltip("Pause")

	elseif arg1=="ShowAll" then
		if recap.Opt.SelfView.value then
			recap.Opt.SelfView.value = false
		end
		if recap.Opt.View.value=="Last" then
			Recap_SetView("All")
		else
			Recap_SetView("Last")
		end
		Recap_SetColumns()
		Recap_OnTooltip("ShowAll")

	elseif arg1=="Options" then
		if RecapOptFrame:IsShown() then
			RecapOptFrame:Hide()
		else
			RecapOptFrame:Show()
			Recap_SetSyncDisplay()
		end

	elseif arg1=="Reset" then

		if recap.Opt.SelfView.value then
			Recap_ResetSelfPlayerAndPets()
			Recap_SetView()

		elseif recap.Opt.View.value=="All" then
			if ((recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader")) then
				if recap.Opt.SyncState.value == "Leader" then
					-- for a leader, treat this as a restart sync (with existing value for merge pets)
					Recap_StartSync(recap.Opt.SyncMergePets.value)
					DEFAULT_CHAT_FRAME:AddMessage(RECAP_RECAP.." "..RECAP_SYNCHRONIZATION_RESET)
				else
					-- for a member, ignore if in synchronization
					PlaySound("igQuestFailed")
					Recap_WarnNotAllowedDuringSync()
				end
			else
				-- wipe everything --
				RecapPanel_Hide(1)
				Recap_ResetAllCombatants(false)
				recap_temp.HoldFight = "Normal"
				Recap_MakeFriends()
				Recap_SetView()
			end

		elseif recap.Opt.View.value=="Last" then
			if ((recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader")) then
				-- ignore if in synchronization
				PlaySound("igQuestFailed")
				Recap_WarnNotAllowedDuringSync()
			else
				-- Hawksy: add a confirm dialog
				StaticPopupDialogs["recap_temp.CONFIRMLASTRESET"] = {
					text = TEXT(recap_temp.Localize.ConfirmLastReset),
					button1 = TEXT(ACCEPT),
					button2 = TEXT(CANCEL),
					OnAccept = function()
						-- Hawksy: silent repair
						if not recap.Combatant then
							recap.Combatant = {}
						end
						-- SUBTRACT the last fight --
						for i in pairs(recap.Combatant) do
							if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
								if recap.Combatant[i].WasInLast then
									Recap_ResetLastFight(i)
								end
							end
						end
						recap_temp.HoldFight = "Normal"
						recap[recap_temp.p].TotalDuration = recap[recap_temp.p].TotalDuration - recap[recap_temp.p].LastDuration
						recap[recap_temp.p].LastDuration = 0
						recap[recap_temp.p].TotalDurationIn = recap[recap_temp.p].TotalDurationIn - recap[recap_temp.p].LastDurationIn
						recap[recap_temp.p].LastDurationIn = 0
						recap[recap_temp.p].TotalDurationHeal = recap[recap_temp.p].TotalDurationHeal - recap[recap_temp.p].LastDurationHeal
						recap[recap_temp.p].LastDurationHeal = 0
						Recap_SetView()
					end,
					OnHide = function()
					end,
					OnCancel = function()
					end,
					timeout = 0,
					showAlert = 1,
					whileDead = 1
				}
				StaticPopup_Show("recap_temp.CONFIRMLASTRESET")
			end
		end

	elseif arg1=="Hold" then
		if recap.Opt.View.value=="Last" then
			if recap_temp.HoldFight=="Normal" then
				recap_temp.HoldFight = "HoldNext"
			elseif recap_temp.HoldFight=="Holding" then
				recap_temp.HoldFight = "Normal"
				-- Update the panel
				Recap_EndFight(true)
				Recap_CheckJoinSync()
				Recap_CheckSendSummary()
				Recap_CheckProcessSyncData()
			end
			Recap_SetView()
		end

	elseif arg1=="SaveAllSet" then
		RecapSetEditBox:ClearFocus()
		if filen and filen~="" then
			filen = string.gsub(filen,"\"","'") -- convert " to '
			filen = string.gsub(filen,"%[","(") -- convert [ to (
			filen = string.gsub(filen,"%]",")") -- convert ] to )
		end
		RecapSetEditBox:SetText("")
		Recap_SaveCombatants(filen, false)
		Recap_BuildFightSets()
		recap_temp.FightSetSelected = 0
		RecapFightSetsScrollBar_Update()

	-- Hawksy: added function to save only the Last Fight data set
	elseif arg1=="SaveLastSet" then
		RecapSetEditBox:ClearFocus()
		if filen and filen~="" then
			filen = string.gsub(filen,"\"","'") -- convert " to '
			filen = string.gsub(filen,"%[","(") -- convert [ to (
			filen = string.gsub(filen,"%]",")") -- convert ] to )
		end
		RecapSetEditBox:SetText("")
		Recap_SaveCombatants(filen, true)
		Recap_BuildFightSets()
		recap_temp.FightSetSelected = 0
		RecapFightSetsScrollBar_Update()

	elseif arg1=="QuickSaveAllSet" then
		filen = RECAP_ALL_FIGHTS.." "..date()
		Recap_SaveCombatants(filen, false)
		RecapSetEditBox:SetText("")
		Recap_BuildFightSets()
		recap_temp.FightSetSelected = 0
		RecapFightSetsScrollBar_Update()

	elseif arg1=="QuickSaveLastSet" then
		filen = RECAP_LAST_FIGHT.." "..date()
		Recap_SaveCombatants(filen, true)
		RecapSetEditBox:SetText("")
		Recap_BuildFightSets()
		recap_temp.FightSetSelected = 0
		RecapFightSetsScrollBar_Update()

	elseif arg1=="LoadSet" then
		if ((recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader")) then
			-- ignore if in synchronization
			PlaySound("igQuestFailed")
			Recap_WarnNotAllowedDuringSync()
		else
			RecapSetEditBox:SetText("")
			RecapSetEditBox:ClearFocus()
			recap_temp.OutgoingDetailSelected = 0
			recap_temp.IncomingDetailSelected = 0
			recap_temp.OtherDetailSelected = 0
			recap_temp.Selected = 0
			Recap_LoadCombatants(filen)
			recap_temp.HoldFight = "Normal"
			Recap_MakeFriends()
			Recap_SetView("All")
		end

	elseif arg1=="DeleteSet" then
		RecapSetEditBox:SetText("")
		RecapSetEditBox:ClearFocus()
		recap_set[filen] = nil -- keep as nil
		Recap_BuildFightSets()

	elseif arg1=="SelfView" then
		recap.Opt.SelfView.value = not recap.Opt.SelfView.value
		if recap.Opt.SelfView.value then
			RecapHoldButton:Hide()
		else
			if recap.Opt.View.value=="Last" then
				Recap_SetView("Last")
			else
				Recap_SetView("All")
			end
		end
		RecapPanel_Hide(1)
		Recap_SetColumns()
		Recap_OnTooltip("SelfView")
	end
end

function RecapHeader_OnEnter()

	-- Hawksy: lock OnEnter-OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local msg=string.gsub(this:GetName(),"RecapHeader_","Header")
	Recap_OnTooltip(msg)

	-- Hawksy: unlock OnEnter-OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function RecapHeader_OnMouseUp(arg1)

	if arg1=="RightButton" then
		RecapCreateMenu(recap_temp.Localize.ColumnMenu, nil)
	end
end

-- sort column, toggle dir for new headers, name=ascending, rest=descending
function RecapHeader_OnClick()

	local text = ""
	local arg1 = string.gsub(this:GetName(),"RecapHeader_","")

	if not IsShiftKeyDown() then
		if recap.Opt.SortBy.value==string.gsub(arg1,"P$","") then
			recap.Opt.SortDir.value = not recap.Opt.SortDir.value
		else
			recap.Opt.SortBy.value = string.gsub(arg1,"P$","")
			if arg1=="Name" then
				recap.Opt.SortDir.value = true
			else
				recap.Opt.SortDir.value = false
			end
		end
		Recap_SortList()
		RecapScrollBar_Update()

	elseif IsShiftKeyDown() and recap_temp.ListSize>1 and arg1~="Name" then

		recap.Opt.SortBy.value = arg1
		recap.Opt.SortDir.value = false
		Recap_SortList()
		RecapScrollBar_Update()

		if recap.Opt.SpamRows.value then
			Recap_PostSpamRows(arg1, nil)
		else
			Recap_InsertChat(Recap_PostSpamLine(arg1))
		end

	elseif IsShiftKeyDown() and not ChatFrameEditBox:IsVisible() then
		-- Hawksy: will now happen seldom
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.RankUsage)
	end

end

function Recap_List_OnClick(arg1)

	local id, index, thisCombatant
	local _G = getfenv(0)

	id = this:GetID()
	index = id + FauxScrollFrame_GetOffset(RecapScrollBar)

	if index<recap_temp.ListSize then
		thisCombatant = recap_temp.List[index].Name
		recap_temp.OutgoingDetailSelected = 0
		recap_temp.IncomingDetailSelected = 0
		recap_temp.OtherDetailSelected = 0
		if recap_temp.Selected==index and arg1=="LeftButton" then
			recap_temp.Selected = 0
			_G["RecapList"..id]:UnlockHighlight()
			RecapPanel_Hide()
		else
			-- Hawksy: guard code
			if thisCombatant then
				recap_temp.Selected = index
				RecapScrollBar_Update()
				RecapPanel_Show(thisCombatant)
			else
				recap_temp.Selected = 0
				_G["RecapList"..id]:UnlockHighlight()
				RecapPanel_Hide()
			end
		end

		if arg1=="RightButton" and IsShiftKeyDown() and not IsControlKeyDown() then
			-- Hawksy: modified to post Recent Incoming events when Recent Data Mode is active
			if recap.Opt.RecentData.value then
				Recap_PostRecentRows(false, thisCombatant, nil, nil)
			else
				Recap_InsertChat("Recap: "..thisCombatant.." ".._G["RecapList"..id.."_DPS"]:GetText().." "..RECAP_DPS.." ")
			end

		elseif arg1=="RightButton" and IsControlKeyDown() and not IsShiftKeyDown() then
			-- Hawksy: modified to post Recent Outgoing events when Recent Data Mode is active
			if recap.Opt.RecentData.value then
				Recap_PostRecentRows(true, thisCombatant, nil, nil)
			else
				Recap_InsertChat("Recap: "..thisCombatant.." ".._G["RecapList"..id.."_DPS"]:GetText().." "..RECAP_DPS.." ")
			end

		elseif arg1=="LeftButton" and IsShiftKeyDown() then
			Recap_InsertChat(string.format(recap_temp.Localize.VerboseLinkStart,
								thisCombatant,
								_G["RecapList"..id.."_DmgIn"]:GetText(),
								_G["RecapList"..id.."_TimeIn"]:GetText(),
								_G["RecapList"..id.."_DPSIn"]:GetText(),
								_G["RecapList"..id.."_Kills"]:GetText(),
								_G["RecapList"..id.."_DmgOut"]:GetText(),
								_G["RecapList"..id.."_Time"]:GetText(),
								_G["RecapList"..id.."_DPS"]:GetText(),
								_G["RecapList"..id.."_MaxHit"]:GetText(),
								_G["RecapList"..id.."_Heal"]:GetText(),
								_G["RecapList"..id.."_TimeHeal"]:GetText(),
								_G["RecapList"..id.."_HPS"]:GetText() ).." "..recap_temp.Localize.For.." "..
								recap_temp.LastAll[recap.Opt.View.value])

		elseif arg1=="RightButton" and not IsShiftKeyDown() and not IsControlKeyDown() then
			-- Hawksy: guard code
			if thisCombatant and recap.Combatant[thisCombatant] and recap.Combatant[thisCombatant].Friend then
				if recap.Opt.View.value == "Last" then
					RecapCreateMenu(recap_temp.Localize.DropMenu, 1)
				elseif recap.Combatant[thisCombatant].Locked then
					RecapCreateMenu(recap_temp.Localize.DropUnlockMenu, 1)
				else
					RecapCreateMenu(recap_temp.Localize.DropLockMenu, 1)
				end
			else
				if recap.Opt.View.value == "Last" then
					RecapCreateMenu(recap_temp.Localize.AddMenu, 1)
				elseif recap.Combatant[thisCombatant].Locked then
					RecapCreateMenu(recap_temp.Localize.AddUnlockMenu, 1)
				else
					RecapCreateMenu(recap_temp.Localize.AddLockMenu, 1)
				end
			end
		end
	end
end

-- Hawksy: added to allow click reporting off the totals line
function Recap_List_Totals_OnClick(arg1)

	local i, thisCombatant
	local _G = getfenv(0)

	recap_temp.OutgoingDetailSelected = 0
	recap_temp.IncomingDetailSelected = 0
	recap_temp.OtherDetailSelected = 0
	for i=1,15 do
		_G["RecapList"..i]:UnlockHighlight()
	end

	-- Hawksy: control+shift+right-click on total line posts all recent combat event information
	-- to avoid impossible spam, this function only posts to "Self" (the player console)
	-- with a lot of events, it will be very slow
	-- this feature is deliberately undocumented
	if arg1=="RightButton" and IsShiftKeyDown() and IsControlKeyDown() then
		recap_temp.Selected = 0
		RecapPanel_Hide()
		if recap.Opt.RecentData.value then
			Recap_PostRecentRows(true, nil, nil, true)
		end

	elseif arg1=="LeftButton" and IsShiftKeyDown() then
		recap_temp.Selected = 0
		RecapPanel_Hide()
		Recap_InsertChat(string.format(recap_temp.Localize.VerboseFriendlyTotalStart,
							_G["RecapTotals_DmgIn"]:GetText(),
							_G["RecapTotals_TimeIn"]:GetText(),
							_G["RecapTotals_DPSIn"]:GetText(),
							_G["RecapTotals_Kills"]:GetText(),
							_G["RecapTotals_DmgOut"]:GetText(),
							_G["RecapTotals_Time"]:GetText(),
							_G["RecapTotals_DPS"]:GetText(),
							_G["RecapTotals_MaxHit"]:GetText(),
							_G["RecapTotals_Heal"]:GetText(),
							_G["RecapTotals_TimeHeal"]:GetText(),
							_G["RecapTotals_HPS"]:GetText() ).." "..recap_temp.Localize.For.." "..
							recap_temp.LastAll[recap.Opt.View.value])
		-- create totals for the other (non-friendly) combatants
		local dmgin, kills, dmgout, heal, maxhit, duration, durationIn, DurationHeal
		dmgin = 0
		kills = 0
		dmgout = 0
		heal = 0
		maxhit = 0
		if recap.Opt.View.value=="Last" then
			duration = recap[recap_temp.p].LastDuration
			durationIn = recap[recap_temp.p].LastDurationIn
			durationHeal = recap[recap_temp.p].LastDurationHeal
		else
			duration = recap[recap_temp.p].TotalDuration
			durationIn = recap[recap_temp.p].TotalDurationIn
			durationHeal = recap[recap_temp.p].TotalDurationHeal
		end
		if duration>0 or durationIn>0 or durationHeal>0 then
			for j=1,recap_temp.ListSize-1 do
				thisCombatant = recap_temp.List[j].Name
				if thisCombatant and recap.Combatant[thisCombatant] and not recap.Combatant[thisCombatant].Friend then
					if recap_temp.List[j].MaxHit > maxhit then
						maxhit = recap_temp.List[j].MaxHit
					end
					dmgin = dmgin + recap_temp.List[j].DmgIn
					kills = kills + recap_temp.List[j].Kills
					dmgout = dmgout + recap_temp.List[j].DmgOut
					heal = heal + recap_temp.List[j].Heal
				end
			end
			Recap_InsertChat(string.format(recap_temp.Localize.VerboseNonFriendlyTotalStart,
								dmgin,
								_G["RecapTotals_TimeIn"]:GetText(),
								(((durationIn>0) and Recap_Div1(dmgin,durationIn)) or 0),
								kills,
								dmgout,
								_G["RecapTotals_Time"]:GetText(),
								(((duration>0) and Recap_Div1(dmgout,duration)) or 0),
								maxhit,
								heal,
								_G["RecapTotals_TimeHeal"]:GetText(),
								(((durationHeal>0) and Recap_Div1(heal,durationHeal)) or 0)).." "..recap_temp.Localize.For.." "..
								recap_temp.LastAll[recap.Opt.View.value])
		end

	elseif (arg1=="LeftButton") and not IsControlKeyDown() and not IsShiftKeyDown() then
		if recap.Opt.ShowPanel.value then
			if (recap_temp.Selected == recap_temp.FriendlyIndex) and RecapPanel:IsShown() then
				-- toggle off
				recap_temp.Selected = 0
				_G["RecapList"..id]:UnlockHighlight()
				RecapPanel_Hide()
			else
				-- calculate our special combatant "Friendly Total" (on demand only, since it takes a significant amount of time)
				Recap_ClearGrandTotal(recap_temp.FriendlyTotal)
				Recap_DoGrandTotal(recap_temp.FriendlyTotal)
				-- show our special combatant "Friendly Total"
				recap_temp.Selected = recap_temp.FriendlyIndex -- non-zero value to keep the panel showing and indicate which total we are showing
				RecapScrollBar_Update()
				RecapPanel_Populate(recap_temp.FriendlyTotal)
				RecapPanel:Show()
			end
		end

	elseif (arg1=="LeftButton") and IsControlKeyDown() and not IsShiftKeyDown() then
		if recap.Opt.ShowPanel.value then
			if (recap_temp.Selected == recap_temp.NonFriendlyIndex) and RecapPanel:IsShown() then
				-- toggle off
				recap_temp.Selected = 0
				_G["RecapList"..id]:UnlockHighlight()
				RecapPanel_Hide()
			else
				-- calculate our special combatant "Non Friendly Total" (on demand only, since it takes a significant amount of time)
				Recap_ClearGrandTotal(recap_temp.NonFriendlyTotal)
				Recap_DoGrandTotal(recap_temp.NonFriendlyTotal)
				-- show our special combatant "Non Friendly Total"
				recap_temp.Selected = recap_temp.NonFriendlyIndex -- non-zero value to keep the panel showing and indicate which total we are showing
				RecapScrollBar_Update()
				RecapPanel_Populate(recap_temp.NonFriendlyTotal)
				RecapPanel:Show()
			end
		end
	end
end

function Recap_ClearGrandTotal(thisTotal)

	local i, j, iTotal

	-- create and clear the hidden combatant
	if not recap.Combatant[thisTotal] then
		recap.Combatant[thisTotal] = {}
	end
	iTotal = recap.Combatant[thisTotal]
	iTotal.TotalTime = 0
	iTotal.TotalTimeIn = 0
	iTotal.TotalTimeHeal = 0
	iTotal.TotalMaxHit = 0
	iTotal.TotalKills = 0
	iTotal.TotalRawHeal = 0
	iTotal.TotalOverHeal = 0
	iTotal.TotalDmgIn = 0
	iTotal.TotalDmgOut = 0
	iTotal.LastTime = 0
	iTotal.LastTimeIn = 0
	iTotal.LastTimeHeal = 0
	iTotal.LastMaxHit = 0
	iTotal.LastKills = 0
	iTotal.LastRawHeal = 0
	iTotal.LastOverHeal = 0
	iTotal.LastDmgIn = 0
	iTotal.LastDmgOut = 0
	if thisTotal == recap_temp.FriendlyTotal then
		iTotal.Friend = true
	else
		iTotal.Friend = false
	end
	iTotal.IncomingDetail = {} -- breaks table.sort if we try to re-use the table by setting individual values to nil
	iTotal.OutgoingDetail = {} -- breaks table.sort if we try to re-use the table by setting individual values to nil
	iTotal.OtherDetail = {} -- breaks table.sort if we try to re-use the table by setting individual values to nil
end

function Recap_DoGrandTotal(thisTotal)

	local i, iCombatant, iTotal

	-- accumulate
	for i in pairs(recap.Combatant) do
		if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
			iCombatant = recap.Combatant[i]
			if (iCombatant.LastDmgIn==0 and iCombatant.LastDmgOut==0 and
				iCombatant.TotalDmgOut==0 and iCombatant.TotalDmgIn==0 and
				iCombatant.LastRawHeal==0 and iCombatant.TotalRawHeal==0) or
			   (recap.Opt.SelfData.value and not Recap_CheckForSelf(i, iCombatant.OwnedBy)) or
			   (recap.Opt.PartyData.value and not Recap_CheckForParty(i, iCombatant.OwnedBy)) or
			   (recap.Opt.HideFoe.value and not iCombatant.Friend) or
			   (recap.Opt.HideYardTrash.value and (not iCombatant.Friend and iCombatant.TotalKills>1)) or
			   (iCombatant.Ignore) then
				-- ignore unwanted combatants (if-statement code duplicated from earlier)

			elseif not recap.Opt.HideZero.value or ((iCombatant.TotalTime>recap_temp.MinTime) or (iCombatant.TotalTimeIn>recap_temp.MinTime) or (iCombatant.TotalTimeHeal>recap_temp.MinTime)) then
				if (thisTotal == recap_temp.FriendlyTotal) and iCombatant.Friend then
					-- this is a valid friendly combatant; add their Outgoing, Incoming, and Other Details to a hidden combatant called "Friendly Total"
					Recap_AccumulateGrandTotal(thisTotal, i)
				elseif (thisTotal == recap_temp.NonFriendlyTotal) and not iCombatant.Friend then
					-- this is a valid non-friendly combatant; add their Outgoing, Incoming, and Other Details to a hidden combatant called "Non Friendly Total"
					Recap_AccumulateGrandTotal(thisTotal, i)
				end
			end
		end
	end

	-- per second values
	iTotal = recap.Combatant[thisTotal]
	if iTotal.TotalTime > recap_temp.MinTime then
		iTotal.TotalDPS = Recap_Div1(iTotal.TotalDmgOut,iTotal.TotalTime)
	else
		iTotal.TotalDPS = 0
	end
	if iTotal.TotalTimeIn > recap_temp.MinTime then
		iTotal.TotalDPSIn = Recap_Div1(iTotal.TotalDmgIn,iTotal.TotalTimeIn)
	else
		iTotal.TotalDPSIn = 0
	end
	if iTotal.TotalTimeHeal > recap_temp.MinTime then
		iTotal.TotalHPS = Recap_Div1((iTotal.TotalRawHeal - iTotal.TotalOverHeal),iTotal.TotalTimeHeal)
	else
		iTotal.TotalHPS = 0
	end
	if iTotal.LastTime > recap_temp.MinTime then
		iTotal.LastDPS = Recap_Div1(iTotal.LastDmgOut,iTotal.LastTime)
	else
		iTotal.LastDPS = 0
	end
	if iTotal.LastTimeIn > recap_temp.MinTime then
		iTotal.LastDPSIn = Recap_Div1(iTotal.LastDmgIn,iTotal.LastTimeIn)
	else
		iTotal.LastDPSIn = 0
	end
	if iTotal.LastTimeHeal > recap_temp.MinTime then
		iTotal.LastHPS = Recap_Div1((iTotal.LastRawHeal - iTotal.LastOverHeal),iTotal.LastTimeHeal)
	else
		iTotal.LastHPS = 0
	end

end

function Recap_AccumulateGrandTotal(thisTotal, thisCombatant)

	local i, iTotal, iCombatant

	iTotal = recap.Combatant[thisTotal]
	iCombatant = recap.Combatant[thisCombatant]

	-- accumulate across all combatants into a single hidden combatant named "Friendly Total"
	for i in pairs(iCombatant.IncomingDetail) do
		if not iTotal.IncomingDetail[i] then
			iTotal.IncomingDetail[i] = {}
		end
		Recap_AccumulateDetails(iTotal.IncomingDetail[i], iCombatant.IncomingDetail[i])
	end

	for i in pairs(iCombatant.OutgoingDetail) do
		if not iTotal.OutgoingDetail[i] then
			iTotal.OutgoingDetail[i] = {}
		end
		Recap_AccumulateDetails(iTotal.OutgoingDetail[i], iCombatant.OutgoingDetail[i])
	end

	for i in pairs(iCombatant.OtherDetail) do
		if not iTotal.OtherDetail[i] then
			iTotal.OtherDetail[i] = {}
		end
		Recap_AccumulateDetails(iTotal.OtherDetail[i], iCombatant.OtherDetail[i])
	end

	-- include some total values
	iTotal.TotalTime = (iTotal.TotalTime or 0) + iCombatant.TotalTime
	iTotal.TotalTimeIn = (iTotal.TotalTimeIn or 0) + iCombatant.TotalTimeIn
	iTotal.TotalTimeHeal = (iTotal.TotalTimeHeal or 0) + iCombatant.TotalTimeHeal
	iTotal.TotalMaxHit = math.max((iTotal.TotalMaxHit or 0),iCombatant.TotalMaxHit)
	iTotal.TotalKills = (iTotal.TotalKills or 0) + iCombatant.TotalKills
	iTotal.TotalRawHeal = (iTotal.TotalRawHeal or 0) + iCombatant.TotalRawHeal
	iTotal.TotalOverHeal = (iTotal.TotalOverHeal or 0) + iCombatant.TotalOverHeal
	iTotal.TotalDmgIn = (iTotal.TotalDmgIn or 0) + iCombatant.TotalDmgIn
	iTotal.TotalDmgOut = (iTotal.TotalDmgOut or 0) + iCombatant.TotalDmgOut
	if iCombatant.WasInLast then
		iTotal.LastTime = (iTotal.LastTime or 0) + iCombatant.LastTime
		iTotal.LastTimeIn = (iTotal.LastTimeIn or 0) + iCombatant.LastTimeIn
		iTotal.LastTimeHeal = (iTotal.LastTimeHeal or 0) + iCombatant.LastTimeHeal
		iTotal.LastMaxHit = math.max((iTotal.LastMaxHit or 0),iCombatant.LastMaxHit)
		iTotal.LastKills = (iTotal.LastKills or 0) + iCombatant.LastKills
		iTotal.LastRawHeal = (iTotal.LastRawHeal or 0) + iCombatant.LastRawHeal
		iTotal.LastOverHeal = (iTotal.LastOverHeal or 0) + iCombatant.LastOverHeal
		iTotal.LastDmgIn = (iTotal.LastDmgIn or 0) + iCombatant.LastDmgIn
		iTotal.LastDmgOut = (iTotal.LastDmgOut or 0) + iCombatant.LastDmgOut
	end
end

function Recap_AccumulateDetails(iTotalDetail, iDetail)

	local j

	for j in pairs(iDetail) do
		if (j == "Element") then
			-- fancy merge
			iTotalDetail[j] = Recap_MergeElements(iTotalDetail[j], iDetail[j])
		elseif (j == "Attribute") then
			-- direct copy (not a composite string)
			iTotalDetail[j] = iDetail[j]
		elseif (j == "PrevTime") or (j == "PrevTimeDur") then
			-- ignore
		elseif (j == "GlancesMin") or (j == "HitsMin") or (j == "TicksMin") or (j == "CritsMin") or (j == "CrushMin") then
			-- minima
			iTotalDetail[j] = Recap_Min(iTotalDetail[j], iDetail[j])
		elseif (j == "GlancesMax") or (j == "HitsMax") or (j == "TicksMax") or (j == "CritsMax") or (j == "CrushMax") then
			-- maxima
			iTotalDetail[j] = math.max((iTotalDetail[j] or 0), iDetail[j])
		else
			-- everything else is a straight sum
			iTotalDetail[j] = (iTotalDetail[j] or 0) + iDetail[j]
		end
	end
end

local mergeElements = {}
function Recap_MergeElements(first, second)

	if not first then
		return second
	elseif not second then
		return first
	else
		-- need to do something fancier
		local i, result
		for i in pairs(mergeElements) do
			mergeElements[i] = nil
		end
		for i in string.gmatch(first, "[^%+]+") do
			mergeElements[i] = true
		end
		for i in string.gmatch(second, "[^%+]+") do
			mergeElements[i] = true
		end
		result = nil
		for i in pairs(mergeElements) do
			if mergeElements[i] then
				if not result then
					result = i
				else
					result = result.."+"..i
				end
			end
		end
		return result
	end
end


--[[ Tooltip functions ]]

function Recap_Tooltip(arg1,arg2)

	if recap_temp.Loaded and recap.Opt.ShowTooltips.value then
		if not recap.Opt.TooltipFollow.value then
			GameTooltip_SetDefaultAnchor(GameTooltip,this)
		else
			GameTooltip:SetOwner(this,Recap_TooltipAnchor())
		end
		GameTooltip:SetText(arg1)
		GameTooltip:AddLine(arg2, .75, .75, .75, 1)
		GameTooltip:Show()
	end
end

-- returns line1,line2 of a tooltip for arg1 in OptList (for generic static tooltips)
function Recap_GetTooltip(arg1)

	local i

	if arg1 then
		for i in pairs(recap_temp.OptList) do
			if recap_temp.OptList[i][1]==arg1 then
				return recap_temp.OptList[i][2],recap_temp.OptList[i][3]
			end
		end
	end
end

-- arg1=tooltip index, arg2=optional addition to header
function Recap_OnTooltip(arg1,arg2)

	local line1, line2

	if arg1=="Status" then
		Recap_Tooltip(recap_temp.Recap.." "..recap_temp.Localize.Status..": "..recap_temp.StoppedActiveIdle[recap.Opt.State.value],recap_temp.Localize.StatusTooltip)
	elseif arg1=="Close" then
		if recap.Opt.State.value=="Stopped" then
			Recap_OnTooltip("ExitRecap")
		else
			Recap_OnTooltip("HideWindow")
		end
	elseif arg1=="Minimize" then
		if recap.Opt.Minimized.value then
			Recap_OnTooltip("ExpandWindow")
		else
			Recap_OnTooltip("MinimizeWindow")
		end
	elseif arg1=="Pin" then
		if recap.Opt.Pinned.value then
			Recap_OnTooltip("UnPinWindow")
		else
			Recap_OnTooltip("PinWindow")
		end
	elseif arg1=="Pause" then
		if recap.Opt.State.value=="Stopped" then
			Recap_OnTooltip("Resume")
		else
			Recap_OnTooltip("PauseMonitoring")
		end
	elseif arg1=="ShowAll" then
		if recap.Opt.View.value=="Last" then
			Recap_OnTooltip("ShowAllFights")
		else
			Recap_OnTooltip("ShowLastFight")
		end
	elseif arg1=="HeaderName" then
		if recap.Opt.View.value=="Last" then
			Recap_OnTooltip("CombatLast")
		else
			Recap_OnTooltip("CombatAll")
		end
	elseif arg1=="Reset" then
		if recap.Opt.SelfView.value then
			Recap_OnTooltip("ResetSelfView")
		elseif recap.Opt.View.value=="Last" then
			Recap_OnTooltip("ResetLastFight")
		else
			Recap_OnTooltip("ResetAllTotals")
		end

	elseif arg1=="Hold" then
		if recap.Opt.View.value=="Last" then
			if recap_temp.HoldFight=="Normal" then
				Recap_OnTooltip("HoldFight")
			elseif recap_temp.HoldFight=="Holding" then
				Recap_OnTooltip("ResumeUpdate")
			end
		end

	elseif arg1=="SelfView" then
		if recap.Opt.SelfView.value then
			Recap_OnTooltip("HideSelfView")
		else
			Recap_OnTooltip("ShowSelfView")
		end
	else
		line1,line2 = Recap_GetTooltip(arg1)
		if line1 and line2 then
			if arg2 then
				Recap_Tooltip(line1..": "..arg2,line2)
			else
				Recap_Tooltip(line1, line2)
			end
		end
	end
end

function Recap_List_OnEnter()

	-- Hawksy: lock OnEnter-OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local index, id, class, faction, secondLine, thisCombatant, iCombatant

	id = this:GetID()
	index = id + FauxScrollFrame_GetOffset(RecapScrollBar)

	thisCombatant = recap_temp.List[index].Name
	if thisCombatant then
		iCombatant = recap.Combatant[thisCombatant]
		class = recap_temp.Localize.ClassName[iCombatant.Class]
		faction = recap_temp.Localize.FactionName[iCombatant.Faction]
		secondLine = ""
		if class then
			secondLine = class
		end
		if faction then
			secondLine = secondLine.." ("..faction..")"
		end
		if iCombatant.OwnedBy then
			secondLine = secondLine.." ("..recap_temp.Localize.OwnedBy.." "..iCombatant.OwnedBy..")"
		end
		Recap_Tooltip(thisCombatant, secondLine)
	end
	if (index<recap_temp.ListSize) and recap_temp.Selected==0 then
		if thisCombatant then
			RecapPanel_Show(thisCombatant)
		end
	end

	-- Hawksy: unlock OnEnter-OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function Recap_Totals_OnEnter()

	-- Hawksy: lock OnEnter-OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local i, thisCombatant, iTotal
	local r = recap_temp.ColorWhite.r
	local g = recap_temp.ColorWhite.g
	local b = recap_temp.ColorWhite.b
	local other_kills = 0

	if recap.Opt.ShowTooltips.value then

		if recap.Opt.TooltipFollow.value then
			GameTooltip:SetOwner(this,Recap_TooltipAnchor())
		else
			GameTooltip_SetDefaultAnchor(GameTooltip,this)
		end

		GameTooltip:AddLine(recap_temp.Localize.Totals.." "..recap_temp.Localize.For.." "..recap_temp.LastAll[recap.Opt.View.value])

		if recap_temp.ListSize>1 then

			for i=1,(recap_temp.ListSize-1) do
				-- Hawksy: guard code
				thisCombatant = recap_temp.List[i].Name
				if thisCombatant and recap.Combatant[thisCombatant] and (not recap.Combatant[thisCombatant].Friend) then
					other_kills = other_kills + recap_temp.List[i].Kills
				end
			end
			iTotal = recap_temp.List[recap_temp.ListSize]
			GameTooltip:AddDoubleLine(RECAP_COMBATANTS..":",recap_temp.ListSize-1,r,g,b,r,g,b)
			GameTooltip:AddDoubleLine(" ") -- blank line
			GameTooltip:AddDoubleLine(RECAP_DAMAGE_OUT..":",iTotal.DmgOut,recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b,recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			GameTooltip:AddDoubleLine(RECAP_TIME_FIGHTING..":",Recap_FormatTime(iTotal.Time),recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b,recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			GameTooltip:AddDoubleLine(RECAP_DPS_OUT..":",string.format("%.1f",iTotal.TotDPSOut),recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b,recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			GameTooltip:AddDoubleLine(RECAP_MAX_HIT..":",iTotal.MaxHit,r,g,b,r,g,b)
			GameTooltip:AddDoubleLine(RECAP_KILLS..":",other_kills,r,g,b,r,g,b)
			GameTooltip:AddDoubleLine(" ") -- blank line
			GameTooltip:AddDoubleLine(RECAP_DAMAGE_IN..":",iTotal.DmgIn,recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b,recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			GameTooltip:AddDoubleLine(RECAP_TIME_DAMAGED..":",Recap_FormatTime(iTotal.TimeIn),recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b,recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			GameTooltip:AddDoubleLine(RECAP_DPS_IN..":",string.format("%.1f",iTotal.TotDPSIn),recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b,recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			GameTooltip:AddDoubleLine(RECAP_DEATHS..":",iTotal.Kills,r,g,b,r,g,b)
			GameTooltip:AddDoubleLine(" ") -- blank line
			GameTooltip:AddDoubleLine(RECAP_HEALS..":",iTotal.Heal,recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b,recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			GameTooltip:AddDoubleLine(RECAP_TIME_HEALING..":",Recap_FormatTime(iTotal.TimeHeal),recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b,recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			GameTooltip:AddDoubleLine(RECAP_HPS_OUT..":",string.format("%.1f",iTotal.TotHPS),recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b,recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			GameTooltip:AddDoubleLine(" ") -- blank line
			GameTooltip:AddDoubleLine(RECAP_GROUP_FIGHT_TIME..":",Recap_FormatTime(iTotal.OverallDuration),recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b,recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			GameTooltip:AddDoubleLine(RECAP_GROUP_DPS..":",string.format("%.1f",iTotal.OverallDPS),recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b,recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
		else
			GameTooltip:AddLine(RECAP_COMBATANTS..": "..recap_temp.Localize.None,r,g,b)
		end

		GameTooltip:Show()
	end

	-- Hawksy: unlock OnEnter-OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function Recap_TooltipAnchor()

	local anchor = "ANCHOR_LEFT"

	if GetCursorPosition("UIParent")<(UIParent:GetWidth()/2) then
		anchor = "ANCHOR_RIGHT"
	end
	return anchor
end

--[[ Plug-in functions ]]

function Recap_GetIB_Tooltip()
	return recap_temp.LastAll[recap.Opt.View.value]..":\n"..RECAP_YOUR_DPS..": "..RecapMinYourDPS_Text:GetText().."\n"..RECAP_MAX_HIT..": "..RecapTotals_MaxHit:GetText().."\n"..RECAP_TOTAL_DPS_OUT..": "..RecapTotals_DPS:GetText().."\n"..RECAP_TOTAL_DPS_IN..": "..RecapTotals_TotDPSIn:GetText()
end

function Recap_UpdatePlugins()

	local yourdps = RecapMinYourDPS_Text:GetText()
	local dpsin = RecapMinDPSIn_Text:GetText()
	local dpsout = RecapMinDPSOut_Text:GetText()

	if IB_Recap_Update then
		IB_Recap_Update("DPS: "..yourdps)
	end

	if TitanPanelRecap_Update then
		TitanPanelRecap_Update(recap.Opt.State.value,yourdps,dpsin,dpsout)
	end
end

--[[ Context menu functions ]]

function RecapMenu_OnUpdate(arg1)

	local i
	local _G = getfenv(0)

	if not recap_temp.MenuTimer or MouseIsOver(RecapMenu) then
		recap_temp.MenuTimer = 0
	end

	recap_temp.MenuTimer = recap_temp.MenuTimer + arg1
	-- Hawksy: doubled the time that a menu hangs around
	if recap_temp.MenuTimer > 0.5 then
		recap_temp.MenuTimer = 0
		RecapMenu:Hide()
		if recap_temp.SelectedEffect then
			for i=1,15 do
				_G["RecapSelf"..i]:UnlockHighlight()
			end
		end
	end
end

function RecapMenu_SetCheck(id,arg1)

	if arg1 then
		_G["RecapMenu"..id.."_Check"]:Show()
	else
		_G["RecapMenu"..id.."_Check"]:Hide()
	end
end

function RecapMenu_OnClick()

	local i
	local id, index = this:GetID()
	local _G = getfenv(0)

	if recap_temp.Menu==recap_temp.Localize.ColumnMenu or recap_temp.Menu==recap_temp.Localize.MinMenu or recap_temp.Menu==recap_temp.Localize.EffectOptMenu then
		index = recap_temp.Menu[id].Info -- .Info of menu item selected
		if recap.Opt[index] then
			recap.Opt[index].value = not recap.Opt[index].value
			_G["RecapOptCheck_"..recap_temp.Menu[id].Info]:SetChecked(0)
			if recap.Opt[index].value then
				_G["RecapOptCheck_"..recap_temp.Menu[id].Info]:SetChecked(1)
			end
			RecapMenu_SetCheck(id,recap.Opt[index].value)
			if recap.Opt.Minimized.value then
				Recap_Minimize()
			else
				Recap_Maximize()
			end
		elseif index=="Pin" then
			recap.Opt.Pinned.value = not recap.Opt.Pinned.value
			Recap_SetButtons()
			RecapMenu:Hide()
		end
	end

	-- popup menu
	if recap_temp.Menu==recap_temp.Localize.AddMenu or recap_temp.Menu==recap_temp.Localize.AddLockMenu or recap_temp.Menu==recap_temp.Localize.AddUnlockMenu or
	   recap_temp.Menu==recap_temp.Localize.DropMenu or recap_temp.Menu==recap_temp.Localize.DropLockMenu or recap_temp.Menu==recap_temp.Localize.DropUnlockMenu then
		RecapMenu:Hide()
		index = recap_temp.List[recap_temp.Selected].Name -- name selected

		if id==1 and ((recap_temp.Menu==recap_temp.Localize.DropMenu) or (recap_temp.Menu==recap_temp.Localize.DropLockMenu) or (recap_temp.Menu==recap_temp.Localize.DropUnlockMenu)) and index~=recap_temp.Player then
			-- Make them no longer a 'friendly combatant'
			recap.Combatant[index].Friend = false

		elseif id==1 and index~=recap_temp.Player then
			-- Toggle their 'friendly combatant' status
			recap.Combatant[index].Friend = not recap.Combatant[index].Friend

		elseif id==2 and recap.Opt.View.value=="All" then -- reset
			if ((recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader")) then
				-- ignore if in synchronization
				PlaySound("igQuestFailed")
				Recap_WarnNotAllowedDuringSync()
			else
				-- Reset a combatant, meaning force all their numbers to zero
				recap.Combatant[index] = nil -- keep as nil
				Recap_CreateBlankCombatant(index)
			end

		elseif id==2 and recap.Opt.View.value=="Last" then
			if ((recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader")) then
				-- ignore if in synchronization
				PlaySound("igQuestFailed")
				Recap_WarnNotAllowedDuringSync()
			else
				-- Partially Reset a combatant, meaning SUBTRACT their last fight information
				if recap.Combatant[index].WasInLast then
					-- Hawksy: add a confirm dialog
					StaticPopupDialogs["recap_temp.CONFIRMLASTRESET"] = {
						text = TEXT(recap_temp.Localize.ConfirmLastReset),
						button1 = TEXT(ACCEPT),
						button2 = TEXT(CANCEL),
						OnAccept = function()
							-- SUBTRACT the last fight --
							Recap_ResetLastFight(index)
							-- force redisplay (not sure if there might not be a cheaper call)
							Recap_Maximize()
						end,
						OnHide = function()
						end,
						OnCancel = function()
						end,
						timeout = 0,
						showAlert = 1,
						whileDead = 1
					}
					StaticPopup_Show("recap_temp.CONFIRMLASTRESET")
				end
			end

		elseif id==3 and index~=recap_temp.Player then
			if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
				-- don't ignore a combatant when in synchronization
				PlaySound("igQuestFailed")
				Recap_WarnNotAllowedDuringSync()
			else
				-- Put this combatant on 'ignore' (after forcing all their numbers to zero)
				recap.Combatant[index] = nil -- keep as nil
				Recap_CreateBlankCombatant(index)
				recap.Combatant[index].Ignore = true
			end

		elseif id==4 then
			-- Toggle their Locked status (makes them stick around through an All Fights Reset)
			recap.Combatant[index].Locked = not recap.Combatant[index].Locked
			-- Set the Locked status of any pets to match
			for i in pairs(recap.Combatant) do
				if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
					if recap.Combatant[i].OwnedBy == index then
						recap.Combatant[i].Locked = recap.Combatant[index].Locked
					end
				end
			end
		end

		Recap_SetView()
	else
		-- remove this effect
		if recap_temp.Menu==recap_temp.Localize.EffectMenu then
			RecapMenu:Hide()
			for i=1,15 do
				_G["RecapSelf"..i]:UnlockHighlight()
			end
			if recap_temp.SelectedEffect then
				-- could be for player, or could be for one of the player's pets
				if (string.sub(recap_temp.SelectedEffect,1,1)=="1") or (string.sub(recap_temp.SelectedEffect,1,1)=="3") then
					-- player
					if recap.Self[recap_temp.s][recap_temp.SelectedEffect] then
						recap.Self[recap_temp.s][recap_temp.SelectedEffect] = nil -- keep as nil
					end
				else
					-- player's pet
					local selfType, selfName, selfEffect, selfIndex
					_,_,selfType,selfName,selfEffect = string.find(recap_temp.SelectedEffect, "^(.)(.+): (.+)")
					selfIndex = recap_temp.s..":"..selfName
					if selfType=="2" then
						selfEffect = "1"..selfEffect
					else
						selfEffect = "3"..selfEffect
					end
					if recap.Self[selfIndex][selfEffect] then
						recap.Self[selfIndex][selfEffect] = nil -- keep as nil
					end
				end
				recap_temp.SelectedEffect = false
				Recap_SetView()
			end
		end

		-- other actions
		if recap_temp.Menu==recap_temp.Localize.DetailMenu then
			if recap_temp.OutgoingDetailsList and recap_temp.OutgoingDetailsList[1][recap_temp.Menu[id].Info] then
				recap.Opt.OutgoingPanelDetail.value = recap_temp.Menu[id].Info
				if recap_temp.Selected == recap_temp.FriendlyIndex then
					RecapPanel_Populate(recap_temp.FriendlyTotal)
				elseif recap_temp.Selected == recap_temp.NonFriendlyIndex then
					RecapPanel_Populate(recap_temp.NonFriendlyTotal)
				else
					RecapPanel_Populate(recap_temp.List[recap_temp.Selected].Name)
				end
			end
			RecapMenu:Hide()
		end
		if recap_temp.Menu==recap_temp.Localize.DetailMenu then
			if recap_temp.IncomingDetailsList and recap_temp.IncomingDetailsList[1][recap_temp.Menu[id].Info] then
				recap.Opt.IncomingPanelDetail.value = recap_temp.Menu[id].Info
				if recap_temp.Selected == recap_temp.FriendlyIndex then
					RecapPanel_Populate(recap_temp.FriendlyTotal)
				elseif recap_temp.Selected == recap_temp.NonFriendlyIndex then
					RecapPanel_Populate(recap_temp.NonFriendlyTotal)
				else
					RecapPanel_Populate(recap_temp.List[recap_temp.Selected].Name)
				end
			end
			RecapMenu:Hide()
		end
	end
end

-- offset=1 to move window so it's under pointer
function RecapCreateMenu(menu, offset)

	local i, cy, x, y, lines

	recap_temp.Menu = menu

	if recap_temp.Menu==recap_temp.Localize.ColumnMenu or recap_temp.Menu==recap_temp.Localize.MinMenu or recap_temp.Menu==recap_temp.Localize.EffectOptMenu then
		for i in pairs(recap_temp.Menu) do
			if recap.Opt[recap_temp.Menu[i].Info] then
				recap_temp.Menu[i].Check = recap.Opt[recap_temp.Menu[i].Info].value
			end
		end
	end

	if recap_temp.Menu==recap_temp.Localize.MinMenu then
		recap_temp.Menu[1].Check = recap.Opt.Pinned.value
	end

	if recap_temp.Menu==recap_temp.Localize.DetailMenu then
		for i in pairs(recap_temp.Menu) do
			recap_temp.Menu[i].Check = recap.Opt.OutgoingPanelDetail.value==recap_temp.Menu[i].Info
		end
	end
	if recap_temp.Menu==recap_temp.Localize.DetailMenu then
		for i in pairs(recap_temp.Menu) do
			recap_temp.Menu[i].Check = recap.Opt.IncomingPanelDetail.value==recap_temp.Menu[i].Info
		end
	end

	lines = min(#recap_temp.Menu,recap_temp.MaxMenuLines)

	cy = 44 -- 12+32
	for i=1,lines do
		_G["RecapMenu"..i.."_Text"]:SetText(recap_temp.Menu[i].Text)
		RecapMenu_SetCheck(i,recap_temp.Menu[i].Check)
		_G["RecapMenu"..i]:Show()
		cy = cy + 14
	end
	for i=(lines+1),recap_temp.MaxMenuLines do
		_G["RecapMenu"..i]:Hide()
	end

	RecapMenu:SetHeight(cy)
	RecapDropSubFrame:SetHeight(cy-32)
	RecapMenu:ClearAllPoints()

	x,y = GetCursorPosition()
	if recap.Opt.GrowUpwards.value and not offset then
		y = this:GetBottom()*UIParent:GetScale()
		RecapMenu:SetPoint("BOTTOMRIGHT","UIParent","BOTTOMLEFT",(x/UIParent:GetScale())+20,(y/UIParent:GetScale())-4)
	else
		y = this:GetTop()*UIParent:GetScale()
		RecapMenu:SetPoint("TOPRIGHT","UIParent","BOTTOMLEFT",(x/UIParent:GetScale())+20,(y/UIParent:GetScale())+(offset and 16 or 4))
	end

	RecapMenu:Show()

end

function RecapMenu_OnEnter()

	-- Hawksy: lock OnEnter-OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	id = this:GetID()
	if recap_temp.Menu[id] and recap_temp.Menu[id].Info then
		if recap_temp.Menu==recap_temp.Localize.AddLockMenu or recap_temp.Menu==recap_temp.Localize.AddUnlockMenu or recap_temp.Menu==recap_temp.Localize.DropLockMenu or recap_temp.Menu==recap_temp.Localize.DropUnlockMenu then
			Recap_OnTooltip(recap_temp.Menu[id].Info,recap_temp.List[recap_temp.Selected].Name)
		elseif recap_temp.Menu==recap_temp.Localize.EffectMenu and recap_temp.SelectedEffect then
			Recap_OnTooltip(recap_temp.Menu[id].Info,string.sub(recap_temp.SelectedEffect,2))
		else
			Recap_OnTooltip(recap_temp.Menu[id].Info)
		end
	end

	-- Hawksy: unlock OnEnter-OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function RecapEffectsHeader_OnEnter()

	-- Hawksy: lock OnEnter-OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local s

	_,_,s = string.find(this:GetName(), "RecapHeader_(.+)")
	if s then
		Recap_OnTooltip("Header"..s)
	end

	-- Hawksy: unlock OnEnter-OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function RecapSelf_OnMouseUp(arg1)

	recap_temp.SelectedEffect = false
	Recap_OnMouseUp(arg1)
end

function RecapEffectsHeader_OnMouseUp(arg1)

	if arg1=="RightButton" then
		RecapCreateMenu(recap_temp.Localize.EffectOptMenu, nil)
	end
end

function RecapEffectsHeader_OnClick()

	local i, lineno
	local alltext, text = ""
	local hasdata
	local arg1 = string.gsub(this:GetName(),"RecapHeader_","")
	local print_chat = SendChatMessage
	local chatchan, chatnum

	-- Hawksy: new code to support sorting effects on a per column basis
	if not IsShiftKeyDown() then
		if recap.Opt.ESortBy.value==string.gsub(arg1,"$","") then
			recap.Opt.ESortDir.value = not recap.Opt.ESortDir.value
		else
			recap.Opt.ESortBy.value = string.gsub(arg1,"$","")
			if (arg1=="EName") or (arg1=="EElement") then
				recap.Opt.ESortDir.value = true
			else
				recap.Opt.ESortDir.value = false
			end
		end
		Recap_SortSelfList()
		RecapScrollBar_Update()

	elseif IsShiftKeyDown() and recap_temp.SelfListSize>1 then

		recap.Opt.ESortBy.value = arg1

		-- only report for columns other than EName
		if recap.Opt.ESortBy.value~="EName" then

			if (arg1=="EName") or (arg1=="EElement") then
				recap.Opt.ESortDir.value = true
			else
				recap.Opt.ESortDir.value = false
			end
			Recap_SortSelfList()
			RecapScrollBar_Update()

			chatchan, chatnum = Recap_GetChannelInfo(nil, nil)
			if chatchan=="Self" then
				print_chat = Recap_Print
			end

			-- If the Clip box on the Options : Reports panel is open, post there instead of to chat or console
			if RecapClipEditBox:IsVisible() then
				print_chat = Recap_LineToClipboard
			end

			alltext = (recap.Opt.SpamRows.value and "__ Recap: " or "Recap: ")..recap_temp.Player..": "..Recap_GetTooltip("Header"..arg1)..(recap.Opt.SpamRows.value and " __" or ":")
			lineno = 0
			for i=1,recap_temp.SelfListSize-1 do
				if recap_temp.SelfList[i][arg1]~="0.0%" and recap_temp.SelfList[i][arg1]~="--" and tostring(recap_temp.SelfList[i][arg1])~="0" then
					if not hasdata and recap.Opt.SpamRows.value then
						print_chat(alltext, chatchan, nil, chatnum)
					end
					hasdata = true
					text = string.sub(recap_temp.SelfList[i].EName,2)..(((string.sub(recap_temp.SelfList[i].EName,1,1)=="3") or (string.sub(recap_temp.SelfList[i].EName,1,1)=="4")) and "+" or "")..(recap.Opt.SpamRows.value and ": " or " ")..recap_temp.SelfList[i][arg1]..(arg1=="ETotal" and (" ("..recap_temp.SelfList[i].ETotalP..")") or "")
					if not recap.Opt.SpamRows.value then
						alltext = alltext.." ["..text.."]"
					else
						lineno = lineno + 1
						text = tostring(lineno)..". "..text
						print_chat(text, chatchan, nil, chatnum)
					end
				end
			end
			if not recap.Opt.SpamRows.value and hasdata then
				Recap_InsertChat(alltext)
			end
		end

	else
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Localize.RankUsage)
	end
end


function RecapSelf_OnClick(arg1)

	local id = this:GetID()
	local index = id + FauxScrollFrame_GetOffset(RecapScrollBar)
	local linkformat = recap_temp.Localize.OutgoingDamageDetailLink
	local text = ""

	if arg1=="LeftButton" and IsShiftKeyDown() and index<recap_temp.SelfListSize then
		if (string.sub(recap_temp.SelfList[index].EName,1,1)=="3") or (string.sub(recap_temp.SelfList[index].EName,1,1)=="4") then
			linkformat = recap_temp.Localize.OutgoingHealDetailLink
		end
		text = string.format(linkformat,recap_temp.Player,string.sub(recap_temp.SelfList[index].EName,2),(recap_temp.SelfList[index].EElement or "?"),recap_temp.SelfList[index].ETotal,recap_temp.SelfList[index].ETotalP,recap_temp.Player )..", "..RECAP_MAX_HIT..": "..recap_temp.SelfList[index].EMaxAll..((recap_temp.SelfList[index].ECritsP~="--") and (", "..RECAP_CRIT_RATE..": "..recap_temp.SelfList[index].ECritsP) or "")
		Recap_InsertChat(text," ")

	elseif arg1=="RightButton" and index<recap_temp.SelfListSize and IsControlKeyDown() and not IsShiftKeyDown() then
		-- Hawksy: post Recent Outgoing events for this effect for this combatant when Recent Data Mode is active
		if recap.Opt.RecentData.value then
			Recap_PostRecentRows(true, recap_temp.Player, string.sub(recap_temp.SelfList[index].EName,2), nil)
		end

	elseif arg1=="RightButton" and index<recap_temp.SelfListSize and not IsShiftKeyDown() and not IsControlKeyDown() then
		this:LockHighlight()
		recap_temp.SelectedEffect = recap_temp.SelfList[index].EName
		RecapCreateMenu(recap_temp.Localize.EffectMenu, 1)
	end
end


Recap_lua_372 = true
