--[[ Auxiliary functions, assorted ]]

--[[ local tables ]]

-- keys for (det)ails
-- Hawksy: added nine additional miss types, and crushing blows
-- Hawksy: added nine values for Other details
-- Hawksy: added three glancing types, and twelve partial damage types, and five minima
-- (no more single alpha letters available)
local detkey =	{ ["HitsDmg"]="d", ["Hits"]="n", ["HitsMax"]="x", ["CritsDmg"]="D", ["Crits"]="N",
				  ["CritsMax"]="X", ["CritsEvents"]="e", ["Missed"]="m", ["TicksDmg"]="t", ["TicksMax"]="T", ["Ticks"]="i",
				  ["Dodged"]="G", ["Parried"]="P", ["Blocked"]="B", ["Absorbed"]="A", ["Deflected"]="F",
				  ["Evaded"]="V", ["Resisted"]="R", ["Reflected"]="S", ["Immune"]="I",
				  ["Crushes"]="r", ["CrushDmg"]="s", ["CrushMax"]="u",
				  ["Total"]="v", ["Max"]="w", ["Dispels"]="y", ["PrevTime"]="z", ["ICount"]="a", ["ITotal"]="b", ["DCount"]="c",
				  ["DTotal"]="f", ["PrevTimeDur"]="g", ["GlancesDmg"]="h", ["Glances"]="j", ["GlancesMax"]="k",
				  ["PAbsorbs"]="l", ["PAbsorbsDmg"]="o", ["PBlocks"]="p", ["PBlocksDmg"]="q", ["PVulnerable"]="C",
				  ["PVulnerableDmg"]="E", ["PResists25"]="H", ["PResists25Dmg"]="J", ["PResists50"]="K",
				  ["PResists50Dmg"]="L", ["PResists75"]="M", ["PResists75Dmg"]="O",
				  ["GlancesMin"]="Q", ["HitsMin"]="U", ["CritsMin"]="W", ["CrushMin"]="Y", ["TicksMin"]="Z" }

-- keys for (inc)oming data sets
-- Hawksy: added NonMeleeTicks -- for Incoming, will eventually be obsolete
local inckey = { ["MeleeDamage"]="a", ["MeleeMax"]="b", ["MeleeHits"]="c", ["MeleeCrits"]="d", ["MeleeMissed"]="e",
				 ["MeleeDodged"]="f", ["MeleeParried"]="g", ["MeleeBlocked"]="h", ["NonMeleeDamage"]="i",
				 ["NonMeleeMax"]="j", ["NonMeleeHits"]="k", ["NonMeleeCrits"]="l", ["NonMeleeMissed"]="m",
				 ["NonMeleeTicks"]="n" }

-- Hawksy: when dumping events to WOWChatLog, emit keys in a canonical order
local canonicalKeys = { "type", "source", "victim", "skill", "amount", "element", "isCrit", "isCrushing", "isDOT",
						"isGlancing", "missType", "isItem", "isSplit", "amountAbsorb", "amountBlock", "amountResist",
						"amountVulnerable", "damageType", "amountRank", "attribute", "isFailed", "isBegin", "isPerform",
						"sourceSkill", "sourceGained", "amountGained", "attributeGained" }

-- Hawksy: events with zero damage or healing will be ignored if they have *only* the following info keys
--   the "isSplit" is for some split damage spells, and the "isCrit" can happen with Vampiric Embrace
local zeroEventKeys = { "message", "source", "victim", "type", "amount", "skill", "element", "isDOT", "isSplit", "isCrit" }


--[[ Auxiliary functions, assorted ]]


function Recap_Round0(a)
	return math.floor(a + 0.5)
end
function Recap_Div0(a, b)
	return math.floor(a/b + 0.5)
end
function Recap_Round1(a)
	return math.floor(a * 10 + 0.5) / 10
end
function Recap_Div1(a, b)
	return math.floor(a/b * 10 + 0.5) / 10
end
function Recap_Div2(a, b)
	return math.floor(a/b * 100 + 0.5) / 100
end
function Recap_Min(a, b)
	if a and b then
		return math.min(a, b)
	elseif a then
		return a
	else
		return b
	end
end

function Recap_MergePets()
	if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
		if recap.Opt.SyncMergePets.value == "on" then
			-- synchronization active and merge pets is on
			return true
		end
	else
		if recap.Opt.MergePets.value then
			-- synchronization not active and merge pets is on
			return true
		end
	end
	return false
end

-- do our own error display since WoW 2.1 will have lua error display off by default
function Recap_TrapError(message, frame)
	debuginfo();
	if string.find(message, "Recap", 1, true) then
		-- this is an error with the string "Recap" in it, pretend it is ours
		if (not ScriptErrors:IsVisible()) then
			-- but only if there is not already an error message visible
			local whichParser = "ParserLib11"
			if ParserLib.parser30 then
				-- ckknight says this will indicate if the Parser-3.0 compatibility layer is being used
				whichParser = "Parser30"
			end
			ScriptErrors_Message:SetText("Recap "..tostring(Recap_Version).." ("..whichParser.." loaded): "..message)
			ScriptErrors:Show()
			if recap_temp.snap then
				Screenshot()
			end
		end
	else
		-- probably not a Recap error, pass it to the original error handler
		if recap_Original_ErrorHandler then
			return recap_Original_ErrorHandler(message, frame)
		end
	end
end

function Recap_AuthorDebug(msg)
	if (recap_temp.p == "HawksyElune") or (recap_temp.p == "GalassaElune") or (recap_temp.p == "MetrognomeElune") then
		DEFAULT_CHAT_FRAME:AddMessage(msg)
	end
end

-- Hawksy: function for setting the event range
function Recap_SetEventRange(setMax)

	if setMax then
		-- maximum
		SetCVar("CombatDeathLogRange", 200)
		SetCVar("CombatLogRangeParty", 200)
		SetCVar("CombatLogRangePartyPet", 200)
		SetCVar("CombatLogRangeFriendlyPlayers", 200)
		SetCVar("CombatLogRangeFriendlyPlayersPets", 200)
		SetCVar("CombatLogRangeHostilePlayers", 200)
		SetCVar("CombatLogRangeHostilePlayersPets", 200)
		SetCVar("CombatLogRangeCreature", 200)
	else
		-- Blizzard default
		SetCVar("CombatDeathLogRange", 60)
		SetCVar("CombatLogRangeParty", 50)
		SetCVar("CombatLogRangePartyPet", 50)
		SetCVar("CombatLogRangeFriendlyPlayers", 50)
		SetCVar("CombatLogRangeFriendlyPlayersPets", 50)
		SetCVar("CombatLogRangeHostilePlayers", 50)
		SetCVar("CombatLogRangeHostilePlayersPets", 50)
		SetCVar("CombatLogRangeCreature", 30)
	end

	-- Note: there is no need to reloadui to get the new ranges to take effect
end

-- Hawksy: Special case for combined DoT:HoT spells where the healing event comes via ParserLib as if it were a non-HoT
-- This code is designed to correct existing errors after the fact
-- TODO: at the moment it doesn't correct the element subtotal
function Recap_CorrectDualDoTHoT()

	local i, thisTypeEffect, thisType, healEffect, damageEffect, iCombatant, iHealEffect

	-- first for the Combatant table
	for i in pairs(recap.Combatant) do
		if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
			iCombatant = recap.Combatant[i]
			-- guard code
			if not iCombatant.OutgoingDetail then
				iCombatant.OutgoingDetail = {}
			end
			for thisTypeEffect in pairs(iCombatant.OutgoingDetail) do
				-- checking every effect for every combatant
				thisType = tonumber(string.sub(thisTypeEffect,1,1))
				if (thisType == 3) or (thisType == 4) then
					-- only check from healing effects
					healEffect = thisTypeEffect
					iHealEffect = iCombatant.OutgoingDetail[healEffect]
					-- compute the matching damage effect, and see if it exists
					damageEffect = tostring(thisType-2)..string.sub(healEffect,2)
					if iCombatant.OutgoingDetail[damageEffect] then
						-- there is a matching damage effect, so this is a dual damage:healing effect
						if iCombatant.OutgoingDetail[damageEffect].TicksDmg and iHealEffect.HitsDmg then
							-- the damage half of the effect is a DoT and the healing half is (incorrectly) non-HoT healing
							-- convert the healing half of the effect into a HoT
							iHealEffect.Ticks = (iHealEffect.Ticks or 0) + (iHealEffect.CritsEvents or 0)
							iHealEffect.Hits = nil -- keep as nil
							iHealEffect.CritsEvents = nil -- keep as nil
							iHealEffect.TicksDmg = (iHealEffect.TicksDmg or 0) + (iHealEffect.HitsDmg or 0)
							iHealEffect.HitsDmg = nil -- keep as nil
							iHealEffect.TicksMax = math.max((iHealEffect.TicksMax or 0),(iHealEffect.HitsMax or 0))
							iHealEffect.HitsMax = nil -- keep as nil
							iHealEffect.TicksMin = Recap_Min(iHealEffect.TicksMin,iHealEffect.HitsMin)
							iHealEffect.HitsMin = nil -- keep as nil
						end
					end
				end
			end
			-- guard code
			if not iCombatant.IncomingDetail then
				iCombatant.IncomingDetail = {}
			end
			for thisTypeEffect in pairs(iCombatant.IncomingDetail) do
				-- checking every effect for every combatant
				thisType = tonumber(string.sub(thisTypeEffect,1,1))
				if (thisType == 3) or (thisType == 4) then
					-- only check from healing effects
					healEffect = thisTypeEffect
					iHealEffect = iCombatant.IncomingDetail[healEffect]
					-- compute the matching damage effect (in the *Outgoing* table), and see if it exists
					damageEffect = tostring(thisType-2)..string.sub(healEffect,2)
					if iCombatant.OutgoingDetail[damageEffect] then
						-- there is a matching damage effect, so this is a dual damage:healing effect
						if iCombatant.OutgoingDetail[damageEffect].TicksDmg and iHealEffect.HitsDmg then
							-- the damage half of the effect is a DoT and the healing half is (incorrectly) non-HoT healing
							-- convert the healing half of the effect into a HoT
							iHealEffect.Ticks = (iHealEffect.Ticks or 0) + (iHealEffect.CritsEvents or 0)
							iHealEffect.Hits = nil -- keep as nil
							iHealEffect.CritsEvents = nil -- keep as nil
							iHealEffect.TicksDmg = (iHealEffect.TicksDmg or 0) + (iHealEffect.HitsDmg or 0)
							iHealEffect.HitsDmg = nil -- keep as nil
							iHealEffect.TicksMax = math.max((iHealEffect.TicksMax or 0),(iHealEffect.HitsMax or 0))
							iHealEffect.HitsMax = nil -- keep as nil
							iHealEffect.TicksMin = Recap_Min(iHealEffect.TicksMin,iHealEffect.HitsMin)
							iHealEffect.HitsMin = nil -- keep as nil
						end
					end
				end
			end
		end
	end

	-- second for the Self table (is Outgoing only)
	for thisTypeEffect in pairs(recap.Self[recap_temp.s]) do
		thisType = tonumber(string.sub(thisTypeEffect,1,1))
		if (thisType == 3) or (thisType == 4) then
			-- only check from healing effects
			healEffect = thisTypeEffect
			iHealEffect = recap.Self[recap_temp.s][healEffect]
			-- compute the matching damage effect, and see if it exists
			damageEffect = tostring(thisType-2)..string.sub(healEffect,2)
			if recap.Self[recap_temp.s][damageEffect] then
				-- there is a matching damage effect, so this is a dual damage:healing effect
				if recap.Self[recap_temp.s][damageEffect].TicksDmg and iHealEffect.HitsDmg then
					-- the damage half of the effect is a DoT and the healing half is non-HoT healing
					-- convert the healing half of the effect into a HoT
					iHealEffect.Ticks = (iHealEffect.Ticks or 0) + (iHealEffect.CritsEvents or 0)
					iHealEffect.Hits = nil -- keep as nil
					iHealEffect.CritsEvents = nil -- keep as nil
					iHealEffect.TicksDmg = (iHealEffect.TicksDmg or 0) + (iHealEffect.HitsDmg or 0)
					iHealEffect.HitsDmg = nil -- keep as nil
					iHealEffect.TicksMax = math.max((iHealEffect.TicksMax or 0),(iHealEffect.HitsMax or 0))
					iHealEffect.HitsMax = nil -- keep as nil
					iHealEffect.TicksMin = Recap_Min(iHealEffect.TicksMin,iHealEffect.HitsMin)
					iHealEffect.HitsMin = nil -- keep as nil
				end
			end
		end
	end
	-- not at the moment being done for the player's pets
end

-- Hawksy: special case correction to add OutgoingDetail.Ticks and Self.Ticks (conversion from v3.56 or earlier to v 3.57 or later)
function Recap_CorrectForTicks()

	local i, thisTypeEffect, subtotal, iCombatant, iEffect

	-- Hawksy: silent repair
	if not recap.Combatant then
		recap.Combatant = {}
	end
	-- first for the Combatant table
	for i in pairs(recap.Combatant) do
		if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
			iCombatant = recap.Combatant[i]
			for thisTypeEffect in pairs(iCombatant.OutgoingDetail) do
				iEffect = iCombatant.OutgoingDetail[thisTypeEffect]
				if iEffect.TicksDmg and not iEffect.Ticks then
					-- DoT:HoT without a tick count, add one
					if iEffect.HitsDmg then
						-- DoT:HoT with a direct damage component
						subtotal = math.floor(((iEffect.CritsEvents or 0) - (iEffect.Crits or 0))/2)
					else
						subtotal = (iEffect.CritsEvents or 0) - (iEffect.Crits or 0)
					end
					iEffect.Ticks = (iEffect.Hits or 0)
					iEffect.Hits = subtotal
					iEffect.CritsEvents = subtotal + (iEffect.Crits or 0)
				end
			end
			for thisTypeEffect in pairs(iCombatant.IncomingDetail) do
				iEffect = iCombatant.IncomingDetail[thisTypeEffect]
				if iEffect.TicksDmg and not iEffect.Ticks then
					-- DoT:HoT without a tick count, add one
					if iEffect.HitsDmg then
						-- DoT:HoT with a direct damage component
						subtotal = math.floor(((iEffect.CritsEvents or 0) - (iEffect.Crits or 0))/2)
					else
						subtotal = (iEffect.CritsEvents or 0) - (iEffect.Crits or 0)
					end
					iEffect.Ticks = (iEffect.Hits or 0)
					iEffect.Hits = subtotal
					iEffect.CritsEvents = subtotal + (iEffect.Crits or 0)
				end
			end
		end
	end

	-- Hawksy: silent repairs
	if not recap.Self then
		recap.Self = {}
	end
	if not recap.Self[recap_temp.s] then
		recap.Self[recap_temp.s] = {}
	end
	-- second for the Self table
	for thisTypeEffect in pairs(recap.Self[recap_temp.s]) do
		iEffect = recap.Self[recap_temp.s][thisTypeEffect]
		if iEffect.TicksDmg and not iEffect.Ticks then
			-- DoT:HoT without a tick count, add one
			if iEffect.HitsDmg then
				-- DoT:HoT with a direct damage component
				subtotal = math.floor(((iEffect.CritsEvents or 0) - (iEffect.Crits or 0))/2)
			else
				subtotal = (iEffect.CritsEvents or 0) - (iEffect.Crits or 0)
			end
			iEffect.Ticks = (iEffect.Hits or 0)
			iEffect.Hits = subtotal
			iEffect.CritsEvents = subtotal + (iEffect.Crits or 0)
		end
	end
end

-- Hawksy: migrate Detail to OutgoingDetail; and Incoming to IncomingDetail (for 3.59 and later)
function Recap_MigrateDetails()

	local i, thisEffect, j, iCombatant, iEffect

	-- Hawksy: silent repair
	if not recap.Combatant then
		recap.Combatant = {}
	end
	for i in pairs(recap.Combatant) do
		if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
			iCombatant = recap.Combatant[i]
			if iCombatant.Detail then
				if not iCombatant.OutgoingDetail then
					iCombatant.OutgoingDetail = {}
				end
				for thisEffect in pairs(iCombatant.Detail) do
					if not iCombatant.OutgoingDetail[thisEffect] then
						iCombatant.OutgoingDetail[thisEffect] = {}
					end
					for j in pairs(iCombatant.Detail[thisEffect]) do
						iCombatant.OutgoingDetail[thisEffect][j] = iCombatant.Detail[thisEffect][j]
					end
				end
				iCombatant.Detail = nil -- keep as nil
			end

			if iCombatant.Incoming then
				if not iCombatant.IncomingDetail then
					iCombatant.IncomingDetail = {}
				end
				if iCombatant.Incoming.MeleeDamage then
					if not iCombatant.IncomingDetail["1Melee"] then
						iCombatant.IncomingDetail["1Melee"] = {}
					end
					iEffect = iCombatant.IncomingDetail["1Melee"]
					iEffect.Element = "Melee"
					iEffect.HitsDmg = iCombatant.Incoming.MeleeDamage
					if iCombatant.Incoming.MeleeHits then
						iEffect.Hits = iCombatant.Incoming.MeleeHits
						iEffect.CritsEvents = iCombatant.Incoming.MeleeHits
					end
					if iCombatant.Incoming.MeleeCrits then
						iEffect.Crits = iCombatant.Incoming.MeleeCrits
						iEffect.CritsEvents = (iEffect.CritsEvents or 0) + iCombatant.Incoming.MeleeCrits
					end
					if iCombatant.Incoming.MeleeMax then
						iEffect.HitsMax = iCombatant.Incoming.MeleeMax
					end
					if iCombatant.Incoming.MeleeMissed then
						iEffect.Missed = iCombatant.Incoming.MeleeMissed
					end
					if iCombatant.Incoming.MeleeBlocked then
						iEffect.Blocked = iCombatant.Incoming.MeleeBlocked
					end
					if iCombatant.Incoming.MeleeParried then
						iEffect.Parried = iCombatant.Incoming.MeleeParried
					end
					if iCombatant.Incoming.MeleeDodged then
						iEffect.Dodged = iCombatant.Incoming.MeleeDodged
					end
				end

				if iCombatant.Incoming.NonMeleeDamage then
					if not iCombatant.IncomingDetail["1Non-Melee"] then
						iCombatant.IncomingDetail["1Non-Melee"] = {}
					end
					iEffect = iCombatant.IncomingDetail["1Non-Melee"]
					iEffect.Element = "Non-Melee"
					iEffect.HitsDmg = iCombatant.Incoming.NonMeleeDamage
					if iCombatant.Incoming.NonMeleeHits then
						iEffect.Hits = iCombatant.Incoming.NonMeleeHits
						iEffect.CritsEvents = iCombatant.Incoming.NonMeleeHits
					end
					if iCombatant.Incoming.NonMeleeCrits then
						iEffect.Crits = iCombatant.Incoming.NonMeleeCrits
						iEffect.CritsEvents = (iEffect.CritsEvents or 0) + iCombatant.Incoming.NonMeleeCrits
					end
					if iCombatant.Incoming.NonMeleeMax then
						iEffect.HitsMax = iCombatant.Incoming.NonMeleeMax
					end
					if iCombatant.Incoming.NonMeleeTicks then
						iEffect.Ticks = iCombatant.Incoming.NonMeleeTicks
					end
					if iCombatant.Incoming.NonMeleeMissed then
						iEffect.Missed = iCombatant.Incoming.NonMeleeMissed
					end
				end

				iCombatant.Incoming = nil -- keep as nil
			end
		end
	end
end


-- Hawksy: dump ParserLib events to WOWChatLog.txt
function Recap_DumpMessage(message)
	-- include a line count so that we can detect if any lines get dropped
	recap.dumplines = recap.dumplines + 1
	-- limit message to 244 leaving plenty of room for the line number
	ChatThrottleLib:SendChatMessage("BULK", "Recap", tostring(recap.dumplines)..";"..string.sub(message,1,244), "CHANNEL", nil, recap_temp.LogChatnum)
end

function Recap_CreateMessage(event)
	local j, k, ri, value, printable, whichParser, knownIndex

	if ParserLib.parser30 then
		-- ckknight says this will indicate if the Parser-3.0 compatibility layer is being used
		whichParser = "P30"
	else
		whichParser = "PL11"
	end
	printable = event..";"..tostring(Recap_Version)..";"..whichParser
	ri = ParserLib:GetInstance("1.1").info -- undocumented
	if ri and (type(ri) == "table") then -- needed since Parser-3.0 can override ParserLib, and doesn't support this function
		-- first, 'known' keys in a fixed order
		for j=1,#canonicalKeys do
			k = canonicalKeys[j]
			if ri[k] then
				value = tostring(ri[k])
				value = string.gsub(value,"\ ","\_") -- convert blank to underscore
				printable = printable..";"..tostring(k).."="..value
			end
		end
		-- second, 'unknown' keys if any, in arbitrary order
		for k in pairs(ri) do
			if (type(k) == "number") or (k == "pattern") then
				-- ignore numeric keys and 'pattern'
			else
				knownIndex = false
				for j=1,#canonicalKeys do
					if k == canonicalKeys[j] then
						-- the index is one of the 'known' ones
						knownIndex = true
						break
					end
				end
				if not knownIndex then
					-- unknown index
					value = tostring(ri[k])
					value = string.gsub(value,"\ ","\_") -- convert blank to underscore
					printable = printable..";"..tostring(k).."="..value
				end
			end
		end
	end
	return printable
end


function Recap_CheckUnexpectedEvent()
	-- ignore zero events (damage or healing) that have keys only from zeroEventKeys
	local j, k, ri, allOkayIndexes
	ri = ParserLib:GetInstance("1.1").info -- undocumented
	if ri and (type(ri) == "table") then -- needed since Parser-3.0 can override ParserLib, and doesn't support this function
		allOkayIndexes = true
		for k in pairs(ri) do
			if (type(k) == "number") or (k == "pattern") then
				-- ignore numeric keys and 'pattern'
			else
				allOkayIndexes = false
				for j=1,#zeroEventKeys do
					if k == zeroEventKeys[j] then
						-- the index is one of the okay ones
						allOkayIndexes = true
						break
					end
				end
				if not allOkayIndexes then
					-- we got through zeroEventKeys without a match for this index
					break
				end
			end
		end
		if allOkayIndexes then
			-- probably an uninteresting event, do nothing, ignore the event
		else
			-- an unexpected event, post diagnostic information
			local printable
			printable = Recap_CreateMessage(event)
			printable = "Recap ignoring unexpected zero amount event: "..printable -- used to include arg1 which had the message
			if recap.dump then
				Recap_DumpMessage(printable)
			end
			printable = string.gsub(printable,"\;","\ ") -- convert semicolon to blank for ordinary output
			DEFAULT_CHAT_FRAME:AddMessage(printable)
			if recap_temp.snap then
				Screenshot()
			end
		end
	end
end


-- Hawksy: ought to be obsolete with ParserLib, but still firing occasionally
function Recap_CheckForDuplicateSource(source)

	local starting
	local duplicate = false

	-- Hawksy: sources of "Your <spellname>" and "<critter>'s <spellname>" appear
	--   to be duplicates. They show up in the uncorrected code as combatants.
	--   They are not added to the main combatant damage. It appears to be safe
	--   to ignore these sources.  The damage these false combatants do is shown
	--   as 'melee' damage.
	-- These tests are for English, and are NOT localized since I don't know what
	--   to look for in the French and German patterns.
	starting = string.find(source, "^Your .+")
	if starting == 1 then
		duplicate = true
	end
	starting = string.find(source, "^.+'s .+")
	if starting == 1 then
		duplicate = true
	end

	if not duplicate then
		-- not a duplicate
		return false
	else
		-- this is a known duplicate, reject it
		if recap.debug or recap.dump then
			local message
			message = "Recap ignoring known duplicate source: ("..tostring(arg1)..") "..tostring(source)
			DEFAULT_CHAT_FRAME:AddMessage(message)
			if recap.dump then
				Recap_DumpMessage(message)
			end
		end
		return true
	end
end

function Recap_CheckForDuplicateEffect(effect)

	local starting
	local duplicate = false

	-- Hawksy: effects such as "<spellname> critically" appear to be duplicates,
	--   and it appears to be safe to ignore these effects.  Note that these
	--   duplicate effects seem to happen for healing crits only.
	-- These tests are for English and German at the moment.
	starting = string.find(effect, " critically")
	if starting then
		duplicate = true
	end
	-- German
	starting = string.find(effect, " kritisch")
	if starting then
		duplicate = true
	end

	if not duplicate then
		-- not a duplicate
		return false
	else
		-- this is a known duplicate, reject it
		if recap.debug or recap.dump then
			local message
			message = "Recap ignoring known duplicate effect: ("..tostring(arg1)..") "..tostring(effect)
			DEFAULT_CHAT_FRAME:AddMessage(message)
			if recap.dump then
				Recap_DumpMessage(message)
			end
		end
		return true
	end
end


-- Hawksy: is this the player (or pet)?
function Recap_CheckForSelf(combatant, ownedBy)

	local okayToShow = false

	if combatant == recap_temp.Player then
		-- player
		okayToShow = true
	elseif ownedBy and (ownedBy == recap_temp.Player) then
		-- player's pet
		okayToShow = true
	end

	return okayToShow
end

-- Hawksy: is this combatant (or pet) in party?
function Recap_CheckForParty(combatant, ownedBy)

	local okayToShow = false
	local i, u, uName

	okayToShow = Recap_CheckForSelf(combatant, ownedBy)

	for i=1,5 do
		u = "party"..i
		if UnitExists(u) then
			uName = (UnitName(u))
			if uName then
				if combatant == uName then
					-- party member
					okayToShow = true
				elseif ownedBy and (ownedBy == uName) then
					-- party member's pet
					okayToShow = true
				end
			end
		end
	end

	return okayToShow
end


--[[ Misc functions ]]

-- if this is a pet, pass owner's name via ownedby
function Recap_AddFriend(thisCombatant, ownedby)

	if thisCombatant and thisCombatant~=UNKNOWNOBJECT and recap.Combatant[thisCombatant] then
		recap.Combatant[thisCombatant].Friend = true
		if ownedby and ownedby~=UNKNOWNOBJECT then
			-- pet could potentially do damage without owner, add owner if not already present
			if not recap.Combatant[ownedby] then
				Recap_CreateBlankCombatant(ownedby)
				recap.Combatant[ownedby].Friend = true
			end
			if not recap_temp.Last[ownedby] then
				recap_temp.Last[ownedby] = { StartOut = 0, EndOut = 0, StartIn = 0, EndIn = 0, StartHeal = 0, EndHeal = 0, DmgIn = 0, DmgOut = 0, MaxHit = 0, Kills = 0, RawHeal = 0, OverHeal = 0 }
			end
			if thisCombatant == ownedby then
				-- pet and owner have the same name, ensure that we always treat the name as an owner without a pet, not as a pet
				recap.Combatant[thisCombatant].OwnsPet = nil -- keep as nil
				recap.Combatant[thisCombatant].OwnedBy = nil -- keep as nil
			else
				-- if the name of the pet (thisCombatant) is also the name of a player this will turn that name into a pet
				--   if a friendly name is processed on the second pass from Recap_MakeFriends the name will be turned back into a player
				--   anomaly: if the player is non-friendly, they will 'vanish' and their effects will be counted as the pet's effects
				recap.Combatant[thisCombatant].OwnedBy = ownedby
				recap.Combatant[ownedby].OwnsPet = thisCombatant
				-- if pet belongs to player, ensure that Self is set up for this pet
				if ownedby == recap_temp.Player then
					local petIndex
					petIndex = recap_temp.s..":"..thisCombatant
					if not recap.Self[petIndex] then
						recap.Self[petIndex] = {}
					end
				end
			end
		elseif ownedby~=UNKNOWNOBJECT then
			-- ownedby is nil, so thisCombatant is not a pet -- ensure that we treat the name as an owner, not as a pet
			local owner = recap.Combatant[thisCombatant].OwnedBy
			if owner then
				-- thisCombatant has an owner, break the chain (player always beats pet)
				recap.Combatant[owner].OwnsPet = nil -- keep as nil
				recap.Combatant[thisCombatant].OwnedBy = nil -- keep as nil
			end
		end
	end
end

function Recap_MakeFriends()

	local i, j

	-- Hawksy: fix here and everywhere else UnitName is called.
	--   Since patch 1.12 UnitName returns two values, name and realm;
	--   and if included as an argument will insert two arguments into
	--   the call, which for this particular call would make the combatant
	--   appear to be the pet of their realm (!).  This has caused problems
	--   in battlegrounds.  The fix is to force UnitName to return one value
	--   only, which is done by wrapping it in parentheses in the context of
	--   a call.  I've done the wrapping everywhere, whether necessary or not.

	-- to avoid anomalies when players have the same names as pets, do pets first, then players
	-- what then happens is that NAME is marked as a pet (by Recap_AddFriend), then re-marked as a player (by Recap_AddFriend)
	if UnitExists("pet") then
		Recap_GetUnitInfo("pet")
		Recap_AddFriend((UnitName("pet")), (UnitName("player")))
	end
	for i=1,4 do
		j = "partypet"..i
		if UnitExists(j) then
			Recap_GetUnitInfo(j)
			Recap_AddFriend((UnitName(j)), (UnitName("party"..i)))
		end
	end
	if (GetNumRaidMembers()>0) then
		for i=1,40 do
			j = "raidpet"..i
			if UnitExists(j) then
				Recap_GetUnitInfo(j)
				Recap_AddFriend((UnitName(j)), (UnitName("raid"..i)))
			end
		end
	end

	-- now do players (player with same name as pet will overwrite the pet designation)
	Recap_GetUnitInfo("player")
	Recap_AddFriend(recap_temp.Player, nil)
	for i=1,4 do
		j = "party"..i
		if UnitExists(j) then
			Recap_GetUnitInfo(j)
			Recap_AddFriend((UnitName(j)), nil)
		end
	end
	if (GetNumRaidMembers()>0) then
		for i=1,40 do
			j = "raid"..i
			if UnitExists(j) then
				Recap_GetUnitInfo(j)
				Recap_AddFriend((UnitName(j)), nil)
			end
		end
	end
end

-- returns a string of seconds converted to 0:00:00 format
function Recap_FormatTime(arg1)

	local hours, minutes, seconds
	local text

	seconds = Recap_Round0(arg1)
	hours = math.floor(seconds/3600)
	seconds = seconds - hours*3600
	minutes = math.floor(seconds/60)
	seconds = seconds - minutes*60

	return ((hours>0) and (hours..":") or "") .. string.format("%02d:",minutes) .. string.format("%02d",seconds)
end

-- returns a string of seconds converted to 00:00.0 format (nearest tenth of a second, hours not included)
function Recap_FormatTimeRecent(arg1)

	local hours, minutes, seconds, deciseconds
	local text

	seconds = math.floor(arg1)
	deciseconds = Recap_Round0(10 * (arg1 - seconds))
	if deciseconds == 10 then
		seconds = seconds + 1
		deciseconds = 0
	end
	hours = math.floor(seconds/3600)
	seconds = seconds - hours*3600
	minutes = math.floor(seconds/60)
	seconds = seconds - minutes*60

	return string.format("%02d:",minutes) .. string.format("%02d",seconds) .. string.format(".%01d",deciseconds)
end

function Recap_SetClassIcon(id,class)

	local item
	local _G = getfenv(0)

	item = _G["RecapList"..id.."_Class"]
	if class and recap_temp.ClassIcons[class] then
		item:SetTexCoord(recap_temp.ClassIcons[class].left,recap_temp.ClassIcons[class].right,recap_temp.ClassIcons[class].top,recap_temp.ClassIcons[class].bottom)
	else
		item:SetTexCoord(.9,1,.9,1)
	end
end

function Recap_SetFactionIcon(id,faction)

	local item
	local _G = getfenv(0)

	item = _G["RecapList"..id.."_Faction"]
	if faction and recap_temp.FactionIcons[faction] then
		item:SetTexture(recap_temp.FactionIcons[faction])
	else
		item:SetTexture("")
	end
end

function Recap_SetRecapFrameBackdrop()
	if RecapFrame then
		if recap.Opt.OpaqueBackground.value then
			-- took quite a while to find an actual solid black background file
			RecapFrame:SetBackdrop({bgFile = "Interface\\CharacterFrame\\UI-Party-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
									tile = true, tileSize = 16, edgeSize = 16, insets = { left = 4, right = 4, top = 4, bottom = 4 }})
			RecapOptFrame:SetBackdrop({bgFile = "Interface\\CharacterFrame\\UI-Party-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
									tile = true, tileSize = 16, edgeSize = 16, insets = { left = 4, right = 4, top = 4, bottom = 4 }})
			RecapPanel:SetBackdrop({bgFile = "Interface\\CharacterFrame\\UI-Party-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
									tile = true, tileSize = 16, edgeSize = 16, insets = { left = 4, right = 4, top = 4, bottom = 4 }})
		else
			RecapFrame:SetBackdrop({bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
									tile = true, tileSize = 16, edgeSize = 16, insets = { left = 4, right = 4, top = 4, bottom = 4 }})
			RecapOptFrame:SetBackdrop({bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
									tile = true, tileSize = 16, edgeSize = 16, insets = { left = 4, right = 4, top = 4, bottom = 4 }})
			RecapPanel:SetBackdrop({bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
									tile = true, tileSize = 16, edgeSize = 16, insets = { left = 4, right = 4, top = 4, bottom = 4 }})
		end
	end
end

function Recap_GetUnitInfo(unit)

	local u=(UnitName(unit))

	if u then

		local iCombatant = recap.Combatant[u]

		if iCombatant then
			_,iCombatant.Class = UnitClass(unit) -- returns localized class
			_,iCombatant.Faction = UnitFactionGroup(unit) -- returns localized faction
			if UnitCreatureFamily(unit) and iCombatant.Faction then
				iCombatant.Class = "Pet"
			end
			iCombatant.Level = UnitLevel(unit)
		end

		if unit~="target" and unit~="mouseover" then

			-- Hawksy: create the 'recap_temp.Last' table for this unit so that we
			--   can have HP information from the beginning of a fight, in order to
			--   be able to calculate more accurate overhealing information.  The
			--   code will add party members or raid members, who would normally
			--   soon become combatants anyway.  We're just doing it a bit early.
			--   See also the call to Recap_MakeFriends in Recap_StartFight.
			if not iCombatant then
				Recap_CreateBlankCombatant(u)
			end
			if not recap_temp.Last[u] then
				recap_temp.Last[u] = { StartOut = 0, EndOut = 0, StartIn = 0, EndIn = 0, StartHeal = 0, EndHeal = 0, DmgIn = 0, DmgOut = 0, MaxHit = 0, Kills = 0, RawHeal = 0, OverHeal = 0 }
			end
			if UnitIsVisible(unit) then
				recap_temp.Last[u].HP = UnitHealth(unit)
				recap_temp.Last[u].MaxHP = UnitHealthMax(unit)
			else
				recap_temp.Last[u].HP = nil -- discredit -- keep as nil
			end
		end
	end
end

-- creates a new .Combatant[thisCombatant]
function Recap_CreateBlankCombatant(thisCombatant)

	if not recap.Combatant[thisCombatant] then

		-- Hawksy: modified following to be correct and complete (to reduce 'nil' errors)
		recap.Combatant[thisCombatant] = {
			TotalTime = 0,
			TotalTimeIn = 0,
			TotalTimeHeal = 0,
			TotalDmgIn = 0,
			TotalDmgOut = 0,
			TotalDPS = 0,
			TotalDPSIn = 0,
			TotalHPS = 0,
			TotalMaxHit = 0,
			TotalKills = 0,
			TotalRawHeal = 0,
			TotalOverHeal = 0,
			LastTime = 0,
			LastTimeIn = 0,
			LastTimeHeal = 0,
			LastDmgIn = 0,
			LastDmgOut = 0,
			LastDPS = 0,
			LastDPSIn = 0,
			LastHPS = 0,
			LastMaxHit = 0,
			LastKills = 0,
			LastRawHeal = 0,
			LastOverHeal = 0,
			WasInLast = false,
			WasInCurrent = false,
			Friend = false,
			Locked = false,
			Ignore = false,
			OwnedBy = false,
			OwnsPet = false,
			Name = false,
			Class = false,
			Faction = false,
			Level = false,
			Incoming = nil, -- Hawksy: will become obsolete
			Detail = nil, -- Hawksy: will become obsolete
			OutgoingDetail = {},
			IncomingDetail = {},
			OtherDetail = {}
		}
	end

	if not recap_temp.Last[thisCombatant] then
		recap_temp.Last[thisCombatant] = { StartOut = 0, EndOut = 0, StartIn = 0, EndIn = 0, StartHeal = 0, EndHeal = 0, DmgIn = 0, DmgOut = 0, MaxHit = 0, Kills = 0, RawHeal = 0, OverHeal = 0 }
	end
end

-- populates .Opt with a copy of DefaultOpt
function Recap_LoadDefaultOpt()

	local i, j

	recap.Opt = {}
	for i in pairs(recap_temp.DefaultOpt) do
		recap.Opt[i] = {}
		for j in pairs(recap_temp.DefaultOpt[i]) do
			recap.Opt[i][j] = recap_temp.DefaultOpt[i][j]
		end
	end
end

function Recap_SaveOpt(user)

	local i, j

	if user and string.len(user)>0 then
		recap[user].Opt = {}
		for i in pairs(recap.Opt) do
			text = ""
			for j in pairs(recap.Opt[i]) do
				text = text..tostring(j).."="..tostring(recap.Opt[i][j]).." "
			end
			recap[user].Opt[i] = text
		end
	end
end

function Recap_LoadOpt(user)

	local i, j

	if user and recap[user] and recap[user].Opt then
		for i in pairs(recap[user].Opt) do
			if recap_temp.DefaultOpt[i] then
				for t,v in string.gmatch(recap[user].Opt[i],"(%w+)=(%d-%--%w+) ") do
					if v=="true" then
						v = true
					elseif v=="false" then
						v = false
					elseif tonumber(v) then
						-- haven't yet figured it out, but this may bork non-integer numbers, forcing them to false
						v = tonumber(v)
					end
					if t=="type" and recap_temp.DefaultOpt[i][t] then
						v = recap_temp.DefaultOpt[i][t]
					end
					if recap.Opt[i] then
						recap.Opt[i][t] = v
					end
				end
			end
		end

		-- Hawksy: populate current user with any Default options that are missing,
		--   to prevent many 'nil' errors result from missing options.
		for i in pairs(recap_temp.DefaultOpt) do
			if not recap[user].Opt[i] then
				recap[user].Opt[i] = {}
				for j in pairs(recap_temp.DefaultOpt[i]) do
					recap[user].Opt[i][j] = recap_temp.DefaultOpt[i][j]
				end
			end
		end

		-- Hawksy: fix for 3.59 and later
		if recap[user].Opt["PanelDetail"] then
			recap[user].Opt["PanelDetail"] = nil -- keep as nil
		end
		if recap_temp.DefaultOpt["PanelDetail"] then
			recap_temp.DefaultOpt["PanelDetail"] = nil -- keep as nil
		end
	end
end

-- saves current data to data set "filen"
function Recap_SaveCombatants(filen, lastfightonly)

	local i, j, k
	local sd = {}

	if not filen or string.len(filen)<1 then
		return
	end

	recap_set[filen] = {}
	recap_set[filen].TimeStamp = date()
	if not lastfightonly then
		recap_set[filen].TotalDuration = recap[recap_temp.p].TotalDuration
		recap_set[filen].TotalDurationIn = recap[recap_temp.p].TotalDurationIn
		recap_set[filen].TotalDurationHeal = recap[recap_temp.p].TotalDurationHeal
	else
		recap_set[filen].TotalDuration = recap[recap_temp.p].LastDuration
		recap_set[filen].TotalDurationIn = recap[recap_temp.p].LastDurationIn
		recap_set[filen].TotalDurationHeal = recap[recap_temp.p].LastDurationHeal
	end
	recap_set[filen].Combatant = {}

	-- Hawksy: silent repair
	if not recap.Combatant then
		recap.Combatant = {}
	end
	for i in pairs(recap.Combatant) do
		if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) and
		   (not recap.Opt.SaveFriends.value or recap.Combatant[i].Friend) and
		   (not lastfightonly or recap.Combatant[i].WasInLast) then

			-- Hawksy: upgrade to 3.54, new values TotalTimeIn and TotalTimeHeal
			if not recap.Combatant[i].TotalTimeIn then
				recap.Combatant[i].TotalTimeIn = 0
			end
			if not recap.Combatant[i].TotalTimeHeal then
				recap.Combatant[i].TotalTimeHeal = 0
			end

			-- Hawksy: upgrade to 3.61, new values TotalRawHeal and TotalOverHeal
			if not recap.Combatant[i].TotalRawHeal then
				recap.Combatant[i].TotalRawHeal = 0
			end
			if not recap.Combatant[i].TotalOverHeal then
				recap.Combatant[i].TotalOverHeal = 0
			end

			if not lastfightonly then
				recap_set[filen].Combatant[i] = string.format("%s %d %d %d %d %d %d %.3f %.3f %.3f ~%d %d %d",
												tostring(recap.Combatant[i].Friend),
												recap.Combatant[i].TotalDmgIn,
												recap.Combatant[i].TotalDmgOut,
												recap.Combatant[i].TotalMaxHit,
												recap.Combatant[i].TotalRawHeal,
												recap.Combatant[i].TotalOverHeal,
												recap.Combatant[i].TotalKills,
												recap.Combatant[i].TotalTime,
												recap.Combatant[i].TotalTimeIn,
												recap.Combatant[i].TotalTimeHeal,
												Recap_MakeKey(recap.Combatant[i].Faction),
												Recap_MakeKey(recap.Combatant[i].Class),
												recap.Combatant[i].Level or 0)
			else
				-- Hawksy: added to write Last Fight data only
				-- Note: personal details saved are for all fights, not just the last fight
				recap_set[filen].Combatant[i] = string.format("%s %d %d %d %d %d %d %.3f %.3f %.3f ~%d %d %d",
												tostring(recap.Combatant[i].Friend),
												recap.Combatant[i].LastDmgIn,
												recap.Combatant[i].LastDmgOut,
												recap.Combatant[i].LastMaxHit,
												recap.Combatant[i].LastRawHeal,
												recap.Combatant[i].LastOverHeal,
												recap.Combatant[i].LastKills,
												recap.Combatant[i].LastTime,
												recap.Combatant[i].LastTimeIn,
												recap.Combatant[i].LastTimeHeal,
												Recap_MakeKey(recap.Combatant[i].Faction),
												Recap_MakeKey(recap.Combatant[i].Class),
												recap.Combatant[i].Level or 0)
			end
			if recap.Opt.LightData and not recap.Opt.LightData.value then

				-- Hawksy: silent repair
				if not recap.Combatant[i].OutgoingDetail then
					recap.Combatant[i].OutgoingDetail = {}
				end
				for j in pairs(recap.Combatant[i].OutgoingDetail) do
					recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i].."["..j..">"
					if recap.Combatant[i].OutgoingDetail[j].Element then
						recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i]..recap.Combatant[i].OutgoingDetail[j].Element..">"
					else
						recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i].."?"..">"
					end
					for k in pairs(recap.Combatant[i].OutgoingDetail[j]) do
						if k~="Element" then
							recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i]..detkey[k]..recap.Combatant[i].OutgoingDetail[j][k]
						end
					end
					recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i].."]"
				end

				-- Hawksy: silent repair
				if not recap.Combatant[i].IncomingDetail then
					recap.Combatant[i].IncomingDetail = {}
				end
				for j in pairs(recap.Combatant[i].IncomingDetail) do
					recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i].."*"..j..">"
					if recap.Combatant[i].IncomingDetail[j].Element then
						recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i]..recap.Combatant[i].IncomingDetail[j].Element..">"
					else
						recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i].."?"..">"
					end
					for k in pairs(recap.Combatant[i].IncomingDetail[j]) do
						if k~="Element" then
							recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i]..detkey[k]..recap.Combatant[i].IncomingDetail[j][k]
						end
					end
					recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i].."*"
				end

				-- Hawksy: silent repair
				if not recap.Combatant[i].OtherDetail then
					recap.Combatant[i].OtherDetail = {}
				end
				for j in pairs(recap.Combatant[i].OtherDetail) do
					recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i].."#"..j..">"
					if recap.Combatant[i].OtherDetail[j].Attribute then
						recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i]..recap.Combatant[i].OtherDetail[j].Attribute..">"
					else
						recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i].."?"..">"
					end
					for k in pairs(recap.Combatant[i].OtherDetail[j]) do
						if k~="Attribute" then
							recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i]..detkey[k]..recap.Combatant[i].OtherDetail[j][k]
						end
					end
					recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i].."#"
				end

			end
		end
	end

	fight_count = 0
	for i in pairs(recap_set[filen].Combatant) do
		fight_count = fight_count + 1
	end
	recap_set[filen].ListSize = fight_count
end


function Recap_MakeKey(arg1)

	local i
	local key = 0

	for i in pairs(recap_temp.Keys) do
		if arg1==recap_temp.Keys[i] then
			key = i
		end
	end
	return key
end

function Recap_GetKey(arg1)

	local key = nil -- keep as nil

	if not arg1 or not tonumber(arg1) or arg1==0 then
		key=nil -- keep as nil
	else
		key=recap_temp.Keys[tonumber(arg1)]
	end
	return key
end

function Recap_LoadCombatants(filen)

	local i, j, k
	local sd = {} -- temp holding place for fields that may not exist
	local found, set_friend, set_dmgin, set_dmgout, set_maxhit, set_rawheal, set_overheal, set_heal, set_kills, set_timeOut, set_timeIn, set_timeHeal, set_faction, set_class, set_level, set_element, set_attribute
	local okayToLoad

	if not filen or string.len(filen)<1 or not recap_set[filen] then
		return
	end

	Recap_ResetAllCombatants(false)

	-- Hawksy: guard code and update of durations to version 3.59 and later
	if not recap[recap_temp.p].TotalDurationIn then
		recap[recap_temp.p].TotalDurationIn = recap[recap_temp.p].TotalDuration
	end
	if not recap[recap_temp.p].TotalDurationHeal then
		recap[recap_temp.p].TotalDurationHeal = recap[recap_temp.p].TotalDuration
	end
	recap[recap_temp.p].TotalDuration = recap[recap_temp.p].TotalDuration + recap_set[filen].TotalDuration
	if recap_set[filen].TotalDurationIn then
		recap[recap_temp.p].TotalDurationIn = recap[recap_temp.p].TotalDurationIn + recap_set[filen].TotalDurationIn
	else
		recap[recap_temp.p].TotalDurationIn = recap[recap_temp.p].TotalDurationIn + recap_set[filen].TotalDuration
	end
	if recap_set[filen].TotalDurationHeal then
		recap[recap_temp.p].TotalDurationHeal = recap[recap_temp.p].TotalDurationHeal + recap_set[filen].TotalDurationHeal
	else
		recap[recap_temp.p].TotalDurationHeal = recap[recap_temp.p].TotalDurationHeal + recap_set[filen].TotalDuration
	end

	-- Hawksy: silent repair
	if not recap_set[filen].Combatant then
		recap_set[filen].Combatant = {}
	end
	for i in pairs(recap_set[filen].Combatant) do

		if not recap.Combatant[i] then
			Recap_CreateBlankCombatant(i)
		end

		-- Hawksy: format from 3.61 and later
		found,_,set_friend,set_dmgin,set_dmgout,set_maxhit,set_rawheal,set_overheal,set_kills,set_timeOut,set_timeIn,set_timeHeal = string.find(recap_set[filen].Combatant[i], "(%w+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+%.?%d+) (%d+%.?%d+) (%d+%.?%d+)")
		if found then
			if set_friend == "true" then
				recap.Combatant[i].Friend = true
			end
			if tonumber(set_maxhit) > recap.Combatant[i].TotalMaxHit then
				recap.Combatant[i].TotalMaxHit = tonumber(set_maxhit)
			end
			recap.Combatant[i].TotalDmgIn = recap.Combatant[i].TotalDmgIn + tonumber(set_dmgin)
			recap.Combatant[i].TotalDmgOut = recap.Combatant[i].TotalDmgOut + tonumber(set_dmgout)
			recap.Combatant[i].TotalKills = recap.Combatant[i].TotalKills + tonumber(set_kills)
			recap.Combatant[i].TotalRawHeal = recap.Combatant[i].TotalRawHeal + tonumber(set_rawheal)
			recap.Combatant[i].TotalOverHeal = math.min((recap.Combatant[i].TotalOverHeal + tonumber(set_overheal)),recap.Combatant[i].TotalRawHeal)
			recap.Combatant[i].TotalTime = recap.Combatant[i].TotalTime + tonumber(set_timeOut)
			if recap.Combatant[i].TotalTime > recap_temp.MinTime then
				recap.Combatant[i].TotalDPS = Recap_Div1(recap.Combatant[i].TotalDmgOut,recap.Combatant[i].TotalTime)
			else
				recap.Combatant[i].TotalDPS = 0
			end
			recap.Combatant[i].TotalTimeIn = recap.Combatant[i].TotalTimeIn + tonumber(set_timeIn)
			if recap.Combatant[i].TotalTimeIn > recap_temp.MinTime then
				recap.Combatant[i].TotalDPSIn = Recap_Div1(recap.Combatant[i].TotalDmgIn,recap.Combatant[i].TotalTimeIn)
			else
				recap.Combatant[i].TotalDPSIn = 0
			end
			recap.Combatant[i].TotalTimeHeal = recap.Combatant[i].TotalTimeHeal + tonumber(set_timeHeal)
			if recap.Combatant[i].TotalTimeHeal > recap_temp.MinTime then
				recap.Combatant[i].TotalHPS = Recap_Div1((recap.Combatant[i].TotalRawHeal - recap.Combatant[i].TotalOverHeal),recap.Combatant[i].TotalTimeHeal)
			else
				recap.Combatant[i].TotalHPS = 0
			end
		else
			-- Hawksy: format from 3.54 and later
			found,_,set_friend,set_dmgin,set_dmgout,set_maxhit,set_heal,set_kills,set_timeOut,set_timeIn,set_timeHeal = string.find(recap_set[filen].Combatant[i], "(%w+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+%.?%d+) (%d+%.?%d+) (%d+%.?%d+)")
			if found then
				if set_friend == "true" then
					recap.Combatant[i].Friend = true
				end
				if tonumber(set_maxhit) > recap.Combatant[i].TotalMaxHit then
					recap.Combatant[i].TotalMaxHit = tonumber(set_maxhit)
				end
				recap.Combatant[i].TotalDmgIn = recap.Combatant[i].TotalDmgIn + tonumber(set_dmgin)
				recap.Combatant[i].TotalDmgOut = recap.Combatant[i].TotalDmgOut + tonumber(set_dmgout)
				recap.Combatant[i].TotalKills = recap.Combatant[i].TotalKills + tonumber(set_kills)
				-- details of overhealing lost on update to 3.61
				recap.Combatant[i].TotalRawHeal = recap.Combatant[i].TotalRawHeal + tonumber(set_heal)
				recap.Combatant[i].TotalOverHeal = 0
				recap.Combatant[i].TotalTime = recap.Combatant[i].TotalTime + tonumber(set_timeOut)
				if recap.Combatant[i].TotalTime > recap_temp.MinTime then
					recap.Combatant[i].TotalDPS = Recap_Div1(recap.Combatant[i].TotalDmgOut,recap.Combatant[i].TotalTime)
				else
					recap.Combatant[i].TotalDPS = 0
				end
				recap.Combatant[i].TotalTimeIn = recap.Combatant[i].TotalTimeIn + tonumber(set_timeIn)
				if recap.Combatant[i].TotalTimeIn > recap_temp.MinTime then
					recap.Combatant[i].TotalDPSIn = Recap_Div1(recap.Combatant[i].TotalDmgIn,recap.Combatant[i].TotalTimeIn)
				else
					recap.Combatant[i].TotalDPSIn = 0
				end
				recap.Combatant[i].TotalTimeHeal = recap.Combatant[i].TotalTimeHeal + tonumber(set_timeHeal)
				if recap.Combatant[i].TotalTimeHeal > recap_temp.MinTime then
					recap.Combatant[i].TotalHPS = Recap_Div1(recap.Combatant[i].TotalRawHeal,recap.Combatant[i].TotalTimeHeal)
				else
					recap.Combatant[i].TotalHPS = 0
				end
			else
				-- Hawksy: load format from 3.53 and earlier
				found,_,set_friend,set_dmgin,set_dmgout,set_maxhit,set_heal,set_kills,set_timeOut = string.find(recap_set[filen].Combatant[i], "(%w+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+%.?%d+)")
				if found then
					if set_friend == "true" then
						recap.Combatant[i].Friend = true
					end
					if tonumber(set_maxhit) > recap.Combatant[i].TotalMaxHit then
						recap.Combatant[i].TotalMaxHit = tonumber(set_maxhit)
					end
					recap.Combatant[i].TotalDmgIn = recap.Combatant[i].TotalDmgIn + tonumber(set_dmgin)
					recap.Combatant[i].TotalDmgOut = recap.Combatant[i].TotalDmgOut + tonumber(set_dmgout)
					recap.Combatant[i].TotalKills = recap.Combatant[i].TotalKills + tonumber(set_kills)
					-- details of overhealing lost on update to 3.61
					recap.Combatant[i].TotalRawHeal = recap.Combatant[i].TotalRawHeal + tonumber(set_heal)
					recap.Combatant[i].TotalOverHeal = 0
					recap.Combatant[i].TotalTime = recap.Combatant[i].TotalTime + tonumber(set_timeOut)
					if recap.Combatant[i].TotalTime > recap_temp.MinTime then
						recap.Combatant[i].TotalDPS = Recap_Div1(recap.Combatant[i].TotalDmgOut,recap.Combatant[i].TotalTime)
					else
						recap.Combatant[i].TotalDPS = 0
					end
					recap.Combatant[i].TotalTimeIn = 0
					recap.Combatant[i].TotalDPSIn = 0
					recap.Combatant[i].TotalTimeHeal = 0
					recap.Combatant[i].TotalHPS = 0
				end
			end
		end
		found,_,set_faction,set_class,set_level = string.find(recap_set[filen].Combatant[i], "~(%d+) (%d+) (%d+)")
		if not found then
			found,_,set_faction,set_class = string.find(recap_set[filen].Combatant[i], "~(%d+) (%d+)")
		end
		if found then
			recap.Combatant[i].Faction = Recap_GetKey(set_faction)
			recap.Combatant[i].Class = Recap_GetKey(set_class)
			recap.Combatant[i].Level = recap.Combatant[i].Level or set_level or 0
		end

		if not recap.Opt.LightData.value then

			-- process incoming: ^a000b000c000^
			for k in pairs(sd) do sd[k] = nil end -- wipe temp buffer -- keep as nil
			found,_,j = string.find(recap_set[filen].Combatant[i], "%^(.-)%^")
			if found then -- new saved method
				for k in pairs(inckey) do
					_,_,sd[k] = string.find(j, inckey[k].."(%d+)")
				end
			elseif string.find(recap_set[filen].Combatant[i], "%^%d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+") then
				-- old saved method
				found,_,sd.MeleeDamage,sd.MeleeMax,sd.MeleeHits,sd.MeleeCrits,sd.MeleeMissed,sd.MeleeDodged,sd.MeleeParried,sd.MeleeBlocked,
					sd.NonMeleeDamage,sd.NonMeleeMax,sd.NonMeleeHits,sd.NonMeleeCrits,sd.NonMeleeMissed =
						string.find(recap_set[filen].Combatant[i], "%^(%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+)")
			end
			if found then
				if sd.MeleeDamage and tonumber(sd.MeleeDamage)>0 then
					if not recap.Combatant[thisCombatant].IncomingDetail["1Melee"] then
						recap.Combatant[thisCombatant].IncomingDetail["1Melee"] = {}
					end
					recap.Combatant[thisCombatant].IncomingDetail["1Melee"].Element = "Melee"
					recap.Combatant[thisCombatant].IncomingDetail["1Melee"].HitsDmg = sd.MeleeDamage
					if sd.MeleeHits then
						recap.Combatant[thisCombatant].IncomingDetail["1Melee"].Hits = sd.MeleeHits
						recap.Combatant[thisCombatant].IncomingDetail["1Melee"].CritsEvents = sd.MeleeHits
					end
					if sd.MeleeCrits then
						recap.Combatant[thisCombatant].IncomingDetail["1Melee"].Crits = sd.MeleeCrits
						recap.Combatant[thisCombatant].IncomingDetail["1Melee"].CritsEvents = (recap.Combatant[thisCombatant].IncomingDetail["1Melee"].CritsEvents or 0) + sd.MeleeCrits
					end
					if sd.MeleeMax then
						recap.Combatant[thisCombatant].IncomingDetail["1Melee"].HitsMax = sd.MeleeMax
					end
					if sd.MeleeMissed then
						recap.Combatant[thisCombatant].IncomingDetail["1Melee"].Missed = sd.MeleeMissed
					end
					if sd.MeleeBlocked then
						recap.Combatant[thisCombatant].IncomingDetail["1Melee"].Blocked = sd.MeleeBlocked
					end
					if sd.MeleeParried then
						recap.Combatant[thisCombatant].IncomingDetail["1Melee"].Parried = sd.MeleeParried
					end
					if sd.MeleeDodged then
						recap.Combatant[thisCombatant].IncomingDetail["1Melee"].Dodged = sd.MeleeDodged
					end
				end
				if sd.NonMeleeDamage and tonumber(sd.NonMeleeDamage)>0 then
					if not recap.Combatant[thisCombatant].IncomingDetail["1Non-Melee"] then
						recap.Combatant[thisCombatant].IncomingDetail["1Non-Melee"] = {}
					end
					recap.Combatant[thisCombatant].IncomingDetail["1Non-Melee"].Element = "Non-Melee"
					recap.Combatant[thisCombatant].IncomingDetail["1Non-Melee"].HitsDmg = sd.NonMeleeDamage
					if sd.NonMeleeHits then
						recap.Combatant[thisCombatant].IncomingDetail["1Non-Melee"].Hits = sd.NonMeleeHits
						recap.Combatant[thisCombatant].IncomingDetail["1Non-Melee"].CritsEvents = sd.NonMeleeHits
					end
					if sd.NonMeleeCrits then
						recap.Combatant[thisCombatant].IncomingDetail["1Non-Melee"].Crits = sd.NonMeleeCrits
						recap.Combatant[thisCombatant].IncomingDetail["1Non-Melee"].CritsEvents = (recap.Combatant[thisCombatant].IncomingDetail["1Non-Melee"].CritsEvents or 0) + sd.NonMeleeCrits
					end
					if sd.NonMeleeMax then
						recap.Combatant[thisCombatant].IncomingDetail["1Non-Melee"].HitsMax = sd.NonMeleeMax
					end
					if sd.NonMeleeTicks then
						recap.Combatant[thisCombatant].IncomingDetail["1Non-Melee"].Ticks = sd.NonMeleeTicks
					end
					if sd.NonMeleeMissed then
						recap.Combatant[thisCombatant].IncomingDetail["1Non-Melee"].Missed = sd.NonMeleeMissed
					end
				end
			end

			-- test whether old format Detail or new format Outgoing Detail (from 3.59)
			local newFormat = false
			for j in string.gmatch(recap_set[filen].Combatant[i],"%[.-%]") do
				found,_,set_name,set_element = string.find(j, "%[(.-)%>(.-)%>")
				if found then
					newFormat = true
				end
				break
			end
			if newFormat then
				-- process Outgoing Detail: [name>element>a000b000c000]
				for j in string.gmatch(recap_set[filen].Combatant[i],"%[.-%]") do
					for k in pairs(sd) do sd[k] = nil end -- wipe temp buffer -- keep as nil
					found,_,set_name,set_element = string.find(j, "%[(.-)%>(.-)%>")
					if found then -- new saved method
						for k in pairs(detkey) do
							_,_,sd[k] = string.find(j, detkey[k].."(%d+)")
						end
						-- Hawksy: skip effects that have zero damage
						local total
						total = (sd.GlancesDmg or 0) + (sd.HitsDmg or 0) + (sd.CritsDmg or 0) + (sd.CrushDmg or 0) + (sd.TicksDmg or 0)
						if total == 0 then
							found = false
						end
					end
					if found then
						recap.Combatant[i].OutgoingDetail[set_name] = {}
						recap.Combatant[i].OutgoingDetail[set_name].Element = set_element
						for k in pairs(sd) do
							if sd[k] and tonumber(sd[k])>0 then
								recap.Combatant[i].OutgoingDetail[set_name][k] = tonumber(sd[k])
							end
						end
					end
				end
			else
				-- process Detail: [name:a000b000c000]
				for j in string.gmatch(recap_set[filen].Combatant[i],"%[.-%]") do
					for k in pairs(sd) do sd[k] = nil end -- wipe temp buffer
					found,_,set_name = string.find(j, "%[(.-)%>")
					if found then -- new saved method
						recap.Combatant[i].OutgoingDetail[set_name] = {}
						for k in pairs(detkey) do
							_,_,sd[k] = string.find(j, detkey[k].."(%d+)")
						end
					else
						found,_,set_name = string.find(j, "%[(.-) %d")
						if found then -- old saved method
							recap.Combatant[i].OutgoingDetail[set_name] = {}
							_,_,sd.HitsDmg,sd.Hits,sd.HitsMax,sd.CritsDmg,sd.Crits,sd.CritsMax,sd.CritsEvents,sd.Missed =
								string.find(j, "%[.+ (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+)") -- old method
						end
					end
					if found then
						recap.Combatant[i].OutgoingDetail[set_name].Element = "?"
						for k in pairs(sd) do
							if sd[k] and tonumber(sd[k])>0 then
								recap.Combatant[i].OutgoingDetail[set_name][k] = tonumber(sd[k])
							end
						end
					end
				end
			end

			-- process Incoming Detail: *name>element>a000b000c000*
			for j in string.gmatch(recap_set[filen].Combatant[i],"%*.-%*") do
				for k in pairs(sd) do sd[k] = nil end -- wipe temp buffer -- keep as nil
				found,_,set_name,set_element = string.find(j, "%*(.-)%>(.-)%>")
				if found then -- new saved method
					for k in pairs(detkey) do
						_,_,sd[k] = string.find(j, detkey[k].."(%d+)")
					end
					-- Hawksy: skip effects that have zero damage
					local total
					total = (sd.GlancesDmg or 0) + (sd.HitsDmg or 0) + (sd.CritsDmg or 0) + (sd.CrushDmg or 0) + (sd.TicksDmg or 0)
					if total == 0 then
						found = false
					end
				end
				if found then
					recap.Combatant[i].IncomingDetail[set_name] = {}
					recap.Combatant[i].IncomingDetail[set_name].Element = set_element
					for k in pairs(sd) do
						if sd[k] and tonumber(sd[k])>0 then
							recap.Combatant[i].IncomingDetail[set_name][k] = tonumber(sd[k])
						end
					end
				end
			end

			-- process Other Detail: #name>attribute>a000b000c000#
			for j in string.gmatch(recap_set[filen].Combatant[i],"%#.-%#") do
				for k in pairs(sd) do sd[k] = nil end -- wipe temp buffer -- keep as nil
				found,_,set_name,set_attribute = string.find(j, "%#(.-)%>(.-)%>")
				if found then -- new saved method
					for k in pairs(detkey) do
						_,_,sd[k] = string.find(j, detkey[k].."(%d+)")
					end
					recap.Combatant[i].OtherDetail[set_name] = {}
					if set_attribute ~= "?" then
						recap.Combatant[i].OtherDetail[set_name].Attribute = set_attribute
					end
					for k in pairs(sd) do
						if sd[k] and tonumber(sd[k])>0 then
							recap.Combatant[i].OtherDetail[set_name][k] = tonumber(sd[k])
						end
					end
				end
			end

			-- for the player, ensure that recap.Self has an entry for every OutgoingDetail
			if (i == recap_temp.Player) then
				-- Hawksy: silent repair
				if not recap.Combatant[i].OutgoingDetail then
					recap.Combatant[i].OutgoingDetail = {}
				end
				for j in pairs(recap.Combatant[i].OutgoingDetail) do
					if not recap.Self[recap_temp.s][j] then
						recap.Self[recap_temp.s][j] = {}
					end
				end
			end
		end
	end

	-- Hawksy: special case migration of Detail to OutgoingDetail, and Incoming to IncomingDetail for 3.59 and later, prior to saving
	Recap_MigrateDetails()

	-- Hawksy: special case correction to add OutgoingDetail.Ticks, OutgoingDetail.Ticks, and Self.Ticks
	Recap_CorrectForTicks()
end

function Recap_ResetAllCombatants(overrideLocks)

	local current_state = recap.Opt.State.value
	RecapUpdateFrame:Hide() -- disable OnUpdate
	recap.Opt.State.value = "Stopped"

	-- scan combatants and save a copy of any combatant marked as Locked (unless in synchronization)
	local i
	local local_save = {}
	-- Hawksy: silent repair
	if not recap.Combatant then
		recap.Combatant = {}
	end
	if not overrideLocks then
		for i in pairs(recap.Combatant) do
			if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
				if recap.Combatant[i].Locked then
					local_save[i] = recap.Combatant[i]
				end
			end
		end
	end
	-- clear all combatants
	recap.Combatant = {}
	-- restore the saved combatants (unless in synchronization)
	if not overrideLocks then
		for i in pairs(local_save) do
			recap.Combatant[i] = local_save[i]
		end
	end
	recap_temp.Last = {}
	for i in pairs(recap.Combatant) do
		if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
			recap_temp.Last[i] = { StartOut = 0, EndOut = 0, StartIn = 0, EndIn = 0, StartHeal = 0, EndHeal = 0, DmgIn = 0, DmgOut = 0, MaxHit = 0, Kills = 0, RawHeal = 0, OverHeal = 0 }
		end
	end
	recap[recap_temp.p].LastDuration = 0
	recap[recap_temp.p].LastDurationIn = 0
	recap[recap_temp.p].LastDurationHeal = 0
	recap[recap_temp.p].TotalDuration = 0
	recap[recap_temp.p].TotalDurationIn = 0
	recap[recap_temp.p].TotalDurationHeal = 0
	recap_temp.FightStart = 0
	recap_temp.FightEnd = 0
	recap_temp.FightStartIn = 0
	recap_temp.FightEndIn = 0
	recap_temp.FightStartHeal = 0
	recap_temp.FightEndHeal = 0
	recap_temp.ListSize = 0
	recap_temp.List = {}

	-- Hawksy: if Recent Data Mode is active, clear the buffers
	if recap.Opt.RecentData.value then
		Recap_CreateRecent()
	else
		recap_temp.Recent = {}
	end

	-- make sure end of fight delay timer is off
	recap_temp.EndFightDelayTimer = -1

	-- need to set idle timer if we were in combat
	if current_state == "Active" then
		-- start idle timers
		if not recap.Opt.LimitFights.value and recap.Opt.IdleFight.value then
			recap_temp.IdleTimer = 0
		end
		if not recap.Opt.LimitFights.value then
			recap_temp.HiddenIdleTimer = 0
		end
	end

	recap.Opt.State.value = current_state
	RecapUpdateFrame:Show() -- enable OnUpdate
end

function Recap_LoadSelf()

	recap.Self = recap.Self or {}
	recap.Self[recap_temp.s] = recap.Self[recap_temp.s] or {}
	-- not at the moment being done for the player's pets

	recap_temp.SelfListSize = 1
	recap_temp.SelfList = {}
end

local oldkeys = { ["Krieger"]="WARRIOR", ["Magier"]="MAGE", ["Schurke"]="ROGUE", ["Druide"]="DRUID",
				  ["J\195\164ger"]="HUNTER", ["Schamane"]="SHAMAN", ["Priester"]="PRIEST", ["Hexenmeister"]="WARLOCK",
				  ["Guerrier"]="WARRIOR", ["Voleur"]="ROGUE", ["Chasseur"]="HUNTER", ["Chaman"]="SHAMAN",
				  ["Prêtre"]="PRIEST", ["Démoniste"]="WARLOCK", ["Familier"]="Pet" }
function Recap_InitializeData()

	local i, j, k, l, m
	local temp_user = recap_temp.p

	if not recap.DataVersion then
		-- wipe all pre-2.6 data
		recap = {}
		recap.DataVersion = 3.2
	elseif recap.DataVersion < 3.0 then
		-- Hawksy: silent repair
		if not recap.Combatant then
			recap.Combatant = {}
		end
		-- convert 2.6 to 3.0, make classes uppercase
		for i in pairs(recap.Combatant) do
			if recap.Combatant[i].Faction=="Allianz" then
				recap.Combatant[i].Faction = "Alliance"
			end
			if recap.Combatant[i].Class then
				-- convert classes to english, which all locales will use for icon reference
				if oldkeys[recap.Combatant[i].Class] then
					recap.Combatant[i].Class = oldkeys[recap.Combatant[i].Class]
				end
				recap.Combatant[i].Class = string.upper(recap.Combatant[i].Class)
			end
		end
		recap.DataVersion = 3.0
	end

	-- If no Opt, create a default one
	if not recap.Opt then
		Recap_LoadDefaultOpt()
	end

	-- if this user hasn't been made before
	if not recap[recap_temp.p] then
		recap[recap_temp.p] = {}
		recap[recap_temp.p].LastDuration = 0
		recap[recap_temp.p].LastDurationIn = 0
		recap[recap_temp.p].LastDurationHeal = 0
		recap[recap_temp.p].TotalDuration = 0
		recap[recap_temp.p].TotalDurationIn = 0
		recap[recap_temp.p].TotalDurationHeal = 0
	end

	-- if this is a different or first-time user, or if this is first run of a new version,
	-- save the current combatants and options and then load them based on default.
	-- any changes in combatant or option format must come with a version change
	if (recap.User ~= recap_temp.p) or recap.DataVersion<Recap_Version then

		-- if we have data from previous user, save them
		if recap.User then

			-- Hawksy: special case migration of Detail to OutgoingDetail, and Incoming to IncomingDetail for 3.59 and later, prior to saving
			if Recap_Version >= 3.59 then
				Recap_MigrateDetails()
			end

			recap_temp.p = recap.User
			Recap_SaveCombatants("UserData:"..recap.User, false)
			Recap_SaveOpt(recap.User)
			recap_temp.p = temp_user
		end
		recap.User = recap_temp.p

		-- wipe slate, load defaults and overlay saved data ontop
		Recap_LoadDefaultOpt()
		Recap_ResetAllCombatants(false)

		-- convert data sets to new format (see readme)
		if recap.DataVersion < 3.2 then
			for i in pairs(recap_set) do
				if recap_set[i].Combatant then
					j = recap_set[i].TimeStamp
					Recap_LoadCombatants(i)
					for k in pairs(recap_set[i].Combatant) do
						l = (l or 0) + string.len(recap_set[i].Combatant[k])
					end
					Recap_SaveCombatants(i, false)
					for k in pairs(recap_set[i].Combatant) do
						m = (m or 0) + string.len(recap_set[i].Combatant[k])
					end
					recap_set[i].TimeStamp = j
				end
			end
			recap.DataVersion = 3.2
		end

		Recap_LoadCombatants("UserData:"..recap_temp.p)
		Recap_LoadOpt(recap_temp.p)

		recap.DataVersion = Recap_Version
	end

	-- Hawksy: silent repair
	if not recap.Combatant then
		recap.Combatant = {}
	end
	for i in pairs(recap.Combatant) do
		if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
			recap_temp.Last[i] = { StartOut = 0, EndOut = 0, StartIn = 0, EndIn = 0, StartHeal = 0, EndHeal = 0, DmgIn = 0, DmgOut = 0, MaxHit = 0, Kills = 0, RawHeal = 0, OverHeal = 0 }
		end
	end

	-- remove current UserData: - it's now "live" in recap.Combatant and elsewhere
	if recap_set and recap_set["UserData:"..recap_temp.p] then
		recap_set["UserData:"..recap_temp.p] = nil -- keep as nil
	end

	Recap_LoadSelf()

	-- Hawksy: special case migration of Detail to OutgoingDetail, and Incoming to IncomingDetail for 3.59 and later, prior to saving
	Recap_MigrateDetails()

	-- Hawksy: special case correction for some damage:healing DoT:HoT spells
	Recap_CorrectDualDoTHoT()

	-- Hawksy: special case correction to add OutgoingDetail.Ticks, OutgoingDetail.Ticks, and Self.Ticks
	Recap_CorrectForTicks()
end


--[[ Fight report functions ]]

function Recap_AutoPostGetStatID(arg1)

	if arg1=="Damage" then
		return "DmgOut"
	elseif arg1=="Tanking" then
		return "DmgIn"
	elseif arg1=="Healing" then
		return "Heal"
	end
	return "DPS"
end

function Recap_PostFight()

	local print_chat = SendChatMessage
	local i, chatchan, chatnum, text

	if recap.Opt.SpamRows.value then
		Recap_PostSpamRows(Recap_AutoPostGetStatID(recap.Opt.AutoPost.Stat), true)
	else
		chatchan = recap.Opt.AutoPost.Channel
		-- de-localize the channel name
		for i=1,5 do -- hardcoded
			if chatchan == recap_temp.Localize.ChannelDropList[i] then
				chatchan = recap_temp.CanonicalChannelDropList[i]
				break
			end
		end
		if not chatchan then
			chatchan = "Self"
		end
		_,_,chatnum = string.find(chatchan, "(%d+)")
		if chatnum then
			chatchan = "CHANNEL"
		end
		if chatchan=="Self" then
			print_chat = Recap_Print
		end

		-- If the Clip box on the Options : Reports panel is open, post there instead of to chat or console
		if RecapClipEditBox:IsVisible() then
			print_chat = Recap_LineToClipboard
		end

		text = Recap_PostSpamLine(Recap_AutoPostGetStatID(recap.Opt.AutoPost.Stat))
		if text and string.find(text, "%d") then
			print_chat(Recap_PostSpamLine(Recap_AutoPostGetStatID(recap.Opt.AutoPost.Stat)), chatchan, nil, chatnum)
		end
	end

end

function Recap_PostLeader()

	local print_chat = SendChatMessage
	local i, chatchan, chatnum, leader, statid, thisCombatant

	if recap_temp.ListSize>2 then
		chatchan = recap.Opt.AutoPost.Channel
		-- de-localize the channel name
		for i=1,5 do -- hardcoded
			if chatchan == recap_temp.Localize.ChannelDropList[i] then
				chatchan = recap_temp.CanonicalChannelDropList[i]
				break
			end
		end
		if not chatchan then
			chatchan = "Self"
		end
		_,_,chatnum = string.find(chatchan, "(%d+)")
		if chatnum then
			chatchan = "CHANNEL"
		end
		if chatchan=="Self" then
			print_chat = Recap_Print
		end

		-- If the Clip box on the Options : Reports panel is open, post there instead of to chat or console
		if RecapClipEditBox:IsVisible() then
			print_chat = Recap_LineToClipboard
		end

		statid = Recap_AutoPostGetStatID(recap.Opt.AutoPost.Stat)

		if not recap.Combatant[recap_temp.StatLeader] then -- in case leader reset
			recap_temp.StatLeader = false
		end

		-- Hawksy: guard code
		thisCombatant = recap_temp.List[1].Name
		if (not recap_temp.StatLeader) and thisCombatant and recap.Combatant[thisCombatant] and recap.Combatant[thisCombatant].Friend then
			recap_temp.StatLeader = thisCombatant
			leader = recap_temp.StatLeader -- spam initial leader
		end

		if recap_temp.StatLeader then
			for i in pairs(recap.Combatant) do
				if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
					if (recap.Combatant[i]["Total"..statid] > recap.Combatant[recap_temp.StatLeader]["Total"..statid]) and recap.Combatant[i].Friend then
						leader = i
					end
				end
			end
			if leader then
				recap_temp.StatLeader = leader
				print_chat(string.format(recap_temp.Localize.NewLeader.." "..recap_temp.Localize.For.." %s: %s "..recap_temp.Localize.With.." %s",recap.Opt.AutoPost.Stat,leader,Recap_FormatStat(statid,leader)), chatchan, nil, chatnum)
			end
		end
	end
end


-- returns 'text' to be spammed in autopost or linked in chat
function Recap_PostSpamLine(arg1)

	local text, headertext, i

	text = ""
	headertext = "Recap: "..recap_temp.Localize.LinkRank[arg1].." "..recap_temp.Localize.For.." "..recap_temp.LastAll[recap.Opt.View.value]..":"
	for i=1,recap.Opt.MaxRank.value do
		-- Hawksy: guard code
		if recap_temp.List[i] then
			thisCombatant = recap_temp.List[i].Name
			if (i<recap_temp.ListSize) and thisCombatant and recap.Combatant[thisCombatant] and recap.Combatant[thisCombatant].Friend then
				if recap_temp.List[i][arg1]>0 then
					if headertext then
						text = headertext
						headertext = nil -- keep as nil
					end
					text = text.." ["..i.."."..thisCombatant.." "..Recap_FormatStat(arg1,i).."]"
				end
			end
		end
	end
	return text
end

-- Hawksy: function to package channel info calculation
function Recap_GetChannelInfo(recapAuto, forceSelf)

	local chatchan, chatnum

	chatchan = false
	chatnum = false
	if recapAuto then
		chatchan = recap.Opt.AutoPost.Channel
		-- de-localize the channel name
		for i=1,5 do -- hardcoded
			if chatchan == recap_temp.Localize.ChannelDropList[i] then
				chatchan = recap_temp.CanonicalChannelDropList[i]
				break
			end
		end
	else
		if forceSelf then
			chatchan = "Self"
		else
			if ChatFrameEditBox:IsVisible() then
				chatchan = ChatFrameEditBox:GetAttribute("chatType")
				if chatchan=="WHISPER" then
					chatnum = ChatFrameEditBox:GetAttribute("tellTarget")
				elseif chatchan=="CHANNEL" then
					chatnum = ChatFrameEditBox:GetAttribute("channelTarget")
				end
			end
		end
	end
	if not chatchan then
		chatchan = "Self"
	end
	if not chatnum then
		_,_,chatnum = string.find(chatchan, "(%d+)")
		if chatnum then
			chatchan = "CHANNEL"
		end
	end

	return chatchan, chatnum
end

-- arg1 = stat to report by (DPS, DmgOut, etc)
function Recap_PostSpamRows(arg1, recapAuto)

	local i
	local headertext = nil -- keep as nil
	local print_chat = SendChatMessage
	local chatchan, chatnum

	arg1 = string.gsub(arg1,"P$","") -- strip trailing P from % ids

	chatchan, chatnum = Recap_GetChannelInfo(recapAuto, nil)
	if chatchan=="Self" then
		print_chat = Recap_Print
	end

	-- If the Clip box on the Options : Reports panel is open, post there instead of to chat or console
	if RecapClipEditBox:IsVisible() then
		print_chat = Recap_LineToClipboard
	end

	headertext = "__ Recap: "..recap_temp.Localize.LinkRank[arg1].." "..recap_temp.Localize.For.." "..recap_temp.LastAll[recap.Opt.View.value].." __"
	for i=1,recap.Opt.MaxRank.value do
		-- Hawksy: guard code
		if recap_temp.List[i] and recap_temp.List[i].Name then
			thisCombatant = recap_temp.List[i].Name
			if (i<recap_temp.ListSize) and thisCombatant and recap.Combatant[thisCombatant] and recap.Combatant[thisCombatant].Friend then
				text = i..". "..thisCombatant..": "..Recap_FormatStat(arg1,i)
				if recap_temp.List[i][arg1] > 0 then
					if headertext then
						print_chat(headertext, chatchan, nil, chatnum)
						headertext = nil -- keep as nil
					end
					print_chat(text, chatchan, nil, chatnum)
				end
			end
		end
	end
end

-- Hawksy: two functions added to report Recent Events
function Recap_PostRecentRows(outgoing, filterCombatant, filterEffect, forceSelf)

	local i, first, last, index, message, sentEOF, iCount, iStart, maxLines
	local print_chat = SendChatMessage
	local chatchan, chatnum

	chatchan, chatnum = Recap_GetChannelInfo(nil, forceSelf)
	-- block posting to channels that would be excessively annoying to others
	--   (also blocks other channels that happen to have BlockChannels as substrings -- tough)
	if chatchan then
		for i=1,#recap_temp.Localize.BlockChannels do
			if string.find(string.lower(chatchan), recap_temp.Localize.BlockChannels[i], 1, true) then
				PlaySound("igQuestFailed")
				return
			end
		end
	end
	if chatchan=="Self" then
		print_chat = Recap_Print
	end

	-- If the Clip box on the Options : Reports panel is open, post there instead of to chat or console
	-- Writing to the Clipboard seems to be slow, and it has a maximum size of 32000 letters
	-- It takes about 10 seconds to post 400 lines
	maxLines = recap_temp.MaxRecentEventLinesChat
	if RecapClipEditBox:IsVisible() then
		print_chat = Recap_LineToClipboard
		maxLines = recap_temp.MaxRecentEventLinesClipboard
	end

	-- check backwards until we have found up to maxLines
	iCount = 2 -- bookend lines
	if recap_temp.Recent then
		first = recap_temp.Recent.First
		iStart = recap_temp.Recent.Last
		if not ((iStart == first) and not (recap_temp.Recent[iStart] and recap_temp.Recent[iStart].T)) then
			sentEOF = false
			for index=1, recap.Opt.RecentEventCount.value do
				if recap_temp.Recent[iStart].S == "**EOF**" then
					if sentEOF ~= true then
						iCount = iCount + 1
						if iCount >= maxLines then
							break
						end
						sentEOF = true
					end
				else
					if Recap_CheckRecentEvent(iStart, outgoing, filterCombatant, filterEffect) then
						iCount = iCount + 1
						if iCount >= maxLines then
							break
						end
						sentEOF = false
					end
				end
				if (iStart == first) then
					break
				end
				if iStart <= 1 then
					iStart = recap.Opt.RecentEventCount.value
				else
					iStart = iStart - 1
				end
			end
		end
	end

	-- now generate output forwards
	if recap_temp.Recent then
		i = iStart
		last = recap_temp.Recent.Last
		print_chat("Recap: ----  vvvv  ----", chatchan, nil, chatnum)
		if (i == last) and not (recap_temp.Recent[i] and recap_temp.Recent[i].T) then
			-- no data at all
		else
			-- data exists, post the rows (there may be none due to filtering)
			sentEOF = true -- set to skip any initial 'end of fight' markers
			for index=1, recap.Opt.RecentEventCount.value do
				if recap_temp.Recent[i].S == "**EOF**" then
					-- maybe post an 'end of fight' marker
					if sentEOF == true then
						-- avoid sending two in a row, and avoid leading 'end of fight' indicators
					else
						-- special 'end of fight' message
						message = Recap_FormatTimeRecent(recap_temp.Recent[i].T).." --------------------------------------------------"
						print_chat(message, chatchan, nil, chatnum)
						sentEOF = true
					end
				else
					-- not an 'end of fight'
					message = Recap_FormatRecentEvent(i, outgoing, filterCombatant, filterEffect)
					-- if filterCombatant or filterEffect fail, the message gets returned as nil
					if message then
						print_chat(message, chatchan, nil, chatnum)
						sentEOF = false
					end
				end
				if (i == last) then
					-- just posted the most recent one, leave the loop
					break
				end
				-- increment circular pointer
				if i >= recap.Opt.RecentEventCount.value then
					i = 1
				else
					i = i + 1
				end
			end
		end
		print_chat("Recap: ----  ^^^^  ----", chatchan, nil, chatnum)
	end
end

function Recap_CheckRecentEvent(i, outgoing, filterCombatant, filterEffect)

	local source = recap_temp.Recent[i].S
	local dest = recap_temp.Recent[i].D
	local printEffect = string.sub(recap_temp.Recent[i].E,2)
	local okayCombatant = false
	if filterCombatant then
		if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
			if (recap.Opt.SyncMergePets.value == "on") and recap.Combatant[filterCombatant].OwnsPet then
				if outgoing and source and recap.Combatant[source] and (recap.Combatant[source].OwnedBy == filterCombatant) then
					okayCombatant = true
				elseif not outgoing and recap.Combatant[dest] and (recap.Combatant[dest].OwnedBy == filterCombatant) then
					okayCombatant = true
				end
			end
		else
			if recap.Opt.MergePets.value and recap.Combatant[filterCombatant].OwnsPet then
				if outgoing and source and recap.Combatant[source] and (recap.Combatant[source].OwnedBy == filterCombatant) then
					okayCombatant = true
				elseif not outgoing and recap.Combatant[dest] and (recap.Combatant[dest].OwnedBy == filterCombatant) then
					okayCombatant = true
				end
			end
		end
		if outgoing and source and (filterCombatant == source) then
			okayCombatant = true
		elseif not outgoing and (filterCombatant == dest) then
			okayCombatant = true
		end
	else
		okayCombatant = true
	end
	if okayCombatant == false then
		return nil
	end
	if filterEffect and (filterEffect ~= printEffect) then
		return nil
	end
	return true
end

function Recap_FormatRecentEvent(i, outgoing, filterCombatant, filterEffect)

	local printTime = Recap_FormatTimeRecent(recap_temp.Recent[i].T)
	local source = recap_temp.Recent[i].S
	local dest = recap_temp.Recent[i].D
	local effectType = string.sub(recap_temp.Recent[i].E,1,1)
	local printEffect = string.sub(recap_temp.Recent[i].E,2)
	local printElement
	if recap_temp.Recent[i].L then
		printElement = recap_temp.Recent[i].L
	else
		printElement = "?"
	end
	local amountNumeric = false
	if type(recap_temp.Recent[i].A) == "number" then
		amountNumeric = true
	end
	local printAmount = tostring(recap_temp.Recent[i].A)
	local printCrit
	printCrit = ""
	if recap_temp.Recent[i].C then
		-- critical blow
		printCrit = "*"
	end
	if recap_temp.Recent[i].R then
		-- crushing blow
		printCrit = "**"
	end

	local okayCombatant = false
	if filterCombatant then
		-- filtering by combatant
		if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
			-- synchronization active
			if (recap.Opt.SyncMergePets.value == "on") and recap.Combatant[filterCombatant].OwnsPet then
				-- merge is active and combatant has pets, see if this source or destination is one of their pets
				if outgoing and source and recap.Combatant[source] and (recap.Combatant[source].OwnedBy == filterCombatant) then
					okayCombatant = true
				elseif not outgoing and recap.Combatant[dest] and (recap.Combatant[dest].OwnedBy == filterCombatant) then
					okayCombatant = true
				end
			end
		else
			-- not synchronization
			if recap.Opt.MergePets.value and recap.Combatant[filterCombatant].OwnsPet then
				-- merge is active and combatant has pets, see if this source or destination is one of their pets
				if outgoing and source and recap.Combatant[source] and (recap.Combatant[source].OwnedBy == filterCombatant) then
					okayCombatant = true
				elseif not outgoing and recap.Combatant[dest] and (recap.Combatant[dest].OwnedBy == filterCombatant) then
					okayCombatant = true
				end
			end
		end
		-- check the combatant
		if outgoing and source and (filterCombatant == source) then
			okayCombatant = true
		elseif not outgoing and (filterCombatant == dest) then
			okayCombatant = true
		end
	else
		-- no filtering by combatant
		okayCombatant = true
	end
	if okayCombatant == false then
		return nil
	end

	-- filter by effect
	if filterEffect and (filterEffect ~= printEffect) then
		return nil
	end

	-- prepare the message
	if source then
		if amountNumeric then
			if effectType == "1" then
				-- damage
				if (printEffect == printElement) or (printElement == recap_temp.Localize.ElementOther) then
					return printTime.." "..source.." ("..printEffect..") => "..dest.." ( -"..printAmount..printCrit.." )"
				else
					return printTime.." "..source.." ("..printEffect.." ("..printElement..")) => "..dest.." ( -"..printAmount..printCrit.." )"
				end
			else
				-- heal
				if (printEffect == printElement) or (printElement == recap_temp.Localize.ElementOther) then
					return printTime.." "..source.." ("..printEffect..") => "..dest.." ( +"..printAmount..printCrit.." )"
				else
					return printTime.." "..source.." ("..printEffect.." ("..printElement..")) => "..dest.." ( +"..printAmount..printCrit.." )"
				end
			end
		else
			if printAmount == "cast" then
				-- successful cast, target not available, not sure how best to display this
				return printTime.." "..source.." ("..printEffect..") => "..dest
			else
				return printTime.." "..source.." ("..printEffect..") => "..dest.." ( "..printAmount.." )"
			end
		end
	else
		if amountNumeric then
			if effectType == "1" then
				-- damage
				if (printEffect == printElement) or (printElement == recap_temp.Localize.ElementOther) then
					return printTime.." ("..printEffect..") => "..dest.." ( -"..printAmount..printCrit.." )"
				else
					return printTime.." ("..printEffect.." ("..printElement..")) => "..dest.." ( -"..printAmount..printCrit.." )"
				end
			else
				-- heal
				if (printEffect == printElement) or (printElement == recap_temp.Localize.ElementOther) then
					return printTime.." ("..printEffect..") => "..dest.." ( +"..printAmount..printCrit.." )"
				else
					return printTime.." ("..printEffect.." ("..printElement..")) => "..dest.." ( +"..printAmount..printCrit.." )"
				end
			end
		else
			if printAmount == "cast" then
				-- successful cast, target not available, not sure how best to display this
				return printTime.." ("..printEffect..") => "..dest
			else
				return printTime.." ("..printEffect..") => "..dest.." ( "..printAmount.." )"
			end
		end
	end
end

-- returns a formatted string for arg1 stat ("DPS" "Time" etc) for combatant arg2 (number is .List[], non-number is name)
function Recap_FormatStat(arg1,arg2)

	local i

	arg1 = string.gsub(arg1,"P$","")

	if not tonumber(arg2) then

		for i=1,(recap_temp.ListSize-1) do
			if recap_temp.List[i].Name==arg2 then
				arg2 = i
				i = recap_temp.ListSize
			end
		end
	end

	if tonumber(arg2) then

		if arg1=="DPS" then
			return string.format("%.1f",recap_temp.List[arg2].DPS)
		elseif arg1=="DPSIn" then
			return string.format("%.1f",recap_temp.List[arg2].DPSIn)
		elseif arg1=="HPS" then
			return string.format("%.1f",recap_temp.List[arg2].HPS)
		elseif arg1=="DPSvsAll" then
			return string.format("%.1f",recap_temp.List[arg2].DPSvsAll)
		elseif arg1=="Time" then
			return Recap_FormatTime(recap_temp.List[arg2].Time)
		elseif arg1=="TimeIn" then
			return Recap_FormatTime(recap_temp.List[arg2].TimeIn)
		elseif arg1=="TimeHeal" then
			return Recap_FormatTime(recap_temp.List[arg2].TimeHeal)
		elseif arg1=="DmgOut" then
			return recap_temp.List[arg2].DmgOut.." ("..recap_temp.List[arg2].DmgOutP.."%)"
		elseif arg1=="DmgIn" then
			return recap_temp.List[arg2].DmgIn.." ("..recap_temp.List[arg2].DmgInP.."%)"
		elseif arg1=="Heal" then
			return recap_temp.List[arg2].Heal.." ("..recap_temp.List[arg2].HealP.."%)".." ("..recap_temp.List[arg2].Over.."%>)"
		elseif arg1=="Over" then
			return recap_temp.List[arg2].Over.."%"
		else
			return recap_temp.List[arg2][arg1]
		end
	end

	return ""
end

function Recap_Print(arg1)
	DEFAULT_CHAT_FRAME:AddMessage(tostring(arg1))
end

RecapAux_lua_372 = true
