--[[ Fight functions ]]

function Recap_CreateCombatantAndUpdateTimers(thisCombatant, event_time, timeType)

	-- guard code
	if not thisCombatant then
		return
	end

	-- if thisCombatant not a .Combatant, add an entry for them
	if not recap.Combatant[thisCombatant] then
		Recap_CreateBlankCombatant(thisCombatant)
	end

	-- if they haven't fought this session, add a .Last entry for them (only do it once, we get called repeatedly)
	if not recap_temp.Last[thisCombatant] then
		recap_temp.Last[thisCombatant] = { StartOut = 0, EndOut = 0, StartIn = 0, EndIn = 0, StartHeal = 0, EndHeal = 0, DmgIn = 0, DmgOut = 0, MaxHit = 0, Kills = 0, RawHeal = 0, OverHeal = 0 }
	end

	local iLast = recap_temp.Last[thisCombatant]

	-- Hawksy: If we get here for any reason we might as well start the overall fight timers
	--   (Recap Incoming.32 would only do so for a source doing damage, or for a spellcast start)
	--   (Is healing after a fight, thus starting a 'new' fight, any different from someone ganking a toad?)
	-- Hawksy: now three independent overall fight timers
	if timeType == "Out" then
		if recap_temp.FightStart==0 then
			recap_temp.FightStart = event_time
		end
		recap_temp.FightEnd = event_time
	end
	if timeType == "In" then
		if recap_temp.FightStartIn==0 then
			recap_temp.FightStartIn = event_time
		end
		recap_temp.FightEndIn = event_time
	end
	if timeType == "Heal" then
		if recap_temp.FightStartHeal==0 then
			recap_temp.FightStartHeal = event_time
		end
		recap_temp.FightEndHeal = event_time
	end

	-- Hawksy: three independent timers per combatant for damage dealt, damage received, and healing done
	--   (at the moment no timer for healing received)
	if timeType == "Out" then
		if iLast.StartOut==0 then
			iLast.StartOut = event_time
		end
		iLast.EndOut = event_time
	elseif timeType == "In" then
		if iLast.StartIn==0 then
			iLast.StartIn = event_time
		end
		iLast.EndIn = event_time
	elseif timeType == "Heal" then
		if iLast.StartHeal==0 then
			iLast.StartHeal = event_time
		end
		iLast.EndHeal = event_time
	end

	-- anyone who does damage, takes damage, or heals will be flagged as having
	--   been in the fight, but simply getting healed still doesn't count
	recap.Combatant[thisCombatant].WasInCurrent = true
end

function Recap_StartFight()

	local i

	if recap.Opt.AutoHide.value and not recap.Opt.Minimized.value then
		RecapFrame_Hide()
	end

	-- Hawksy: only clear WasInCurrent once per fight
	if	not recap_temp.ResetWasInCurrent then
		-- Hawksy: silent repair
		if not recap.Combatant then
			recap.Combatant = {}
		end
		for i in pairs(recap.Combatant) do
			if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
				recap.Combatant[i].WasInCurrent = false
			end
		end
		recap_temp.ResetWasInCurrent = true
	end

	-- start idle timers
	if not recap.Opt.LimitFights.value and recap.Opt.IdleFight.value then
		recap_temp.IdleTimer = 0
	end
	if not recap.Opt.LimitFights.value then
		recap_temp.HiddenIdleTimer = 0
	end

	-- make sure end of fight delay timer is off
	recap_temp.EndFightDelayTimer = -1

	-- Hawksy: check friends at start of fight to get HP for more accurate
	--   calculation of overhealing.  See also Recap_GetUnitInfo.
	Recap_MakeFriends()

	-- clear any existing synchronization update data (done outside the spinlock, I'm hoping that's safe)
	recap_temp.SyncData = {}

	Recap_SetState("Active")

end

function Recap_AddPetToOwner(pet, owner)

	local combatantPet = recap.Combatant[pet]
	local combatantOwner = recap.Combatant[owner]
	local lastPet = recap_temp.Last[pet]
	local lastOwner = recap_temp.Last[owner]

	-- Hawksy: guard code
	if not combatantPet then
		return
	end
	if not combatantOwner then
		return
	end
	if not lastPet then
		return
	end
	if not lastOwner then
		return
	end

	-- Hawksy: now three independent times, for damage, incoming, and healing
	if (combatantPet.WasInCurrent and (lastPet.StartOut > 0)) and
	   (combatantOwner.WasInCurrent and (lastOwner.StartOut > 0)) then
		-- both pet and owner were in the fight and did damage
		lastOwner.StartOut = math.min(lastPet.StartOut,lastOwner.StartOut)
		lastOwner.EndOut = math.max(lastPet.EndOut,lastOwner.EndOut)
	elseif (combatantPet.WasInCurrent and (lastPet.StartOut > 0)) and
		   (not combatantOwner.WasInCurrent or (lastOwner.StartOut == 0)) then
		-- the pet was in the fight and did damage, the owner did no damage
		lastOwner.StartOut = lastPet.StartOut
		lastOwner.EndOut = lastPet.EndOut
	end
	if (combatantPet.WasInCurrent and (lastPet.StartIn > 0)) and
	   (combatantOwner.WasInCurrent and (lastOwner.StartIn > 0)) then
		-- both pet and owner were in the fight and took damage
		lastOwner.StartIn = math.min(lastPet.StartIn,lastOwner.StartIn)
		lastOwner.EndIn = math.max(lastPet.EndIn,lastOwner.EndIn)
	elseif (combatantPet.WasInCurrent and (lastPet.StartIn > 0)) and
		   (not combatantOwner.WasInCurrent or (lastOwner.StartIn == 0)) then
		-- the pet was in the fight and took damage, the owner took no damage
		lastOwner.StartIn = lastPet.StartIn
		lastOwner.EndIn = lastPet.EndIn
	end
	if (combatantPet.WasInCurrent and (lastPet.StartHeal > 0)) and
	   (combatantOwner.WasInCurrent and (lastOwner.StartHeal > 0)) then
		-- both pet and owner were in the fight and did healing
		lastOwner.StartHeal = math.min(lastPet.StartHeal,lastOwner.StartHeal)
		lastOwner.EndHeal = math.max(lastPet.EndHeal,lastOwner.EndHeal)
	elseif (combatantPet.WasInCurrent and (lastPet.StartHeal > 0)) and
		   (not combatantOwner.WasInCurrent or (lastOwner.StartHeal == 0)) then
		-- the pet was in the fight and did healing, the owner did no healing
		lastOwner.StartHeal = lastPet.StartHeal
		lastOwner.EndHeal = lastPet.EndHeal
	end

	lastOwner.DmgIn = lastOwner.DmgIn + lastPet.DmgIn
	lastOwner.DmgOut = lastOwner.DmgOut + lastPet.DmgOut
	lastOwner.MaxHit = math.max(lastOwner.MaxHit,lastPet.MaxHit)
	-- Hawksy: add in heals and kills
	lastOwner.Kills = lastOwner.Kills + lastPet.Kills
	lastOwner.RawHeal = lastOwner.RawHeal + lastPet.RawHeal
	lastOwner.OverHeal = math.min((lastOwner.OverHeal + lastPet.OverHeal),lastOwner.RawHeal)

	lastPet.StartOut = 0
	lastPet.EndOut = 0
	lastPet.StartIn = 0
	lastPet.EndIn = 0
	lastPet.StartHeal = 0
	lastPet.EndHeal = 0
	lastPet.DmgIn = 0
	lastPet.DmgOut = 0
	lastPet.MaxHit = 0
	lastPet.Kills = 0
	lastPet.RawHeal = 0
	lastPet.OverHeal = 0

	combatantPet.WasInCurrent = false
	combatantOwner.WasInCurrent = true -- if the owner wasn't in the fight, the owner is now

	-- Hawksy: effects are now treated separately

end

function Recap_AccumulateCombatant(i, iCombatant, iLast)

	local iLastDmgIn, iLastDmgOut, iLastTimeOut, iLastTimeIn, iLastTimeHeal, iLastMaxHit, iLastKills, iLastRawHeal, iLastOverHeal

	iLastDmgIn = iLast.DmgIn
	iLastDmgOut = iLast.DmgOut
	-- these three if statements shouldn't be necessary, but I'm getting paranoid about the reported bad Time values
	if iLast.StartOut > 0 and (iLast.EndOut > iLast.StartOut) then
		iLastTimeOut = iLast.EndOut - iLast.StartOut
	else
		iLastTimeOut = 0
	end
	if iLast.StartIn > 0 and (iLast.EndIn > iLast.StartIn) then
		iLastTimeIn = iLast.EndIn - iLast.StartIn
	else
		iLastTimeIn = 0
	end
	if iLast.StartHeal > 0 and (iLast.EndHeal > iLast.StartHeal) then
		iLastTimeHeal = iLast.EndHeal - iLast.StartHeal
	else
		iLastTimeHeal = 0
	end
	iLastMaxHit = iLast.MaxHit
	iLastKills = iLast.Kills
	iLastRawHeal = iLast.RawHeal
	iLastOverHeal = math.min(iLast.OverHeal,iLastRawHeal)

	-- Hawksy: If there is non-zero damage or healing, force the matching time to be
	--   (somewhat arbitrarily) at least as large as the global cooldown ("DeemedTime").
	--   This gives some duration to single-shot fights (which are accumulated), keeping
	--   the anomalous DPS down a bit, without displaying them or their DPS.
	if (iLastDmgOut > 0) and (iLastTimeOut < recap_temp.DeemedTime) then
		iLastTimeOut = recap_temp.DeemedTime
	end
	if (iLastDmgIn > 0) and (iLastTimeIn < recap_temp.DeemedTime) then
		iLastTimeIn = recap_temp.DeemedTime
	end
	if (iLastRawHeal > 0) and (iLastTimeHeal < recap_temp.DeemedTime) then
		iLastTimeHeal = recap_temp.DeemedTime
	end

	-- warning (possibly temporary) if any Last time is greater than 30 minutes (1800 seconds) (English only atm)
	if (iLastTimeOut > 1800) or (iLastTimeIn > 1800) or (iLastTimeHeal > 1800) then
		DEFAULT_CHAT_FRAME:AddMessage("Recap: Warning: One or more times for Last Fight for "..i.." exceeded 30 minutes.  Did the fight really last that long?  Or is there an error in Recap?  Or did something unusual happen recently with your computer or your toon?")
		if recap_temp.snap then
			Screenshot()
		end
	end

	if recap_temp.LastFightSignificant then
		iCombatant.LastDmgIn = iLastDmgIn
		iCombatant.LastDmgOut = iLastDmgOut
		iCombatant.LastTime = iLastTimeOut
		iCombatant.LastTimeIn = iLastTimeIn
		iCombatant.LastTimeHeal = iLastTimeHeal
		iCombatant.LastMaxHit = iLastMaxHit
		iCombatant.LastKills = iLastKills
		iCombatant.LastRawHeal = iLastRawHeal
		iCombatant.LastOverHeal = math.min(iLastOverHeal,iLastRawHeal)
	end

	iLast.StartOut = 0
	iLast.EndOut = 0
	iLast.StartIn = 0
	iLast.EndIn = 0
	iLast.StartHeal = 0
	iLast.EndHeal = 0
	iLast.DmgIn = 0
	iLast.DmgOut = 0
	iLast.MaxHit = 0
	iLast.Kills = 0
	iLast.RawHeal = 0
	iLast.OverHeal = 0

	iCombatant.TotalDmgIn = iCombatant.TotalDmgIn + iLastDmgIn
	iCombatant.TotalDmgOut = iCombatant.TotalDmgOut + iLastDmgOut
	iCombatant.TotalTime = iCombatant.TotalTime + iLastTimeOut
	iCombatant.TotalTimeIn = iCombatant.TotalTimeIn + iLastTimeIn
	iCombatant.TotalTimeHeal = iCombatant.TotalTimeHeal + iLastTimeHeal
	iCombatant.TotalMaxHit = math.max(iLastMaxHit,iCombatant.TotalMaxHit)
	iCombatant.TotalKills = iCombatant.TotalKills + iLastKills
	iCombatant.TotalRawHeal = iCombatant.TotalRawHeal + (iLastRawHeal or 0)
	iCombatant.TotalOverHeal = math.min((iCombatant.TotalOverHeal + (iLastOverHeal or 0)),iCombatant.TotalRawHeal)

	-- Hawksy: three flavours of "per second" calculation
	if recap_temp.LastFightSignificant then
		if iLastTimeOut > recap_temp.MinTime then
			iCombatant.LastDPS = Recap_Div1(iLastDmgOut,iLastTimeOut)
		else
			iCombatant.LastDPS = 0
		end
		if iLastTimeIn > recap_temp.MinTime then
			iCombatant.LastDPSIn = Recap_Div1(iLastDmgIn,iLastTimeIn)
		else
			iCombatant.LastDPSIn = 0
		end
		if iLastTimeHeal > recap_temp.MinTime then
			iCombatant.LastHPS = Recap_Div1((iLastRawHeal - iLastOverHeal),iLastTimeHeal)
		else
			iCombatant.LastHPS = 0
		end
	end
	if iCombatant.TotalTime > recap_temp.MinTime then
		iCombatant.TotalDPS = Recap_Div1(iCombatant.TotalDmgOut,iCombatant.TotalTime)
	else
		iCombatant.TotalDPS = 0
	end
	if iCombatant.TotalTimeIn > recap_temp.MinTime then
		iCombatant.TotalDPSIn = Recap_Div1(iCombatant.TotalDmgIn,iCombatant.TotalTimeIn)
	else
		iCombatant.TotalDPSIn = 0
	end
	if iCombatant.TotalTimeHeal > recap_temp.MinTime then
		iCombatant.TotalHPS = Recap_Div1((iCombatant.TotalRawHeal - iCombatant.TotalOverHeal),iCombatant.TotalTimeHeal)
	else
		iCombatant.TotalHPS = 0
	end

	iCombatant.WasInCurrent = false
	if recap_temp.LastFightSignificant then
		iCombatant.WasInLast = true
	end
end

-- used to remove pet effects when Merge Pets has been set to off, after having been on
function Recap_RemovePetEffects()

	local i, j, effectNum

	-- Hawksy: silent repair
	if not recap.Combatant then
		recap.Combatant = {}
	end
	for i in pairs(recap.Combatant) do
		if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
			local iCombatant = recap.Combatant[i]
			-- Hawksy: silent repair
			if not iCombatant.OutgoingDetail then
				iCombatant.OutgoingDetail = {}
			end
			if not iCombatant.IncomingDetail then
				iCombatant.IncomingDetail = {}
			end
			if not iCombatant.OtherDetail then
				iCombatant.OtherDetail = {}
			end
			for j in pairs(iCombatant.OutgoingDetail) do
				effectNum = string.sub(j,1,1)
				if (effectNum == "2") or (effectNum == "4") or (effectNum == "6") or (effectNum == "8") then
					-- remove pet effect
					iCombatant.OutgoingDetail[j] = nil -- keep as nil
				end
			end
			for j in pairs(iCombatant.IncomingDetail) do
				effectNum = string.sub(j,1,1)
				if (effectNum == "2") or (effectNum == "4") or (effectNum == "6") or (effectNum == "8") then
					-- remove pet effect
					iCombatant.IncomingDetail[j] = nil -- keep as nil
				end
			end
			for j in pairs(iCombatant.OtherDetail) do
				effectNum = string.sub(j,1,1)
				if (effectNum == "2") or (effectNum == "4") or (effectNum == "6") or (effectNum == "8") then
					-- remove pet effect
					iCombatant.OtherDetail[j] = nil -- keep as nil
				end
			end
		end
	end
	-- and personal details
	-- Hawksy: with 3.67 this next bit is now only for cleanup: no new effects of this type will be created
	for j in pairs(recap.Self[recap_temp.s]) do
		effectNum = string.sub(j,1,1)
		if (effectNum == "2") or (effectNum == "4") then
			-- remove pet effect
			recap.Self[recap_temp.s][j] = nil -- keep as nil
		end
	end
end

function Recap_CopyPetEffectsToOwner(pet, owner)

	local i
	local peteffect
	local type, effect
	local petDetail
	local combatantPet = recap.Combatant[pet]

	if not recap.Opt.LightData.value then
		for i in pairs(combatantPet.OutgoingDetail) do
			type = tonumber(string.sub(i,1,1))
			effect = string.sub(i,2)
			peteffect = tostring(type+1)..pet..": "..effect
			petDetail = combatantPet.OutgoingDetail[i]
			recap.Combatant[owner].OutgoingDetail[peteffect] = {
				Hits = petDetail.Hits, HitsDmg = petDetail.HitsDmg,
				HitsMax = petDetail.HitsMax, Crits = petDetail.Crits,
				CritsEvents = petDetail.CritsEvents, CritsDmg = petDetail.CritsDmg,
				CritsMax = petDetail.CritsMax, Missed = petDetail.Missed,
				Ticks = petDetail.Ticks, TicksDmg = petDetail.TicksDmg,
				TicksMax = petDetail.TicksMax, Element = petDetail.Element,
				Dodged = petDetail.Dodged, Parried = petDetail.Parried,
				Blocked = petDetail.Blocked, Absorbed = petDetail.Absorbed,
				Deflected = petDetail.Deflected, Evaded = petDetail.Evaded,
				Resisted = petDetail.Resisted, Reflected = petDetail.Reflected,
				Immune = petDetail.Immune, PrevTime = petDetail.PrevTime,
				ICount = petDetail.ICount, ITotal = petDetail.ITotal,
				Glances = petDetail.Glances, GlancesDmg = petDetail.GlancesDmg,
				GlancesMax = petDetail.GlancesMax, GlancesMin = petDetail.GlancesMin,
				HitsMin = petDetail.HitsMin, CritsMin = petDetail.CritsMin,
				TicksMin = petDetail.TicksMin }
		end
		for i in pairs(combatantPet.IncomingDetail) do
			type = tonumber(string.sub(i,1,1))
			effect = string.sub(i,2)
			peteffect = tostring(type+1)..pet..": "..effect
			petDetail = combatantPet.IncomingDetail[i]
			recap.Combatant[owner].IncomingDetail[peteffect] = {
				Hits = petDetail.Hits, HitsDmg = petDetail.HitsDmg,
				HitsMax = petDetail.HitsMax, Crits = petDetail.Crits,
				CritsEvents = petDetail.CritsEvents, CritsDmg = petDetail.CritsDmg,
				CritsMax = petDetail.CritsMax, Missed = petDetail.Missed,
				Ticks = petDetail.Ticks, TicksDmg = petDetail.TicksDmg,
				TicksMax = petDetail.TicksMax, Element = petDetail.Element,
				Dodged = petDetail.Dodged, Parried = petDetail.Parried,
				Blocked = petDetail.Blocked, Absorbed = petDetail.Absorbed,
				Deflected = petDetail.Deflected, Evaded = petDetail.Evaded,
				Resisted = petDetail.Resisted, Reflected = petDetail.Reflected,
				Immune = petDetail.Immune, PrevTime = petDetail.PrevTime,
				ICount = petDetail.ICount, ITotal = petDetail.ITotal,
				Glances = petDetail.Glances, GlancesDmg = petDetail.GlancesDmg,
				GlancesMax = petDetail.GlancesMax, GlancesMin = petDetail.GlancesMin,
				HitsMin = petDetail.HitsMin, CritsMin = petDetail.CritsMin,
				TicksMin = petDetail.TicksMin }
		end
		if recap.Opt.OtherData.value then
			for i in pairs(combatantPet.OtherDetail) do
				type = tonumber(string.sub(i,1,1))
				effect = string.sub(i,2)
				peteffect = tostring(type+1)..pet..": "..effect
				petDetail = combatantPet.OtherDetail[i]
				recap.Combatant[owner].OtherDetail[peteffect] = {
					Hits = petDetail.Hits, Total = petDetail.Total,
					Max = petDetail.Max, Attribute = petDetail.Attribute,
					Missed = petDetail.Missed, Dispels = petDetail.Dispels,
					PrevTime = petDetail.PrevTime, PrevTimeDur = petDetail.PrevTimeDur,
					ICount = petDetail.ICount, ITotal = petDetail.ITotal,
					DCount = petDetail.DCount, DTotal = petDetail.DTotal }
			end
		end
	end
end

function Recap_EndFight(forceEnd)

	local i, iPlayer, iCombatant, iLast
	local iDurationOut, iDurationIn, iDurationHeal

	-- turn off idle and end of fight delay timers
	recap_temp.IdleTimer = -1
	recap_temp.HiddenIdleTimer = -1
	recap_temp.EndFightDelayTimer = -1

	if forceEnd or recap.Opt.State.value=="Active" then

		local debug_time = GetTime()

		Recap_MakeFriends()

		-- Hawksy: three states for HoldFight
		--   Normal: do nothing different from the usual Recap behaviour
		--   HoldNext: process this end of fight as usual; then hold further fights by switching to Holding
		--   Holding: currently holding and awaiting release
		if forceEnd or (recap_temp.HoldFight == "Normal") or (recap_temp.HoldFight == "HoldNext") then

			RecapUpdateFrame:Hide() -- disable OnUpdate
			recap.Opt.State.value = "Stopped" -- suspend logging to calculate

			-- Hawksy: silent repairs
			if not recap_temp.Last then
				recap_temp.Last = {}
			end
			if not recap.Combatant then
				recap.Combatant = {}
			end

			-- merge pets if appropriate
			if Recap_MergePets() then
				-- add all pets to owners
				for i in pairs(recap_temp.Last) do
					iCombatant = recap.Combatant[i]
					if iCombatant.OwnedBy and iCombatant.WasInCurrent then
						Recap_AddPetToOwner(i, iCombatant.OwnedBy) -- assert: clears pet's WasInCurrent, sets owner's WasInCurrent
						Recap_CopyPetEffectsToOwner(i, iCombatant.OwnedBy)
					end
				end
			end

			-- Hawksy: scan all combatants in Last to see if any dealt or took damage for more than the significant time
			--   If so, then we will update combatant Last Fight information
			--   If not, we won't update combatant Last Fight information, but we will still accumulate to the All Fights information
			recap_temp.LastFightSignificant = false
			for i in pairs(recap_temp.Last) do
				iCombatant = recap.Combatant[i]
				if iCombatant.WasInCurrent then
					iLast = recap_temp.Last[i]
					if (iLast.DmgOut > 0) and (iLast.StartOut > 0) and ((iLast.EndOut - iLast.StartOut) > recap_temp.LastFightSignificantTime) then
						-- damage dealt, for more than the significant time
						recap_temp.LastFightSignificant = true
						break
					end
					if (iLast.DmgIn > 0) and (iLast.StartIn > 0) and ((iLast.EndIn - iLast.StartIn) > recap_temp.LastFightSignificantTime) then
						-- damage taken, for more than the significant time
						recap_temp.LastFightSignificant = true
						break
					end
				end
			end

			-- if we are going to have fresh Last Fight data, clear all WasInLast values
			if recap_temp.LastFightSignificant then
				for i in pairs(recap.Combatant) do
					if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
						recap.Combatant[i].WasInLast = false
					end
				end
			end

			-- accumulate all combatants who were in the Last Fight
			for i in pairs(recap_temp.Last) do
				iCombatant = recap.Combatant[i]
				iLast = recap_temp.Last[i]
				if iCombatant.WasInCurrent then
					Recap_AccumulateCombatant(i, iCombatant, iLast) -- assert: clears WasInCurrent, sets WasInLast if LastFightSignificant
				end
				-- ensure that every combatant has their 'Last' record cleared, even if not in current fight
				iLast.StartOut = 0
				iLast.EndOut = 0
				iLast.StartIn = 0
				iLast.EndIn = 0
				iLast.StartHeal = 0
				iLast.EndHeal = 0
				iLast.DmgIn = 0
				iLast.DmgOut = 0
				iLast.MaxHit = 0
				iLast.Kills = 0
				iLast.RawHeal = 0
				iLast.OverHeal = 0
			end

			-- update the overall (All Fights) durations
			iPlayer = recap[recap_temp.p]
			if recap_temp.FightStart>0 and (recap_temp.FightEnd > recap_temp.FightStart) then
				iDurationOut = recap_temp.FightEnd - recap_temp.FightStart
			else
				iDurationOut = 0
			end
			if recap_temp.FightStartIn>0 and (recap_temp.FightEndIn > recap_temp.FightStartIn) then
				iDurationIn = recap_temp.FightEndIn - recap_temp.FightStartIn
			else
				iDurationIn = 0
			end
			if recap_temp.FightStartHeal>0 and (recap_temp.FightEndHeal > recap_temp.FightStartHeal) then
				iDurationHeal = recap_temp.FightEndHeal - recap_temp.FightStartHeal
			else
				iDurationHeal = 0
			end
			if recap_temp.LastFightSignificant then
				iPlayer.LastDuration = iDurationOut
				iPlayer.LastDurationIn = iDurationIn
				iPlayer.LastDurationHeal = iDurationHeal
			end
			iPlayer.TotalDuration = iPlayer.TotalDuration + iDurationOut
			iPlayer.TotalDurationIn = iPlayer.TotalDurationIn + iDurationIn
			iPlayer.TotalDurationHeal = iPlayer.TotalDurationHeal + iDurationHeal
			recap_temp.FightStart = 0
			recap_temp.FightEnd = 0
			recap_temp.FightStartIn = 0
			recap_temp.FightEndIn = 0
			recap_temp.FightStartHeal = 0
			recap_temp.FightEndHeal = 0

			-- clear effect interval timers
			Recap_ClearEffectIntervalTimers()

			-- Hawksy: if the Last Fight panel is being held, this supports multiple fights happening in the background
			recap_temp.ResetWasInCurrent = false

			-- Hawksy: if Recent Data Mode is active, add an 'end of fight' event (signalled by false values for source and dest)
			--   (if holding the Last Fight panel this could be later than otherwise)
			if recap.Opt.RecentData.value then
				Recap_AddRecentEvent(debug_time, false, false, false, false, false, false, false, false)
			end

			-- Hawksy: special case correction for some dual damage:healing tickers
			Recap_CorrectDualDoTHoT()

			-- If holding the Last Fight panel then postings will be later than otherwise
			if recap.Opt.AutoPost.value then
				recap.Opt.SortBy.value = Recap_AutoPostGetStatID(recap.Opt.AutoPost.Stat)
				recap.Opt.SortDir.value = false
			end

			Recap_ConstructList()
			Recap_SetState("Idle")

			-- Hawksy: refresh the Blizzard party frame at the end of a fight (if the group changes during a fight the frame is not always updated)
			-- Hawksy: yeah, I know this doesn't really belong in a combat analysis addon like Recap
			if recap.Opt.RefreshPartyFrame.value then
				ShowPartyFrame()
			end

			if recap.Opt.AutoPost.value then
				Recap_PostFight()
			end

			if recap.Opt.AutoLeader.value then
				Recap_PostLeader()
			end

			if recap.debug then
				DEFAULT_CHAT_FRAME:AddMessage("(recap.debug) Fight processed in "..string.format("%.4f",GetTime()-debug_time).." secs for "..(recap_temp.ListSize-1).." combatants.")
			end

		end

		-- If we just set the Last Fight, and need to hold it, set the state variable
		if (recap_temp.HoldFight == "HoldNext") then
			recap_temp.HoldFight = "Holding"
		end

		-- If we are holding the Last Fight, display the Last Fight panel at every end of fight (until they click the Resume button)
		if (recap_temp.HoldFight == "Holding") then
			recap.Opt.View.value = "Last"
			recap.Opt.SelfView.value = false
			Recap_Maximize()
		end

		if recap.Opt.AutoHide.value and not recap.Opt.Minimized.value then
			RecapFrame_Show()
			if recap.Opt.AutoFade.value then
				recap_temp.FadeTimer = 0
			end
		end

		-- Hawksy: force a garbage collection, as this is probably the best time, just after the end of a fight
		if recap.Opt.CollectGarbage.value then
			if recap.debug then
				-- update numbers for all addons
				UpdateAddOnMemoryUsage()
				-- Print memory usage for Recap specifically
				DEFAULT_CHAT_FRAME:AddMessage("(recap.debug) Recap memory before garbage collection "..string.format("%.3f MB",GetAddOnMemoryUsage("Recap")/1000))
			end
			local gc_time = GetTime()
			collectgarbage("collect")
			if recap.debug then
				local total, forRecap, text
				DEFAULT_CHAT_FRAME:AddMessage("(recap.debug) Garbage collection took "..string.format("%.3f secs",GetTime()-gc_time))
				total = collectgarbage("count")/1000
				text = ""
				text = text.."(recap.debug) Total memory "..string.format("%.3f MB",total)
				-- update memory amounts for all addons
				UpdateAddOnMemoryUsage()
				forRecap = GetAddOnMemoryUsage("Recap")/1000
				text = text.." -- Recap "..string.format("%.3f MB (%.2f%%)",forRecap,100*forRecap/total)
				DEFAULT_CHAT_FRAME:AddMessage(text)
				text = ""
				total = GetScriptCPUUsage()/1000
				text = text.."(recap.debug) Total addon CPU time "..string.format("%.3f secs",total)
				-- update CPU times for all addons
				UpdateAddOnCPUUsage()
				forRecap = GetAddOnCPUUsage("Recap")/1000
				text = text.." -- Recap "..string.format("%.3f secs (%.2f%%)",forRecap,100*forRecap/total)
				DEFAULT_CHAT_FRAME:AddMessage(text)
				text = ""
				forRecap = GetAddOnCPUUsage("Omen")/1000
				text = text.."Omen "..string.format("(%.2f%%) ",100*forRecap/total)
				forRecap = GetAddOnCPUUsage("CT_RaidAssist")/1000
				text = text.."CT_RaidAssist "..string.format("(%.2f%%) ",100*forRecap/total)
				forRecap = GetAddOnCPUUsage("Clique")/1000
				text = text.."Clique "..string.format("(%.2f%%) ",100*forRecap/total)
				forRecap = GetAddOnCPUUsage("MapNotes")/1000
				text = text.."MapNotes "..string.format("(%.2f%%) ",100*forRecap/total)
				forRecap = GetAddOnCPUUsage("ItemRack")/1000
				text = text.."ItemRack "..string.format("(%.2f%%) ",100*forRecap/total)
				forRecap = (GetAddOnCPUUsage("Sanity2")+GetAddOnCPUUsage("SanityItemCache"))/1000
				text = text.."Sanity2 "..string.format("(%.2f%%) ",100*forRecap/total)
				forRecap = (GetAddOnCPUUsage("DBM_API")+GetAddOnCPUUsage("DBM_GUI")+GetAddOnCPUUsage("DBM_Karazhan")+GetAddOnCPUUsage("DBM_Other"))/1000
				text = text.."Deadly Boss Mods "..string.format("(%.2f%%) ",100*forRecap/total)
				DEFAULT_CHAT_FRAME:AddMessage(text)
				if (recap_temp.p == "HawksyElune") or (recap_temp.p == "GalassaElune") or (recap_temp.p == "MetrognomeElune") then
					-- for me only, reset CPU profiling numbers at the end of every fight
					ResetCPUUsage()
				end
			end
		end

	end
end

function Recap_ClearAsIfEndOfFight()
	-- ensure that fight timers are cleared (had previously only been cleared on end of fight)
	recap_temp.FightStart = 0
	recap_temp.FightEnd = 0
	recap_temp.FightStartIn = 0
	recap_temp.FightEndIn = 0
	recap_temp.FightStartHeal = 0
	recap_temp.FightEndHeal = 0
	if not recap_temp.Last then
		recap_temp.Last = {}
	end
	for i in pairs(recap_temp.Last) do
		-- ensure that every 'Last' record is cleared
		local iLast = recap_temp.Last[i]
		iLast.StartOut = 0
		iLast.EndOut = 0
		iLast.StartIn = 0
		iLast.EndIn = 0
		iLast.StartHeal = 0
		iLast.EndHeal = 0
		iLast.DmgIn = 0
		iLast.DmgOut = 0
		iLast.MaxHit = 0
		iLast.Kills = 0
		iLast.RawHeal = 0
		iLast.OverHeal = 0
	end
	-- clear effect interval timers
	Recap_ClearEffectIntervalTimers()
	-- turn off idle and end of fight delay timers
	recap_temp.IdleTimer = -1
	recap_temp.HiddenIdleTimer = -1
	recap_temp.EndFightDelayTimer = -1
end

-- clear all of the effect interval timers, since intervals are only meaningful when measured within a single fight
-- the effect duration timers, however, are not reset, since the effects can last between fights
function Recap_ClearEffectIntervalTimers()

	local i, j

	-- Hawksy: silent repair
	if not recap.Combatant then
		recap.Combatant = {}
	end
	for i in pairs(recap.Combatant) do
		if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
			local iCombatant = recap.Combatant[i]
			-- Hawksy: silent repair
			if not iCombatant.OutgoingDetail then
				iCombatant.OutgoingDetail = {}
			end
			if not iCombatant.IncomingDetail then
				iCombatant.IncomingDetail = {}
			end
			if not iCombatant.OtherDetail then
				iCombatant.OtherDetail = {}
			end
			for j in pairs(iCombatant.OutgoingDetail) do
				if iCombatant.OutgoingDetail[j].PrevTime then
					iCombatant.OutgoingDetail[j].PrevTime = 0
				end
			end
			for j in pairs(iCombatant.IncomingDetail) do
				if iCombatant.IncomingDetail[j].PrevTime then
					iCombatant.IncomingDetail[j].PrevTime = 0
				end
			end
			for j in pairs(iCombatant.OtherDetail) do
				if iCombatant.OtherDetail[j].PrevTime then
					iCombatant.OtherDetail[j].PrevTime = 0
				end
			end
		end
	end
end

-- adds combat message parsings (from ParserLib) to .Last running totals
ParserLib_SELF = 103
ParserLib_MELEE = 112
ParserLib_DAMAGESHIELD = 113
ParserLib_NONE = 200 -- (source is absent)

function Recap_DamageEvent(event, info)

	if recap.Opt.State.value~="Stopped" then

		local source, dest, damage, effect, element, sourceCombatant, destCombatant, iLast
		local event_time = GetTime()

		if recap.dump then
			local printable
			printable = Recap_CreateMessage(event)
			Recap_DumpMessage("DAMAGE;"..printable)
		end

		source = info.source
		if tonumber(source) == ParserLib_SELF then
			source = recap_temp.Player
		end
		if tonumber(source) == ParserLib_NONE then
			-- damage with no source
			source = false
		end
		dest = info.victim
		if tonumber(dest) == ParserLib_SELF then
			dest = recap_temp.Player
		end

		damage = 0
		damage = tonumber(info.amount)
		effect = info.skill
		if tonumber(effect) == ParserLib_MELEE then
			effect = RECAP_MELEE
		end
		if tonumber(effect) == ParserLib_DAMAGESHIELD then
			effect = recap_temp.Localize.DamageShield
		end

		-- default element is Melee or Other when Blizzard doesn't specify
		if effect == RECAP_MELEE then
			element = RECAP_MELEE
		else
			element = recap_temp.Localize.ElementOther
		end
		if info.element then
			element = info.element
		end

		-- Hawksy: check for known duplicates
		if source then
			if Recap_CheckForDuplicateSource(source) then
				return
			end
		end
		if Recap_CheckForDuplicateEffect(effect) then
			return
		end

		if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
			-- in synchronization, don't ignore anyone (if 'ignore' they will still be hidden on the Fights panel)
		else
			if (source and recap.Combatant[source] and recap.Combatant[source].Ignore) then
				-- source is an ignored combatant, pretend the hit didn't happen at all
				return
			end
			if (dest and recap.Combatant[dest] and recap.Combatant[dest].Ignore) then
				-- dest is an ignored combatant, pretend the hit didn't happen at all
				return
			end
		end

		-- possibly start the fight
		if not recap.Opt.LimitFights.value and recap.Opt.State.value=="Idle" then
			Recap_StartFight()
		end

		-- if the fight is ongoing, start or restart the idle timers
		if recap.Opt.State.value=="Active" and recap.Opt.IdleFight.value then
			recap_temp.IdleTimer = 0
		end
		if recap.Opt.State.value=="Active" then
			recap_temp.HiddenIdleTimer = 0
		end

		-- if the fight is ongoing and if the end of fight delay timer is running, restart the end of fight delay timer
		--   (the player left combat, perhaps temporarily, but the fight continues)
		if recap.Opt.State.value=="Active" and recap_temp.EndFightDelayTimer~=-1 then
			recap_temp.EndFightDelayTimer = 0
		end

		-- does a combatant have "do not store" status due to the setting of options?
		if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") or (recap.Opt.StoreFriendsPartySelf.value == false) then
			-- in synchronization, or the StoreFriendsPartySelf option is off, carry on as normal
		else
			-- the StoreFriendsPartySelf option is on
			if source then
				-- note that 'friends' include former party members, hostiles controlled for a while, and combatants explicitly named as friends
				if recap.Opt.HideFoe.value and recap.Combatant[source] and not recap.Combatant[source].Friend then
					source = nil
				elseif recap.Opt.PartyData.value and not Recap_CheckForParty(source, (recap.Combatant[source] and recap.Combatant[source].OwnedBy)) then
					source = nil
				elseif recap.Opt.SelfData.value and not Recap_CheckForSelf(source, (recap.Combatant[source] and recap.Combatant[source].OwnedBy)) then
					source = nil
				end
			end
			if dest then
				-- note that 'friends' include former party members, hostiles controlled for a while, and combatants explicitly named as friends
				if recap.Opt.HideFoe.value and recap.Combatant[dest] and not recap.Combatant[dest].Friend then
					dest = nil
				elseif recap.Opt.PartyData.value and not Recap_CheckForParty(dest, (recap.Combatant[dest] and recap.Combatant[dest].OwnedBy)) then
					dest = nil
				elseif recap.Opt.SelfData.value and not Recap_CheckForSelf(dest, (recap.Combatant[dest] and recap.Combatant[dest].OwnedBy)) then
					dest = nil
				end
			end
			-- are both source and dest now nil?
			if source == nil and dest == nil then
				-- ignore entirely
				return
			end
		end

		-- Hawksy: events are already almost entirely filtered down to damage or miss, so let's start the timers
		if source then
			Recap_CreateCombatantAndUpdateTimers(source, event_time, "Out")
		end
		if dest then
			Recap_CreateCombatantAndUpdateTimers(dest, event_time, "In")
		end

		-- table lookup shortcuts
		if source then
			sourceCombatant = recap.Combatant[source]
		else
			sourceCombatant = nil
		end
		if dest then
			destCombatant = recap.Combatant[dest]
		else
			destCombatant = nil
		end

		-- remove friend flag from hostiles no longer charmed or otherwise controlled
		--   (this is the only place where this happens automatically)
		--   (note that this causes friends who damage other friends (e.g. with 'you are the bomb' effects) to be classed as hostiles
		--    until the next call to Recap_MakeFriends)
		if sourceCombatant and destCombatant then
			if sourceCombatant.Friend and destCombatant.Friend then
				sourceCombatant.Friend = false
			end
		end

		if damage and (damage > 0) then
			-- process totals

			if (source == dest) then
				-- Hawksy: ignore the Outgoing component of damage to self (such as Shadow Word: Death damage to self) (otherwise would inflate DPS)
			else
				if source then
					if dest and ((sourceCombatant.OwnsPet == dest) or (sourceCombatant.OwnedBy == dest)) then
						-- Hawksy: ignore the Outgoing component of damage to or from a combatant's pet (Soul Link does this) (otherwise would inflate DPS)
					else
						-- due to the code above that makes anyone who damages a friend into a (temporary) enemy,
						--   we can't simply ignore friend to friend damage here
						iLast = recap_temp.Last[source]
						iLast.DmgOut = iLast.DmgOut + damage
						if damage > iLast.MaxHit then
							iLast.MaxHit = damage
						end
					end
				end
			end

			if dest then
				iLast = recap_temp.Last[dest]
				iLast.DmgIn = iLast.DmgIn + damage
				if iLast.HP then
					iLast.HP = iLast.HP - damage
				end
			end

			if not recap.Opt.LightData.value then
				-- determine whether it could have critted
				if string.find(event, "PERIODIC", 1, true) then
					-- periodic event, assume not crit-capable, pass 'nil' in the crit slot (and in the crush and glance slots) -- keep crit and crush and glance as 'nil', not false
					Recap_CreateEffect(event_time, source, dest, effect, 1, damage, nil, info.isDOT, element, false, nil, nil, info.amountAbsorb, info.amountBlock, info.amountResist, info.amountVulnerable)
				else
					-- assume all others are crit or crush or glance capable
					local critty, crushy
					critty = info.isCrit or false
					crushy = info.isCrushing or false
					glancy = info.isGlancing or false
					Recap_CreateEffect(event_time, source, dest, effect, 1, damage, critty, info.isDOT, element, false, crushy, glancy, info.amountAbsorb, info.amountBlock, info.amountResist, info.amountVulnerable)
				end
			end

			if recap.Opt.RecentData.value then
				Recap_AddRecentEvent(event_time, source, dest, 1, effect, element, damage, info.isCrit, info.isCrushing)
			end
		else
			-- no actual damage, probably a miss
			if not recap.Opt.LightData.value then
				if (source == dest) then
					-- Hawksy: ignore misses related to self-damage (if any)
				else
					if (info.type == "miss") then
						local typeEffect, total
						typeEffect = "1"..effect
						total = 0
						if sourceCombatant and sourceCombatant.OutgoingDetail and sourceCombatant.OutgoingDetail[typeEffect] then
							local iEffect = sourceCombatant.OutgoingDetail[typeEffect]
							total = (iEffect.GlancesDmg or 0) + (iEffect.HitsDmg or 0) + (iEffect.CritsDmg or 0) + (iEffect.CrushDmg or 0) + (iEffect.TicksDmg or 0)
						end
						if total > 0 then
							-- this effect has done damage, so count the miss directly
							-- register the miss as a non-crit-capable event -- keep crit and crush and glance as 'nil', not false
							Recap_CreateEffect(event_time, source, dest, effect, 1, 0, nil, false, element, info.missType, nil, nil, nil, nil, nil, nil)

							if recap.Opt.RecentData.value then
								local missText = (info.missType or "miss")
								Recap_AddRecentEvent(event_time, source, dest, 1, effect, element, missText, false, false)
							end
						else
							-- this effect has not done damage, treat it as a miss for any matching cast, debuff, or buff
							local missText = (info.missType or "miss")
							Recap_CreateMissEffect(source, dest, effect, missText)
						end
					else
						-- Hawksy: no damage and not a miss
						Recap_CheckUnexpectedEvent()
					end
				end
			end
		end
	end
end


function Recap_EnvironmentDamageEvent(event, info)

	if recap.Opt.State.value~="Stopped" then

		local dest, damage, effect, element, iLast
		local event_time = GetTime()

		if recap.dump then
			local printable
			printable = Recap_CreateMessage(event)
			Recap_DumpMessage("ENVIRONMENT;"..printable)
		end

		dest = info.victim
		if tonumber(dest) == ParserLib_SELF then
			dest = recap_temp.Player
		end

		-- does a combatant have "do not store" status due to the setting of options?
		if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") or (recap.Opt.StoreFriendsPartySelf.value == false) then
			-- in synchronization, or the StoreFriendsPartySelf option is off, carry on as normal
		else
			-- the StoreFriendsPartySelf option is on
			if dest then
				-- note that 'friends' include former party members, hostiles controlled for a while, and combatants explicitly named as friends
				if recap.Opt.HideFoe.value and recap.Combatant[dest] and not recap.Combatant[dest].Friend then
					-- ignore entirely
					return
				elseif recap.Opt.PartyData.value and not Recap_CheckForParty(dest, (recap.Combatant[dest] and recap.Combatant[dest].OwnedBy)) then
					-- ignore entirely
					return
				elseif recap.Opt.SelfData.value and not Recap_CheckForSelf(dest, (recap.Combatant[dest] and recap.Combatant[dest].OwnedBy)) then
					-- ignore entirely
					return
				end
			end
		end

		-- table lookup shortcut
		destCombatant = recap.Combatant[dest]

		damage = 0
		damage = tonumber(info.amount)
		effect = RECAP_ENVIRONMENT
		-- ParserLib seems to send damageType = nil sometimes
		if info.damageType then
			element = string.upper(string.sub(info.damageType,1,1))..string.sub(info.damageType,2)
		else
			element = recap_temp.Localize.ElementOther
		end

		if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
			-- in synchronization, don't ignore anyone (if 'ignore' they will still be hidden on the Fights panel)
		else
			if (destCombatant and destCombatant.Ignore) then
				-- dest is an ignored combatant, pretend the hit didn't happen at all
				return
			end
		end


		-- possibly start the fight
		if not recap.Opt.LimitFights.value and recap.Opt.State.value=="Idle" then
			Recap_StartFight()
		end

		-- if the fight is ongoing, start or restart the idle timers
		if recap.Opt.State.value=="Active" and recap.Opt.IdleFight.value then
			recap_temp.IdleTimer = 0
		end
		if recap.Opt.State.value=="Active" then
			recap_temp.HiddenIdleTimer = 0
		end

		-- Hawksy: start the timers
		Recap_CreateCombatantAndUpdateTimers(dest, event_time, "In")

		if damage and (damage > 0) then
			iLast = recap_temp.Last[dest]
			iLast.DmgIn = iLast.DmgIn + damage
			if iLast.HP then
				iLast.HP = iLast.HP - damage
			end

			if not recap.Opt.LightData.value then
				-- environment effect
				Recap_CreateEffect(event_time, false, dest, effect, 1, damage, false, false, element, false, false, false, info.amountAbsorb, info.amountBlock, info.amountResist, info.amountVulnerable)
			end

			if recap.Opt.RecentData.value then
				Recap_AddRecentEvent(event_time, false, dest, 1, effect, element, damage, false, false)
			end
		else
			-- no actual damage, probably a miss
			if not recap.Opt.LightData.value then
				if info.amountAbsorb and (info.amountAbsorb > 0) then
					-- entire amount absorbed
					-- register the miss as a non-crit-capable event -- keep crit and crush and glance as 'nil', not false
					-- ( this one happened, an ordinary fire that did 12 damage and was entirely absorbed )
					Recap_CreateEffect(event_time, false, dest, effect, 1, 0, nil, false, element, "absorb", nil, nil, nil, nil, nil, nil)
				elseif info.amountBlock and (info.amountBlock > 0) then
					-- entire amount blocked
					-- ( this might never occur )
					Recap_CreateEffect(event_time, false, dest, effect, 1, 0, nil, false, element, "block", nil, nil, nil, nil, nil, nil)
				elseif info.amountResist and (info.amountResist > 0) then
					-- entire amount resisted
					-- ( this might never occur )
					Recap_CreateEffect(event_time, false, dest, effect, 1, 0, nil, false, element, "resist", nil, nil, nil, nil, nil, nil)
				else
					-- Hawksy: no damage and not a miss
					Recap_CheckUnexpectedEvent()
				end
			end
		end
	end
end

function Recap_CreateEffect(event_time, source, dest, effect, type, total, did_crit, is_dot, element, missType, did_crush, did_glance, absorb, block, resist, vulnerable)

	local iEffect, sourceCombatant, destCombatant, typeEffect, elementEffect, damageEffect

	-- Hawksy: parameter is_dot is not currently used

	-- Hawksy: guard code
	if not effect then
		return
	end

	-- table lookup shortcuts
	if source then
		sourceCombatant = recap.Combatant[source]
	else
		sourceCombatant = nil
	end
	if dest then
		destCombatant = recap.Combatant[dest]
	else
		destCombatant = nil
	end

	typeEffect = type..effect

	-- some misses will get here with Other or "?" as the element
	if missType and ((element == recap_temp.Localize.ElementOther) or (element == "?")) then
		-- if so attempt a correction by looking for a non-default element in the corresponding Outgoing effect
		if sourceCombatant and sourceCombatant.OutgoingDetail and sourceCombatant.OutgoingDetail["1"..effect] and sourceCombatant.OutgoingDetail["1"..effect].Element then
			for i in string.gmatch(sourceCombatant.OutgoingDetail["1"..effect].Element, "[^%+]+") do
				if (i ~= recap_temp.Localize.ElementOther) and (i ~= "?") then
					element = i
					break
				end
			end
		end
		-- if still Other or "?" then check among the corresponding Incoming effects
		if (element == recap_temp.Localize.ElementOther) or (element == "?") then
			if destCombatant and destCombatant.IncomingDetail and destCombatant.IncomingDetail["1"..effect] and destCombatant.IncomingDetail["1"..effect].Element then
				for i in string.gmatch(destCombatant.IncomingDetail["1"..effect].Element, "[^%+]+") do
					if (i ~= recap_temp.Localize.ElementOther) and (i ~= "?") then
						element = i
						break
					end
				end
			end
		end
	end
	if type == 1 then
		-- keep element subtotals separate for damage and healing
		elementEffect = "5"..recap_temp.Localize.Subtotal..": "..element
		-- overall damage total
		damageEffect = "7"..RECAP_TOTAL..": "..RECAP_DAMAGE
	elseif type == 3 then
		-- overall healing total
		elementEffect = "7"..RECAP_TOTAL..": "..recap_temp.Localize.ElementHealing
	end

	-- is the source the player or one of the player's pets?
	local selfIndex = nil -- keep as nil
	if source then
		if source == recap_temp.Player then
			selfIndex = recap_temp.s
		elseif sourceCombatant.OwnedBy == recap_temp.Player then
			selfIndex = recap_temp.s..":"..source
		end
	end

	if (did_crit==true) then
		if source and ((source ~= dest) or (type == 3)) then
			-- ignore the Outgoing component of self-damage (such as Shadow Word: Death damage to self) (otherwise would inflate DPS)
			Recap_MakeOutgoingDetail(sourceCombatant, typeEffect)
			iEffect = sourceCombatant.OutgoingDetail[typeEffect]
			Recap_AddCrits(iEffect, total)
			Recap_AddElement(iEffect, element)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			-- element subtotal
			Recap_MakeOutgoingDetail(sourceCombatant, elementEffect)
			iEffect = sourceCombatant.OutgoingDetail[elementEffect]
			Recap_AddCrits(iEffect, total)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			if type == 1 then
				-- damage subtotal
				Recap_MakeOutgoingDetail(sourceCombatant, damageEffect)
				iEffect = sourceCombatant.OutgoingDetail[damageEffect]
				Recap_AddCrits(iEffect, total)
				Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
				Recap_CalcInterval(iEffect, event_time)
			end

			if selfIndex then
				Recap_MakeSelfDetail(selfIndex, typeEffect)
				iEffect = recap.Self[selfIndex][typeEffect]
				iEffect.CritsEvents = (iEffect.CritsEvents or 0) + 1
				iEffect.Crits = (iEffect.Crits or 0) + 1
				iEffect.CritsDmg = (iEffect.CritsDmg or 0) + total
				iEffect.CritsMax = math.max(total,iEffect.CritsMax or 0)
				if recap.gmin then
					-- for the cognoscenti, factor back in any partials to get a gross minimum
					iEffect.CritsMin = Recap_Min(total+(absorb or 0)+(block or 0)+(resist or 0)-(vulnerable or 0),iEffect.CritsMin)
				else
					-- normal net minimum
					iEffect.CritsMin = Recap_Min(total,iEffect.CritsMin)
				end
				Recap_AddSelfElement(iEffect, element)
			end
		end
		if destCombatant then
			Recap_MakeIncomingDetail(destCombatant, typeEffect)
			iEffect = destCombatant.IncomingDetail[typeEffect]
			Recap_AddCrits(iEffect, total)
			Recap_AddElement(iEffect, element)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			-- element subtotal
			Recap_MakeIncomingDetail(destCombatant, elementEffect)
			iEffect = destCombatant.IncomingDetail[elementEffect]
			Recap_AddCrits(iEffect, total)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			if type == 1 then
				-- damage subtotal
				Recap_MakeIncomingDetail(destCombatant, damageEffect)
				iEffect = destCombatant.IncomingDetail[damageEffect]
				Recap_AddCrits(iEffect, total)
				Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
				Recap_CalcInterval(iEffect, event_time)
			end
		end
	elseif (did_crush==true) then
		if source and (source ~= dest) then
			-- ignore the Outgoing component of self-damage (such as Shadow Word: Death damage to self) (otherwise would inflate DPS)
			Recap_MakeOutgoingDetail(sourceCombatant, typeEffect)
			iEffect = sourceCombatant.OutgoingDetail[typeEffect]
			Recap_AddCrushes(iEffect, total)
			Recap_AddElement(iEffect, element)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			-- element subtotal
			Recap_MakeOutgoingDetail(sourceCombatant, elementEffect)
			iEffect = sourceCombatant.OutgoingDetail[elementEffect]
			Recap_AddCrushes(iEffect, total)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			-- damage subtotal
			Recap_MakeOutgoingDetail(sourceCombatant, damageEffect)
			iEffect = sourceCombatant.OutgoingDetail[damageEffect]
			Recap_AddCrushes(iEffect, total)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)

			-- self and pets can never crush, so there is nothing done with Self
		end
		if destCombatant then
			Recap_MakeIncomingDetail(destCombatant, typeEffect)
			iEffect = destCombatant.IncomingDetail[typeEffect]
			Recap_AddCrushes(iEffect, total)
			Recap_AddElement(iEffect, element)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			-- element subtotal
			Recap_MakeIncomingDetail(destCombatant, elementEffect)
			iEffect = destCombatant.IncomingDetail[elementEffect]
			Recap_AddCrushes(iEffect, total)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			if type == 1 then
				-- damage subtotal
				Recap_MakeIncomingDetail(destCombatant, damageEffect)
				iEffect = destCombatant.IncomingDetail[damageEffect]
				Recap_AddCrushes(iEffect, total)
				Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
				Recap_CalcInterval(iEffect, event_time)
			end
		end
	elseif (did_glance==true) then
		if source and (source ~= dest) then
			-- ignore the Outgoing component of self-damage (such as Shadow Word: Death damage to self) (otherwise would inflate DPS)
			Recap_MakeOutgoingDetail(sourceCombatant, typeEffect)
			iEffect = sourceCombatant.OutgoingDetail[typeEffect]
			Recap_AddGlances(iEffect, total)
			Recap_AddElement(iEffect, element)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			-- element subtotal
			Recap_MakeOutgoingDetail(sourceCombatant, elementEffect)
			iEffect = sourceCombatant.OutgoingDetail[elementEffect]
			Recap_AddGlances(iEffect, total)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			-- damage subtotal
			Recap_MakeOutgoingDetail(sourceCombatant, damageEffect)
			iEffect = sourceCombatant.OutgoingDetail[damageEffect]
			Recap_AddGlances(iEffect, total)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)

			if selfIndex then
				Recap_MakeSelfDetail(selfIndex, typeEffect)
				iEffect = recap.Self[selfIndex][typeEffect]
				iEffect.CritsEvents = (iEffect.CritsEvents or 0) + 1
				iEffect.Glances = (iEffect.Glances or 0) + 1
				iEffect.GlancesDmg = (iEffect.GlancesDmg or 0) + total
				iEffect.GlancesMax = math.max(total,iEffect.GlancesMax or 0)
				if recap.gmin then
					-- for the cognoscenti, factor back in any partials to get a gross minimum
					iEffect.GlancesMin = Recap_Min(total+(absorb or 0)+(block or 0)+(resist or 0)-(vulnerable or 0),iEffect.GlancesMin)
				else
					-- normal net minimum
					iEffect.GlancesMin = Recap_Min(total,iEffect.GlancesMin)
				end
				Recap_AddSelfElement(iEffect, element)
			end
		end
		if destCombatant then
			Recap_MakeIncomingDetail(destCombatant, typeEffect)
			iEffect = destCombatant.IncomingDetail[typeEffect]
			Recap_AddGlances(iEffect, total)
			Recap_AddElement(iEffect, element)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			-- element subtotal
			Recap_MakeIncomingDetail(destCombatant, elementEffect)
			iEffect = destCombatant.IncomingDetail[elementEffect]
			Recap_AddGlances(iEffect, total)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			if type == 1 then
				-- damage subtotal
				Recap_MakeIncomingDetail(destCombatant, damageEffect)
				iEffect = destCombatant.IncomingDetail[damageEffect]
				Recap_AddGlances(iEffect, total)
				Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
				Recap_CalcInterval(iEffect, event_time)
			end
		end
	elseif (did_crit==false) and (did_crush==false) and (did_glance==false) then
		-- hit that could have critted (does not include misses, so that crit percentages are accurate)
		if source and ((source ~= dest) or (type == 3)) then
			-- ignore the Outgoing component of self-damage (such as Shadow Word: Death damage to self) (otherwise would inflate DPS)
			Recap_MakeOutgoingDetail(sourceCombatant, typeEffect)
			iEffect = sourceCombatant.OutgoingDetail[typeEffect]
			Recap_AddHits(iEffect, total)
			Recap_AddElement(iEffect, element)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			-- element subtotal
			Recap_MakeOutgoingDetail(sourceCombatant, elementEffect)
			iEffect = sourceCombatant.OutgoingDetail[elementEffect]
			Recap_AddHits(iEffect, total)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			if type == 1 then
				-- damage subtotal
				Recap_MakeOutgoingDetail(sourceCombatant, damageEffect)
				iEffect = sourceCombatant.OutgoingDetail[damageEffect]
				Recap_AddHits(iEffect, total)
				Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
				Recap_CalcInterval(iEffect, event_time)
			end

			if selfIndex then
				Recap_MakeSelfDetail(selfIndex, typeEffect)
				iEffect = recap.Self[selfIndex][typeEffect]
				iEffect.CritsEvents = (iEffect.CritsEvents or 0) + 1
				iEffect.Hits = (iEffect.Hits or 0) + 1
				iEffect.HitsDmg = (iEffect.HitsDmg or 0) + total
				iEffect.HitsMax = math.max(total,iEffect.HitsMax or 0)
				if recap.gmin then
					-- for the cognoscenti, factor back in any partials to get a gross minimum
					iEffect.HitsMin = Recap_Min(total+(absorb or 0)+(block or 0)+(resist or 0)-(vulnerable or 0),iEffect.HitsMin)
				else
					-- normal net minimum
					iEffect.HitsMin = Recap_Min(total,iEffect.HitsMin)
				end
				Recap_AddSelfElement(iEffect, element)
			end
		end
		if destCombatant then
			Recap_MakeIncomingDetail(destCombatant, typeEffect)
			iEffect = destCombatant.IncomingDetail[typeEffect]
			Recap_AddHits(iEffect, total)
			Recap_AddElement(iEffect, element)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			-- element subtotal
			Recap_MakeIncomingDetail(destCombatant, elementEffect)
			iEffect = destCombatant.IncomingDetail[elementEffect]
			Recap_AddHits(iEffect, total)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			if type == 1 then
				-- damage subtotal
				Recap_MakeIncomingDetail(destCombatant, damageEffect)
				iEffect = destCombatant.IncomingDetail[damageEffect]
				Recap_AddHits(iEffect, total)
				Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
				Recap_CalcInterval(iEffect, event_time)
			end
		end
	elseif (total==0) and (source ~= dest) then
		-- ignore any miss for self damage entirely
		-- miss
		if source then
			Recap_MakeOutgoingDetail(sourceCombatant, typeEffect)
			iEffect = sourceCombatant.OutgoingDetail[typeEffect]
			Recap_AddMisses(iEffect, missType)
			Recap_AddElement(iEffect, element)
			Recap_CalcInterval(iEffect, event_time)
			-- element subtotal
			Recap_MakeOutgoingDetail(sourceCombatant, elementEffect)
			iEffect = sourceCombatant.OutgoingDetail[elementEffect]
			Recap_AddMisses(iEffect, missType)
			Recap_CalcInterval(iEffect, event_time)
			-- damage subtotal
			Recap_MakeOutgoingDetail(sourceCombatant, damageEffect)
			iEffect = sourceCombatant.OutgoingDetail[damageEffect]
			Recap_AddMisses(iEffect, missType)
			Recap_CalcInterval(iEffect, event_time)

			if selfIndex then
				Recap_MakeSelfDetail(selfIndex, typeEffect)
				iEffect = recap.Self[selfIndex][typeEffect]
				if (missType == "miss") then
					iEffect.Missed = (iEffect.Missed or 0) + 1
				elseif (missType == "absorb") then
					iEffect.Absorbed = (iEffect.Absorbed or 0) + 1
				elseif (missType == "dodge") then
					iEffect.Dodged = (iEffect.Dodged or 0) + 1
				elseif (missType == "parry") or (missType == "parr") then
					iEffect.Parried = (iEffect.Parried or 0) + 1
				elseif (missType == "block") then
					iEffect.Blocked = (iEffect.Blocked or 0) + 1
				elseif (missType == "deflect") then
					iEffect.Deflected = (iEffect.Deflected or 0) + 1
				elseif (missType == "evade") then
					iEffect.Evaded = (iEffect.Evaded or 0) + 1
				elseif (missType == "resist") then
					iEffect.Resisted = (iEffect.Resisted or 0) + 1
				elseif (missType == "reflect") then
					iEffect.Reflected = (iEffect.Reflected or 0) + 1
				elseif (missType == "immune") then
					iEffect.Immune = (iEffect.Immune or 0) + 1
				end
				Recap_AddSelfElement(iEffect, element)
			end
		end
		if destCombatant then
			Recap_MakeIncomingDetail(destCombatant, typeEffect)
			iEffect = destCombatant.IncomingDetail[typeEffect]
			Recap_AddMisses(iEffect, missType)
			Recap_AddElement(iEffect, element)
			Recap_CalcInterval(iEffect, event_time)
			-- element subtotal
			Recap_MakeIncomingDetail(destCombatant, elementEffect)
			iEffect = destCombatant.IncomingDetail[elementEffect]
			Recap_AddMisses(iEffect, missType)
			Recap_CalcInterval(iEffect, event_time)
			if type == 1 then
				-- damage subtotal
				Recap_MakeIncomingDetail(destCombatant, damageEffect)
				iEffect = destCombatant.IncomingDetail[damageEffect]
				Recap_AddMisses(iEffect, missType)
				Recap_CalcInterval(iEffect, event_time)
			end
		end
	else
		-- hit that couldn't have critted (must be a DoT or HoT)
		if source and ((source ~= dest) or (type == 3)) then
			-- ignore the Outgoing component of self-damage (such as Shadow Word: Death damage to self) (otherwise would inflate DPS)
			Recap_MakeOutgoingDetail(sourceCombatant, typeEffect)
			iEffect = sourceCombatant.OutgoingDetail[typeEffect]
			Recap_AddTicks(iEffect, total)
			Recap_AddElement(iEffect, element)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			-- element subtotal
			Recap_MakeOutgoingDetail(sourceCombatant, elementEffect)
			iEffect = sourceCombatant.OutgoingDetail[elementEffect]
			Recap_AddTicks(iEffect, total)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			if type == 1 then
				-- damage subtotal
				Recap_MakeOutgoingDetail(sourceCombatant, damageEffect)
				iEffect = sourceCombatant.OutgoingDetail[damageEffect]
				Recap_AddTicks(iEffect, total)
				Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
				Recap_CalcInterval(iEffect, event_time)
			end

			if selfIndex then
				Recap_MakeSelfDetail(selfIndex, typeEffect)
				iEffect = recap.Self[selfIndex][typeEffect]
				iEffect.Ticks = (iEffect.Ticks or 0) + 1
				iEffect.TicksDmg = (iEffect.TicksDmg or 0) + total
				iEffect.TicksMax = math.max(total,iEffect.TicksMax or 0)
				if recap.gmin then
					-- for the cognoscenti, factor back in any partials to get a gross minimum
					iEffect.TicksMin = Recap_Min(total+(absorb or 0)+(block or 0)+(resist or 0)-(vulnerable or 0),iEffect.TicksMin)
				else
					-- normal net minimum
					iEffect.TicksMin = Recap_Min(total,iEffect.TicksMin)
				end
				Recap_AddSelfElement(iEffect, element)
			end
		end
		if destCombatant then
			Recap_MakeIncomingDetail(destCombatant, typeEffect)
			iEffect = destCombatant.IncomingDetail[typeEffect]
			Recap_AddTicks(iEffect, total)
			Recap_AddElement(iEffect, element)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			-- element subtotal
			Recap_MakeIncomingDetail(destCombatant, elementEffect)
			iEffect = destCombatant.IncomingDetail[elementEffect]
			Recap_AddTicks(iEffect, total)
			Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
			Recap_CalcInterval(iEffect, event_time)
			if type == 1 then
				-- damage subtotal
				Recap_MakeIncomingDetail(destCombatant, damageEffect)
				iEffect = destCombatant.IncomingDetail[damageEffect]
				Recap_AddTicks(iEffect, total)
				Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)
				Recap_CalcInterval(iEffect, event_time)
			end
		end
	end
end

-- six auxiliary functions for Recap_CreateEffect to create spots in the tables as necessary
function Recap_MakeIncomingDetail(iCombatant, typeEffect)
	if not iCombatant.IncomingDetail then
		iCombatant.IncomingDetail = {}
	end
	if not iCombatant.IncomingDetail[typeEffect] then
		iCombatant.IncomingDetail[typeEffect] = {}
	end
end
function Recap_MakeOutgoingDetail(iCombatant, typeEffect)
	if not iCombatant.OutgoingDetail then
		iCombatant.OutgoingDetail = {}
	end
	if not iCombatant.OutgoingDetail[typeEffect] then
		iCombatant.OutgoingDetail[typeEffect] = {}
	end
end
function Recap_MakeSelfDetail(selfIndex, typeEffect)
	if not recap.Self[selfIndex] then
		recap.Self[selfIndex] = {}
	end
	if not recap.Self[selfIndex][typeEffect] then
		recap.Self[selfIndex][typeEffect] = {}
	end
end
function Recap_AddCrits(iEffect, total)
	iEffect.CritsEvents = (iEffect.CritsEvents or 0) + 1
	iEffect.Crits = (iEffect.Crits or 0) + 1
	iEffect.CritsDmg = (iEffect.CritsDmg or 0) + total
	iEffect.CritsMax = math.max(total, iEffect.CritsMax or 0)
	iEffect.CritsMin = Recap_Min(total, iEffect.CritsMin)
end
function Recap_AddCrushes(iEffect, total)
	iEffect.CritsEvents = (iEffect.CritsEvents or 0) + 1
	iEffect.Crushes = (iEffect.Crushes or 0) + 1
	iEffect.CrushDmg = (iEffect.CrushDmg or 0) + total
	iEffect.CrushMax = math.max(total, iEffect.CrushMax or 0)
	iEffect.CrushMin = Recap_Min(total, iEffect.CrushMin)
end
function Recap_AddGlances(iEffect, total)
	iEffect.CritsEvents = (iEffect.CritsEvents or 0) + 1
	iEffect.Glances = (iEffect.Glances or 0) + 1
	iEffect.GlancesDmg = (iEffect.GlancesDmg or 0) + total
	iEffect.GlancesMax = math.max(total, iEffect.GlancesMax or 0)
	iEffect.GlancesMin = Recap_Min(total, iEffect.GlancesMin)
end
function Recap_AddHits(iEffect, total)
	iEffect.CritsEvents = (iEffect.CritsEvents or 0) + 1
	iEffect.Hits = (iEffect.Hits or 0) + 1
	iEffect.HitsDmg = (iEffect.HitsDmg or 0) + total
	iEffect.HitsMax = math.max(total, iEffect.HitsMax or 0)
	iEffect.HitsMin = Recap_Min(total, iEffect.HitsMin)end
function Recap_AddTicks(iEffect, total)
	iEffect.Ticks = (iEffect.Ticks or 0) + 1
	iEffect.TicksDmg = (iEffect.TicksDmg or 0) + total
	iEffect.TicksMax = math.max(total, iEffect.TicksMax or 0)
	iEffect.TicksMin = Recap_Min(total, iEffect.TicksMin)
end
function Recap_AddMisses(iEffect, missType)
	if (missType == "miss") then
		iEffect.Missed = (iEffect.Missed or 0) + 1
	elseif (missType == "absorb") then
		iEffect.Absorbed = (iEffect.Absorbed or 0) + 1
	elseif (missType == "dodge") then
		iEffect.Dodged = (iEffect.Dodged or 0) + 1
	elseif (missType == "parry") or (missType == "parr") then
		iEffect.Parried = (iEffect.Parried or 0) + 1
	elseif (missType == "block") then
		iEffect.Blocked = (iEffect.Blocked or 0) + 1
	elseif (missType == "deflect") then
		iEffect.Deflected = (iEffect.Deflected or 0) + 1
	elseif (missType == "evade") then
		iEffect.Evaded = (iEffect.Evaded or 0) + 1
	elseif (missType == "resist") then
		iEffect.Resisted = (iEffect.Resisted or 0) + 1
	elseif (missType == "reflect") then
		iEffect.Reflected = (iEffect.Reflected or 0) + 1
	elseif (missType == "immune") then
		iEffect.Immune = (iEffect.Immune or 0) + 1
	end
end
function Recap_AddElement(iEffect, element)
	if not iEffect.Element then
		-- no element yet, set it
		iEffect.Element = element
	else
		-- if there is a new element that is not 'Other', check whether the new and old elements are the same
		if element and (element ~= recap_temp.Localize.ElementOther) then
			if not string.find(iEffect.Element, element, 1, true) then
				-- new element not found in old element, append it
				-- for example, perhaps the combatant switched wands
				if (iEffect.Element == "?") or (iEffect.Element == recap_temp.Localize.ElementOther) then
					-- replace a "?" or "Other"
					iEffect.Element = element
				else
					-- append to anything else
					iEffect.Element = iEffect.Element.."+"..element
				end
			end
		end
	end
end
function Recap_AddPartials(iEffect, total, absorb, block, resist, vulnerable)

	local proportion

	if absorb and absorb>0 then
		iEffect.PAbsorbs = (iEffect.PAbsorbs or 0) + 1
		iEffect.PAbsorbsDmg = (iEffect.PAbsorbsDmg or 0) + absorb
	end
	if block and block>0 then
		iEffect.PBlocks = (iEffect.PBlocks or 0) + 1
		iEffect.PBlocksDmg = (iEffect.PBlocksDmg or 0) + block
	end
	if resist and resist>0 then
		-- partial resist percentages are quantised to 25%, 50%, and 75% (with some variability)
		proportion = resist / (resist + total)
		if proportion < 0.325 then
			iEffect.PResists25 = (iEffect.PResists25 or 0) + 1
			iEffect.PResists25Dmg = (iEffect.PResists25Dmg or 0) + resist
		elseif proportion < 0.625 then
			iEffect.PResists50 = (iEffect.PResists50 or 0) + 1
			iEffect.PResists50Dmg = (iEffect.PResists50Dmg or 0) + resist
		else
			iEffect.PResists75 = (iEffect.PResists75 or 0) + 1
			iEffect.PResists75Dmg = (iEffect.PResists75Dmg or 0) + resist
		end
	end
	if vulnerable and vulnerable>0 then
		iEffect.PVulnerable = (iEffect.PVulnerable or 0) + 1
		iEffect.PVulnerableDmg = (iEffect.PVulnerableDmg or 0) + vulnerable
	end
end
function Recap_CalcInterval(iEffect, event_time)
	if iEffect.PrevTime and (iEffect.PrevTime > 0) then
		local delta_time = (1000 * event_time) - iEffect.PrevTime
		if (delta_time > 0) and (delta_time < 130000) then
			-- only pay attention if the two events happened less than 130 seconds apart
			-- count the number of time deltas, and sum their values (will be used to produce an average)
			iEffect.ICount = (iEffect.ICount or 0) + 1
			iEffect.ITotal = (iEffect.ITotal or 0) + delta_time
		end
	end
	iEffect.PrevTime = (1000 * event_time)
end
function Recap_AddSelfElement(iEffect, element)
	if not iEffect.Element then
		-- no element yet, set it
		iEffect.Element = element
	else
		-- if there is a new element that is not 'Other', check whether the new and old elements are the same
		if element and (element ~= recap_temp.Localize.ElementOther) then
			if not string.find(iEffect.Element, element, 1, true) then
				-- new element not found in old element, append it
				-- for example, perhaps the combatant switched wands
				if (iEffect.Element == "?") or (iEffect.Element == recap_temp.Localize.ElementOther) then
					-- replace a "?" or "Other"
					iEffect.Element = element
				else
					-- append to anything else
					iEffect.Element = iEffect.Element.."+"..element
				end
			end
		end
	end
end


function Recap_DeathEvent(event, info)

	if recap.Opt.State.value~="Stopped" then

		local j, dest, spot

		if recap.dump then
			local printable
			printable = Recap_CreateMessage(event)
			Recap_DumpMessage("DEATH;"..printable)
		end

		dest = info.victim
		if tonumber(dest) == ParserLib_SELF then
			dest = recap_temp.Player
		end

		if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
			-- in synchronization, don't ignore anyone (if 'ignore' they will still be hidden on the Fights panel)
		else
			if (dest and recap.Combatant[dest] and recap.Combatant[dest].Ignore) then
				-- dest is an ignored combatant, pretend the death didn't happen at all
				return
			end
		end

		-- does a combatant have "do not store" status due to the setting of options?
		if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") or (recap.Opt.StoreFriendsPartySelf.value == false) then
			-- in synchronization, or the StoreFriendsPartySelf option is off, carry on as normal
		else
			-- the StoreFriendsPartySelf option is on
			if dest then
				-- note that 'friends' include former party members, hostiles controlled for a while, and combatants explicitly named as friends
				if recap.Opt.HideFoe.value and recap.Combatant[dest] and not recap.Combatant[dest].Friend then
					-- ignore entirely
					return
				elseif recap.Opt.PartyData.value and not Recap_CheckForParty(dest, (recap.Combatant[dest] and recap.Combatant[dest].OwnedBy)) then
					-- ignore entirely
					return
				elseif recap.Opt.SelfData.value and not Recap_CheckForSelf(dest, (recap.Combatant[dest] and recap.Combatant[dest].OwnedBy)) then
					-- ignore entirely
					return
				end
			end
		end

		-- Hawksy: we often get two death events for a single death, one event with source and one without.
		-- Some testing indicates that the event without the source was more reliable, so we're using that one,
		--   but we will miss some deaths.
		if not (info.source) then
			if info.isItem then
				-- ignore items (such as totems) that are killed
			else
				-- other deaths
				-- check for FD
				if recap.Combatant[dest] and recap.Combatant[dest].Class=="Hunter" then
					if (UnitName("player"))==dest then
						spot = "player"
					end
					for j=1,4 do
						if (UnitName("party"..j))==dest then
							spot = "party"..j
							break
						end
					end
					for j=1,40 do
						if (UnitName("raid"..j))==dest then
							spot = "raid"..j
							break
						end
					end
					if spot then
						if UnitIsFeignDeath(spot) then
							dest = false
						end
					end
				end

				if dest and recap_temp.Last[dest] then
					recap_temp.Last[dest].Kills = recap_temp.Last[dest].Kills + 1
				end
			end
		end
	end
end


function Recap_HealEvent(event, info)

	if recap.Opt.State.value~="Stopped" then

		local source, dest, rawheal, effect, element, actualheal, overheal, sourceCombatant, destCombatant, iLast
		local event_time = GetTime()

		if recap.dump then
			local printable
			printable = Recap_CreateMessage(event)
			Recap_DumpMessage("HEALING;"..printable)
		end

		-- possibly start the fight
		if not recap.Opt.LimitFights.value and recap.Opt.State.value=="Idle" then
			Recap_StartFight()
		end

		-- if the fight is ongoing, start or restart the idle timers
		if recap.Opt.State.value=="Active" and recap.Opt.IdleFight.value then
			recap_temp.IdleTimer = 0
		end
		if recap.Opt.State.value=="Active" then
			recap_temp.HiddenIdleTimer = 0
		end

		-- if the fight is ongoing and if the end of fight delay timer is running, restart the end of fight delay timer
		--   (the player left combat, perhaps temporarily, but the fight continues)
		if recap.Opt.State.value=="Active" and recap_temp.EndFightDelayTimer~=-1 then
			recap_temp.EndFightDelayTimer = 0
		end

		source = info.source
		if tonumber(source) == ParserLib_SELF then
			source = recap_temp.Player
		end
		if tonumber(source) == ParserLib_NONE then
			-- healing with no source
			source = false
		end
		dest = info.victim
		if tonumber(dest) == ParserLib_SELF then
			dest = recap_temp.Player
		end

		rawheal = 0
		rawheal = tonumber(info.amount)
		effect = info.skill
		if tonumber(effect) == ParserLib_MELEE then -- shouldn't happen for a heal, just being paranoid
			effect = RECAP_MELEE
		end
		if tonumber(effect) == ParserLib_DAMAGESHIELD then -- shouldn't happen for a heal, just being paranoid
			effect = recap_temp.Localize.DamageShield
		end

		-- default element is Healing
		element = recap_temp.Localize.ElementHealing
		if info.element then
			element = info.element
		end

		-- Hawksy: check for known duplicates
		if source then
			if Recap_CheckForDuplicateSource(source) then
				return
			end
		end
		if Recap_CheckForDuplicateEffect(effect) then
			return
		end

		if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
			-- in synchronization, don't ignore anyone (if 'ignore' they will still be hidden on the Fights panel)
		else
			if (source and recap.Combatant[source] and recap.Combatant[source].Ignore) then
				-- source is an ignored combatant, pretend the heal didn't happen at all
				return
			end
			if (dest and recap.Combatant[dest] and recap.Combatant[dest].Ignore) then
				-- dest is an ignored combatant, pretend the heal didn't happen at all
				return
			end
		end

		-- does a combatant have "do not store" status due to the setting of options?
		if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") or (recap.Opt.StoreFriendsPartySelf.value == false) then
			-- in synchronization, or the StoreFriendsPartySelf option is off, carry on as normal
		else
			-- the StoreFriendsPartySelf option is on
			if source then
				-- note that 'friends' include former party members, hostiles controlled for a while, and combatants explicitly named as friends
				if recap.Opt.HideFoe.value and recap.Combatant[source] and not recap.Combatant[source].Friend then
					source = nil
				elseif recap.Opt.PartyData.value and not Recap_CheckForParty(source, (recap.Combatant[source] and recap.Combatant[source].OwnedBy)) then
					source = nil
				elseif recap.Opt.SelfData.value and not Recap_CheckForSelf(source, (recap.Combatant[source] and recap.Combatant[source].OwnedBy)) then
					source = nil
				end
			end
			if dest then
				-- note that 'friends' include former party members, hostiles controlled for a while, and combatants explicitly named as friends
				if recap.Opt.HideFoe.value and recap.Combatant[dest] and not recap.Combatant[dest].Friend then
					dest = nil
				elseif recap.Opt.PartyData.value and not Recap_CheckForParty(dest, (recap.Combatant[dest] and recap.Combatant[dest].OwnedBy)) then
					dest = nil
				elseif recap.Opt.SelfData.value and not Recap_CheckForSelf(dest, (recap.Combatant[dest] and recap.Combatant[dest].OwnedBy)) then
					dest = nil
				end
			end
			-- are both source and dest now nil?
			if source == nil and dest == nil then
				-- ignore entirely
				return
			end
		end

		-- Hawksy: now keeping an independent timer for healing, and setting the 'WasInCurrent' flag for the healer
		--   (receiving healing still gets no registration as a combatant)
		--   (note that this means that recipients of heals who do not yet have status will not get IncomingDetail entries)
		if source then
			Recap_CreateCombatantAndUpdateTimers(source, event_time, "Heal")
		end

		-- table lookup shortcuts
		if source then
			sourceCombatant = recap.Combatant[source]
		else
			sourceCombatant = nil
		end
		if dest then
			destCombatant = recap.Combatant[dest]
		else
			destCombatant = nil
		end

		if rawheal and (rawheal > 0) then

			actualheal = rawheal
			overheal = 0
			if dest then
				iLast = recap_temp.Last[dest]
				if iLast and iLast.HP then
					-- reduce 'actualheal' if we know the hit points of the target
					actualheal = math.max(0,math.min(iLast.MaxHP-iLast.HP,rawheal))
					iLast.HP = iLast.HP + actualheal
					overheal = rawheal - actualheal
				end
			end

			-- two hidden columns, gross healing (RawHeal) and overhealing (OverHeal)
			-- a calculated column, Heal, includes actual healing
			if source then
				iLast = recap_temp.Last[source]
				iLast.RawHeal = iLast.RawHeal + rawheal
				iLast.OverHeal = math.min((iLast.OverHeal + overheal),iLast.RawHeal)
			end

			if not recap.Opt.LightData.value then
				-- Hawksy: the effect is credited with the full value of the heal
				-- determine whether it could have critted
				if string.find(event, "PERIODIC", 1, true) then
					-- periodic event, assume not crit-capable, pass 'nil' in the crit slot -- keep crit and crush and glance as 'nil', not false
					Recap_CreateEffect(event_time, source, dest, effect, 3, rawheal, nil, info.isDOT, element, false, nil, nil, nil, nil, nil, nil)
				else
					-- assume all others are crit capable
					local critty
					critty = info.isCrit or false
					Recap_CreateEffect(event_time, source, dest, effect, 3, rawheal, critty, info.isDOT, element, false, false, false, nil, nil, nil, nil)
				end

				if source then
					if overheal>0 and sourceCombatant.OutgoingDetail["3"..effect] then
						-- reuse .Missed detail to store overhealing since heals never miss
						sourceCombatant.OutgoingDetail["3"..effect].Missed = (sourceCombatant.OutgoingDetail["3"..effect].Missed or 0) + overheal
					elseif overheal>0 and sourceCombatant.OutgoingDetail["4"..effect] then
						sourceCombatant.OutgoingDetail["4"..effect].Missed = (sourceCombatant.OutgoingDetail["4"..effect].Missed or 0) + overheal
					end
				end
				if dest then
					if overheal>0 and destCombatant.IncomingDetail["3"..effect] then
						-- reuse .Missed detail to store overhealing since heals never miss
						destCombatant.IncomingDetail["3"..effect].Missed = (destCombatant.IncomingDetail["3"..effect].Missed or 0) + overheal
					elseif overheal>0 and destCombatant.IncomingDetail["4"..effect] then
						destCombatant.IncomingDetail["4"..effect].Missed = (destCombatant.IncomingDetail["4"..effect].Missed or 0) + overheal
					end
				end
			end

			if recap.Opt.RecentData.value then
				Recap_AddRecentEvent(event_time, source, dest, 3, effect, element, rawheal, info.isCrit, false)
			end
		else
			-- Hawksy: no healing, can't be a miss, this is unexpected
			Recap_CheckUnexpectedEvent()
		end
	end
end

-- this handles buffs : debuffs : cleanses : procs : etc.
function Recap_OtherEvent(event, info)

	if recap.Opt.State.value~="Stopped" then

		local source, dest, amount, effect, sourceCombatant, destCombatant
		local event_time = GetTime()

		if recap.dump then
			local printable
			printable = Recap_CreateMessage(event)
			Recap_DumpMessage("OTHER;"..printable)
		end

		source = info.source
		if tonumber(source) == ParserLib_SELF then
			source = recap_temp.Player
		end
		if tonumber(source) == ParserLib_NONE then
			-- effect with no source
			source = false
		end
		dest = info.victim
		if tonumber(dest) == ParserLib_SELF then
			dest = recap_temp.Player
		end

		-- table lookup shortcuts
		if source then
			sourceCombatant = recap.Combatant[source]
		else
			sourceCombatant = nil
		end
		destCombatant = recap.Combatant[dest]

		amount = 0
		amount = tonumber(info.amount)
		effect = info.skill
		if tonumber(effect) == ParserLib_DAMAGESHIELD then
			effect = recap_temp.Localize.DamageShield
		end

		-- Hawksy: check for known duplicates
		if source then
			if Recap_CheckForDuplicateSource(source) then
				return
			end
		end
		if effect then
			if Recap_CheckForDuplicateEffect(effect) then
				return
			end
		end

		if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") then
			-- in synchronization, don't ignore anyone (if 'ignore' they will still be hidden on the Fights panel)
		else
			if (sourceCombatant and sourceCombatant.Ignore) then
				-- source is an ignored combatant, pretend the effect didn't happen at all
				return
			end
			if (destCombatant and destCombatant.Ignore) then
				-- dest is an ignored combatant, pretend the effect didn't happen at all
				return
			end
		end

		-- does a combatant have "do not store" status due to the setting of options?
		if (recap.Opt.SyncState.value == "Member") or (recap.Opt.SyncState.value == "Leader") or (recap.Opt.StoreFriendsPartySelf.value == false) then
			-- in synchronization, or the StoreFriendsPartySelf option is off, carry on as normal
		else
			-- the StoreFriendsPartySelf option is on
			if source then
				-- note that 'friends' include former party members, hostiles controlled for a while, and combatants explicitly named as friends
				if recap.Opt.HideFoe.value and recap.Combatant[source] and not recap.Combatant[source].Friend then
					source = nil
				elseif recap.Opt.PartyData.value and not Recap_CheckForParty(source, (recap.Combatant[source] and recap.Combatant[source].OwnedBy)) then
					source = nil
				elseif recap.Opt.SelfData.value and not Recap_CheckForSelf(source, (recap.Combatant[source] and recap.Combatant[source].OwnedBy)) then
					source = nil
				end
			end
			if dest then
				-- note that 'friends' include former party members, hostiles controlled for a while, and combatants explicitly named as friends
				if recap.Opt.HideFoe.value and recap.Combatant[dest] and not recap.Combatant[dest].Friend then
					dest = nil
				elseif recap.Opt.PartyData.value and not Recap_CheckForParty(dest, (recap.Combatant[dest] and recap.Combatant[dest].OwnedBy)) then
					dest = nil
				elseif recap.Opt.SelfData.value and not Recap_CheckForSelf(dest, (recap.Combatant[dest] and recap.Combatant[dest].OwnedBy)) then
					dest = nil
				end
			end
			-- are both source and dest now nil?
			if source == nil and dest == nil then
				-- ignore entirely
				return
			end
		end

		-- an Other effect does not start a fight
		-- an Other effect does not affect the idle timer
		-- an Other effect does not affect the end of fight delay timer
		-- an Other effect does not create a source or destination combatant, nor affect their combat timers

		if effect then
			if (info.type == "cast") then
				-- not currently doing anything with isBegin or isPerform
				Recap_CreateCastEffect(event_time, source, dest, effect, 1)
			elseif (info.type == "extraattack") then
				Recap_CreateExtraAttackEffect(event_time, source, dest, effect, amount, RECAP_ATTACKS)
			elseif (info.type == "leech") then
				Recap_CreateLeechEffect(event_time, source, dest, effect, amount, info.attribute, info.sourceGained, info.amountGained, info.attributeGained)
			elseif (info.type == "gain") then
				Recap_CreateGainEffect(event_time, source, dest, effect, amount, info.attribute)
			elseif (info.type == "drain") then
				Recap_CreateDrainEffect(event_time, source, dest, effect, amount, info.attribute)
			elseif (info.type == "buff") then
				amount = 1
				if info.amountRank then
					amount = tonumber(info.amountRank)
				end
				-- some buffs will not be counted because the dest does not yet exist
				Recap_CreateBuffEffect(event_time, source, dest, effect, amount)
			elseif (info.type == "debuff") then
				amount = 1
				if info.amountRank then
					amount = tonumber(info.amountRank)
				end
				-- some debuffs will not be counted because the dest does not yet exist
				Recap_CreateDebuffEffect(event_time, source, dest, effect, amount)
			elseif (info.type == "dispel") then
				Recap_CreateDispelEffect(event_time, source, dest, effect, info.isFailed, info.sourceSkill)
			elseif (info.type == "fade") then
				Recap_CreateFadeEffect(event_time, source, dest, effect)
			elseif (info.type == "interrupt") then
				Recap_CreateInterruptEffect(source, dest, RECAP_INTERRUPT)
			elseif (info.type == "durability") then
				-- not currently doing anything with item or isAllItems
				Recap_CreateDurabilityEffect(event_time, source, dest, effect)
			else
				-- screenshot what isn't handled
if recap_temp.snap then
	local printable
	printable = Recap_CreateMessage(event)
	printable = "OTHER_NOT_HANDLED;"..printable
	printable = string.gsub(printable,"\;","\ ") -- convert semicolon to blank for ordinary output
	DEFAULT_CHAT_FRAME:AddMessage(printable)
	Screenshot()
end
			end
		end
	end
end

function Recap_CreateCastEffect(event_time, source, dest, effect, total)

	local typeEffect, iEffect

	if source and recap.Combatant[source] then
		typeEffect = "1"..effect
		Recap_MakeOtherDetail(source, typeEffect)
		iEffect = recap.Combatant[source].OtherDetail[typeEffect]
		iEffect.Hits = (iEffect.Hits or 0) + 1
		Recap_CalcOtherInterval(iEffect, event_time)
	end
	-- we don't know whether a 'cast' event with a dest is a buff or a debuff, so insert it as an unknown (will be coloured white)
	-- inspecting the dump logs suggests that 'cast' events sometimes overlap with buffs and debuffs, so there may be some double counting
	if dest and recap.Combatant[dest] then
		typeEffect = "7"..effect
		Recap_MakeOtherDetail(dest, typeEffect)
		iEffect = recap.Combatant[dest].OtherDetail[typeEffect]
		Recap_AccumulateOtherDetail(iEffect, total, nil)
		Recap_CalcOtherInterval(iEffect, event_time)
	end
end
function Recap_CreateExtraAttackEffect(event_time, source, dest, effect, total, attribute)

	local typeEffect, iEffect

	if dest and recap.Combatant[dest] then
		typeEffect = "5"..effect
		Recap_MakeOtherDetail(dest, typeEffect)
		iEffect = recap.Combatant[dest].OtherDetail[typeEffect]
		Recap_AccumulateOtherDetail(iEffect, total, attribute)
		Recap_CalcOtherInterval(iEffect, event_time)
	end
end
function Recap_CreateLeechEffect(event_time, source, dest, effect, total, attribute, sourceGained, amountGained, attributeGained)

	local typeEffect, iEffect

	if source and recap.Combatant[source] then
		-- this is the implicit 'cast' from the source (who would not be credited if sourceGained were different)
		typeEffect = "1"..effect
		Recap_MakeOtherDetail(source, typeEffect)
		iEffect = recap.Combatant[source].OtherDetail[typeEffect]
		iEffect.Hits = (iEffect.Hits or 0) + 1
		Recap_CalcOtherInterval(iEffect, event_time)
	end
	if sourceGained and recap.Combatant[sourceGained] then
		-- this is the beneficiary of the leech (might not be the same as the source, though it is in all examples seen so far)
		--   (similarly the amounts and attributes are the same in all examples seen so far)
		typeEffect = "5"..effect
		Recap_MakeOtherDetail(sourceGained, typeEffect)
		iEffect = recap.Combatant[sourceGained].OtherDetail[typeEffect]
		Recap_AccumulateOtherDetail(iEffect, amountGained, attributeGained)
		Recap_CalcOtherInterval(iEffect, event_time)
	end
	if dest and recap.Combatant[dest] then
		-- this is the loser from the leech
		typeEffect = "3"..effect
		Recap_MakeOtherDetail(dest, typeEffect)
		iEffect = recap.Combatant[dest].OtherDetail[typeEffect]
		Recap_AccumulateOtherDetail(iEffect, total, attribute)
		Recap_CalcOtherInterval(iEffect, event_time)
	end
end
function Recap_CreateGainEffect(event_time, source, dest, effect, total, attribute)

	local typeEffect, iEffect

	if dest and recap.Combatant[dest] then
		typeEffect = "5"..effect
		Recap_MakeOtherDetail(dest, typeEffect)
		iEffect = recap.Combatant[dest].OtherDetail[typeEffect]
		Recap_AccumulateOtherDetail(iEffect, total, attribute)
		Recap_CalcOtherInterval(iEffect, event_time)
	end
end
function Recap_CreateDrainEffect(event_time, source, dest, effect, total, attribute)

	local typeEffect

	if dest and recap.Combatant[dest] then
		typeEffect = "3"..effect
		Recap_MakeOtherDetail(dest, typeEffect)
		iEffect = recap.Combatant[dest].OtherDetail[typeEffect]
		Recap_AccumulateOtherDetail(iEffect, total, attribute)
		Recap_CalcOtherInterval(iEffect, event_time)
	end
end
function Recap_CreateBuffEffect(event_time, source, dest, effect, total)

	local typeEffect

	if dest and recap.Combatant[dest] then
		typeEffect = "5"..effect
		Recap_MakeOtherDetail(dest, typeEffect)
		iEffect = recap.Combatant[dest].OtherDetail[typeEffect]
		Recap_AccumulateOtherDetail(iEffect, total, nil)
		Recap_CalcOtherInterval(iEffect, event_time)
	else
		-- buffs not recorded before dest has been registered as a combatant
	end
end
function Recap_CreateDebuffEffect(event_time, source, dest, effect, total)

	local typeEffect

	if dest and recap.Combatant[dest] then
		typeEffect = "3"..effect
		Recap_MakeOtherDetail(dest, typeEffect)
		iEffect = recap.Combatant[dest].OtherDetail[typeEffect]
		Recap_AccumulateOtherDetail(iEffect, total, nil)
		Recap_CalcOtherInterval(iEffect, event_time)
	else
		-- debuffs not recorded before dest has been registered as a combatant
	end
end
function Recap_CreateDispelEffect(event_time, source, dest, effect, failed, sourceEffect)

	local typeEffect, iEffect, iDetail

	if source and recap.Combatant[source] and sourceEffect then
		-- the 'dispel' event allows us to count dispel failures
		typeEffect = "1"..sourceEffect
		Recap_MakeOtherDetail(source, typeEffect)
		iEffect = recap.Combatant[source].OtherDetail[typeEffect]
		iEffect.Hits = (iEffect.Hits or 0) + 1
		if failed then
			iEffect.Missed = (iEffect.Missed or 0) + 1
		end
		Recap_CalcOtherInterval(iEffect, event_time)
	end
	if dest and recap.Combatant[dest] then
		if not failed then
			iDetail = recap.Combatant[dest].OtherDetail
			-- check for the existence of a buff or debuff to mark as dispelled
			typeEffect = nil
			if iDetail["3"..effect] then
				typeEffect = "3"..effect
			elseif iDetail["5"..effect] then
				typeEffect = "5"..effect
			elseif iDetail["7"..effect] then
				typeEffect = "7"..effect
			end
			if typeEffect then
				iDetail[typeEffect].Dispels = (iDetail[typeEffect].Dispels or 0) + 1
			end
		else
			-- not recording failed attempts to dispel an effect at the destination
		end
	end
end
function Recap_CreateFadeEffect(event_time, source, dest, effect)

	local typeEffect, iEffect

	if dest and recap.Combatant[dest] then
		-- check for the existence of a cast, buff, debuff, or unknown (in that order)
		typeEffect = nil
		if recap.Combatant[dest].OtherDetail["1"..effect] then
			typeEffect = "1"..effect
		elseif recap.Combatant[dest].OtherDetail["3"..effect] then
			typeEffect = "3"..effect
		elseif recap.Combatant[dest].OtherDetail["5"..effect] then
			typeEffect = "5"..effect
		elseif recap.Combatant[dest].OtherDetail["7"..effect] then
			typeEffect = "7"..effect
		else
			-- unmatched Fade effects (various causes)
		end
		if typeEffect then
			iEffect = recap.Combatant[dest].OtherDetail[typeEffect]
			Recap_CalcOtherDuration(iEffect, event_time)
		end
	end
end
function Recap_CreateMissEffect(source, dest, effect, missText)

	-- currently ignoring missText

	local typeEffect, iDetail

	if source and recap.Combatant[source] then
		-- check for the existence of a cast, buff, debuff, or unknown (in that order)
		typeEffect = nil
		iDetail = recap.Combatant[source].OtherDetail
		if iDetail["1"..effect] then
			typeEffect = "1"..effect
		elseif iDetail["3"..effect] then
			typeEffect = "3"..effect
		elseif iDetail["5"..effect] then
			typeEffect = "5"..effect
		elseif iDetail["7"..effect] then
			typeEffect = "7"..effect
		end
		if typeEffect then
			iDetail[typeEffect].Hits = (iDetail[typeEffect].Hits or 0) + 1
			iDetail[typeEffect].Missed = (iDetail[typeEffect].Missed or 0) + 1
		end
	else
		-- misses not recorded before source has been registered as a combatant
	end
	if dest and recap.Combatant[dest] then
		-- check for the existence of a cast, buff, debuff, or unknown (in that order)
		typeEffect = nil
		iDetail = recap.Combatant[dest].OtherDetail
		if iDetail["1"..effect] then
			typeEffect = "1"..effect
		elseif iDetail["3"..effect] then
			typeEffect = "3"..effect
		elseif iDetail["5"..effect] then
			typeEffect = "5"..effect
		elseif iDetail["7"..effect] then
			typeEffect = "7"..effect
		end
		if typeEffect then
			iDetail[typeEffect].Hits = (iDetail[typeEffect].Hits or 0) + 1
			iDetail[typeEffect].Missed = (iDetail[typeEffect].Missed or 0) + 1
		end
	else
		-- misses not recorded before dest has been registered as a combatant
	end
end
function Recap_CreateInterruptEffect(source, dest, effect)

	local typeEffect

	if source and recap.Combatant[source] then
		typeEffect = "1"..effect
		Recap_MakeOtherDetail(source, typeEffect)
		recap.Combatant[source].OtherDetail[typeEffect].Hits = (recap.Combatant[source].OtherDetail[typeEffect].Hits or 0) + 1
	end
	if dest and recap.Combatant[dest] then
		-- treat an incoming interrupt as a debuff
		typeEffect = "3"..effect
		Recap_MakeOtherDetail(dest, typeEffect)
		recap.Combatant[dest].OtherDetail[typeEffect].Hits = (recap.Combatant[dest].OtherDetail[typeEffect].Hits or 0) + 1
	end
end
function Recap_CreateDurabilityEffect(event_time, source, dest, effect)

	local typeEffect, iEffect

	if source and recap.Combatant[source] then
		typeEffect = "1"..effect
		Recap_MakeOtherDetail(source, typeEffect)
		iEffect = recap.Combatant[source].OtherDetail[typeEffect]
		iEffect.Hits = (iEffect.Hits or 0) + 1
		Recap_CalcOtherInterval(iEffect, event_time)
	end
	if dest and recap.Combatant[dest] then
		typeEffect = "3"..effect
		Recap_MakeOtherDetail(dest, typeEffect)
		iEffect = recap.Combatant[dest].OtherDetail[typeEffect]
		iEffect.Hits = (iEffect.Hits or 0) + 1
		Recap_CalcOtherInterval(iEffect, event_time)
	end
end

function Recap_CalcOtherInterval(iEffect, event_time)
	if iEffect.PrevTime and (iEffect.PrevTime > 0) then
		local delta_time = (1000 * event_time) - iEffect.PrevTime
		if (delta_time > 0) and (delta_time < 130000) then
			-- only pay attention if the two events happened less than 130 seconds apart
			-- count the number of time deltas, and sum their values (will be used to produce an average)
			iEffect.ICount = (iEffect.ICount or 0) + 1
			iEffect.ITotal = (iEffect.ITotal or 0) + delta_time
		end
	end
	iEffect.PrevTime = (1000 * event_time)
	-- set the timer for measuring durations
	iEffect.PrevTimeDur = (1000 * event_time)
end
function Recap_CalcOtherDuration(iEffect, event_time)
	if iEffect.PrevTimeDur and (iEffect.PrevTimeDur > 0) then
		local delta_time = (1000 * event_time) - iEffect.PrevTimeDur
		if (delta_time > 0) and (delta_time < 130000) then
			-- only pay attention if the apparent duration is less than 130 seconds
			-- count the number of time deltas, and sum their values (will be used to produce an average)
			iEffect.DCount = (iEffect.DCount or 0) + 1
			iEffect.DTotal = (iEffect.DTotal or 0) + delta_time
		end
	end
	-- clear the duration timer
	iEffect.PrevTimeDur = 0
end

function Recap_MakeOtherDetail(source, typeEffect)
	if not recap.Combatant[source].OtherDetail then
		recap.Combatant[source].OtherDetail = {}
	end
	if not recap.Combatant[source].OtherDetail[typeEffect] then
		recap.Combatant[source].OtherDetail[typeEffect] = {}
	end
end
function Recap_AccumulateOtherDetail(iEffect, total, attribute)
	iEffect.Hits = (iEffect.Hits or 0) + 1
	iEffect.Total = (iEffect.Total or 0) + total
	iEffect.Max = math.max(total,iEffect.Max or 1)
	if attribute then
		iEffect.Attribute = attribute
	end
end

-- removes combatant i from last fight
function Recap_ResetLastFight(i)

	local iCombatant = recap.Combatant[i]

	iCombatant.TotalTime = iCombatant.TotalTime - iCombatant.LastTime
	iCombatant.TotalTimeIn = iCombatant.TotalTimeIn - iCombatant.LastTimeIn
	iCombatant.TotalTimeHeal = iCombatant.TotalTimeHeal - iCombatant.LastTimeHeal
	iCombatant.TotalDmgIn = iCombatant.TotalDmgIn - iCombatant.LastDmgIn
	iCombatant.TotalDmgOut = iCombatant.TotalDmgOut - iCombatant.LastDmgOut
	iCombatant.TotalRawHeal = iCombatant.TotalRawHeal - iCombatant.LastRawHeal
	iCombatant.TotalOverHeal = math.min((iCombatant.TotalOverHeal - iCombatant.LastOverHeal),iCombatant.TotalRawHeal)
	iCombatant.TotalKills = iCombatant.TotalKills - iCombatant.LastKills
	if iCombatant.TotalTime > recap_temp.MinTime then
		iCombatant.TotalDPS = Recap_Div1(iCombatant.TotalDmgOut,iCombatant.TotalTime)
	else
		iCombatant.TotalDPS = 0
	end
	if iCombatant.TotalTimeIn > recap_temp.MinTime then
		iCombatant.TotalDPSIn = Recap_Div1(iCombatant.TotalDmgIn,iCombatant.TotalTimeIn)
	else
		iCombatant.TotalDPSIn = 0
	end
	if iCombatant.TotalTimeHeal > recap_temp.MinTime then
		iCombatant.TotalHPS = Recap_Div1((iCombatant.TotalRawHeal - iCombatant.TotalOverHeal),iCombatant.TotalTimeHeal)
	else
		iCombatant.TotalHPS = 0
	end
	iCombatant.WasInLast = false
	iCombatant.LastTime = 0
	iCombatant.LastTimeIn = 0
	iCombatant.LastTimeHeal = 0
	iCombatant.LastDmgIn = 0
	iCombatant.LastDmgOut = 0
	iCombatant.LastDPS = 0
	iCombatant.LastDPSIn = 0
	iCombatant.LastHPS = 0
	iCombatant.LastMaxHit = 0
	iCombatant.LastKills = 0
	iCombatant.LastRawHeal = 0
	iCombatant.LastOverHeal = 0
end


-- Hawksy: two functions to support Recent Data Mode
function Recap_CreateRecent()

	local i

	recap_temp.Recent = {}
	i = 1
	while i <= recap.Opt.RecentEventCount.value do
		recap_temp.Recent[i] = {}
		i = i + 1
	end
	recap_temp.Recent.First = 1
	recap_temp.Recent.Last = 1

end

function Recap_AddRecentEvent(event_time, source, dest, effectType, effect, element, amount, crit, crush)

	local i, iRecent

	-- play with the pointers
	if recap_temp.Recent[recap_temp.Recent.Last] and recap_temp.Recent[recap_temp.Recent.Last].T then
		-- events already stored, move Last pointer
		if recap_temp.Recent.Last >= recap.Opt.RecentEventCount.value then
			recap_temp.Recent.Last = 1
		else
			recap_temp.Recent.Last = recap_temp.Recent.Last + 1
		end
		if recap_temp.Recent.First == recap_temp.Recent.Last then
			-- pointers have met, move the First pointer out of the way
			if recap_temp.Recent.First >= recap.Opt.RecentEventCount.value then
				recap_temp.Recent.First = 1
			else
				recap_temp.Recent.First = recap_temp.Recent.First + 1
			end
		end
	else
		-- no event stored yet, store at Last
	end
	i = recap_temp.Recent.Last
	iRecent = recap_temp.Recent[i]
	-- single character keys to keep memory cost down (T = time, S = source D = destination, E = effect, L = element, A = amount, C = crit, R = crush)
	iRecent.T = event_time
	if source or dest then
		iRecent.S = source
		iRecent.D = dest
		iRecent.E = tostring(effectType)..effect
		iRecent.L = element
		iRecent.A = amount
		iRecent.C = crit
		iRecent.R = crush
	else
		-- if source and dest are false, this represents an 'end of fight' event
		iRecent.S = "**EOF**"
		iRecent.D = nil -- keep as nil
		iRecent.E = nil -- keep as nil
		iRecent.L = nil -- keep as nil
		iRecent.A = nil -- keep as nil
		iRecent.C = nil -- keep as nil
		iRecent.R = nil -- keep as nil
	end
end


-- Hawsky: new function to handle ParserLib events
function Recap_ParserLib_OnEvent(event, info)

	if (info.type == "death") then
		-- used to trigger only on CHAT_MSG_COMBAT_FRIENDLY_DEATH and CHAT_MSG_COMBAT_HOSTILE_DEATH
		-- now includes deaths such as "Boom Bot is killed by Suicide.", "Gan'arg Sapper is killed by Power Burn.",
		--    "Gurubashi Bat Rider is killed by Unstable Concoction.", and "Cattneth is killed by Demonic Sacrifice."
		Recap_DeathEvent(event, info)
	elseif (info.type == "hit") or (info.type == "miss") then
		Recap_DamageEvent(event, info)
	elseif (info.type == "environment") then
		Recap_EnvironmentDamageEvent(event, info)
	elseif (info.type == "heal") then
		Recap_HealEvent(event, info)
	elseif (info.type == "buff") or (info.type == "debuff") or (info.type == "cast") or (info.type == "dispel") or (info.type == "gain") or
		   (info.type == "drain") or (info.type == "leech") or (info.type == "fade") or (info.type == "extraattack") or
		   (info.type == "interrupt") or (info.type == "durability") then
		if not recap.Opt.LightData.value and recap.Opt.OtherData.value then
			-- only track this information if not in Light Data Mode, and with the Other Data Option enabled
			Recap_OtherEvent(event, info)
		end
	elseif (info.type == "unknown") then
		if recap_temp.snap then
			local printable, whichParser, value
			if ParserLib.parser30 then
				-- ckknight says this will indicate if the Parser-3.0 compatibility layer is being used
				whichParser = "P30"
			else
				whichParser = "PL11"
			end
			printable = "UNKNOWN;"..tostring(Recap_Version)..";"..whichParser
			value = tostring(info.message)
			value = string.gsub(value,"\ ","\_") -- convert blank to underscore
			printable = printable..";".."message="..value
			printable = string.gsub(printable,"\;","\ ") -- convert semicolon to blank for ordinary output
			DEFAULT_CHAT_FRAME:AddMessage(printable)
			Screenshot()
		end
	else
		-- info currently handled:
			-- source, victim, skill, element, amount, isCrit, isDOT, missType, (isItem for death), isCrushing, isGlancing
			-- amountAbsorb, amountBlock, amountResist, amountVulnerable
			-- attribute (for leech, gain, drain), damageType (for environment), amountRank (for buff or debuff)
			-- sourceSkill (for dispel), isFailed (for dispel), sourceGained (for leech), amountGained (for leech)
			-- attributeGained (for leech)
		-- events currently ignored:
			-- fail
		-- info currently ignored:
			-- isSplit (?)
			-- isBegin, isPerform (for cast)
			-- item, isAllItems (for durability)
			-- reason (for fail)
	end
end

-- Hawksy: moved some events to Recap_ParserLib_OnEvent
function Recap_Combat_OnEvent()

	local source, damageEffect, healEffect, event_time

	if event=="UPDATE_MOUSEOVER_UNIT" then
		Recap_GetUnitInfo("mouseover")
	elseif event=="PLAYER_TARGET_CHANGED" then
		Recap_GetUnitInfo("target")

	-- Hawksy: extensively rewritten
	elseif (event=="UNIT_SPELLCAST_SUCCEEDED") and recap.Opt.State.value~="Stopped" then
		-- this event is said to occur for anyone in the party or raid, and perhaps also the combatant's target.
		--   does not report the target for the spell (only UNIT_SPELLCAST_SENT does that, and it occurs also for failed casts)
		--   includes a lot of extra noise, such as pet attacks, dismounting, adding an Argent Dawn commission, etc.
		--   (2007-01-05: not sure it is catching everything, need to check into it, currently not checking for whether pets get this event)
		source = (UnitName(arg1))
		if source and recap.Combatant[source] then
			-- we have data for this combatant
			damageEffect = "1"..arg2
			healEffect = "3"..arg2
			if recap.Combatant[source].OutgoingDetail[damageEffect] or recap.Combatant[source].OutgoingDetail[healEffect] then
				-- this is a cast with which this combatant has done damage or healing before
				if recap.dump then
					Recap_DumpMessage("CAST;"..event..";"..source..";"..string.gsub(arg2,"\ ","\_"))
				end

				event_time = GetTime()

				-- possibly start the fight
				if not recap.Opt.LimitFights.value and recap.Opt.State.value=="Idle" then
					Recap_StartFight()
				end

				-- if the fight is ongoing, start or restart the idle timers
				if recap.Opt.State.value=="Active" and recap.Opt.IdleFight.value then
					recap_temp.IdleTimer = 0
				end
				if recap.Opt.State.value=="Active" then
					recap_temp.HiddenIdleTimer = 0
				end

				-- if the fight is ongoing and if the end of fight delay timer is running, restart the end of fight delay timer
				--   (the player left combat, perhaps temporarily, but the fight continues)
				if recap.Opt.State.value=="Active" and recap_temp.EndFightDelayTimer~=-1 then
					recap_temp.EndFightDelayTimer = 0
				end

				-- perhaps add the combatant into the fight, and update timers
				-- this takes the time from the moment of successful cast, rather than when it strikes the target
				if recap.Combatant[source].OutgoingDetail[damageEffect] then
					Recap_CreateCombatantAndUpdateTimers(source, event_time, "Out")
					if (recap.Combatant[source].OutgoingDetail[damageEffect].GlancesDmg) or (recap.Combatant[source].OutgoingDetail[damageEffect].HitsDmg) or (recap.Combatant[source].OutgoingDetail[damageEffect].CritsDmg) or (recap.Combatant[source].OutgoingDetail[damageEffect].CrushDmg) then
						-- this has a direct damage component, so it is already being counted; do nothing
					else
						if recap.Combatant[source].OutgoingDetail[damageEffect].TicksDmg then
							-- this is the casting of a DoT (with no direct damage component), so count it as a hit
							recap.Combatant[source].OutgoingDetail[damageEffect].Hits = (recap.Combatant[source].OutgoingDetail[damageEffect].Hits or 0) + 1
							recap.Combatant[source].OutgoingDetail[damageEffect].CritsEvents = (recap.Combatant[source].OutgoingDetail[damageEffect].CritsEvents or 0) + 1
							-- is the source the player or one of the player's pets?
							local selfIndex = nil -- keep as nil
							if source then
								if source == recap_temp.Player then
									selfIndex = recap_temp.s
								elseif recap.Combatant[source].OwnedBy == recap_temp.Player then
									selfIndex = recap_temp.s..":"..source
								end
							end
							if selfIndex then
								Recap_MakeSelfDetail(selfIndex, damageEffect)
								recap.Self[selfIndex][damageEffect].Hits = (recap.Self[selfIndex][damageEffect].Hits or 0) + 1
								recap.Self[selfIndex][damageEffect].CritsEvents = (recap.Self[selfIndex][damageEffect].CritsEvents or 0) + 1
							end

							if recap.Opt.RecentData.value then
								Recap_AddRecentEvent(event_time, source, "?", 1, arg2, false, "cast", false, false)
							end
						end
					end
				end

				-- for heals, just update the timers
				-- a standard heal will get counted as it lands, and a HoT only counts ticks
				if recap.Combatant[source].OutgoingDetail[healEffect] then
					Recap_CreateCombatantAndUpdateTimers(source, event_time, "Heal")
				end
			end
		end
	end
end

function Recap_Register_CombatEvents()

	-- Hawksy: get pointer to ParserLib
	recap_temp.parserlib = ParserLib:GetInstance("1.1")

	if not recap_temp.CombatEventsRegistered then
	-- register damage events
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_CREATURE_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_CREATURE_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_PARTY_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_PARTY_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_FRIENDLYPLAYER_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_FRIENDLYPLAYER_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_HOSTILEPLAYER_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_PARTY_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_PARTY_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_PET_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_PET_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_SELF_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_SELF_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_CREATURE_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_PARTY_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_DAMAGESHIELDS_ON_OTHERS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_DAMAGESHIELDS_ON_SELF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_FRIENDLYPLAYER_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PARTY_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PET_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_SELF_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_PARTY_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE", "Recap_ParserLib_OnEvent")

		-- Hawksy: added event
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_FAILED_LOCALPLAYER", "Recap_ParserLib_OnEvent")

		-- register death events (these aren't the only two, actually -- see ParserLib for the others)
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_FRIENDLY_DEATH", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_HOSTILE_DEATH", "Recap_ParserLib_OnEvent")

		-- register healing events
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_CREATURE_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_PARTY_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_SELF_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PARTY_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PET_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_SELF_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_BUFFS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS", "Recap_ParserLib_OnEvent")

		-- Hawksy: added additional events for buffs etc.
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_AURA_GONE_PARTY", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_AURA_GONE_OTHER", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_AURA_GONE_SELF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_BREAK_AURA", "Recap_ParserLib_OnEvent")

		-- Hawksy: not registered: CHAT_MSG_SPELL_TRADESKILLS, CHAT_MSG_COMBAT_HONOR_GAIN, CHAT_MSG_COMBAT_FACTION_CHANGE,
		--   CHAT_MSG_COMBAT_XP_GAIN, CHAT_MSG_SPELL_ITEM_ENCHANTMENTS

		RecapCombatEvents:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
		RecapCombatEvents:RegisterEvent("PLAYER_TARGET_CHANGED")
		-- Hawksy: change of event
		RecapCombatEvents:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")

		-- register for receiving messages on sync channel
		Recap:RegisterComm(Recap.commPrefix, "GROUP", "OnCommReceive")

		recap_temp.CombatEventsRegistered = 1
	end
end

function Recap_Unregister_CombatEvents()
	if recap_temp.CombatEventsRegistered then
		RecapCombatEvents:UnregisterAllEvents()
		recap_temp.parserlib:UnregisterAllEvents("Recap")
		Recap:UnregisterComm(Recap.commPrefix, "GROUP")
		recap_temp.CombatEventsRegistered = false
	end
end


RecapCombat_lua_372 = true
