--[[ RecapPanel.lua ]]

--[[ local tables ]]

local dockoffset = { ["TOPRIGHTTOPLEFT"] = { x=-4,y=0 },
					 ["BOTTOMRIGHTBOTTOMLEFT"] = { x=-4,y=0 },
					 ["TOPLEFTTOPRIGHT"] = { x=4,y=0 },
					 ["BOTTOMLEFTBOTTOMRIGHT"] = { x=4,y=0 },
					 ["TOPRIGHTBOTTOMRIGHT"] = { x=0,y=-4 },
					 ["BOTTOMRIGHTTOPRIGHT"] = { x=0,y=4 },
					 ["TOPLEFTBOTTOMLEFT"] = { x=0,y=-4 },
					 ["BOTTOMLEFTTOPLEFT"] = { x=0,y=4 } }


--[[ Functions for the popup panel ]]

function RecapPanel_Populate(arg1)
	-- Hawksy: recap.Opt.IncomingPanelDetail.value and recap.Opt.OutgoingPanelDetail.value contain the do-not-translate titles
	--   We need to convert them when creating header lines

	local i, totalmiss, text, found, iCombatant

	iCombatant = recap.Combatant[arg1]
	if iCombatant and not recap.Opt.LightData.value then

		--[[ Populate Panel 1: Incoming details ]]

		recap_temp.IncomingDetailSelected = 0
		RecapPanel_ConstructIncomingDetails(arg1)
		if recap_temp.IncomingDetailsListSize>1 then
			RecapPanel_PopulateIncomingDetails(arg1,1)
		else
			RecapPanel_PopulateIncomingDetails(arg1,0)
		end
		RecapPanelIncomingTotalLabel:SetText(recap_temp.Localize.DetailTitle[recap.Opt.IncomingPanelDetail.value])
		RecapPanelIncomingDetailsScrollBar_Update()

		--[[ Populate Panel 2: Outgoing details ]]

		recap_temp.OutgoingDetailSelected = 0
		RecapPanel_ConstructOutgoingDetails(arg1)
		if recap_temp.OutgoingDetailsListSize>1 then
			RecapPanel_PopulateOutgoingDetails(arg1,1)
		else
			RecapPanel_PopulateOutgoingDetails(arg1,0)
		end
		RecapPanelOutgoingTotalLabel:SetText(recap_temp.Localize.DetailTitle[recap.Opt.OutgoingPanelDetail.value])
		RecapPanelOutgoingDetailsScrollBar_Update()

		--[[ Populate Panel 3: Other details (buffs, debuffs, procs, and so on) ]]

		recap_temp.OtherDetailSelected = 0
		RecapPanel_ConstructOtherDetails(arg1)
		if recap_temp.OtherDetailsListSize>1 then
			RecapPanel_PopulateOtherDetails(arg1,1)
		else
			RecapPanel_PopulateOtherDetails(arg1,0)
		end
		RecapPanelOtherDetailsScrollBar_Update()

	end

	-- populate elements that work in and out of light data mode
	-- do none of these, at the moment, for the hidden combatants "Friendly Total" and "Non Friendly Total"

	if iCombatant then

		--[[ Populate common elements ]]

		if (arg1 == recap_temp.FriendlyTotal) then
			-- special title bar contents
			RecapPanelFaction:SetTexture("")
			RecapPanelLevel:SetText(" ")
			RecapPanelClass:SetTexCoord(.9,1,.9,1)
			RecapPanelName:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			RecapPanelName:SetText(recap_temp.Localize.Friendly.." "..RECAP_COMBATANTS)
		elseif (arg1 == recap_temp.NonFriendlyTotal) then
			-- special title bar contents
			RecapPanelFaction:SetTexture("")
			RecapPanelLevel:SetText(" ")
			RecapPanelClass:SetTexCoord(.9,1,.9,1)
			RecapPanelName:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			RecapPanelName:SetText(recap_temp.Localize.NonFriendly.." "..RECAP_COMBATANTS)
		else
			-- normal title bar contents
			if iCombatant.Faction and recap_temp.FactionIcons[iCombatant.Faction] then
				RecapPanelFaction:SetTexture(recap_temp.FactionIcons[iCombatant.Faction])
			else
				RecapPanelFaction:SetTexture("")
			end
			if iCombatant.Level==-1 then
				RecapPanelLevel:SetText("??")
			elseif tonumber(iCombatant.Level) and tonumber(iCombatant.Level)>0 then
				RecapPanelLevel:SetText(iCombatant.Level)
			else
				RecapPanelLevel:SetText(" ")
			end
			if iCombatant.Class and recap_temp.ClassIcons[iCombatant.Class] then
				RecapPanelClass:SetTexCoord(recap_temp.ClassIcons[iCombatant.Class].left,recap_temp.ClassIcons[iCombatant.Class].right,recap_temp.ClassIcons[iCombatant.Class].top,recap_temp.ClassIcons[iCombatant.Class].bottom)
			else
				RecapPanelClass:SetTexCoord(.9,1,.9,1)
			end
			if iCombatant.Friend then
				RecapPanelName:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			else
				RecapPanelName:SetTextColor(1,1,1)
			end
			RecapPanelName:SetText(arg1)
		end

		--[[ Populate Panel 4: Summary ]]

		RecapPanelLastTimeText:SetText(Recap_FormatTime(iCombatant.LastTime))
		RecapPanelLastTimeInText:SetText(Recap_FormatTime(iCombatant.LastTimeIn))
		RecapPanelLastTimeHealText:SetText(Recap_FormatTime(iCombatant.LastTimeHeal))
		RecapPanelLastMaxText:SetText(iCombatant.LastMaxHit)
		RecapPanelLastDeathsText:SetText(iCombatant.LastKills)
		RecapPanelLastHealsText:SetText((iCombatant.LastRawHeal or 0)-(iCombatant.LastOverHeal or 0))
		RecapPanelLastHPSText:SetText(string.format("%.1f",iCombatant.LastHPS))
		RecapPanelLastDmgInText:SetText(iCombatant.LastDmgIn)
		RecapPanelLastDPSInText:SetText(string.format("%.1f",iCombatant.LastDPSIn))
		RecapPanelLastDmgOutText:SetText(iCombatant.LastDmgOut)
		RecapPanelLastDPSText:SetText(string.format("%.1f",iCombatant.LastDPS))

		RecapPanelAllTimeText:SetText(Recap_FormatTime(iCombatant.TotalTime))
		RecapPanelAllTimeInText:SetText(Recap_FormatTime(iCombatant.TotalTimeIn))
		RecapPanelAllTimeHealText:SetText(Recap_FormatTime(iCombatant.TotalTimeHeal))
		RecapPanelAllMaxText:SetText(iCombatant.TotalMaxHit)
		RecapPanelAllDeathsText:SetText(iCombatant.TotalKills)
		RecapPanelAllHealsText:SetText(iCombatant.TotalRawHeal-iCombatant.TotalOverHeal)
		RecapPanelAllHPSText:SetText(string.format("%.1f",iCombatant.TotalHPS))
		RecapPanelAllDmgInText:SetText(iCombatant.TotalDmgIn)
		RecapPanelAllDPSInText:SetText(string.format("%.1f",iCombatant.TotalDPSIn))
		RecapPanelAllDmgOutText:SetText(iCombatant.TotalDmgOut)
		RecapPanelAllDPSText:SetText(string.format("%.1f",iCombatant.TotalDPS))

		if iCombatant.Friend then
			RecapPanelLastTimeText:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			RecapPanelAllTimeText:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			RecapPanelLastTimeInText:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			RecapPanelAllTimeInText:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			RecapPanelLastTimeHealText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			RecapPanelAllTimeHealText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			RecapPanelLastHealsText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			RecapPanelAllHealsText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			RecapPanelLastHPSText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			RecapPanelAllHPSText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			RecapPanelLastDmgInText:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			RecapPanelAllDmgInText:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			RecapPanelLastDPSInText:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			RecapPanelAllDPSInText:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			RecapPanelLastDmgOutText:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			RecapPanelAllDmgOutText:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			RecapPanelLastDPSText:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			RecapPanelAllDPSText:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
		else
			RecapPanelLastTimeText:SetTextColor(1,1,1)
			RecapPanelAllTimeText:SetTextColor(1,1,1)
			RecapPanelLastTimeInText:SetTextColor(1,1,1)
			RecapPanelAllTimeInText:SetTextColor(1,1,1)
			RecapPanelLastTimeHealText:SetTextColor(1,1,1)
			RecapPanelAllTimeHealText:SetTextColor(1,1,1)
			RecapPanelLastHealsText:SetTextColor(1,1,1)
			RecapPanelAllHealsText:SetTextColor(1,1,1)
			RecapPanelLastHPSText:SetTextColor(1,1,1)
			RecapPanelAllHPSText:SetTextColor(1,1,1)
			RecapPanelLastDmgInText:SetTextColor(1,1,1)
			RecapPanelAllDmgInText:SetTextColor(1,1,1)
			RecapPanelLastDPSInText:SetTextColor(1,1,1)
			RecapPanelAllDPSInText:SetTextColor(1,1,1)
			RecapPanelLastDmgOutText:SetTextColor(1,1,1)
			RecapPanelAllDmgOutText:SetTextColor(1,1,1)
			RecapPanelLastDPSText:SetTextColor(1,1,1)
			RecapPanelAllDPSText:SetTextColor(1,1,1)
		end

	end

end

-- tooltips for individual entries
function RecapPanel_Entry_OnEnter()

	-- Hawksy: lock OnEnter-OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local id = this:GetID()
	local _G = getfenv(0)

	if id and id>0 then
		if this:GetName()=="RecapPanelOutgoingMissP" and RecapPanelOutgoingMissPLabel:GetText()==RECAP_OVERHEALING then
			Recap_OnTooltip("PanelEntryOverheal",_G[this:GetName().."Text"]:GetText())
		else
			Recap_OnTooltip("PanelEntry"..id,_G[this:GetName().."Text"]:GetText())
		end
	end

	-- Hawksy: unlock OnEnter-OnLeave events
	recap_temp.OnEnterOnLeave = false
end

-- clicks on individual fields (the 'magic numbers' here are messy)
function RecapPanel_Entry_OnClick()

	local id, header, text, combatantName
	local i, j, k
	local _G = getfenv(0)

	if recap_temp.Selected == recap_temp.FriendlyIndex then
		combatantName = recap_temp.Localize.Friendly.." "..RECAP_COMBATANTS
	elseif recap_temp.Selected == recap_temp.NonFriendlyIndex then
		combatantName = recap_temp.Localize.NonFriendly.." "..RECAP_COMBATANTS
	else
		combatantName = recap_temp.List[recap_temp.Selected].Name
	end

	id = this:GetID()
	if id and id>0 and _G[this:GetName().."Text"]:GetText()~=" " and _G[this:GetName().."Text"]:GetText()~="--" and _G[this:GetName().."Text"]:GetText()~="?" then
		header = Recap_GetTooltip("PanelEntry"..id)
		if header and IsShiftKeyDown() and recap_temp.Selected~=0 then
			if (id>=70 and id<=79) then
				-- an Outgoing miss, post all miss entries
				local noData = true
				text = "Recap: "..combatantName..": "..RECAP_MISSES
				text = text.." "..recap_temp.Localize.For.." "..RecapPanelOutgoingEffectText:GetText()..":"
				for j in pairs(recap_temp.Localize.MissTypes) do
					k = _G["RecapPanelOutgoingMiss"..j.."Text"]:GetText()
					if (k ~= "--") and (k ~= "0 (0.0%)") then
						text = text.." "..recap_temp.Localize.MissTypes[j].." "..k
						noData = false
					end
				end
				if noData then
					text = text.." 0"
				end
			elseif (id>=55 and id<=64) then
				-- an Incoming miss, post all miss entries not just one
				local noData = true
				text = "Recap: "..combatantName..": "..RECAP_INCOMING.." "..RECAP_MISSES
				text = text.." "..recap_temp.Localize.For.." "..RecapPanelIncomingEffectText:GetText()..":"
				for j in pairs(recap_temp.Localize.MissTypes) do
					k = _G["RecapPanelIncomingMiss"..j.."Text"]:GetText()
					if (k ~= "--") and (k ~= "0 (0.0%)") then
						text = text.." "..recap_temp.Localize.MissTypes[j].." "..k
						noData = false
					end
				end
				if noData then
					text = text.." 0"
				end
			elseif ((id>=25 and id<=42) or (id>=80 and id<=83)) then
				text = "Recap: "..combatantName..": "..header
				text = text..": ".._G[this:GetName().."Text"]:GetText()
			else
				if ((this:GetName()=="RecapPanelOutgoingMissP" and RecapPanelOutgoingMissPLabel:GetText()==RECAP_OVERHEALING) or
					(this:GetName()=="RecapPanelIncomingMissP" and RecapPanelIncomingMissPLabel:GetText()==RECAP_OVERHEALING)) then
					header = RECAP_OVERHEALING
				end
				text = ""
				-- the magic numbers are for PanelEntry..id
				if ((id>=45 and id<=54) or (id>=65 and id<=67) or (id>=90 and id<=104) or (id>=106 and id<=132)) then
					-- I think the 'Outgoing' and 'Incoming' trigger off the xml names, so don't need localization
					if string.find(this:GetName(), "Outgoing", 1, true) and recap_temp.OutgoingDetailsListSize>1 then
						text = "Recap: "..combatantName..": "..RECAP_OUTGOING.." "..header
						text = text.." "..recap_temp.Localize.For.." "..RecapPanelOutgoingEffectText:GetText()
					elseif string.find(this:GetName(), "Incoming", 1, true) and recap_temp.IncomingDetailsListSize>1 then
						text = "Recap: "..combatantName..": "..RECAP_INCOMING.." "..header
						text = text.." "..recap_temp.Localize.For.." "..RecapPanelIncomingEffectText:GetText()
					elseif string.find(this:GetName(), "Other", 1, true) and recap_temp.OtherDetailsListSize>1 then
						text = "Recap: "..combatantName..": "..RECAP_OTHER_EFFECTS..": "..header
						text = text.." "..recap_temp.Localize.For.." "..RecapPanelOtherEffectText:GetText()
					end
				elseif (id == 105) then
					text = "Recap: "..combatantName
				end
				if _G[this:GetName().."Text"]:GetText() then
					text = text..": ".._G[this:GetName().."Text"]:GetText()
				end
			end
			Recap_InsertChat(text)
		end
	end
end

function RecapPanel_Show(name)

	if recap.Opt.ShowPanel.value then

		if recap_temp.Selected~=0 then
			if recap_temp.Selected == recap_temp.FriendlyIndex then
				name = recap_temp.FriendlyTotal
			elseif recap_temp.Selected == recap_temp.NonFriendlyIndex then
				name = recap_temp.NonFriendlyTotal
			else
				name = recap_temp.List[recap_temp.Selected].Name
			end
		end

		RecapPanel_Populate(name)
		RecapPanel:Show()
	end
end

function RecapPanel_Hide(arg1)

	if arg1 then
		recap_temp.Selected = 0
		recap_temp.OutgoingDetailSelected = 0
		recap_temp.IncomingDetailSelected = 0
		recap_temp.OtherDetailSelected = 0
		if not recap.Opt.Minimized.value then
			RecapScrollBar_Update()
		end
	end

	if recap_temp.Selected==0 then
		RecapPanel:Hide()
	end
end

function RecapPanelTab_OnEnter()

	-- Hawksy: lock OnEnter-OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local id = this:GetID()

	if id and id>0 then
		if recap.Opt.LightData.value and id<4 then
			Recap_OnTooltip("PanelTabDisabled"..id)
		else
			Recap_OnTooltip("PanelTab"..id)
		end
	end

	-- Hawksy: unlock OnEnter-OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function RecapPanelTab_OnClick()

	local id = this:GetID()
	if recap.Opt.LightData.value then
		id = 4
	end
	recap.Opt.PanelView.value = id
	RecapPanel_SwitchPanels(id)
end

function RecapPanel_SwitchPanels(panel)

	local i
	local _G = getfenv(0)

	for i=1,4 do
		_G["RecapPanelTab"..i]:UnlockHighlight()
		_G["RecapSubPanel"..i]:Hide()
	end
	_G["RecapPanelTab"..recap.Opt.PanelView.value]:LockHighlight()
	_G["RecapSubPanel"..recap.Opt.PanelView.value]:Show()
end

function RecapPanel_ConstructOutgoingDetails(name)

	local dmgtotal, healtotal, subtotaldmgtotal = 0,0,0
	local iMergePets, i, j, iList, iEffect, iType, effectName, iOwnerEffect
	local amount, landed, attempted, overheal, rawheal, missed, critted, count, sizeIncrement

	-- clear the details
	recap_temp.OutgoingDetailsListSize = 1
	recap_temp.OutgoingDetailsList = {} -- breaks table.sort if we try to re-use the table by setting individual values to nil

	-- to correctly handle the Subtotals and Totals when Merge Pets is on, we do three passes
	-- pass one handles all ordinary owner and pet list entries, plus Subtotal and Total lines when Merge Pets is off (a straight copy is fine)
	-- when Merge Pets is on pass one also does the first stage of owner Subtotal and Total lines
	--   when Merge Pets is on extra hidden fields are added for Subtotal and Total lines to support pass three derived calculations
	-- with Merge Pets is on pass two merges pet Subtotal and Total lines, either adding them in or copying them if the owner has nothing matching
	-- with Merge Pets is on pass three does derived calculations for Subtotals and Totals

	-- is Merge Pets on?
	iMergePets = Recap_MergePets()

	-- pass one
	for i in pairs(recap.Combatant[name].OutgoingDetail) do
		if not recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize] then
			recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize] = {}
		end
		iType = tonumber(string.sub(i,1,1))
		if iMergePets and ((iType == 6) or (iType == 8)) then
			-- skip pet Subtotals and Totals on this pass with Merge Pets on
		elseif iMergePets and ((iType == 5) or (iType == 7)) then
			-- owner Subtotal or Total with Merge Pets on
			iEffect = recap.Combatant[name].OutgoingDetail[i]
			iList = recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize]
			iList.Effect = i
			amount = (iEffect.GlancesDmg or 0) + (iEffect.HitsDmg or 0) + (iEffect.CritsDmg or 0) + (iEffect.CrushDmg or 0) + (iEffect.TicksDmg or 0)
			iList.Total = amount
			iList.Max = math.max(iEffect.TicksMax or 0,math.max(iEffect.HitsMax or 0,math.max(iEffect.CritsMax or 0,math.max(iEffect.CrushMax or 0,iEffect.GlancesMax or 0))))
			iList.Landed = (iEffect.Glances or 0) + (iEffect.Hits or 0) + (iEffect.Crits or 0) + (iEffect.Crushes or 0) + (iEffect.Ticks or 0)
			iList.Critted = (iEffect.Crits or 0)

			-- misses or overhealing (and crit % calcs)
			if (iType == 5) or ((iType == 7) and (string.find(i, RECAP_DAMAGE, 1, true))) then
				missed = (iEffect.Missed or 0) + (iEffect.Blocked or 0) + (iEffect.Dodged or 0) + (iEffect.Parried or 0) +
						(iEffect.Deflected or 0) + (iEffect.Resisted or 0) + (iEffect.Reflected or 0) + (iEffect.Absorbed or 0) +
						(iEffect.Immune or 0) + (iEffect.Evaded or 0)
				iList.Missed = missed
				iList.Attempted = iList.Missed + (iEffect.CritsEvents or 0)
			elseif (iType == 7) and (string.find(i, recap_temp.Localize.ElementHealing, 1, true)) then
				iList.Attempted = (iEffect.CritsEvents or 0)
				-- overhealing (stored in the miss slot) calcs
				iList.OverHeal = (iEffect.Missed or 0)
			end

			-- subtotals for percentage calcs
			if (iType == 5) then
				subtotaldmgtotal = subtotaldmgtotal + amount
			elseif (iType == 7) then
				-- do nothing, answer is always 100%
			end

			-- Subtotal and Total element always blank
			iList.Element = ""

			-- one hidden column for Interval (parallel code populates the fine details)
			iList.ICount = (iEffect.ICount or 0)
			iList.ITotal = (iEffect.ITotal or 0)

			recap_temp.OutgoingDetailsListSize = recap_temp.OutgoingDetailsListSize + 1
		else
			-- standard owner or pet, or Subtotals and Totals with Merge Pets off, simply copy
			iEffect = recap.Combatant[name].OutgoingDetail[i]
			iList = recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize]
			iList.Effect = i
			amount = (iEffect.GlancesDmg or 0) + (iEffect.HitsDmg or 0) + (iEffect.CritsDmg or 0) + (iEffect.CrushDmg or 0) + (iEffect.TicksDmg or 0)
			iList.Total = amount
			iList.Max = math.max(iEffect.TicksMax or 0,math.max(iEffect.HitsMax or 0,math.max(iEffect.CritsMax or 0,math.max(iEffect.CrushMax or 0,iEffect.GlancesMax or 0))))
			landed = (iEffect.Glances or 0) + (iEffect.Hits or 0) + (iEffect.Crits or 0) + (iEffect.Crushes or 0) + (iEffect.Ticks or 0)
			iList.Avg = string.format("%d",(landed>0 and Recap_Div0(amount, landed) or 0))
			critted = (iEffect.Crits or 0)

			-- misses or overhealing
			attempted = 0
			if (iType == 1) or (iType == 2) or (iType == 5) or (iType == 6) or (((iType == 7) or (iType == 8)) and (string.find(i, RECAP_DAMAGE, 1, true))) then
				missed = (iEffect.Missed or 0) + (iEffect.Blocked or 0) + (iEffect.Dodged or 0) + (iEffect.Parried or 0) +
						(iEffect.Deflected or 0) + (iEffect.Resisted or 0) + (iEffect.Reflected or 0) + (iEffect.Absorbed or 0) +
						(iEffect.Immune or 0) + (iEffect.Evaded or 0)
				attempted = missed + (iEffect.CritsEvents or 0)
				if (attempted > 0) then
					iList.MissPOverP = string.format("%.1f%%",Recap_Div1(100*missed, attempted))
				else
					iList.MissPOverP = " "
				end
			elseif (iType == 3) or (iType == 4) or (((iType == 7) or (iType == 8)) and (string.find(i, recap_temp.Localize.ElementHealing, 1, true))) then
				attempted = (iEffect.CritsEvents or 0)
				-- overhealing (stored in the miss slot) calcs
				overheal = (iEffect.Missed or 0)
				rawheal = amount + overheal
				if (rawheal > 0) then
					iList.MissPOverP = string.format("%.1f%%",Recap_Div1(100*overheal, rawheal))
				else
					iList.MissPOverP = " "
				end
			end

			-- crit %
			if attempted>0 then
				iList.CritP = string.format("%.1f%%",Recap_Div1(100*critted, attempted))
			else
				iList.CritP = "--"
			end

			-- subtotals for percentage calcs
			if (iType == 1) or (iType == 2) then
				dmgtotal = dmgtotal + amount
			elseif (iType == 3) or (iType == 4) then
				healtotal = healtotal + amount
			elseif (iType == 5) or (iType == 6) then
				subtotaldmgtotal = subtotaldmgtotal + amount
			elseif (iType == 7) or (iType == 8) then
				-- do nothing, answer is always 100%
			end

			-- add element to the list (hide question mark values)
			if iEffect.Element and (iEffect.Element ~= "?") then
				iList.Element = iEffect.Element
			else
				iList.Element = ""
			end

			-- one hidden column for Interval (parallel code populates the fine details)
			count = (iEffect.ICount or 0)
			if count>0 then
				iList.Interval = string.format("%.1f",Recap_Div1((iEffect.ITotal or 0), 1000*count))
			else
				iList.Interval = "--"
			end

			recap_temp.OutgoingDetailsListSize = recap_temp.OutgoingDetailsListSize + 1
		end
	end
	recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize] = nil -- keep as nil

	-- pass two
	if iMergePets then
		for i in pairs(recap.Combatant[name].OutgoingDetail) do
			if not recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize] then
				recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize] = {}
			end
			iType = tonumber(string.sub(i,1,1))
			if (iType == 6) or (iType == 8) then
				-- pet Subtotal or Total with Merge Pets on
				iEffect = recap.Combatant[name].OutgoingDetail[i]
				iList = recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize]
				sizeIncrement = 1
				-- matching owner effect
				_, _, effectName = string.find(i, "^.-: (.+)$")
				iOwnerEffect = tostring(iType-1)..effectName
				-- see if there is already a matching owner Subtotal or Total, and if so override iList and don't increment the size of the list
				for j=1, recap_temp.OutgoingDetailsListSize-1 do
					if recap_temp.OutgoingDetailsList[j].Effect and (recap_temp.OutgoingDetailsList[j].Effect == iOwnerEffect) then
						iList = recap_temp.OutgoingDetailsList[j]
						sizeIncrement = 0
						break
					end
				end
				iList.Effect = iOwnerEffect
				amount = (iEffect.GlancesDmg or 0) + (iEffect.HitsDmg or 0) + (iEffect.CritsDmg or 0) + (iEffect.CrushDmg or 0) + (iEffect.TicksDmg or 0)
				iList.Total = (iList.Total or 0) + amount
				iList.Max = math.max(iList.Max or 0,math.max(iEffect.TicksMax or 0,math.max(iEffect.HitsMax or 0,math.max(iEffect.CritsMax or 0,math.max(iEffect.CrushMax or 0,iEffect.GlancesMax or 0)))))
				iList.Landed = (iList.Landed or 0) + (iEffect.Glances or 0) + (iEffect.Hits or 0) + (iEffect.Crits or 0) + (iEffect.Crushes or 0) + (iEffect.Ticks or 0)
				iList.Critted = (iList.Critted or 0) + (iEffect.Crits or 0)

				-- misses or overhealing (and crit % calcs)
				if (iType == 6) or ((iType == 8) and (string.find(i, RECAP_DAMAGE, 1, true))) then
					missed = (iEffect.Missed or 0) + (iEffect.Blocked or 0) + (iEffect.Dodged or 0) + (iEffect.Parried or 0) +
							(iEffect.Deflected or 0) + (iEffect.Resisted or 0) + (iEffect.Reflected or 0) + (iEffect.Absorbed or 0) +
							(iEffect.Immune or 0) + (iEffect.Evaded or 0)
					iList.Missed = (iList.Missed or 0) + missed
					iList.Attempted = (iList.Attempted or 0) + missed + (iEffect.CritsEvents or 0)
				elseif (iType == 8) and (string.find(i, recap_temp.Localize.ElementHealing, 1, true)) then
					iList.Attempted = (iList.Attempted or 0) + (iEffect.CritsEvents or 0)
					-- overhealing (stored in the miss slot) calcs
					iList.OverHeal = (iList.OverHeal or 0) + (iEffect.Missed or 0)
				end

				-- subtotals for percentage calcs
				if (iType == 6) then
					subtotaldmgtotal = subtotaldmgtotal + amount
				elseif (iType == 8) then
					-- do nothing, answer is always 100%
				end

				-- Subtotal and Total element always blank
				iList.Element = ""

				-- one hidden column for Interval (parallel code populates the fine details)
				iList.ICount = (iList.ICount or 0) + (iEffect.ICount or 0)
				iList.ITotal = (iList.ITotal or 0) + (iEffect.ITotal or 0)

				recap_temp.OutgoingDetailsListSize = recap_temp.OutgoingDetailsListSize + sizeIncrement
			end
		end
		recap_temp.OutgoingDetailsList[recap_temp.OutgoingDetailsListSize] = nil -- keep as nil
	end

	-- pass three
	if iMergePets then
		for i=1, recap_temp.OutgoingDetailsListSize-1 do
			iList = recap_temp.OutgoingDetailsList[i]
			iType = tonumber(string.sub(iList.Effect,1,1))
			if (iType == 5) or (iType == 7) then
				-- owner Subtotal or Total (now with ** ADDED PETS **) with Merge Pets on
				iList.Avg = string.format("%d",(iList.Landed>0 and Recap_Div0(iList.Total, iList.Landed) or 0))
				if (iType == 5) or ((iType == 7) and (string.find(i, RECAP_DAMAGE, 1, true))) then
					if iList.Attempted and (iList.Attempted > 0) then
						iList.MissPOverP = string.format("%.1f%%",Recap_Div1(100*iList.Missed, iList.Attempted))
					else
						iList.MissPOverP = " "
					end
				elseif (iType == 7) and (string.find(i, recap_temp.Localize.ElementHealing, 1, true)) then
					rawheal = iList.Total + iList.OverHeal
					if (rawheal > 0) then
						iList.MissPOverP = string.format("%.1f%%",Recap_Div1(100*iList.OverHeal, rawheal))
					else
						iList.MissPOverP = " "
					end
				end
				if iList.Attempted and (iList.Attempted > 0) then
					iList.CritP = string.format("%.1f%%",Recap_Div1(100*iList.Critted, iList.Attempted))
				else
					iList.CritP = "--"
				end
				if iList.ICount>0 then
					iList.Interval = string.format("%.1f",Recap_Div1(iList.ITotal, 1000*iList.ICount))
				else
					iList.Interval = "--"
				end
			end
		end
	end

	for i=1, recap_temp.OutgoingDetailsListSize-1 do
		iList = recap_temp.OutgoingDetailsList[i]
		iType = tonumber(string.sub(iList.Effect,1,1))
		if (iType == 1) or (iType == 2) then
			subtotal = dmgtotal
		elseif (iType == 3) or (iType == 4) then
			subtotal = healtotal
		elseif (iType == 5) or (iType == 6) then
			subtotal = subtotaldmgtotal
		elseif (iType == 7) or (iType == 8) then
			-- answer is always 100%
			subtotal = iList.Total
		end
		if subtotal>0 then
			iList.TotalP = Recap_Div0(100*iList.Total, subtotal)
		else
			iList.TotalP = 0
		end
	end

	table.sort(recap_temp.OutgoingDetailsList,RecapPanel_DetailsDamageSort)

end

function RecapPanel_ConstructIncomingDetails(name)

	local dmgtotal, healtotal, subtotaldmgtotal = 0,0,0
	local iMergePets, i, j, iList, iEffect, iType, effectName, iOwnerEffect
	local amount, landed, attempted,  overheal, rawheal, missed, critted, count, sizeIncrement

	-- clear the details
	recap_temp.IncomingDetailsListSize = 1
	recap_temp.IncomingDetailsList = {} -- breaks table.sort if we try to re-use the table by setting individual values to nil

	-- special code to handle pet Subtotals and Totals when Merge Pets is on, see comments in ConstructOutgoingDetails above

	-- is Merge Pets on?
	iMergePets = Recap_MergePets()

	-- pass one
	for i in pairs(recap.Combatant[name].IncomingDetail) do
		if not recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize] then
			recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize] = {}
		end
		iType = tonumber(string.sub(i,1,1))
		if iMergePets and ((iType == 6) or (iType == 8)) then
			-- skip pet Subtotals and Totals on this pass with Merge Pets on
		elseif iMergePets and ((iType == 5) or (iType == 7)) then
			-- owner Subtotal or Total with Merge Pets on
			iEffect = recap.Combatant[name].IncomingDetail[i]
			iList = recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize]
			iList.Effect = i
			amount = (iEffect.GlancesDmg or 0) + (iEffect.HitsDmg or 0) + (iEffect.CritsDmg or 0) + (iEffect.CrushDmg or 0) + (iEffect.TicksDmg or 0)
			iList.Total = amount
			iList.Max = math.max(iEffect.TicksMax or 0,math.max(iEffect.HitsMax or 0,math.max(iEffect.CritsMax or 0,math.max(iEffect.CrushMax or 0,iEffect.GlancesMax or 0))))
			iList.Landed = (iEffect.Glances or 0) + (iEffect.Hits or 0) + (iEffect.Crits or 0) + (iEffect.Crushes or 0) + (iEffect.Ticks or 0)
			iList.Critted = (iEffect.Crits or 0)

			-- misses or overhealing (and crit % calcs)
			if (iType == 5) or ((iType == 7) and (string.find(i, RECAP_DAMAGE, 1, true))) then
				missed = (iEffect.Missed or 0) + (iEffect.Blocked or 0) + (iEffect.Dodged or 0) + (iEffect.Parried or 0) +
						(iEffect.Deflected or 0) + (iEffect.Resisted or 0) + (iEffect.Reflected or 0) + (iEffect.Absorbed or 0) +
						(iEffect.Immune or 0) + (iEffect.Evaded or 0)
				iList.Missed = missed
				iList.Attempted = missed + (iEffect.CritsEvents or 0)
			elseif (iType == 7) and (string.find(i, recap_temp.Localize.ElementHealing, 1, true)) then
				iList.Attempted = (iEffect.CritsEvents or 0)
				-- overhealing (stored in the miss slot) calcs
				iList.OverHeal = (iEffect.Missed or 0)
			end

			-- subtotals for percentage calcs
			if (iType == 5) then
				subtotaldmgtotal = subtotaldmgtotal + amount
			elseif (iType == 7) then
				-- do nothing, answer is always 100%
			end

			-- Subtotal and Total element always blank
			iList.Element = ""

			-- one hidden column for Interval (parallel code populates the fine details)
			iList.ICount = (iEffect.ICount or 0)
			iList.ITotal = (iEffect.ITotal or 0)

			recap_temp.IncomingDetailsListSize = recap_temp.IncomingDetailsListSize + 1
		else
			-- standard owner or pet, or Subtotals and Totals with Merge Pets off, simply copy
			iEffect = recap.Combatant[name].IncomingDetail[i]
			iList = recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize]
			iList.Effect = i
			amount = (iEffect.GlancesDmg or 0) + (iEffect.HitsDmg or 0) + (iEffect.CritsDmg or 0) + (iEffect.CrushDmg or 0) + (iEffect.TicksDmg or 0)
			iList.Total = amount
			iList.Max = math.max(iEffect.TicksMax or 0,math.max(iEffect.HitsMax or 0,math.max(iEffect.CritsMax or 0,math.max(iEffect.CrushMax or 0,iEffect.GlancesMax or 0))))
			landed = (iEffect.Glances or 0) + (iEffect.Hits or 0) + (iEffect.Crits or 0) + (iEffect.Crushes or 0) + (iEffect.Ticks or 0)
			iList.Avg = string.format("%d",(landed>0 and Recap_Div0(amount, landed) or 0))
			critted = (iEffect.Crits or 0)

			-- misses or overhealing (and crit % calcs)
			attempted = 0
			if (iType == 1) or (iType == 2) or (iType == 5) or (iType == 6) or (((iType == 7) or (iType == 8)) and (string.find(i, RECAP_DAMAGE, 1, true))) then
				missed = (iEffect.Missed or 0) + (iEffect.Blocked or 0) + (iEffect.Dodged or 0) + (iEffect.Parried or 0) +
						(iEffect.Deflected or 0) + (iEffect.Resisted or 0) + (iEffect.Reflected or 0) + (iEffect.Absorbed or 0) +
						(iEffect.Immune or 0) + (iEffect.Evaded or 0)
				attempted = missed + (iEffect.CritsEvents or 0)
				if (attempted > 0) then
					iList.MissPOverP = string.format("%.1f%%",Recap_Div1(100*missed, attempted))
				else
					iList.MissPOverP = " "
				end
			elseif (iType == 3) or (iType == 4) or (((iType == 7) or (iType == 8)) and (string.find(i, recap_temp.Localize.ElementHealing, 1, true))) then
				attempted = (iEffect.CritsEvents or 0)
				-- overhealing (stored in the miss slot) calcs
				overheal = (iEffect.Missed or 0)
				rawheal = amount + overheal
				if (rawheal > 0) then
					iList.MissPOverP = string.format("%.1f%%",Recap_Div1(100*overheal, rawheal))
				else
					iList.MissPOverP = " "
				end
			end

			-- crit %
			if attempted>0 then
				iList.CritP = string.format("%.1f%%",Recap_Div1(100*critted, attempted))
			else
				iList.CritP = "--"
			end

			-- subtotals for percentage calcs
			if (iType == 1) or (iType == 2) then
				dmgtotal = dmgtotal + amount
			elseif (iType == 3) or (iType == 4) then
				healtotal = healtotal + amount
			elseif (iType == 5) or (iType == 6) then
				subtotaldmgtotal = subtotaldmgtotal + amount
			elseif (iType == 7) or (iType == 8) then
				-- do nothing, answer is always 100%
			end

			-- add element to the list (hide question mark values)
			if iEffect.Element and (iEffect.Element ~= "?") then
				iList.Element = iEffect.Element
			else
				iList.Element = ""
			end

			-- one hidden column for Interval (parallel code populates the fine details)
			count = (iEffect.ICount or 0)
			if count>0 then
				iList.Interval = string.format("%.1f",Recap_Div1((iEffect.ITotal or 0), 1000*count))
			else
				iList.Interval = "--"
			end

			recap_temp.IncomingDetailsListSize = recap_temp.IncomingDetailsListSize + 1
		end
	end
	recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize] = nil -- keep as nil

	-- pass two
	if iMergePets then
		for i in pairs(recap.Combatant[name].IncomingDetail) do
			if not recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize] then
				recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize] = {}
			end
			iType = tonumber(string.sub(i,1,1))
			if (iType == 6) or (iType == 8) then
				-- pet Subtotal or Total with Merge Pets on
				iEffect = recap.Combatant[name].IncomingDetail[i]
				iList = recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize]
				sizeIncrement = 1
				-- matching owner effect
				_, _, effectName = string.find(i, "^.-: (.+)$")
				iOwnerEffect = tostring(iType-1)..effectName
				-- see if there is already a matching owner Subtotal or Total, and if so override iList and don't increment the size of the list
				for j=1, recap_temp.IncomingDetailsListSize do
					if recap_temp.IncomingDetailsList[j].Effect and (recap_temp.IncomingDetailsList[j].Effect == iOwnerEffect) then
						iList = recap_temp.IncomingDetailsList[j]
						sizeIncrement = 0
						break
					end
				end
				iList.Effect = iOwnerEffect
				amount = (iEffect.GlancesDmg or 0) + (iEffect.HitsDmg or 0) + (iEffect.CritsDmg or 0) + (iEffect.CrushDmg or 0) + (iEffect.TicksDmg or 0)
				iList.Total = (iList.Total or 0) + amount
				iList.Max = math.max(iList.Max or 0,math.max(iEffect.TicksMax or 0,math.max(iEffect.HitsMax or 0,math.max(iEffect.CritsMax or 0,math.max(iEffect.CrushMax or 0,iEffect.GlancesMax or 0)))))
				iList.Landed = (iList.Landed or 0) + (iEffect.Glances or 0) + (iEffect.Hits or 0) + (iEffect.Crits or 0) + (iEffect.Crushes or 0) + (iEffect.Ticks or 0)
				iList.Critted = (iList.Critted or 0) + (iEffect.Crits or 0)

				-- misses or overhealing (and crit % calcs)
				if (iType == 6) or ((iType == 8) and (string.find(i, RECAP_DAMAGE, 1, true))) then
					missed = (iEffect.Missed or 0) + (iEffect.Blocked or 0) + (iEffect.Dodged or 0) + (iEffect.Parried or 0) +
							(iEffect.Deflected or 0) + (iEffect.Resisted or 0) + (iEffect.Reflected or 0) + (iEffect.Absorbed or 0) +
							(iEffect.Immune or 0) + (iEffect.Evaded or 0)
					iList.Missed = (iList.Missed or 0) + missed
					iList.Attempted = (iList.Attempted or 0) + missed + (iEffect.CritsEvents or 0)
				elseif (iType == 8) and (string.find(i, recap_temp.Localize.ElementHealing, 1, true)) then
					iList.Attempted = (iList.Attempted or 0) + (iEffect.CritsEvents or 0)
					-- overhealing (stored in the miss slot) calcs
					iList.OverHeal = (iList.OverHeal or 0) + (iEffect.Missed or 0)
				end

				-- subtotals for percentage calcs
				if (iType == 6) then
					subtotaldmgtotal = subtotaldmgtotal + amount
				elseif (iType == 8) then
					-- do nothing, answer is always 100%
				end

				-- Subtotal and Total element always blank
				iList.Element = ""

				-- one hidden column for Interval (parallel code populates the fine details)
				iList.ICount = (iList.ICount or 0) + (iEffect.ICount or 0)
				iList.ITotal = (iList.ITotal or 0) + (iEffect.ITotal or 0)

				recap_temp.IncomingDetailsListSize = recap_temp.IncomingDetailsListSize + sizeIncrement
			end
		end
		recap_temp.IncomingDetailsList[recap_temp.IncomingDetailsListSize] = nil -- keep as nil
	end

	-- pass three
	if iMergePets then
		for i=1, recap_temp.IncomingDetailsListSize-1 do
			iList = recap_temp.IncomingDetailsList[i]
			iType = tonumber(string.sub(iList.Effect,1,1))
			if (iType == 5) or (iType == 7) then
				-- owner Subtotal or Total (now with ** ADDED PETS **) with Merge Pets on
				iList.Avg = string.format("%d",(iList.Landed>0 and Recap_Div0(iList.Total, iList.Landed) or 0))
				if (iType == 5) or ((iType == 7) and (string.find(i, RECAP_DAMAGE, 1, true))) then
					if iList.Attempted and (iList.Attempted > 0) then
						iList.MissPOverP = string.format("%.1f%%",Recap_Div1(100*iList.Missed, iList.Attempted))
					else
						iList.MissPOverP = " "
					end
				elseif (iType == 7) and (string.find(i, recap_temp.Localize.ElementHealing, 1, true)) then
					rawheal = iList.Total + iList.OverHeal
					if (rawheal > 0) then
						iList.MissPOverP = string.format("%.1f%%",Recap_Div1(100*iList.OverHeal, rawheal))
					else
						iList.MissPOverP = " "
					end
				end
				if iList.Attempted and (iList.Attempted > 0) then
					iList.CritP = string.format("%.1f%%",Recap_Div1(100*iList.Critted, iList.Attempted))
				else
					iList.CritP = "--"
				end
				if iList.ICount>0 then
					iList.Interval = string.format("%.1f",Recap_Div1(iList.ITotal, 1000*iList.ICount))
				else
					iList.Interval = "--"
				end
			end
		end
	end

	for i=1, recap_temp.IncomingDetailsListSize-1 do
		iList = recap_temp.IncomingDetailsList[i]
		iType = tonumber(string.sub(iList.Effect,1,1))
		if (iType == 1) or (iType == 2) then
			subtotal = dmgtotal
		elseif (iType == 3) or (iType == 4) then
			subtotal = healtotal
		elseif (iType == 5) or (iType == 6) then
			subtotal = subtotaldmgtotal
		elseif (iType == 7) or (iType == 8) then
			-- answer is always 100%
			subtotal = iList.Total
		end
		if subtotal>0 then
			iList.TotalP = Recap_Div0(100*iList.Total,subtotal)
		else
			iList.TotalP = 0
		end
	end

	table.sort(recap_temp.IncomingDetailsList,RecapPanel_DetailsDamageSort)

end

function RecapPanel_ConstructOtherDetails(name)

	local i, count, iList, iEffect

	-- clear the details
	recap_temp.OtherDetailsListSize = 1
	recap_temp.OtherDetailsList = {} -- breaks table.sort if we try to re-use the table by setting individual values to nil

	for i in pairs(recap.Combatant[name].OtherDetail) do
		if not recap_temp.OtherDetailsList[recap_temp.OtherDetailsListSize] then
			recap_temp.OtherDetailsList[recap_temp.OtherDetailsListSize] = {}
		end
		iList = recap_temp.OtherDetailsList[recap_temp.OtherDetailsListSize]
		iEffect = recap.Combatant[name].OtherDetail[i]
		iList.Effect = i
		iList.Hits = (iEffect.Hits or 0)
		iList.Total = (iEffect.Total or "--")
		iList.Attribute = (iEffect.Attribute or "--")
		-- two hidden columns for Interval and Duration (parallel code populates the fine details)
		count = (iEffect.ICount or 0)
		if count>0 then
			iList.Interval = string.format("%.1f",Recap_Div1((iEffect.ITotal or 0),1000*count))
		else
			iList.Interval = "--"
		end
		count = (iEffect.DCount or 0)
		if count>0 then
			iList.Duration = string.format("%.1f",Recap_Div1((iEffect.DTotal or 0),1000*count))
		else
			iList.Duration = "--"
		end

		recap_temp.OtherDetailsListSize = recap_temp.OtherDetailsListSize + 1
	end
	recap_temp.OtherDetailsList[recap_temp.OtherDetailsListSize] = nil -- keep as nil

	table.sort(recap_temp.OtherDetailsList,RecapPanel_DetailsHitsSort)

end

function RecapPanel_DetailsDamageSort(e1,e2)

	local effect1,effect2

	-- Hawksy: guard code
	if e1.Effect then
		effect1 = string.sub(e1.Effect,1,1)
	else
		effect1 = "0"
	end
	if e2.Effect then
		effect2 = string.sub(e2.Effect,1,1)
	else
		effect2 = "0"
	end
	if not e1.Total then
		e1.Total = 0
	end
	if not e2.Total then
		e2.Total = 0
	end

	if ((e1.Total>e2.Total) and (effect1==effect2)) or (effect1<effect2) then
		return true
	else
		return false
	end
end

function RecapPanel_DetailsHitsSort(e1,e2)

	local effect1,effect2

	-- Hawksy: guard code
	if e1.Effect then
		effect1 = string.sub(e1.Effect,1,1)
	else
		effect1 = "0"
	end
	if e2.Effect then
		effect2 = string.sub(e2.Effect,1,1)
	else
		effect2 = "0"
	end
	if not e1.Hits then
		e1.Hits = 0
	end
	if not e2.Hits then
		e2.Hits = 0
	end

	if ((e1.Hits>e2.Hits) and (effect1==effect2)) or (effect1<effect2) then
		return true
	else
		return false
	end
end

function RecapPanelOutgoingDetailsScrollBar_Update()

	local i, iType, index, item, r, g, b
	local _G = getfenv(0)

	if recap_temp.Loaded and recap_temp.OutgoingDetailsListSize then

		FauxScrollFrame_Update(RecapPanelOutgoingDetailsScrollBar,recap_temp.OutgoingDetailsListSize-1,7,14)

		for i=1,7 do
			index = i + FauxScrollFrame_GetOffset(RecapPanelOutgoingDetailsScrollBar)
			if index < recap_temp.OutgoingDetailsListSize then
				iType = string.sub(recap_temp.OutgoingDetailsList[index].Effect,1,1)
				if (iType=="1") or (iType=="2") then
					r,g,b = recap_temp.ColorDmgOut.r, recap_temp.ColorDmgOut.g, recap_temp.ColorDmgOut.b
				elseif (iType=="3") or (iType=="4") then
					r,g,b = recap_temp.ColorHeal.r, recap_temp.ColorHeal.g, recap_temp.ColorHeal.b
				elseif (iType=="5") or (iType=="6") then
					r,g,b = recap_temp.ColorDmgOutPale.r, recap_temp.ColorDmgOutPale.g, recap_temp.ColorDmgOutPale.b
				elseif (iType=="7") or (iType=="8") then
					if string.find(recap_temp.OutgoingDetailsList[index].Effect, recap_temp.Localize.ElementHealing, 1, true) then
						r,g,b = recap_temp.ColorHealPale.r, recap_temp.ColorHealPale.g, recap_temp.ColorHealPale.b
					else
						r,g,b = recap_temp.ColorDmgOutPale.r, recap_temp.ColorDmgOutPale.g, recap_temp.ColorDmgOutPale.b
					end
				end
				_G["RecapPanelOutgoingDetail"..i.."_Name"]:SetText(string.sub(recap_temp.OutgoingDetailsList[index].Effect,2))
				_G["RecapPanelOutgoingDetail"..i.."_Name"]:SetTextColor(r,g,b)
				_G["RecapPanelOutgoingDetail"..i.."_Total"]:SetText(recap_temp.OutgoingDetailsList[index][recap.Opt.OutgoingPanelDetail.value])
				if recap.Opt.OutgoingPanelDetail.value=="Total" then
					_G["RecapPanelOutgoingDetail"..i.."_Total"]:SetTextColor(r,g,b)
				else
					_G["RecapPanelOutgoingDetail"..i.."_Total"]:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
				end
				_G["RecapPanelOutgoingDetail"..i.."_TotalP"]:SetText(recap_temp.OutgoingDetailsList[index].TotalP.."%")
				_G["RecapPanelOutgoingDetail"..i.."_TotalP"]:SetTextColor(r,g,b)
				item = _G["RecapPanelOutgoingDetail"..i]
				item:Show()
				if recap_temp.OutgoingDetailSelected == index then
					item:LockHighlight()
				else
					item:UnlockHighlight()
				end
			else
				item = _G["RecapPanelOutgoingDetail"..i]
				item:Hide()
				item:UnlockHighlight()
			end
		end
	end

end

function RecapPanelIncomingDetailsScrollBar_Update()

	local i, iType, index, item, r, g, b
	local _G = getfenv(0)

	if recap_temp.Loaded and recap_temp.IncomingDetailsListSize then

		FauxScrollFrame_Update(RecapPanelIncomingDetailsScrollBar,recap_temp.IncomingDetailsListSize-1,7,14)

		for i=1,7 do
			index = i + FauxScrollFrame_GetOffset(RecapPanelIncomingDetailsScrollBar)
			if index < recap_temp.IncomingDetailsListSize then
				iType = string.sub(recap_temp.IncomingDetailsList[index].Effect,1,1)
				if (iType=="1") or (iType=="2") then
					r,g,b = recap_temp.ColorDmgIn.r, recap_temp.ColorDmgIn.g, recap_temp.ColorDmgIn.b
				elseif (iType=="3") or (iType=="4") then
					r,g,b = recap_temp.ColorHeal.r, recap_temp.ColorHeal.g, recap_temp.ColorHeal.b
				elseif (iType=="5") or (iType=="6") then
					r,g,b = recap_temp.ColorDmgInPale.r, recap_temp.ColorDmgInPale.g, recap_temp.ColorDmgInPale.b
				elseif (iType=="7") or (iType=="8") then
					if string.find(recap_temp.IncomingDetailsList[index].Effect, recap_temp.Localize.ElementHealing, 1, true) then
						r,g,b = recap_temp.ColorHealPale.r, recap_temp.ColorHealPale.g, recap_temp.ColorHealPale.b
					else
						r,g,b = recap_temp.ColorDmgInPale.r, recap_temp.ColorDmgInPale.g, recap_temp.ColorDmgInPale.b
					end
				end
				_G["RecapPanelIncomingDetail"..i.."_Name"]:SetText(string.sub(recap_temp.IncomingDetailsList[index].Effect,2))
				_G["RecapPanelIncomingDetail"..i.."_Name"]:SetTextColor(r,g,b)
				_G["RecapPanelIncomingDetail"..i.."_Total"]:SetText(recap_temp.IncomingDetailsList[index][recap.Opt.IncomingPanelDetail.value])
				if recap.Opt.IncomingPanelDetail.value=="Total" then
					_G["RecapPanelIncomingDetail"..i.."_Total"]:SetTextColor(r,g,b)
				else
					_G["RecapPanelIncomingDetail"..i.."_Total"]:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
				end
				_G["RecapPanelIncomingDetail"..i.."_TotalP"]:SetText(recap_temp.IncomingDetailsList[index].TotalP.."%")
				_G["RecapPanelIncomingDetail"..i.."_TotalP"]:SetTextColor(r,g,b)
				item = _G["RecapPanelIncomingDetail"..i]
				item:Show()
				if recap_temp.IncomingDetailSelected == index then
					item:LockHighlight()
				else
					item:UnlockHighlight()
				end
			else
				item = _G["RecapPanelIncomingDetail"..i]
				item:Hide()
				item:UnlockHighlight()
			end
		end
	end

end

function RecapPanelOtherDetailsScrollBar_Update()

	local i, index, item, r, g, b, iType
	local _G = getfenv(0)

	if recap_temp.Loaded and recap_temp.OtherDetailsListSize then

		FauxScrollFrame_Update(RecapPanelOtherDetailsScrollBar,recap_temp.OtherDetailsListSize-1,7,14)

		for i=1,7 do
			index = i + FauxScrollFrame_GetOffset(RecapPanelOtherDetailsScrollBar)
			if index < recap_temp.OtherDetailsListSize then
				iType = string.sub(recap_temp.OtherDetailsList[index].Effect,1,1)
				-- 1 is outgoing spells, usually non-damaging, being cast
				-- 3 is debuffs or losses (e.g. mana)
				-- 5 is buffs or gains (e.g. mana, extra attacks, happiness)
				-- 7 is debuff or buff, but we don't know which
				if (iType == "1") or (iType == "2") then
					r,g,b = recap_temp.ColorDmgOut.r, recap_temp.ColorDmgOut.g, recap_temp.ColorDmgOut.b
				elseif  (iType == "3") or (iType == "4") then
					r,g,b = recap_temp.ColorDmgIn.r, recap_temp.ColorDmgIn.g, recap_temp.ColorDmgIn.b
				elseif (iType == "5") or (iType == "6") then
					r,g,b = recap_temp.ColorHeal.r, recap_temp.ColorHeal.g, recap_temp.ColorHeal.b
				elseif (iType == "7") or (iType == "8") then
					r,g,b = recap_temp.ColorWhite.r, recap_temp.ColorWhite.g, recap_temp.ColorWhite.b
				end
				_G["RecapPanelOtherDetail"..i.."_Name"]:SetText(string.sub(recap_temp.OtherDetailsList[index].Effect,2))
				_G["RecapPanelOtherDetail"..i.."_Name"]:SetTextColor(r,g,b)
				_G["RecapPanelOtherDetail"..i.."_Hits"]:SetText(recap_temp.OtherDetailsList[index].Hits)
				_G["RecapPanelOtherDetail"..i.."_Hits"]:SetTextColor(r,g,b)
				_G["RecapPanelOtherDetail"..i.."_Total"]:SetText(recap_temp.OtherDetailsList[index].Total)
				_G["RecapPanelOtherDetail"..i.."_Total"]:SetTextColor(r,g,b)
				_G["RecapPanelOtherDetail"..i.."_Attribute"]:SetText(recap_temp.OtherDetailsList[index].Attribute)
				_G["RecapPanelOtherDetail"..i.."_Attribute"]:SetTextColor(r,g,b)
				item = _G["RecapPanelOtherDetail"..i]
				item:Show()
				if recap_temp.OtherDetailSelected == index then
					item:LockHighlight()
				else
					item:UnlockHighlight()
				end
			else
				item = _G["RecapPanelOtherDetail"..i]
				item:Hide()
				item:UnlockHighlight()
			end
		end
	end

end

function RecapPanel_PopulateOutgoingDetails(arg1,sel)

	local effect, i, totalmiss, iCombatant, iEffect, iType

	-- Hawksy: guard code
	if not arg1 then
		return
	end
	if (not sel) or (sel < 0) then
		return
	end
	if not recap_temp.OutgoingDetailsList then
		return
	end
	if not recap.Combatant then
		return
	end
	if not recap.Combatant[arg1] then
		return
	end

	iCombatant = recap.Combatant[arg1]

	if not iCombatant.OutgoingDetail then
		return
	end

	RecapPanelOutgoingEffectText:SetText("--")
	RecapPanelOutgoingElementText:SetText("--")
	RecapPanelOutgoingDamageText:SetText("--")
	RecapPanelOutgoingGlancesText:SetText("--")
	RecapPanelOutgoingHitsText:SetText("--")
	RecapPanelOutgoingCritsText:SetText("--")
	RecapPanelOutgoingCrushesText:SetText("--")
	RecapPanelOutgoingTicksText:SetText("--")
	RecapPanelOutgoingGlancesMinText:SetText("--")
	RecapPanelOutgoingHitsMinText:SetText("--")
	RecapPanelOutgoingCritsMinText:SetText("--")
	RecapPanelOutgoingCrushMinText:SetText("--")
	RecapPanelOutgoingTicksMinText:SetText("--")
	RecapPanelOutgoingGlancesAvgText:SetText("--")
	RecapPanelOutgoingHitsAvgText:SetText("--")
	RecapPanelOutgoingCritsAvgText:SetText("--")
	RecapPanelOutgoingCrushAvgText:SetText("--")
	RecapPanelOutgoingTicksAvgText:SetText("--")
	RecapPanelOutgoingGlancesMaxText:SetText("--")
	RecapPanelOutgoingHitsMaxText:SetText("--")
	RecapPanelOutgoingCritsMaxText:SetText("--")
	RecapPanelOutgoingCrushMaxText:SetText("--")
	RecapPanelOutgoingTicksMaxText:SetText("--")
	RecapPanelOutgoingMissMissedText:SetText("--")
	RecapPanelOutgoingMissDodgedText:SetText("--")
	RecapPanelOutgoingMissParriedText:SetText("--")
	RecapPanelOutgoingMissBlockedText:SetText("--")
	RecapPanelOutgoingMissAbsorbedText:SetText("--")
	RecapPanelOutgoingMissEvadedText:SetText("--")
	RecapPanelOutgoingMissDeflectedText:SetText("--")
	RecapPanelOutgoingMissResistedText:SetText("--")
	RecapPanelOutgoingMissReflectedText:SetText("--")
	RecapPanelOutgoingMissImmuneText:SetText("--")
	RecapPanelOutgoingMissPText:SetText("--")
	RecapPanelOutgoingGlancePText:SetText("--")
	RecapPanelOutgoingCritPText:SetText("--")
	RecapPanelOutgoingCrushPText:SetText("--")
	RecapPanelOutgoingPartAbsorbsText:SetText("--")
	RecapPanelOutgoingPartAbsorbsTotalText:SetText("--")
	RecapPanelOutgoingPartAbsorbsAvgText:SetText("--")
	RecapPanelOutgoingPartBlocksText:SetText("--")
	RecapPanelOutgoingPartBlocksTotalText:SetText("--")
	RecapPanelOutgoingPartBlocksAvgText:SetText("--")
	RecapPanelOutgoingPartResists25Text:SetText("--")
	RecapPanelOutgoingPartResists25TotalText:SetText("--")
	RecapPanelOutgoingPartResists25AvgText:SetText("--")
	RecapPanelOutgoingPartResists50Text:SetText("--")
	RecapPanelOutgoingPartResists50TotalText:SetText("--")
	RecapPanelOutgoingPartResists50AvgText:SetText("--")
	RecapPanelOutgoingPartResists75Text:SetText("--")
	RecapPanelOutgoingPartResists75TotalText:SetText("--")
	RecapPanelOutgoingPartResists75AvgText:SetText("--")
	RecapPanelOutgoingPartVulnerableText:SetText("--")
	RecapPanelOutgoingPartVulnerableTotalText:SetText("--")
	RecapPanelOutgoingPartVulnerableAvgText:SetText("--")
	RecapPanelOutgoingIntervalCountText:SetText("--")
	RecapPanelOutgoingIntervalAvgText:SetText("--")

	if sel == 0 then
		return
	end

	if not recap_temp.OutgoingDetailsList[sel] then
		return
	end

	effect = recap_temp.OutgoingDetailsList[sel].Effect

	-- Hawksy: guard code
	if not effect then
		return
	end

	iType = string.sub(effect,1,1)

	if (iType == "1") or (iType == "2") or (iType == "5") or (iType == "6") or (((iType == "7") or (iType == "8")) and (string.find(effect, RECAP_DAMAGE, 1, true))) then
		RecapPanelOutgoingDamageLabel:SetText(RECAP_DAMAGE)
		RecapPanelOutgoingMissPLabel:SetText(RECAP_MISSES)
	else
		RecapPanelOutgoingDamageLabel:SetText(RECAP_HEAL)
		RecapPanelOutgoingMissPLabel:SetText(RECAP_OVERHEALING)
	end

	-- Hawksy: guard code
	if not iCombatant.OutgoingDetail[effect] then
		return
	end

	iEffect = iCombatant.OutgoingDetail[effect]

	-- effect (repeated so we know which effect we are viewing)
	RecapPanelOutgoingEffectText:SetText(string.sub(effect,2))
	if (iType == "1") or (iType == "2") then
		RecapPanelOutgoingEffectText:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
	elseif (iType=="3") or (iType=="4") then
		RecapPanelOutgoingEffectText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
	elseif (iType=="5") or (iType=="6") then
		RecapPanelOutgoingEffectText:SetTextColor(recap_temp.ColorDmgOutPale.r,recap_temp.ColorDmgOutPale.g,recap_temp.ColorDmgOutPale.b)
	elseif (iType=="7") or (iType=="8") then
		if string.find(effect, recap_temp.Localize.ElementHealing, 1, true) then
			RecapPanelOutgoingEffectText:SetTextColor(recap_temp.ColorHealPale.r,recap_temp.ColorHealPale.g,recap_temp.ColorHealPale.b)
		else
			RecapPanelOutgoingEffectText:SetTextColor(recap_temp.ColorDmgOutPale.r,recap_temp.ColorDmgOutPale.g,recap_temp.ColorDmgOutPale.b)
		end
	end

	-- element (hide question mark values)
	if iEffect.Element and (iEffect.Element ~= "?") then
		RecapPanelOutgoingElementText:SetText(iEffect.Element)
	else
		RecapPanelOutgoingElementText:SetText("")
	end

	-- damage
	RecapPanelOutgoingDamageText:SetText((iEffect.GlancesDmg or 0)+(iEffect.HitsDmg or 0)+(iEffect.CritsDmg or 0)+(iEffect.CrushDmg or 0)+(iEffect.TicksDmg or 0))

	-- glances
	i = (iEffect.Glances or 0)
	if i>0 then
		RecapPanelOutgoingGlancesText:SetText(i)
		RecapPanelOutgoingGlancesMinText:SetText(iEffect.GlancesMin or "--")
		RecapPanelOutgoingGlancesAvgText:SetText(string.format("%d",Recap_Div0((iEffect.GlancesDmg or 0),i)))
		RecapPanelOutgoingGlancesMaxText:SetText(iEffect.GlancesMax or 0)
	elseif iEffect.CritsEvents then
		RecapPanelOutgoingGlancesText:SetText(0)
	end

	-- hits
	i = (iEffect.Hits or 0)
	if i>0 then
		RecapPanelOutgoingHitsText:SetText(i)
		RecapPanelOutgoingHitsMinText:SetText(iEffect.HitsMin or "--")
		RecapPanelOutgoingHitsAvgText:SetText(string.format("%d",Recap_Div0((iEffect.HitsDmg or 0),i)))
		RecapPanelOutgoingHitsMaxText:SetText(iEffect.HitsMax or 0)
	elseif iEffect.CritsEvents then
		RecapPanelOutgoingHitsText:SetText(0)
	end

	-- crits
	i = (iEffect.Crits or 0)
	if i>0 then
		RecapPanelOutgoingCritsText:SetText(i)
		RecapPanelOutgoingCritsMinText:SetText(iEffect.CritsMin or "--")
		RecapPanelOutgoingCritsAvgText:SetText(string.format("%d",Recap_Div0((iEffect.CritsDmg or 0),i)))
		RecapPanelOutgoingCritsMaxText:SetText(iEffect.CritsMax or 0)
	elseif iEffect.CritsEvents then
		RecapPanelOutgoingCritsText:SetText(0)
	end

	-- crushes
	i = (iEffect.Crushes or 0)
	if i>0 then
		RecapPanelOutgoingCrushesText:SetText(i)
		RecapPanelOutgoingCrushMinText:SetText(iEffect.CrushMin or "--")
		RecapPanelOutgoingCrushAvgText:SetText(string.format("%d",Recap_Div0((iEffect.CrushDmg or 0),i)))
		RecapPanelOutgoingCrushMaxText:SetText(iEffect.CrushMax or 0)
	elseif iEffect.CritsEvents then
		RecapPanelOutgoingCrushesText:SetText(0)
	end

	-- ticks
	i = (iEffect.Ticks or 0)
	if i>0 then
		RecapPanelOutgoingTicksText:SetText(i)
		RecapPanelOutgoingTicksMinText:SetText(iEffect.TicksMin or "--")
		RecapPanelOutgoingTicksAvgText:SetText(string.format("%d",Recap_Div0((iEffect.TicksDmg or 0),i)))
		RecapPanelOutgoingTicksMaxText:SetText(iEffect.TicksMax or 0)
	elseif iEffect.TicksMax then
		RecapPanelOutgoingTicksText:SetText(0)
	end

	-- miss or overheal % (and crit % and crush % and glance % calcs)
	if (iType == "1") or (iType == "2") or (iType == "5") or (iType == "6") or (((iType == "7") or (iType == "8")) and (string.find(effect, RECAP_DAMAGE, 1, true))) then
		-- miss
		miss = (iEffect.Missed or 0) + (iEffect.Dodged or 0) + (iEffect.Parried or 0) + (iEffect.Blocked or 0) +
				(iEffect.Deflected or 0) + (iEffect.Resisted or 0) + (iEffect.Reflected or 0) + (iEffect.Absorbed or 0) +
				(iEffect.Immune or 0) + (iEffect.Evaded or 0)
		i = miss + (iEffect.CritsEvents or 0)
		if i>0 then
			RecapPanelOutgoingGlancePText:SetText(((i==iEffect.Glances) and "100%") or string.format("%.1f%%",Recap_Div1(100*(iEffect.Glances or 0),i)))
			RecapPanelOutgoingCritPText:SetText(((i==iEffect.Crits) and "100%") or string.format("%.1f%%",Recap_Div1(100*(iEffect.Crits or 0),i)))
			RecapPanelOutgoingCrushPText:SetText(((i==iEffect.Crushes) and "100%") or string.format("%.1f%%",Recap_Div1(100*(iEffect.Crushes or 0),i)))
			if (miss==i) then
				RecapPanelOutgoingMissPText:SetText(i.." (100%)")
			else
				RecapPanelOutgoingMissPText:SetText(string.format("%d (%.1f%%)",miss,Recap_Div1(100*miss,i)))
			end
		end
	else
		-- crits for heals
		i = (iEffect.CritsEvents or 0)
		if i>0 then
			RecapPanelOutgoingCritPText:SetText(((i==iEffect.Crits) and "100%") or string.format("%.1f%%",Recap_Div1(100*(iEffect.Crits or 0),i)))
		end
		-- Hawksy: change of calculation since effect numbers are full heals, not actual heals
		local heal, overheal
		heal = (iEffect.GlancesDmg or 0)+(iEffect.HitsDmg or 0)+(iEffect.CritsDmg or 0)+(iEffect.CrushDmg or 0)+(iEffect.TicksDmg or 0)
		overheal = (iEffect.Missed or 0)
		if heal>0 then
			RecapPanelOutgoingMissPText:SetText(string.format("%d (%d%%)",iEffect.Missed or 0,Recap_Div0(100*overheal,heal)))
		end
	end

	if (iType == "1") or (iType == "2") or (iType == "5") or (iType == "6") or (((iType == "7") or (iType == "8")) and (string.find(effect, RECAP_DAMAGE, 1, true))) then
		-- non-healing misses
		totalmiss = (iEffect.Missed or 0) + (iEffect.Dodged or 0) + (iEffect.Parried or 0) + (iEffect.Blocked or 0) +
					(iEffect.Deflected or 0) + (iEffect.Resisted or 0) + (iEffect.Reflected or 0) + (iEffect.Absorbed or 0) +
					(iEffect.Immune or 0) + (iEffect.Evaded or 0)
		i = totalmiss + (iEffect.Glances or 0) + (iEffect.Hits or 0) + (iEffect.Crits or 0) + (iEffect.Crushes or 0)
		if i>0 then
			if iEffect.Missed and (iEffect.Missed > 0) then
				RecapPanelOutgoingMissMissedText:SetText(string.format("%d (%.1f%%)", iEffect.Missed, Recap_Div1(100*iEffect.Missed,i)))
			end
			if iEffect.Dodged and (iEffect.Dodged > 0) then
				RecapPanelOutgoingMissDodgedText:SetText(string.format("%d (%.1f%%)", iEffect.Dodged, Recap_Div1(100*iEffect.Dodged,i)))
			end
			if iEffect.Parried and (iEffect.Parried > 0) then
				RecapPanelOutgoingMissParriedText:SetText(string.format("%d (%.1f%%)", iEffect.Parried, Recap_Div1(100*iEffect.Parried,i)))
			end
			if iEffect.Blocked and (iEffect.Blocked > 0) then
				RecapPanelOutgoingMissBlockedText:SetText(string.format("%d (%.1f%%)", iEffect.Blocked, Recap_Div1(100*iEffect.Blocked,i)))
			end
			if iEffect.Deflected and (iEffect.Deflected > 0) then
				RecapPanelOutgoingMissDeflectedText:SetText(string.format("%d (%.1f%%)", iEffect.Deflected, Recap_Div1(100*iEffect.Deflected,i)))
			end
			if iEffect.Resisted and (iEffect.Resisted > 0) then
				RecapPanelOutgoingMissResistedText:SetText(string.format("%d (%.1f%%)", iEffect.Resisted, Recap_Div1(100*iEffect.Resisted,i)))
			end
			if iEffect.Reflected and (iEffect.Reflected > 0) then
				RecapPanelOutgoingMissReflectedText:SetText(string.format("%d (%.1f%%)", iEffect.Reflected, Recap_Div1(100*iEffect.Reflected,i)))
			end
			if iEffect.Absorbed and (iEffect.Absorbed > 0) then
				RecapPanelOutgoingMissAbsorbedText:SetText(string.format("%d (%.1f%%)", iEffect.Absorbed, Recap_Div1(100*iEffect.Absorbed,i)))
			end
			if iEffect.Immune and (iEffect.Immune > 0) then
				RecapPanelOutgoingMissImmuneText:SetText(string.format("%d (%.1f%%)", iEffect.Immune, Recap_Div1(100*iEffect.Immune,i)))
			end
			if iEffect.Evaded and (iEffect.Evaded > 0) then
				RecapPanelOutgoingMissEvadedText:SetText(string.format("%d (%.1f%%)", iEffect.Evaded, Recap_Div1(100*iEffect.Evaded,i)))
			end
		end
	end

	-- partials
	i = (iEffect.PAbsorbs or 0)
	if i>0 then
		RecapPanelOutgoingPartAbsorbsText:SetText(i)
		RecapPanelOutgoingPartAbsorbsTotalText:SetText(iEffect.PAbsorbsDmg or 0)
		RecapPanelOutgoingPartAbsorbsAvgText:SetText(string.format("%d",Recap_Div0((iEffect.PAbsorbsDmg or 0),i)))
	end
	i = (iEffect.PBlocks or 0)
	if i>0 then
		RecapPanelOutgoingPartBlocksText:SetText(i)
		RecapPanelOutgoingPartBlocksTotalText:SetText(iEffect.PBlocksDmg or 0)
		RecapPanelOutgoingPartBlocksAvgText:SetText(string.format("%d",Recap_Div0((iEffect.PBlocksDmg or 0),i)))
	end
	i = (iEffect.PResists25 or 0)
	if i>0 then
		RecapPanelOutgoingPartResists25Text:SetText(i)
		RecapPanelOutgoingPartResists25TotalText:SetText(iEffect.PResists25Dmg or 0)
		RecapPanelOutgoingPartResists25AvgText:SetText(string.format("%d",Recap_Div0((iEffect.PResists25Dmg or 0),i)))
	end
	i = (iEffect.PResists50 or 0)
	if i>0 then
		RecapPanelOutgoingPartResists50Text:SetText(i)
		RecapPanelOutgoingPartResists50TotalText:SetText(iEffect.PResists50Dmg or 0)
		RecapPanelOutgoingPartResists50AvgText:SetText(string.format("%d",Recap_Div0((iEffect.PResists50Dmg or 0),i)))
	end
	i = (iEffect.PResists75 or 0)
	if i>0 then
		RecapPanelOutgoingPartResists75Text:SetText(i)
		RecapPanelOutgoingPartResists75TotalText:SetText(iEffect.PResists75Dmg or 0)
		RecapPanelOutgoingPartResists75AvgText:SetText(string.format("%d",Recap_Div0((iEffect.PResists75Dmg or 0),i)))
	end
	i = (iEffect.PPVulnerable or 0)
	if i>0 then
		RecapPanelOutgoingPartPVulnerableText:SetText(i)
		RecapPanelOutgoingPartVulnerableTotalText:SetText(iEffect.PVulnerableDmg or 0)
		RecapPanelOutgoingPartPVulnerableAvgText:SetText(string.format("%d",Recap_Div0((iEffect.PPVulnerableDmg or 0),i)))
	end

	-- interval count and average (parallel code populates the list)
	count = (iEffect.ICount or 0)
	if count>0 then
		RecapPanelOutgoingIntervalCountText:SetText(count)
		RecapPanelOutgoingIntervalAvgText:SetText(string.format("%.1f",Recap_Div1((iEffect.ITotal or 0),1000*count)))
	end

end

function RecapPanel_PopulateIncomingDetails(arg1,sel)

	local effect, i, totalmiss, iCombatant, iEffect, iType

	-- Hawksy: guard code
	if not arg1 then
		return
	end
	if (not sel) or (sel < 0) then
		return
	end
	if not recap_temp.IncomingDetailsList then
		return
	end
	if not recap.Combatant then
		return
	end
	if not recap.Combatant[arg1] then
		return
	end

	iCombatant = recap.Combatant[arg1]

	if not iCombatant.IncomingDetail then
		return
	end

	RecapPanelIncomingEffectText:SetText("--")
	RecapPanelIncomingElementText:SetText("--")
	RecapPanelIncomingDamageText:SetText("--")
	RecapPanelIncomingGlancesText:SetText("--")
	RecapPanelIncomingHitsText:SetText("--")
	RecapPanelIncomingCritsText:SetText("--")
	RecapPanelIncomingCrushesText:SetText("--")
	RecapPanelIncomingTicksText:SetText("--")
	RecapPanelIncomingGlancesMinText:SetText("--")
	RecapPanelIncomingHitsMinText:SetText("--")
	RecapPanelIncomingCritsMinText:SetText("--")
	RecapPanelIncomingCrushMinText:SetText("--")
	RecapPanelIncomingTicksMinText:SetText("--")
	RecapPanelIncomingGlancesAvgText:SetText("--")
	RecapPanelIncomingHitsAvgText:SetText("--")
	RecapPanelIncomingCritsAvgText:SetText("--")
	RecapPanelIncomingCrushAvgText:SetText("--")
	RecapPanelIncomingTicksAvgText:SetText("--")
	RecapPanelIncomingGlancesMaxText:SetText("--")
	RecapPanelIncomingHitsMaxText:SetText("--")
	RecapPanelIncomingCritsMaxText:SetText("--")
	RecapPanelIncomingCrushMaxText:SetText("--")
	RecapPanelIncomingTicksMaxText:SetText("--")
	RecapPanelIncomingMissMissedText:SetText("--")
	RecapPanelIncomingMissDodgedText:SetText("--")
	RecapPanelIncomingMissParriedText:SetText("--")
	RecapPanelIncomingMissBlockedText:SetText("--")
	RecapPanelIncomingMissAbsorbedText:SetText("--")
	RecapPanelIncomingMissEvadedText:SetText("--")
	RecapPanelIncomingMissDeflectedText:SetText("--")
	RecapPanelIncomingMissResistedText:SetText("--")
	RecapPanelIncomingMissReflectedText:SetText("--")
	RecapPanelIncomingMissImmuneText:SetText("--")
	RecapPanelIncomingMissPText:SetText("--")
	RecapPanelIncomingGlancePText:SetText("--")
	RecapPanelIncomingCritPText:SetText("--")
	RecapPanelIncomingCrushPText:SetText("--")
	RecapPanelIncomingPartAbsorbsText:SetText("--")
	RecapPanelIncomingPartAbsorbsTotalText:SetText("--")
	RecapPanelIncomingPartAbsorbsAvgText:SetText("--")
	RecapPanelIncomingPartBlocksText:SetText("--")
	RecapPanelIncomingPartBlocksTotalText:SetText("--")
	RecapPanelIncomingPartBlocksAvgText:SetText("--")
	RecapPanelIncomingPartResists25Text:SetText("--")
	RecapPanelIncomingPartResists25TotalText:SetText("--")
	RecapPanelIncomingPartResists25AvgText:SetText("--")
	RecapPanelIncomingPartResists50Text:SetText("--")
	RecapPanelIncomingPartResists50TotalText:SetText("--")
	RecapPanelIncomingPartResists50AvgText:SetText("--")
	RecapPanelIncomingPartResists75Text:SetText("--")
	RecapPanelIncomingPartResists75TotalText:SetText("--")
	RecapPanelIncomingPartResists75AvgText:SetText("--")
	RecapPanelIncomingPartVulnerableText:SetText("--")
	RecapPanelIncomingPartVulnerableTotalText:SetText("--")
	RecapPanelIncomingPartVulnerableAvgText:SetText("--")
	RecapPanelIncomingIntervalCountText:SetText("--")
	RecapPanelIncomingIntervalAvgText:SetText("--")

	if sel == 0 then
		return
	end

	if not recap_temp.IncomingDetailsList[sel] then
		return
	end

	effect = recap_temp.IncomingDetailsList[sel].Effect

	-- Hawksy: guard code
	if not effect then
		return
	end

	iType = string.sub(effect,1,1)

	if (iType == "1") or (iType == "2") or (iType == "5") or (iType == "6") or (((iType == "7") or (iType == "8")) and (string.find(effect, RECAP_DAMAGE, 1, true))) then
		RecapPanelIncomingDamageLabel:SetText(RECAP_DAMAGE)
		RecapPanelIncomingMissPLabel:SetText(RECAP_MISSES)
	else
		RecapPanelIncomingDamageLabel:SetText(RECAP_HEAL)
		RecapPanelIncomingMissPLabel:SetText(RECAP_OVERHEALING)
	end

	-- Hawksy: guard code
	if not iCombatant.IncomingDetail[effect] then
		return
	end

	iEffect = iCombatant.IncomingDetail[effect]

	-- effect (repeated so we know which effect we are viewing)
	RecapPanelIncomingEffectText:SetText(string.sub(effect,2))
	if (iType == "1") or (iType == "2") then
		RecapPanelIncomingEffectText:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
	elseif (iType=="3") or (iType=="4") then
		RecapPanelIncomingEffectText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
	elseif (iType=="5") or (iType=="6") then
		RecapPanelIncomingEffectText:SetTextColor(recap_temp.ColorDmgInPale.r,recap_temp.ColorDmgInPale.g,recap_temp.ColorDmgInPale.b)
	elseif (iType=="7") or (iType=="8") then
		if string.find(effect, recap_temp.Localize.ElementHealing, 1, true) then
			RecapPanelIncomingEffectText:SetTextColor(recap_temp.ColorHealPale.r,recap_temp.ColorHealPale.g,recap_temp.ColorHealPale.b)
		else
			RecapPanelIncomingEffectText:SetTextColor(recap_temp.ColorDmgInPale.r,recap_temp.ColorDmgInPale.g,recap_temp.ColorDmgInPale.b)
		end
	end

	-- element (hide question mark values)
	if iEffect.Element and (iEffect.Element ~= "?") then
		RecapPanelIncomingElementText:SetText(iEffect.Element)
	else
		RecapPanelIncomingElementText:SetText("")
	end

	-- damage
	RecapPanelIncomingDamageText:SetText((iEffect.GlancesDmg or 0)+(iEffect.HitsDmg or 0)+(iEffect.CritsDmg or 0)+(iEffect.CrushDmg or 0)+(iEffect.TicksDmg or 0))

	-- glances
	i = (iEffect.Glances or 0)
	if i>0 then
		RecapPanelIncomingGlancesText:SetText(i)
		RecapPanelIncomingGlancesMinText:SetText(iEffect.GlancesMin or "--")
		RecapPanelIncomingGlancesAvgText:SetText(string.format("%d",Recap_Div0((iEffect.GlancesDmg or 0),i)))
		RecapPanelIncomingGlancesMaxText:SetText(iEffect.GlancesMax or 0)
	elseif iEffect.CritsEvents then
		RecapPanelIncomingGlancesText:SetText(0)
	end

	-- hits
	i = (iEffect.Hits or 0)
	if i>0 then
		RecapPanelIncomingHitsText:SetText(i)
		RecapPanelIncomingHitsMinText:SetText(iEffect.HitsMin or "--")
		RecapPanelIncomingHitsAvgText:SetText(string.format("%d",Recap_Div0((iEffect.HitsDmg or 0),i)))
		RecapPanelIncomingHitsMaxText:SetText(iEffect.HitsMax or 0)
	elseif iEffect.CritsEvents then
		RecapPanelIncomingHitsText:SetText(0)
	end

	-- crits
	i = (iEffect.Crits or 0)
	if i>0 then
		RecapPanelIncomingCritsText:SetText(i)
		RecapPanelIncomingCritsMinText:SetText(iEffect.CritsMin or "--")
		RecapPanelIncomingCritsAvgText:SetText(string.format("%d",Recap_Div0((iEffect.CritsDmg or 0),i)))
		RecapPanelIncomingCritsMaxText:SetText(iEffect.CritsMax or 0)
	elseif iEffect.CritsEvents then
		RecapPanelIncomingCritsText:SetText(0)
	end

	-- crushes
	i = (iEffect.Crushes or 0)
	if i>0 then
		RecapPanelIncomingCrushesText:SetText(i)
		RecapPanelIncomingCrushMinText:SetText(iEffect.CrushMin or "--")
		RecapPanelIncomingCrushAvgText:SetText(string.format("%d",Recap_Div0((iEffect.CrushDmg or 0),i)))
		RecapPanelIncomingCrushMaxText:SetText(iEffect.CrushMax or 0)
	elseif iEffect.CritsEvents then
		RecapPanelIncomingCrushesText:SetText(0)
	end

	-- ticks
	i = (iEffect.Ticks or 0)
	if i>0 then
		RecapPanelIncomingTicksText:SetText(i)
		RecapPanelIncomingTicksMinText:SetText(iEffect.TicksMin or "--")
		RecapPanelIncomingTicksAvgText:SetText(string.format("%d",Recap_Div0((iEffect.TicksDmg or 0),i)))
		RecapPanelIncomingTicksMaxText:SetText(iEffect.TicksMax or 0)
	elseif iEffect.TicksMax then
		RecapPanelIncomingTicksText:SetText(0)
	end

	-- miss or overheal % (and crit % and crush % and glance % calcs)
	if (iType == "1") or (iType == "2") or (iType == "5") or (iType == "6") or (((iType == "7") or (iType == "8")) and (string.find(effect, RECAP_DAMAGE, 1, true))) then
		-- miss
		miss = (iEffect.Missed or 0) + (iEffect.Dodged or 0) + (iEffect.Parried or 0) + (iEffect.Blocked or 0) +
				(iEffect.Deflected or 0) + (iEffect.Resisted or 0) + (iEffect.Reflected or 0) + (iEffect.Absorbed or 0) +
				(iEffect.Immune or 0) + (iEffect.Evaded or 0)
		i = miss + (iEffect.CritsEvents or 0)
		if i>0 then
			RecapPanelIncomingGlancePText:SetText(((i==iEffect.Glances) and "100%") or string.format("%.1f%%",Recap_Div1(100*(iEffect.Glances or 0),i)))
			RecapPanelIncomingCritPText:SetText(((i==iEffect.Crits) and "100%") or string.format("%.1f%%",Recap_Div1(100*(iEffect.Crits or 0),i)))
			RecapPanelIncomingCrushPText:SetText(((i==iEffect.Crushes) and "100%") or string.format("%.1f%%",Recap_Div1(100*(iEffect.Crushes or 0),i)))
			if (miss==i) then
				RecapPanelIncomingMissPText:SetText(i.." (100%)")
			else
				RecapPanelIncomingMissPText:SetText(string.format("%d (%.1f%%)",miss,Recap_Div1(100*miss,i)))
			end
		end
	else
		-- crits for heals
		i = (iEffect.CritsEvents or 0)
		if i>0 then
			RecapPanelIncomingCritPText:SetText(((i==iEffect.Crits) and "100%") or string.format("%.1f%%",Recap_Div1(100*(iEffect.Crits or 0),i)))
		end
		-- Hawksy: change of calculation since effect numbers are full heals, not actual heals
		local heal, overheal
		heal = (iEffect.GlancesDmg or 0)+(iEffect.HitsDmg or 0)+(iEffect.CritsDmg or 0)+(iEffect.CrushDmg or 0)+(iEffect.TicksDmg or 0)
		overheal = (iEffect.Missed or 0)
		if heal>0 then
			RecapPanelIncomingMissPText:SetText(string.format("%d (%d%%)",iEffect.Missed or 0,Recap_Div0(100*overheal,heal)))
		end
	end

	if (iType == "1") or (iType == "2") or (iType == "5") or (iType == "6") or (((iType == "7") or (iType == "8")) and (string.find(effect, RECAP_DAMAGE, 1, true))) then
		-- non-healing misses
		totalmiss = (iEffect.Missed or 0) + (iEffect.Dodged or 0) + (iEffect.Parried or 0) + (iEffect.Blocked or 0) +
					(iEffect.Deflected or 0) + (iEffect.Resisted or 0) + (iEffect.Reflected or 0) + (iEffect.Absorbed or 0) +
					(iEffect.Immune or 0) + (iEffect.Evaded or 0)
		i = totalmiss + (iEffect.Glances or 0) + (iEffect.Hits or 0) + (iEffect.Crits or 0) + (iEffect.Crushes or 0)
		if i>0 then
			if iEffect.Missed and (iEffect.Missed > 0) then
				RecapPanelIncomingMissMissedText:SetText(string.format("%d (%.1f%%)", iEffect.Missed, Recap_Div1(100*iEffect.Missed,i)))
			end
			if iEffect.Dodged and (iEffect.Dodged > 0) then
				RecapPanelIncomingMissDodgedText:SetText(string.format("%d (%.1f%%)", iEffect.Dodged, Recap_Div1(100*iEffect.Dodged,i)))
			end
			if iEffect.Parried and (iEffect.Parried > 0) then
				RecapPanelIncomingMissParriedText:SetText(string.format("%d (%.1f%%)", iEffect.Parried, Recap_Div1(100*iEffect.Parried,i)))
			end
			if iEffect.Blocked and (iEffect.Blocked > 0) then
				RecapPanelIncomingMissBlockedText:SetText(string.format("%d (%.1f%%)", iEffect.Blocked, Recap_Div1(100*iEffect.Blocked,i)))
			end
			if iEffect.Deflected and (iEffect.Deflected > 0) then
				RecapPanelIncomingMissDeflectedText:SetText(string.format("%d (%.1f%%)", iEffect.Deflected, Recap_Div1(100*iEffect.Deflected,i)))
			end
			if iEffect.Resisted and (iEffect.Resisted > 0) then
				RecapPanelIncomingMissResistedText:SetText(string.format("%d (%.1f%%)", iEffect.Resisted, Recap_Div1(100*iEffect.Resisted,i)))
			end
			if iEffect.Reflected and (iEffect.Reflected > 0) then
				RecapPanelIncomingMissReflectedText:SetText(string.format("%d (%.1f%%)", iEffect.Reflected, Recap_Div1(100*iEffect.Reflected,i)))
			end
			if iEffect.Absorbed and (iEffect.Absorbed > 0) then
				RecapPanelIncomingMissAbsorbedText:SetText(string.format("%d (%.1f%%)", iEffect.Absorbed, Recap_Div1(100*iEffect.Absorbed,i)))
			end
			if iEffect.Immune and (iEffect.Immune > 0) then
				RecapPanelIncomingMissImmuneText:SetText(string.format("%d (%.1f%%)", iEffect.Immune, Recap_Div1(100*iEffect.Immune,i)))
			end
			if iEffect.Evaded and (iEffect.Evaded > 0) then
				RecapPanelIncomingMissEvadedText:SetText(string.format("%d (%.1f%%)", iEffect.Evaded, Recap_Div1(100*iEffect.Evaded,i)))
			end
		end
	end

	-- partials
	i = (iEffect.PAbsorbs or 0)
	if i>0 then
		RecapPanelIncomingPartAbsorbsText:SetText(i)
		RecapPanelIncomingPartAbsorbsTotalText:SetText(iEffect.PAbsorbsDmg or 0)
		RecapPanelIncomingPartAbsorbsAvgText:SetText(string.format("%d",Recap_Div0((iEffect.PAbsorbsDmg or 0),i)))
	end
	i = (iEffect.PBlocks or 0)
	if i>0 then
		RecapPanelIncomingPartBlocksText:SetText(i)
		RecapPanelIncomingPartBlocksTotalText:SetText(iEffect.PBlocksDmg or 0)
		RecapPanelIncomingPartBlocksAvgText:SetText(string.format("%d",Recap_Div0((iEffect.PBlocksDmg or 0),i)))
	end
	i = (iEffect.PResists25 or 0)
	if i>0 then
		RecapPanelIncomingPartResists25Text:SetText(i)
		RecapPanelIncomingPartResists25TotalText:SetText(iEffect.PResists25Dmg or 0)
		RecapPanelIncomingPartResists25AvgText:SetText(string.format("%d",Recap_Div0((iEffect.PResists25Dmg or 0),i)))
	end
	i = (iEffect.PResists50 or 0)
	if i>0 then
		RecapPanelIncomingPartResists50Text:SetText(i)
		RecapPanelIncomingPartResists50TotalText:SetText(iEffect.PResists50Dmg or 0)
		RecapPanelIncomingPartResists50AvgText:SetText(string.format("%d",Recap_Div0((iEffect.PResists50Dmg or 0),i)))
	end
	i = (iEffect.PResists75 or 0)
	if i>0 then
		RecapPanelIncomingPartResists75Text:SetText(i)
		RecapPanelIncomingPartResists75TotalText:SetText(iEffect.PResists75Dmg or 0)
		RecapPanelIncomingPartResists75AvgText:SetText(string.format("%d",Recap_Div0((iEffect.PResists75Dmg or 0),i)))
	end
	i = (iEffect.PPVulnerable or 0)
	if i>0 then
		RecapPanelIncomingPartPVulnerableText:SetText(i)
		RecapPanelIncomingPartVulnerableTotalText:SetText(iEffect.PVulnerableDmg or 0)
		RecapPanelIncomingPartPVulnerableAvgText:SetText(string.format("%d",Recap_Div0((iEffect.PPVulnerableDmg or 0),i)))
	end

	-- interval count and average (parallel code populates the list)
	count = (iEffect.ICount or 0)
	if count>0 then
		RecapPanelIncomingIntervalCountText:SetText(count)
		RecapPanelIncomingIntervalAvgText:SetText(string.format("%.1f",Recap_Div1((iEffect.ITotal or 0),1000*count)))
	end

end

function RecapPanel_PopulateOtherDetails(arg1,sel)

	local effect, count, iCombatant, iEffect, iType

	-- Hawksy: guard code
	if not arg1 then
		return
	end
	if (not sel) or (sel < 0) then
		return
	end
	if not recap_temp.OtherDetailsList then
		return
	end
	if not recap.Combatant then
		return
	end
	if not recap.Combatant[arg1] then
		return
	end

	iCombatant = recap.Combatant[arg1]

	if not iCombatant.OtherDetail then
		return
	end

	RecapPanelOtherEffectText:SetText("--")
	RecapPanelOtherAttributeText:SetText("--")
	RecapPanelOtherTotalText:SetText("--")
	RecapPanelOtherHitsText:SetText("--")
	RecapPanelOtherAvgText:SetText("--")
	RecapPanelOtherMaxText:SetText("--")
	RecapPanelOtherMissesText:SetText("--")
	RecapPanelOtherDispelsText:SetText("--")
	RecapPanelOtherIntervalCountText:SetText("--")
	RecapPanelOtherIntervalAvgText:SetText("--")
	RecapPanelOtherDurationCountText:SetText("--")
	RecapPanelOtherDurationAvgText:SetText("--")

	if sel == 0 then
		return
	end

	if not recap_temp.OtherDetailsList[sel] then
		return
	end

	effect = recap_temp.OtherDetailsList[sel].Effect

	-- Hawksy: guard code
	if not effect then
		return
	end

	iType = string.sub(effect,1,1)

	-- Hawksy: guard code
	if not iCombatant.OtherDetail[effect] then
		return
	end

	iEffect = iCombatant.OtherDetail[effect]

	-- effect (repeated so we know which effect we are viewing)
	RecapPanelOtherEffectText:SetText(string.sub(effect,2))
	if (iType == "1") or (iType == "2") then
		RecapPanelOtherEffectText:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
	elseif  (iType == "3") or (iType == "4") then
		RecapPanelOtherEffectText:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
	elseif (iType == "5") or (iType == "6") then
		RecapPanelOtherEffectText:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
	elseif (iType == "7") or (iType == "8") then
		RecapPanelOtherEffectText:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
	end

	-- attribute
	if iEffect.Attribute then
		RecapPanelOtherAttributeText:SetText(iEffect.Attribute)
	end

	-- total, hits, misses, average, max
	count = (iEffect.Hits or 0)
	if count>0 then
		RecapPanelOtherHitsText:SetText(count)
		RecapPanelOtherMissesText:SetText((iEffect.Missed or 0))
		RecapPanelOtherTotalText:SetText(iEffect.Total or 0)
		RecapPanelOtherAvgText:SetText(string.format("%d",Recap_Div0((iEffect.Total or 0),count)))
		RecapPanelOtherMaxText:SetText(iEffect.Max or 0)
	else
		RecapPanelOtherHitsText:SetText(0)
	end

	-- dispels
	if iEffect.Dispels and (iEffect.Dispels>0) then
		RecapPanelOtherDispelsText:SetText(iEffect.Dispels)
	end

	-- interval count and average (parallel code populates the list)
	count = (iEffect.ICount or 0)
	if count>0 then
		RecapPanelOtherIntervalCountText:SetText(count)
		RecapPanelOtherIntervalAvgText:SetText(string.format("%.1f",Recap_Div1((iEffect.ITotal or 0),1000*count)))
	end

	-- duration count and average (parallel code populates the list)
	count = (iEffect.DCount or 0)
	if count>0 then
		RecapPanelOtherDurationCountText:SetText(count)
		RecapPanelOtherDurationAvgText:SetText(string.format("%.1f",Recap_Div1((iEffect.DTotal or 0),1000*count)))
	end

end

function RecapPanel_OutgoingDetail_OnEnter()

	-- Hawksy: lock OnEnter-OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local id = this:GetID()
	local index = id + FauxScrollFrame_GetOffset(RecapPanelOutgoingDetailsScrollBar)

	if (index<recap_temp.OutgoingDetailsListSize) and recap_temp.OutgoingDetailSelected==0 then
		if recap_temp.Selected == recap_temp.FriendlyIndex then
			RecapPanel_PopulateOutgoingDetails(recap_temp.FriendlyTotal,index)
		elseif recap_temp.Selected == recap_temp.NonFriendlyIndex then
			RecapPanel_PopulateOutgoingDetails(recap_temp.NonFriendlyTotal,index)
		else
			RecapPanel_PopulateOutgoingDetails(RecapPanelName:GetText(),index)
		end
	end

	-- Hawksy: unlock OnEnter-OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function RecapPanel_OutgoingDetail_OnLeave()

	-- Hawksy: lock OnEnter-OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	if recap_temp.OutgoingDetailSelected==0 then
		RecapPanel_PopulateOutgoingDetails(nil,0)
	end

	-- Hawksy: unlock OnEnter-OnLeave events
	recap_temp.OnEnterOnLeave = false
end

-- clicks on individual effect lines
function RecapPanel_OutgoingDetail_OnClick(arg1)

	local id = this:GetID()
	local index = id + FauxScrollFrame_GetOffset(RecapPanelOutgoingDetailsScrollBar)
	local _G = getfenv(0)

	if arg1=="LeftButton" and IsShiftKeyDown() and index<recap_temp.OutgoingDetailsListSize then
		local iType = string.sub(recap_temp.OutgoingDetailsList[index].Effect,1,1)
		if (iType=="3") or (iType=="4") or (((iType=="7") or (iType=="8")) and (string.find(recap_temp.OutgoingDetailsList[index].Effect, recap_temp.Localize.ElementHealing, 1, true))) then
			-- healing, ordinary, subtotal, or total
			Recap_InsertChat(string.format(recap_temp.Localize.OutgoingHealDetailLink,
								RecapPanelName:GetText(),
								_G["RecapPanelOutgoingDetail"..id.."_Name"]:GetText(),
								recap_temp.OutgoingDetailsList[index].Total,
								_G["RecapPanelOutgoingDetail"..id.."_TotalP"]:GetText(),
								RecapPanelName:GetText() ))
		elseif (iType=="5") or (iType=="6") or (((iType=="7") or (iType=="8")) and (string.find(recap_temp.OutgoingDetailsList[index].Effect, RECAP_DAMAGE, 1, true))) then
			-- damage, subtotal or total
			Recap_InsertChat(string.format(recap_temp.Localize.OutgoingDamageDetailLinkNoElement,
								RecapPanelName:GetText(),
								_G["RecapPanelOutgoingDetail"..id.."_Name"]:GetText(),
								recap_temp.OutgoingDetailsList[index].Total,
								_G["RecapPanelOutgoingDetail"..id.."_TotalP"]:GetText(),
								RecapPanelName:GetText() ))
		else
			-- damage, ordinary
			Recap_InsertChat(string.format(recap_temp.Localize.OutgoingDamageDetailLink,
								RecapPanelName:GetText(),
								_G["RecapPanelOutgoingDetail"..id.."_Name"]:GetText(),
								recap_temp.OutgoingDetailsList[index].Element,
								recap_temp.OutgoingDetailsList[index].Total,
								_G["RecapPanelOutgoingDetail"..id.."_TotalP"]:GetText(),
								RecapPanelName:GetText() ))
		end

	elseif arg1=="RightButton" and index<recap_temp.OutgoingDetailsListSize and IsControlKeyDown() and not IsShiftKeyDown() then
		-- Hawksy: post Recent Outgoing events for this effect for this combatant when Recent Data Mode is active
		if recap.Opt.RecentData.value then
			Recap_PostRecentRows(true, RecapPanelName:GetText(), _G["RecapPanelOutgoingDetail"..id.."_Name"]:GetText(), nil)
		end

	elseif recap_temp.OutgoingDetailSelected==index then
		recap_temp.OutgoingDetailSelected = 0
		RecapPanelOutgoingDetailsScrollBar_Update()
		RecapPanel_PopulateOutgoingDetails(nil,0)

	else
		recap_temp.OutgoingDetailSelected = index
		if recap_temp.Selected == recap_temp.FriendlyIndex then
			RecapPanel_PopulateOutgoingDetails(recap_temp.FriendlyTotal,index)
		elseif recap_temp.Selected == recap_temp.NonFriendlyIndex then
			RecapPanel_PopulateOutgoingDetails(recap_temp.NonFriendlyTotal,index)
		else
			RecapPanel_PopulateOutgoingDetails(RecapPanelName:GetText(),index)
		end
		RecapPanelOutgoingDetailsScrollBar_Update()
	end
end

function RecapPanel_IncomingDetail_OnEnter()

	-- Hawksy: lock OnEnter-OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local id = this:GetID()
	local index = id + FauxScrollFrame_GetOffset(RecapPanelIncomingDetailsScrollBar)

	if (index<recap_temp.IncomingDetailsListSize) and recap_temp.IncomingDetailSelected==0 then
		if recap_temp.Selected == recap_temp.FriendlyIndex then
			RecapPanel_PopulateIncomingDetails(recap_temp.FriendlyTotal,index)
		elseif recap_temp.Selected == recap_temp.NonFriendlyIndex then
			RecapPanel_PopulateIncomingDetails(recap_temp.NonFriendlyTotal,index)
		else
			RecapPanel_PopulateIncomingDetails(RecapPanelName:GetText(),index)
		end
	end

	-- Hawksy: unlock OnEnter-OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function RecapPanel_IncomingDetail_OnLeave()

	-- Hawksy: lock OnEnter-OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	if recap_temp.IncomingDetailSelected==0 then
		RecapPanel_PopulateIncomingDetails(nil,0)
	end

	-- Hawksy: unlock OnEnter-OnLeave events
	recap_temp.OnEnterOnLeave = false
end

-- clicks on individual effect lines
function RecapPanel_IncomingDetail_OnClick(arg1)

	local id = this:GetID()
	local index = id + FauxScrollFrame_GetOffset(RecapPanelIncomingDetailsScrollBar)
	local _G = getfenv(0)

	if arg1=="LeftButton" and IsShiftKeyDown() and index<recap_temp.IncomingDetailsListSize then
		local iType = string.sub(recap_temp.IncomingDetailsList[index].Effect,1,1)
		if (iType=="3") or (iType=="4") or (((iType=="7") or (iType=="8")) and (string.find(recap_temp.IncomingDetailsList[index].Effect, recap_temp.Localize.ElementHealing, 1, true))) then
			-- healing, ordinary, subtotal, or total
			Recap_InsertChat(string.format(recap_temp.Localize.IncomingHealDetailLink,
								_G["RecapPanelIncomingDetail"..id.."_Name"]:GetText(),
								RecapPanelName:GetText(),
								recap_temp.IncomingDetailsList[index].Total,
								_G["RecapPanelIncomingDetail"..id.."_TotalP"]:GetText(),
								RecapPanelName:GetText() ))
		elseif (iType=="5") or (iType=="6") or (((iType=="7") or (iType=="8")) and (string.find(recap_temp.IncomingDetailsList[index].Effect, RECAP_DAMAGE, 1, true))) then
			-- damage, subtotal or total
			Recap_InsertChat(string.format(recap_temp.Localize.IncomingDamageDetailLinkNoElement,
								_G["RecapPanelIncomingDetail"..id.."_Name"]:GetText(),
								RecapPanelName:GetText(),
								recap_temp.IncomingDetailsList[index].Total,
								_G["RecapPanelIncomingDetail"..id.."_TotalP"]:GetText(),
								RecapPanelName:GetText() ))
		else
			-- damage, ordinary
			Recap_InsertChat(string.format(recap_temp.Localize.IncomingDamageDetailLink,
								_G["RecapPanelIncomingDetail"..id.."_Name"]:GetText(),
								recap_temp.IncomingDetailsList[index].Element,
								RecapPanelName:GetText(),
								recap_temp.IncomingDetailsList[index].Total,
								_G["RecapPanelIncomingDetail"..id.."_TotalP"]:GetText(),
								RecapPanelName:GetText() ))
		end

	elseif arg1=="RightButton" and index<recap_temp.IncomingDetailsListSize and IsControlKeyDown() and not IsShiftKeyDown() then
		-- Hawksy: post Recent Incoming events for this effect for this combatant when Recent Data Mode is active
		if recap.Opt.RecentData.value then
			Recap_PostRecentRows(true, RecapPanelName:GetText(), _G["RecapPanelIncomingDetail"..id.."_Name"]:GetText(), nil)
		end

	elseif recap_temp.IncomingDetailSelected==index then
		recap_temp.IncomingDetailSelected = 0
		RecapPanelIncomingDetailsScrollBar_Update()
		RecapPanel_PopulateIncomingDetails(nil,0)

	else
		recap_temp.IncomingDetailSelected = index
		if recap_temp.Selected == recap_temp.FriendlyIndex then
			RecapPanel_PopulateIncomingDetails(recap_temp.FriendlyTotal,index)
		elseif recap_temp.Selected == recap_temp.NonFriendlyIndex then
			RecapPanel_PopulateIncomingDetails(recap_temp.NonFriendlyTotal,index)
		else
			RecapPanel_PopulateIncomingDetails(RecapPanelName:GetText(),index)
		end
		RecapPanelIncomingDetailsScrollBar_Update()
	end
end

function RecapPanel_OtherDetail_OnEnter()

	-- Hawksy: lock OnEnter-OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local id = this:GetID()
	local index = id + FauxScrollFrame_GetOffset(RecapPanelOtherDetailsScrollBar)

	if (index<recap_temp.OtherDetailsListSize) and recap_temp.OtherDetailSelected==0 then
		if recap_temp.Selected == recap_temp.FriendlyIndex then
			RecapPanel_PopulateOtherDetails(recap_temp.FriendlyTotal,index)
		elseif recap_temp.Selected == recap_temp.NonFriendlyIndex then
			RecapPanel_PopulateOtherDetails(recap_temp.NonFriendlyTotal,index)
		else
			RecapPanel_PopulateOtherDetails(RecapPanelName:GetText(),index)
		end
	end

	-- Hawksy: unlock OnEnter-OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function RecapPanel_OtherDetail_OnLeave()

	-- Hawksy: lock OnEnter-OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	if recap_temp.OtherDetailSelected==0 then
		RecapPanel_PopulateOtherDetails(nil,0)
	end

	-- Hawksy: unlock OnEnter-OnLeave events
	recap_temp.OnEnterOnLeave = false
end

-- clicks on individual effect lines
function RecapPanel_OtherDetail_OnClick(arg1)

	local id = this:GetID()
	local index = id + FauxScrollFrame_GetOffset(RecapPanelOtherDetailsScrollBar)
	local linkformat = recap_temp.Localize.OtherDetailLink
	local _G = getfenv(0)

	if arg1=="LeftButton" and IsShiftKeyDown() and index<recap_temp.OtherDetailsListSize then
		local iType, typeText
		iType = string.sub(recap_temp.OtherDetailsList[index].Effect,1,1)
		typeText = recap_temp.Localize.OtherDetailType[tonumber(iType)]
		-- Hawksy: considerable variability in the message output
		local variableBit = ""
		if recap_temp.OtherDetailsList[index].Total and (recap_temp.OtherDetailsList[index].Total ~= "--") then
			variableBit = "("..string.lower(RECAP_TOTAL).." "..recap_temp.OtherDetailsList[index].Total
			if recap_temp.OtherDetailsList[index].Attribute ~= "--" then
				variableBit = variableBit.." "..recap_temp.OtherDetailsList[index].Attribute
			end
		end
		if recap_temp.OtherDetailsList[index].Interval and (recap_temp.OtherDetailsList[index].Interval ~= "--") then
			if (variableBit == "") then
				variableBit = "("
			else
				variableBit = variableBit..", "
			end
			variableBit = variableBit..RECAP_ESTIMATED_INTERVAL.." "..recap_temp.OtherDetailsList[index].Interval.." s"
		end
		if recap_temp.OtherDetailsList[index].Duration and (recap_temp.OtherDetailsList[index].Duration ~= "--") then
			if (variableBit == "") then
				variableBit = "("
			else
				variableBit = variableBit..", "
			end
			variableBit = variableBit..RECAP_ESTIMATED_DURATION.." "..recap_temp.OtherDetailsList[index].Duration.." s"
		end
		if (variableBit ~= "") then
			variableBit = variableBit..")"
		end
		Recap_InsertChat(string.format(linkformat,RecapPanelName:GetText(),
							typeText,
							_G["RecapPanelOtherDetail"..id.."_Name"]:GetText(),
							recap_temp.OtherDetailsList[index].Hits,
							variableBit ))

		-- currently Other details are not included in Recent Data

	elseif recap_temp.OtherDetailSelected==index then
		recap_temp.OtherDetailSelected = 0
		RecapPanelOtherDetailsScrollBar_Update()
		RecapPanel_PopulateOtherDetails(nil,0)

	else
		recap_temp.OtherDetailSelected = index
		if recap_temp.Selected == recap_temp.FriendlyIndex then
			RecapPanel_PopulateOtherDetails(recap_temp.FriendlyTotal,index)
		elseif recap_temp.Selected == recap_temp.NonFriendlyIndex then
			RecapPanel_PopulateOtherDetails(recap_temp.NonFriendlyTotal,index)
		else
			RecapPanel_PopulateOtherDetails(RecapPanelName:GetText(),index)
		end
		RecapPanelOtherDetailsScrollBar_Update()
	end
end

function RecapPanel_OnMouseDown(arg1)

	if recap_temp.Loaded and arg1=="LeftButton" then
		RecapPanel:StartMoving()
	end
end

function RecapPanel_OnMouseUp(arg1)

	if recap_temp.Loaded and arg1=="LeftButton" then
		RecapPanel:StopMovingOrSizing()

		-- check for docking
		recap.Opt.PanelAnchor.value = false

		if Recap_Near(RecapFrame:GetRight(),RecapPanel:GetLeft()) then
			if Recap_Near(RecapFrame:GetTop(),RecapPanel:GetTop()) then
				recap.Opt.PanelAnchor = { type="Flag", value=true, Main="TOPRIGHT", Panel="TOPLEFT" }
			elseif Recap_Near(RecapFrame:GetBottom(),RecapPanel:GetBottom()) then
				recap.Opt.PanelAnchor = { type="Flag", value=true, Main="BOTTOMRIGHT", Panel="BOTTOMLEFT"}
			end
		elseif Recap_Near(RecapFrame:GetLeft(),RecapPanel:GetRight()) then
			if Recap_Near(RecapFrame:GetTop(),RecapPanel:GetTop()) then
				recap.Opt.PanelAnchor = { type="Flag", value=true, Main="TOPLEFT", Panel="TOPRIGHT" }
			elseif Recap_Near(RecapFrame:GetBottom(),RecapPanel:GetBottom()) then
				recap.Opt.PanelAnchor = { type="Flag", value=true, Main="BOTTOMLEFT", Panel="BOTTOMRIGHT" }
			end
		elseif Recap_Near(RecapFrame:GetRight(),RecapPanel:GetRight()) then
			if Recap_Near(RecapFrame:GetTop(),RecapPanel:GetBottom()) then
				recap.Opt.PanelAnchor = { type="Flag", value=true, Main="TOPRIGHT", Panel="BOTTOMRIGHT" }
			elseif Recap_Near(RecapFrame:GetBottom(),RecapPanel:GetTop()) then
				recap.Opt.PanelAnchor = { type="Flag", value=true, Main="BOTTOMRIGHT", Panel="TOPRIGHT" }
			end
		elseif Recap_Near(RecapFrame:GetLeft(),RecapPanel:GetLeft()) then
			if Recap_Near(RecapFrame:GetTop(),RecapPanel:GetBottom()) then
				recap.Opt.PanelAnchor = { type="Flag", value=true, Main="TOPLEFT", Panel="BOTTOMLEFT" }
			elseif Recap_Near(RecapFrame:GetBottom(),RecapPanel:GetTop()) then
				recap.Opt.PanelAnchor = { type="Flag", value=true, Main="BOTTOMLEFT", Panel="TOPLEFT" }
			end
		end

		if recap.Opt.PanelAnchor.value then
			RecapPanel:ClearAllPoints()
			RecapPanel:SetPoint(recap.Opt.PanelAnchor.Panel,"RecapFrame",recap.Opt.PanelAnchor.Main,Recap_PanelOffset("x"),Recap_PanelOffset("y"))
		end

	end
end

function Recap_PanelOffset(arg1)

	local anchor

	anchor = recap.Opt.PanelAnchor.Main..recap.Opt.PanelAnchor.Panel
	if dockoffset[anchor] and arg1 then
		return dockoffset[anchor][arg1]
	else
		return 0
	end
end

-- returns true if the two values are close to each other
function Recap_Near(arg1,arg2)

	local isnear = false

	if (math.max(arg1,arg2)-math.min(arg1,arg2)) < 15 then
	isnear = true
	end

	return isnear
end

function RecapOutgoingDetailHeader_OnMouseUp(arg1)

	if arg1=="RightButton" then
		RecapCreateMenu(recap_temp.Localize.DetailMenu)
	end
end

function RecapIncomingDetailHeader_OnMouseUp(arg1)

	if arg1=="RightButton" then
		RecapCreateMenu(recap_temp.Localize.DetailMenu)
	end
end

-- Hawksy: changed the behaviour of this function -- it now outputs rather than leaving
--   stuff in the buffer -- I know that wasn't the original design, but I think that
--   I prefer sending rather than concatenating -- if someone wants to switch it back
--   replace the "print_chat" just after "<255" with "ChatFrameEditBox:Insert" again,
--   removing the ", chatchan, nil, chatnum".
--   However, if you do this be aware that posting to "Self" (the console) will behave
--   strangely, since the "Self" chat edit box isn't visible so you can't see that it is
--   perhaps half full of something.  You'll post to "Self" and nothing will appear to
--   happen until the hidden chat edit box fills up.  Then you see stuff you might have
--   already forgotten you put there.
-- this substitutes ChatFrameEditBox:Insert.  If 'msg' will fit in the current editbox,
-- it will :Insert.  If not, it will automatically send 'msg' over the current chatType
-- across as many lines as needed, breaking lines by 'delimiter', a single-character
-- string.  (if no delimiter is given it will use "]").  If a delimeter isn't found, it
-- will hard-break every 255 characters.
function Recap_InsertChat(msg,delimiter)

	local cx, i, m_start, m_end, done
	local print_chat = SendChatMessage
	local chatchan, chatnum

	delimiter = string.sub(delimiter or "]",1,1)

	chatchan, chatnum = Recap_GetChannelInfo(nil, nil)
	if chatchan=="Self" then
		print_chat = Recap_Print
	end

	-- If the Clip box on the Options : Reports panel is open, post there instead of to chat or console
	if RecapClipEditBox:IsVisible() then
		print_chat = Recap_LineToClipboard
	end

	-- Hawksy: this will presumably usually return zero
	cx = string.len(ChatFrameEditBox:GetText())

	if cx+string.len(msg)<255 then
		-- Hawksy: modified to force output, not leave it sitting in the chat edit box
		print_chat(msg, chatchan, nil, chatnum)
	else
		if cx>0 then
			print_chat(ChatFrameEditBox:GetText(), chatchan, nil, chatnum)
			ChatFrameEditBox:SetText("")
		end

		cx = string.len(msg)
		m_start = 1

		while not done do
			m_end = false
			for i=m_start,cx do
				if ((string.sub(msg,i,i)==delimiter and (i-m_start)<255)) or ((i-m_start<255) and i==cx) then
					m_end = i
				end
				if (i-m_start)>254 and not m_end then
					m_end = i
				end
			end
			print_chat(string.sub(msg,m_start,m_end), chatchan, nil, chatnum)
			m_start = m_end+1
			if m_start>cx then
				done = true
			end
		end
	end

end

-- click on header on detail panel, dump that header info for all effects
function RecapOutgoingDetailHeader_OnClick()

	local i, lineno, combatantName, iList
	local text, alltext = "",""
	local print_chat = SendChatMessage
	local chatchan, chatnum

	if arg1=="LeftButton" and IsShiftKeyDown() and (recap_temp.Selected ~= 0) and recap_temp.OutgoingDetailsListSize>1 then

		if recap_temp.Selected == recap_temp.FriendlyIndex then
			combatantName = recap_temp.Localize.Friendly.." "..RECAP_COMBATANTS
		elseif recap_temp.Selected == recap_temp.NonFriendlyIndex then
			combatantName = recap_temp.Localize.NonFriendly.." "..RECAP_COMBATANTS
		else
			combatantName = recap_temp.List[recap_temp.Selected].Name
		end

		chatchan, chatnum = Recap_GetChannelInfo(nil, nil)
		if chatchan=="Self" then
			print_chat = Recap_Print
		end

		-- If the Clip box on the Options : Reports panel is open, post there instead of to chat or console
		if RecapClipEditBox:IsVisible() then
			print_chat = Recap_LineToClipboard
		end

		alltext = (recap.Opt.SpamRows.value and "__ Recap: " or "Recap: ")..combatantName..": "..Recap_GetTooltip(recap.Opt.OutgoingPanelDetail.value)..(recap.Opt.SpamRows.value and " __" or ":")
		lineno = 0
		if recap.Opt.SpamRows.value then
			print_chat(alltext, chatchan, nil, chatnum)
		end
		for i=1,recap_temp.OutgoingDetailsListSize-1 do
			iList = recap_temp.OutgoingDetailsList[i]
			if iList[recap.Opt.OutgoingPanelDetail.value]~="0.0%" and iList[recap.Opt.OutgoingPanelDetail.value]~="--" then
				local iType = string.sub(iList.Effect,1,1)
				text = string.sub(iList.Effect,2)..(recap.Opt.SpamRows.value and ": " or " ")..(((iType=="3") or (iType=="4")) and "+" or "")..iList[recap.Opt.OutgoingPanelDetail.value]..(recap.Opt.OutgoingPanelDetail.value=="Total" and (" ("..iList.TotalP.."%)") or "")
				if not recap.Opt.SpamRows.value then
					alltext = alltext.." ["..text.."]"
				else
					lineno = lineno + 1
					text = tostring(lineno)..". "..text
					print_chat(text, chatchan, nil, chatnum)
				end
			end
		end
		if not recap.Opt.SpamRows.value then
			Recap_InsertChat(alltext)
		end
	end
end

-- click on header on detail panel, dump that header info for all effects
function RecapIncomingDetailHeader_OnClick()

	local i, lineno, combatantName
	local text, alltext = "",""
	local print_chat = SendChatMessage
	local chatchan, chatnum

	if arg1=="LeftButton" and IsShiftKeyDown() and (recap_temp.Selected ~= 0) and recap_temp.IncomingDetailsListSize>1 then

		if recap_temp.Selected == recap_temp.FriendlyIndex then
			combatantName = recap_temp.Localize.Friendly.." "..RECAP_COMBATANTS
		elseif recap_temp.Selected == recap_temp.NonFriendlyIndex then
			combatantName = recap_temp.Localize.NonFriendly.." "..RECAP_COMBATANTS
		else
			combatantName = recap_temp.List[recap_temp.Selected].Name
		end

		chatchan, chatnum = Recap_GetChannelInfo(nil, nil)
		if chatchan=="Self" then
			print_chat = Recap_Print
		end

		-- If the Clip box on the Options : Reports panel is open, post there instead of to chat or console
		if RecapClipEditBox:IsVisible() then
			print_chat = Recap_LineToClipboard
		end

		alltext = (recap.Opt.SpamRows.value and "__ Recap: " or "Recap: ")..combatantName..": "..RECAP_INCOMING.." "..Recap_GetTooltip(recap.Opt.IncomingPanelDetail.value)..(recap.Opt.SpamRows.value and " __" or ":")
		lineno = 0
		if recap.Opt.SpamRows.value then
			print_chat(alltext, chatchan, nil, chatnum)
		end
		for i=1,recap_temp.IncomingDetailsListSize-1 do
			if recap_temp.IncomingDetailsList[i][recap.Opt.IncomingPanelDetail.value]~="0.0%" and recap_temp.IncomingDetailsList[i][recap.Opt.IncomingPanelDetail.value]~="--" then
				local iType = string.sub(recap_temp.IncomingDetailsList[i].Effect,1,1)
				text = string.sub(recap_temp.IncomingDetailsList[i].Effect,2)..(recap.Opt.SpamRows.value and ": " or " ")..(((iType=="3") or (iType=="4")) and "+" or "")..recap_temp.IncomingDetailsList[i][recap.Opt.IncomingPanelDetail.value]..(recap.Opt.IncomingPanelDetail.value=="Total" and (" ("..recap_temp.IncomingDetailsList[i].TotalP.."%)") or "")
				if not recap.Opt.SpamRows.value then
					alltext = alltext.." ["..text.."]"
				else
					lineno = lineno + 1
					text = tostring(lineno)..". "..text
					print_chat(text, chatchan, nil, chatnum)
				end
			end
		end
		if not recap.Opt.SpamRows.value then
			Recap_InsertChat(alltext)
		end
	end
end

-- click on header on detail panel, dump that header info for all effects
function RecapOtherDetailHeader_OnClick(colName)

	local i, lineno, combatantName
	local text, alltext = "",""
	local print_chat = SendChatMessage
	local chatchan, chatnum, iType, typeText

	if arg1=="LeftButton" and IsShiftKeyDown() and (recap_temp.Selected ~= 0) and recap_temp.OtherDetailsListSize>1 then

		if recap_temp.Selected == recap_temp.FriendlyIndex then
			combatantName = recap_temp.Localize.Friendly.." "..RECAP_COMBATANTS
		elseif recap_temp.Selected == recap_temp.NonFriendlyIndex then
			combatantName = recap_temp.Localize.NonFriendly.." "..RECAP_COMBATANTS
		else
			combatantName = recap_temp.List[recap_temp.Selected].Name
		end

		chatchan, chatnum = Recap_GetChannelInfo(nil, nil)
		if chatchan=="Self" then
			print_chat = Recap_Print
		end

		-- If the Clip box on the Options : Reports panel is open, post there instead of to chat or console
		if RecapClipEditBox:IsVisible() then
			print_chat = Recap_LineToClipboard
		end

		alltext = (recap.Opt.SpamRows.value and "__ Recap: " or "Recap: ")..combatantName..": "..RECAP_OTHER_EFFECTS..": "..(((colName == "Hits") and RECAP_HITS) or ((colName == "Total") and RECAP_TOTAL))..(recap.Opt.SpamRows.value and " __" or ":")
		lineno = 0
		if recap.Opt.SpamRows.value then
			print_chat(alltext, chatchan, nil, chatnum)
		end
		for i=1,recap_temp.OtherDetailsListSize-1 do
			iType = string.sub(recap_temp.OtherDetailsList[i].Effect,1,1)
			typeText = recap_temp.Localize.OtherDetailType[tonumber(iType)]
			if (colName == "Hits") then
				text = typeText..": "..string.sub(recap_temp.OtherDetailsList[i].Effect,2)..(recap.Opt.SpamRows.value and ": " or " ")..recap_temp.OtherDetailsList[i].Hits
			elseif (colName == "Total") then
				text = typeText..": "..string.sub(recap_temp.OtherDetailsList[i].Effect,2)..(recap.Opt.SpamRows.value and ": " or " ")..recap_temp.OtherDetailsList[i].Total
				if recap_temp.OtherDetailsList[i].Attribute ~= "--" then
					text = text.." "..recap_temp.OtherDetailsList[i].Attribute
				end
			end
			if not recap.Opt.SpamRows.value then
				alltext = alltext.." ["..text.."]"
			else
				lineno = lineno + 1
				text = tostring(lineno)..". "..text
				print_chat(text, chatchan, nil, chatnum)
			end
		end
		if not recap.Opt.SpamRows.value then
			Recap_InsertChat(alltext)
		end
	end
end

RecapPanel_lua_372 = true
