--[[ Synchronization functions ]]

function Recap_SetSyncDisplay()

	if not recap_temp.Loaded then
		return
	end

	if recap.Opt.SyncState.value == "Ignore" then
		RecapOpt_SyncState:SetText(RECAP_SYNC_STATE_IGNORE)
		RecapOpt_SyncStateDetail:SetText("")
		RecapOpt_SyncStateLastUpdate:SetText("")
		RecapStartSyncMergePetsOffButton:Disable()
		RecapStartSyncMergePetsOnButton:Disable()
		RecapManualSyncButton:Disable()
		RecapBecomeLeaderButton:Disable()
		RecapListMembersButton:Disable()
	elseif recap.Opt.SyncState.value == "Ready" then
		RecapOpt_SyncState:SetText(RECAP_SYNC_STATE_READY)
		RecapOpt_SyncStateDetail:SetText("")
		RecapOpt_SyncStateLastUpdate:SetText("")
		RecapStartSyncMergePetsOffButton:Enable()
		RecapStartSyncMergePetsOnButton:Enable()
		RecapManualSyncButton:Disable()
		RecapBecomeLeaderButton:Disable()
		RecapListMembersButton:Disable()
	elseif recap.Opt.SyncState.value == "Member" then
		RecapOpt_SyncState:SetText(RECAP_SYNC_STATE_MEMBER)
		Recap_SetSyncStateDetail()
		RecapStartSyncMergePetsOffButton:Enable()
		RecapStartSyncMergePetsOnButton:Enable()
		RecapManualSyncButton:Disable()
		RecapBecomeLeaderButton:Enable()
		RecapListMembersButton:Enable()
	elseif recap.Opt.SyncState.value == "Leader" then
		RecapOpt_SyncState:SetText(RECAP_SYNC_STATE_LEADER)
		Recap_SetSyncStateDetail()
		RecapStartSyncMergePetsOffButton:Enable()
		RecapStartSyncMergePetsOnButton:Enable()
		RecapManualSyncButton:Enable()
		RecapBecomeLeaderButton:Disable()
		RecapListMembersButton:Enable()
	else
		-- safety code
		recap.Opt.EnableSync.value = false
		recap.Opt.SyncState.value = "Ignore"
		-- clear all sync values (including the spinlock)
		recap.Opt.SyncTimestamp.value = false
		recap.Opt.SyncLeader.value = false
		recap.Opt.SyncMergePets.value = false
		recap.Opt.SyncUpdateWhen.value = false
		recap_temp.SyncInviteTimestamp = false
		recap_temp.SyncInviteLeader = false
		recap_temp.SyncInviteMergePets = false
		recap_temp.SyncInviteWhen = false
		recap_temp.SyncInviteNewOngoing = false
		recap_temp.AskingJoinSync = false
		recap_temp.SyncData = {}
		recap_temp.SyncMembers = {}
		recap_temp.OnCommReceive = false
		RecapOpt_SyncState:SetText(RECAP_SYNC_STATE_IGNORE)
		RecapOpt_SyncStateDetail:SetText("")
		RecapOpt_SyncStateLastUpdate:SetText("")
		RecapStartSyncMergePetsOffButton:Disable()
		RecapStartSyncMergePetsOnButton:Disable()
		RecapManualSyncButton:Disable()
		RecapBecomeLeaderButton:Disable()
		RecapListMembersButton:Disable()
	end
end

function Recap_SetSyncStateDetail()

	local text

	text = ""
	if recap.Opt.SyncTimestamp.value then
		text = text..recap_temp.Localize.Timestamp..": "..tostring(recap.Opt.SyncTimestamp.value)
	end
	if recap.Opt.SyncState.value and (recap.Opt.SyncState.value == "Member") and recap.Opt.SyncLeader.value then
		-- somehow (don't know how) SyncLeader is losing its value
		-- details for leader distinguished by not listing leader
		text = text.." "..recap_temp.Localize.Leader..": "..Recap_SyncParseMemberRealm(recap.Opt.SyncLeader.value)
	end
	if recap.Opt.SyncMergePets.value and (recap.Opt.SyncMergePets.value == "on") then
		text = text.."  ("..recap_temp.Localize.MergePetsOn..")"
	else
		text = text.."  ("..recap_temp.Localize.MergePetsOff..")"
	end
	RecapOpt_SyncStateDetail:SetText(text)
	text = ""
	if recap.Opt.SyncUpdateWhen.value then
		text = recap_temp.Localize.LastUpdateWas..": "..Recap_FormatTime(GetTime()-(recap.Opt.SyncUpdateWhen.value/1000)).." "..recap_temp.Localize.Ago
	end
	RecapOpt_SyncStateLastUpdate:SetText(text)
end

function Recap_SyncParseMemberRealm(nameRealm)

	local found, name, realm

	realm = ""
	found, _, name, realm = string.find(nameRealm, "^(%u%l+)(.*)")
	if found and (realm ~= "") then
		return name.." ("..realm..")"
	else
		return nameRealm
	end
end

function Recap_StartSync(mergePets)

	-- paranoia check
	if (recap.Opt.SyncState.value == "Ignore") then
		-- bail if ignoring synchronization
		return
	end

	if (recap.Opt.State.value == "Stopped") then
		-- we've got Recap turned off at the moment (okay to start if Recap is in combat, invitation will be presented when people are out of combat)
		PlaySound("igQuestFailed")
		return
	end

	-- lock OnCommReceive event
	if recap_temp.OnCommReceive == true then
		PlaySound("igQuestFailed")
		return
	end
	recap_temp.OnCommReceive = true

	-- abandon existing synchronization (if any), start a new one as leader
	recap.Opt.SyncState.value = "Leader"
	recap.Opt.SyncTimestamp.value = math.floor(1000*GetTime()) -- integer thousandths
	recap.Opt.SyncLeader.value = recap_temp.p -- player name with realm
	recap.Opt.SyncMergePets.value = mergePets
	recap.Opt.SyncUpdateWhen.value = recap.Opt.SyncTimestamp.value
	recap_temp.AskingJoinSync = false
	recap_temp.SyncData = {}
	recap_temp.SyncMembers = {}
	Recap_UpdateSyncMember(recap_temp.p, Recap_Sync_Version)
	Recap_SetSyncDisplay()

	-- reset All Fights
	RecapPanel_Hide(1)
	Recap_ResetAllCombatants(true) -- force reset even of locked combatants
	recap_temp.HoldFight = "Normal"
	Recap_MakeFriends()

	-- perhaps remove pet effects
	if recap.Opt.SyncMergePets.value == "off" then
		Recap_RemovePetEffects()
	end

	-- display
	Recap_SetState(false)
	Recap_SetView()

	-- issue invitation (version, timestamp, leader, mergePets)
	if not Recap:SendCommMessage("GROUP", recap_temp.Invite, Recap_Sync_Version, recap.Opt.SyncTimestamp.value, recap.Opt.SyncLeader.value, recap.Opt.SyncMergePets.value) then
		-- not sent
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Recap..": "..recap_temp.Localize.FailSendInvite)
	end
--Recap:SendCommMessage("WHISPER", UnitName("player"), recap_temp.Invite, Recap_Sync_Version, recap.Opt.SyncTimestamp.value, recap.Opt.SyncLeader.value, recap.Opt.SyncMergePets.value)

	-- unlock OnCommReceive event
	recap_temp.OnCommReceive = false
end

function Recap_ManualSync()

	-- paranoia check
	if not (recap.Opt.SyncState.value == "Leader") then
		-- bail if not Leader
		return
	end

	if (recap.Opt.State.value == "Stopped") or (recap.Opt.State.value == "Active") or (recap_temp.HoldFight == "Holding") then
		-- we've got Recap turned off at the moment, or are in combat, or are holding the last fight
		PlaySound("igQuestFailed")
		return
	end

	-- lock OnCommReceive event
	if recap_temp.OnCommReceive == true then
		PlaySound("igQuestFailed")
		return
	end
	recap_temp.OnCommReceive = true

	Recap_SendSummary()

	-- unlock OnCommReceive event
	recap_temp.OnCommReceive = false
end

function Recap_BecomeLeader()

	-- paranoia check
	if not (recap.Opt.SyncState.value == "Member") then
		-- bail if not Member
		return
	end

	if (recap.Opt.State.value == "Stopped") or (recap.Opt.State.value == "Active") or (recap_temp.HoldFight == "Holding") then
		-- we've got Recap turned off at the moment, or are in combat, or are holding the last fight
		PlaySound("igQuestFailed")
		return
	end

	-- lock OnCommReceive event
	if recap_temp.OnCommReceive == true then
		PlaySound("igQuestFailed")
		return
	end
	recap_temp.OnCommReceive = true

	-- take over an existing synchronization as leader
	recap.Opt.SyncState.value = "Leader"
	recap.Opt.SyncLeader.value = recap_temp.p -- player name with realm
	Recap_SetSyncDisplay()

	-- do a manual synchronization to let other players know about the change of leader
	Recap_SendSummary()

	-- unlock OnCommReceive event
	recap_temp.OnCommReceive = false
end

-- standard receipt of Recap synchronization messages
function Recap:OnCommReceive(prefix, sender, distribution, request, version, timestamp, leader, mergePets,
							 Name, Time, TimeIn, TimeHeal, DmgOut, DmgIn, RawHeal, OverHeal, MaxHit, Kills)

	if recap.Opt.SyncState.value == "Ignore" then
		-- we're ignoring synchronization
		-- (TODO: future optimisation would be to dynamically Register and Unregister from the channel)
		return
	end

	if recap.Opt.State.value == "Stopped" then
		-- we've got Recap turned off at the moment
		return
	end

	if not (version == Recap_Sync_Version) then
		-- incompatible version (at moment exact match required)
		if (version > Recap_Sync_Version) then
			-- running an outdated version
			DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Recap..": "..recap_temp.Localize.OutOfDateSyncVersion)
		end
		return
	end

	-- lock OnCommReceive event
	if recap_temp.OnCommReceive == true then return end
	recap_temp.OnCommReceive = true

	-- State machine based on SyncState and request

	-- Invite messages (Ready, Member, Leader)
	if request == recap_temp.Invite then
		if recap.Opt.State.value == "Active" then
			-- in combat, remember the request (overwrites any earlier memo)
			Recap_RememberInvite(timestamp, leader, mergePets, "new")
		elseif recap.Opt.State.value == "Idle" then
			-- idle (not in combat), okay to pop the question (but only if not already asking it)
			if not recap_temp.AskingJoinSync then
				recap_temp.AskingJoinSync = true
				Recap_AskJoinSync(timestamp, leader, mergePets, "new")
			end
		end
		recap_temp.OnCommReceive = false -- unlock OnCommReceive event
		return
	end

	-- Ready (Summary, treated as an invitation)
	if recap.Opt.SyncState.value == "Ready" then
		-- not currently a member of a synchronization
		if request == recap_temp.Summary then
			-- treat a Summary record as an implicit invitation
			if recap.Opt.State.value == "Active" then
				-- in combat, remember the invitation for later (overwrites any earlier memo)
				Recap_RememberInvite(timestamp, leader, mergePets, "ongoing")
			else
				-- idle (not in combat), okay to pop the question (but only if not already asking it)
				if not recap_temp.AskingJoinSync then
					recap_temp.AskingJoinSync = true
					Recap_AskJoinSync(timestamp, leader, mergePets, "ongoing")
				end
			end
			-- note that until the player actually accepts the invite, the Summary data are discarded
		end
		recap_temp.OnCommReceive = false -- unlock OnCommReceive event
		return
	end
	-- assert: Member or Leader only

	-- assert: further processing must match on timestamp
	if not (recap.Opt.SyncTimestamp.value == timestamp) then
		-- no match, bail early
		recap_temp.OnCommReceive = false -- unlock OnCommReceive event
		return
	end

	-- Accept and Decline messages (only reported to leader)
	if request == recap_temp.Accept then
		if recap.Opt.SyncState.value == "Leader" then
			DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Recap..": "..sender.." "..recap_temp.Localize.AcceptedSync)
		end
		-- note this person as part of the sync
		Recap_UpdateSyncMember(sender, Recap_Sync_Version)
		recap_temp.OnCommReceive = false -- unlock OnCommReceive event
		return
	end
	if request == recap_temp.Decline then
		if recap.Opt.SyncState.value == "Leader" then
			DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Recap..": "..sender.." "..recap_temp.Localize.DeclinedSync)
		end
		recap_temp.OnCommReceive = false -- unlock OnCommReceive event
		return
	end
	-- assert: Reply or Summary messages only

	if not ((request == recap_temp.Reply) or (request == recap_temp.Summary)) then
		recap_temp.OnCommReceive = false -- unlock OnCommReceive event
		return
	end

	-- assert: everyone needs to be on the same page for synchronization to work reliably
	-- assert: the All Fights panel is stable from the end of one fight until just before the accumulation at the end of the next fight
	-- assert: we're going to be extra cautious, and only process and respond to updates until the beginning of the next fight
	-- assert: short inter-fight gaps will probably mean incomplete synchronization

	-- assert: if we're holding the All Fights panel we aren't on the same page, bail (don't even memorize update data)
	if recap_temp.HoldFight == "Holding" then
		recap_temp.OnCommReceive = false -- unlock OnCommReceive event
		return
	end

	-- Update who this is from
	if request == recap_temp.Reply then
		Recap_UpdateSyncMember(sender, Recap_Sync_Version)
	elseif request == recap_temp.Summary then
		Recap_UpdateSyncMember(recap.Opt.SyncLeader.value, Recap_Sync_Version)
	end

	-- if we're in combat, remember the data for later on (whether Summary or Reply) (overwrites any earlier memo)
	if recap.Opt.State.value == "Active" then
		Recap_RememberSyncData(timestamp, Name, Time, TimeIn, TimeHeal, DmgOut, DmgIn, RawHeal, OverHeal, MaxHit, Kills)
		recap_temp.OnCommReceive = false -- unlock OnCommReceive event
		return
	end

	-- Reply messages
	if request == recap_temp.Reply then
		-- ignore any possible new leader information from a Reply in case it is stale-dated
		Recap_ProcessReply(Name, Time, TimeIn, TimeHeal, DmgOut, DmgIn, RawHeal, OverHeal, MaxHit, Kills)
		recap_temp.OnCommReceive = false -- unlock OnCommReceive event
		return
	end
	-- assert: Summary messages only

	if not (request == recap_temp.Summary) then
		recap_temp.OnCommReceive = false -- unlock OnCommReceive event
		return
	end

	-- check for new leader
	if not (recap.Opt.SyncLeader.value == leader) then
		-- new leader (if we were the Leader, we silently become a Member)
		recap.Opt.SyncState.value = "Member"
		recap.Opt.SyncLeader.value = leader
		Recap_SetSyncDisplay()
	end

	-- and finally, process the Summary update
	Recap_ProcessSummary(Name, Time, TimeIn, TimeHeal, DmgOut, DmgIn, RawHeal, OverHeal, MaxHit, Kills)

	-- unlock OnCommReceive event
	recap_temp.OnCommReceive = false
end

function Recap_RememberInvite(timestamp, leader, mergePets, newOngoing)
	recap_temp.SyncInviteTimestamp = timestamp
	recap_temp.SyncInviteLeader = leader
	recap_temp.SyncInviteMergePets = mergePets
	recap_temp.SyncInviteWhen = math.floor(1000*GetTime()) -- integer thousandths
	recap_temp.SyncInviteNewOngoing = newOngoing
end

function Recap_RememberSyncData(timestamp, Name, Time, TimeIn, TimeHeal, DmgOut, DmgIn, RawHeal, OverHeal, MaxHit, Kills)
	if not recap_temp.SyncData[Name] then
		recap_temp.SyncData[Name] = {}
	end
	recap_temp.SyncData[Name].Timestamp = timestamp
	recap_temp.SyncData[Name].Time = math.max((recap_temp.SyncData[Name].Time or 0), (Time or 0))
	recap_temp.SyncData[Name].TimeIn = math.max((recap_temp.SyncData[Name].TimeIn or 0), (TimeIn or 0))
	recap_temp.SyncData[Name].TimeHeal = math.max((recap_temp.SyncData[Name].TimeHeal or 0), (TimeHeal or 0))
	recap_temp.SyncData[Name].DmgOut = math.max((recap_temp.SyncData[Name].DmgOut or 0), (DmgOut or 0))
	recap_temp.SyncData[Name].DmgIn = math.max((recap_temp.SyncData[Name].DmgIn or 0), (DmgIn or 0))
	recap_temp.SyncData[Name].RawHeal = math.max((recap_temp.SyncData[Name].RawHeal or 0), (RawHeal or 0))
	recap_temp.SyncData[Name].OverHeal = math.max((recap_temp.SyncData[Name].OverHeal or 0), (OverHeal or 0))
	recap_temp.SyncData[Name].MaxHit = math.max((recap_temp.SyncData[Name].MaxHit or 0), (MaxHit or 0))
	recap_temp.SyncData[Name].Kills = math.max((recap_temp.SyncData[Name].Kills or 0), (Kills or 0))
	recap_temp.SyncData[Name].When = math.floor(1000*GetTime()) -- integer thousandths
end

function Recap_CheckJoinSync()
	-- see if there is an outstanding invitation to a synchronization
	-- called immediately after EndFight

	if recap.Opt.SyncState.value == "Ignore" then
		-- we're ignoring synchronization
		return
	end

	if (recap.Opt.State.value == "Stopped") or (recap.Opt.State.value == "Active") then
		-- we've got Recap turned off at the moment, or are in combat (okay to be holding the last fight)
		return
	end

	if not (recap_temp.SyncInviteTimestamp and recap_temp.SyncInviteLeader and recap_temp.SyncInviteMergePets and recap_temp.SyncInviteWhen and recap_temp.SyncInviteNewOngoing) then
		-- paranoia check that the saved invite info is present and complete
		return
	end

	-- lock OnCommReceive event
	if recap_temp.OnCommReceive == true then return end
	recap_temp.OnCommReceive = true

	if (GetTime()-(recap_temp.SyncInviteWhen/1000)) <= 180 then
		-- within the time limit (in seconds)
		-- pop the question (we were out of combat when Recap_CheckJoinSync was called)
		local timestamp, leader, mergePets, newOngoing
		timestamp = recap_temp.SyncInviteTimestamp
		leader = recap_temp.SyncInviteLeader
		mergePets = recap_temp.SyncInviteMergePets
		newOngoing = recap_temp.SyncInviteNewOngoing
		recap_temp.SyncInviteTimestamp = false
		recap_temp.SyncInviteLeader = false
		recap_temp.SyncInviteMergePets = false
		recap_temp.SyncInviteWhen = false
		recap_temp.SyncInviteNewOngoing = false
		recap_temp.AskingJoinSync = true
		Recap_AskJoinSync(timestamp, leader, mergePets, newOngoing)
	end

	-- unlock OnCommReceive event
	recap_temp.OnCommReceive = false
end

function Recap_AskJoinSync(timestamp, leader, mergePets, newOngoing)

	local msg

	msg = recap_temp.Recap..": "..leader.." "..recap_temp.Localize.InvitesYouToSynchronize
	if mergePets == "on" then
		msg = msg.."  ("..recap_temp.Localize.MergePetsOn..")"
	else
		msg = msg.."  ("..recap_temp.Localize.MergePetsOff..")"
	end
	if newOngoing == "new" then
		msg = msg.."\n\n--  "..recap_temp.Localize.NewSynchronization.."  --"
	else
		msg = msg.."\n\n--  "..recap_temp.Localize.OngoingSynchronization.."  --"
	end
	StaticPopupDialogs["recap_temp.ASKJOINSYNC"] = {
		text = TEXT(msg),
		button1 = TEXT(ACCEPT),
		button2 = TEXT(CANCEL),
		OnAccept = function()
			-- join the synchronization
			Recap_JoinSync(timestamp, leader, mergePets)
			-- send accept message
			if not Recap:SendCommMessage("GROUP", recap_temp.Accept, Recap_Sync_Version, timestamp, leader, mergePets) then
				-- not sent
				DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Recap..": "..recap_temp.Localize.FailSendAccept)
			end
--Recap:SendCommMessage("WHISPER", UnitName("player"), recap_temp.Accept, Recap_Sync_Version, timestamp, leader, mergePets)
			-- free the spinlock
			recap_temp.AskingJoinSync = false
		end,
		OnCancel = function()
			-- send decline message
			if not Recap:SendCommMessage("GROUP", recap_temp.Decline, Recap_Sync_Version, timestamp, leader, mergePets) then
				-- not sent
				DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Recap..": "..recap_temp.Localize.FailSendAccept)
			end
--Recap:SendCommMessage("WHISPER", UnitName("player"), recap_temp.Decline, Recap_Sync_Version, timestamp, leader, mergePets)
			-- free the spinlock
			recap_temp.AskingJoinSync = false
		end,
		timeout = 0,
		showAlert = 1,
		whileDead = 1
	}
	StaticPopup_Show("recap_temp.ASKJOINSYNC") -- Note: NOT a blocking call

end

function Recap_JoinSync(timestamp, leader, mergePets)

	recap.Opt.SyncState.value = "Member"
	recap.Opt.SyncTimestamp.value = timestamp
	recap.Opt.SyncLeader.value = leader
	recap.Opt.SyncMergePets.value = mergePets
	recap.Opt.SyncUpdateWhen.value = math.floor(1000*GetTime()) -- integer thousandths
	recap_temp.SyncData = {}
	recap_temp.SyncMembers = {}
	Recap_UpdateSyncMember(leader, Recap_Sync_Version)
	Recap_UpdateSyncMember(recap_temp.p, Recap_Sync_Version)
	Recap_SetSyncDisplay()

	-- reset All Fights
	RecapPanel_Hide(1)
	Recap_ResetAllCombatants(true) -- force reset even of locked combatants
	recap_temp.HoldFight = "Normal"
	Recap_MakeFriends()

	-- perhaps remove pet effects
	if recap.Opt.SyncMergePets.value == "off" then
		Recap_RemovePetEffects()
	end

	-- display
	Recap_SetState(false)
	Recap_SetView()

end

function Recap_CheckSendSummary()
	-- see if we as leader need to broadcast a Summary
	-- called immediately after EndFight

	if not (recap.Opt.SyncState.value == "Leader") then
		-- bail if not Leader
		return
	end

	if (recap.Opt.State.value == "Stopped") or (recap.Opt.State.value == "Active") or (recap_temp.HoldFight == "Holding") then
		-- we've got Recap turned off at the moment, or are in combat, or are holding the last fight (All Fights not up to date)
		return
	end

	if math.max(1+GetNumPartyMembers(),GetNumRaidMembers()) == 1 then
		-- we're not in a group, skip the summary (can still send one manually)
		return
	end

	-- lock OnCommReceive event
	if recap_temp.OnCommReceive == true then return end
	recap_temp.OnCommReceive = true

	-- do it
	Recap_SendSummary()

	-- unlock OnCommReceive event
	recap_temp.OnCommReceive = false
end

function Recap_SendSummary()

	local i, iCombatant, dataSent

	dataSent = false
	for i in pairs(recap.Combatant) do
		if (i ~= recap_temp.FriendlyTotal) and (i ~= recap_temp.NonFriendlyTotal) then
			iCombatant = recap.Combatant[i]
			-- one message per combatant (version, timestamp, leader, mergePets, summary values)
			-- send times as integer thousandths for more compact transmission
			if Recap:SendCommMessage("GROUP", recap_temp.Summary, Recap_Sync_Version, recap.Opt.SyncTimestamp.value, recap.Opt.SyncLeader.value, recap.Opt.SyncMergePets.value,
									  i,
									  math.floor(1000*iCombatant.TotalTime), math.floor(1000*iCombatant.TotalTimeIn), math.floor(1000*iCombatant.TotalTimeHeal),
									  iCombatant.TotalDmgOut, iCombatant.TotalDmgIn, iCombatant.TotalRawHeal, iCombatant.TotalOverHeal,
									  iCombatant.TotalMaxHit, iCombatant.TotalKills) then
				dataSent = true
			else
				-- not sent
				DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Recap..": "..recap_temp.Localize.FailSendSummary)
				-- leave loop, no point in continuing after one failure
				break
			end
--Recap:SendCommMessage("WHISPER", UnitName("player"), recap_temp.Summary, Recap_Sync_Version, recap.Opt.SyncTimestamp.value, recap.Opt.SyncLeader.value, recap.Opt.SyncMergePets.value,
--							  i,
--							  math.floor(1000*iCombatant.TotalTime), math.floor(1000*iCombatant.TotalTimeIn), math.floor(1000*iCombatant.TotalTimeHeal),
--							  iCombatant.TotalDmgOut, iCombatant.TotalDmgIn, iCombatant.TotalRawHeal, iCombatant.TotalOverHeal,
--							  iCombatant.TotalMaxHit, iCombatant.TotalKills)
		end
	end

	if dataSent then
		-- turn on sync status light blink
		if recap_temp.SyncStatusLightBlink == 0 then
			recap_temp.SyncStatusLightBlink = 1
		end
	end

	-- update our own time as leader
	Recap_UpdateSyncMember(recap_temp.p, Recap_Sync_Version)

	-- remember the local time of the latest update activity
	recap.Opt.SyncUpdateWhen.value = math.floor(1000*GetTime()) -- integer thousandths
	Recap_SetSyncDisplay()
end

function Recap_CheckProcessSyncData()
	-- see if we need to process any remembered updates
	-- could include Summary or Reply data, and we treat both kinds of update data as if it were Summary data (so in this case we might Reply to a Reply)
	-- called immediately after EndFight

	local Name, i, iEffect

	if ((recap.Opt.SyncState.value == "Ignore") or (recap.Opt.SyncState.value == "Ready")) then
		-- we're not in synchronization
		return
	end

	if (recap.Opt.State.value == "Stopped") or (recap.Opt.State.value == "Active") or (recap_temp.HoldFight == "Holding") then
		-- we've got Recap turned off at the moment, or are in combat, or are holding the last fight (All Fights not up to date)
		return
	end

	-- lock OnCommReceive event
	if recap_temp.OnCommReceive == true then return end
	recap_temp.OnCommReceive = true

	-- scan for saved SyncData
	for Name in pairs(recap_temp.SyncData) do
		iEffect = recap_temp.SyncData[Name]
		if iEffect.Timestamp then
			-- data present
			if iEffect.Timestamp == recap.Opt.SyncTimestamp.value then
				-- data matches our timestamp (paranoia check)
				if (GetTime()-(iEffect.When/1000)) <= 180 then
					-- within the time limit (in seconds)
					Recap_ProcessSummary(Name, iEffect.Time, iEffect.TimeIn, iEffect.TimeHeal,
										iEffect.DmgOut, iEffect.DmgIn, iEffect.RawHeal,
										iEffect.OverHeal, iEffect.MaxHit, iEffect.Kills)
				end
			end
		end
		-- clear the data
		for i in pairs(iEffect) do
			iEffect[i] = nil
		end
	end

	-- unlock OnCommReceive event
	recap_temp.OnCommReceive = false
end

function Recap_ProcessSummary(Name, Time, TimeIn, TimeHeal, DmgOut, DmgIn, RawHeal, OverHeal, MaxHit, Kills)

	local upTime, upTimeIn, upTimeHeal, upDmgOut, upDmgIn, upRawHeal, upOverHeal, upMaxHit, upKills
	upTime = false
	upTimeIn = false
	upTimeHeal = false
	upDmgOut = false
	upDmgIn = false
	upRawHeal = false
	upOverHeal = false
	upMaxHit = false
	upKills = false

	if not recap.Combatant[Name] then
		-- new combatant
		Recap_CreateBlankCombatant(Name)
	end

	-- if an incoming value is larger than our value, it replaces our value
	-- if our value is larger, we'll send that out
	-- the times come in as integer thousandths for more compact transmission
	if Time >= math.floor(1000*recap.Combatant[Name].TotalTime) then
		recap.Combatant[Name].TotalTime = Time/1000
	else
--Recap_AuthorDebug("Reply my Time "..tostring(recap.Combatant[Name].TotalTime).." beats the summary "..tostring(Time/1000).." for "..tostring(Name))
		upTime = math.floor(1000*recap.Combatant[Name].TotalTime)
	end
	if TimeIn >= math.floor(1000*recap.Combatant[Name].TotalTimeIn) then
		recap.Combatant[Name].TotalTimeIn = TimeIn/1000
	else
--Recap_AuthorDebug("Reply my TimeIn "..tostring(recap.Combatant[Name].TotalTimeIn).." beats the summary "..tostring(TimeIn/1000).." for "..tostring(Name))
		upTimeIn = math.floor(1000*recap.Combatant[Name].TotalTimeIn)
	end
	if TimeHeal >= math.floor(1000*recap.Combatant[Name].TotalTimeHeal) then
		recap.Combatant[Name].TotalTimeHeal = TimeHeal/1000
	else
--Recap_AuthorDebug("Reply my TimeHeal "..tostring(recap.Combatant[Name].TotalTimeHeal).." beats the summary "..tostring(TimeHeal/1000).." for "..tostring(Name))
		upTimeHeal = math.floor(1000*recap.Combatant[Name].TotalTimeHeal)
	end
	if DmgOut >= recap.Combatant[Name].TotalDmgOut then
		recap.Combatant[Name].TotalDmgOut = DmgOut
	else
--Recap_AuthorDebug("Reply my DmgOut "..tostring(recap.Combatant[Name].TotalDmgOut).." beats the summary "..tostring(DmgOut).." for "..tostring(Name))
		upDmgOut = recap.Combatant[Name].TotalDmgOut
	end
	if DmgIn >= recap.Combatant[Name].TotalDmgIn then
		recap.Combatant[Name].TotalDmgIn = DmgIn
	else
--Recap_AuthorDebug("Reply my DmgIn "..tostring(recap.Combatant[Name].TotalDmgIn).." beats the summary "..tostring(DmgIn).." for "..tostring(Name))
		upDmgIn = recap.Combatant[Name].TotalDmgIn
	end
	if RawHeal >= recap.Combatant[Name].TotalRawHeal then
		recap.Combatant[Name].TotalRawHeal = RawHeal
	else
--Recap_AuthorDebug("Reply my RawHeal "..tostring(recap.Combatant[Name].TotalRawHeal).." beats the summary "..tostring(RawHeal).." for "..tostring(Name))
		upRawHeal = recap.Combatant[Name].TotalRawHeal
	end
	if OverHeal >= recap.Combatant[Name].TotalOverHeal then
		recap.Combatant[Name].TotalOverHeal = OverHeal
	else
--Recap_AuthorDebug("Reply my OverHeal "..tostring(recap.Combatant[Name].TotalOverHeal).." beats the summary "..tostring(OverHeal).." for "..tostring(Name))
		upOverHeal = recap.Combatant[Name].TotalOverHeal
	end
	if MaxHit >= recap.Combatant[Name].TotalMaxHit then
		recap.Combatant[Name].TotalMaxHit = MaxHit
	else
--Recap_AuthorDebug("Reply my MaxHit "..tostring(recap.Combatant[Name].TotalMaxHit).." beats the summary "..tostring(MaxHit).." for "..tostring(Name))
		upMaxHit = recap.Combatant[Name].TotalMaxHit
	end
	if Kills >= recap.Combatant[Name].TotalKills then
		recap.Combatant[Name].TotalKills = Kills
	else
--Recap_AuthorDebug("Reply my Kills "..tostring(recap.Combatant[Name].TotalKills).." beats the summary "..tostring(Kills).." for "..tostring(Name))
		upKills = recap.Combatant[Name].TotalKills
	end

	-- silently adjust overhealing number if necessary (could leave us out of sync)
	recap.Combatant[Name].TotalOverHeal = math.min(recap.Combatant[Name].TotalOverHeal,recap.Combatant[Name].TotalRawHeal)

	if upTime or upTimeIn or upTimeHeal or upDmgOut or upDmgIn or upRawHeal or upOverHeal or upMaxHit or upKills then
		-- one message per combatant (version, timestamp, leader, mergePets, summary values)
		-- values that do not need updating are sent as 'false'
		if not Recap:SendCommMessage("GROUP", recap_temp.Reply, Recap_Sync_Version, recap.Opt.SyncTimestamp.value, recap.Opt.SyncLeader.value, recap.Opt.SyncMergePets.value,
								     Name, upTime, upTimeIn, upTimeHeal, upDmgOut, upDmgIn, upRawHeal, upOverHeal, upMaxHit, upKills) then
			-- not sent (could be a lot of these messages)
			DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Recap..": "..recap_temp.Localize.FailSendReply)
		end
--Recap:SendCommMessage("WHISPER", UnitName("player"), recap_temp.Reply, Recap_Sync_Version, recap.Opt.SyncTimestamp.value, recap.Opt.SyncLeader.value, recap.Opt.SyncMergePets.value,
--								  Name, upTime, upTimeIn, upTimeHeal, upDmgOut, upDmgIn, upRawHeal, upOverHeal, upMaxHit, upKills)
	else
		-- we're not sending any update for this combatant
		-- if this is a summary record for the leader of the synchronization (i.e. only once per Summary set),
		--   then send a Reply (with no actual updates) to show that we are still here
		local tempName
		_, _, tempName = string.find(recap.Opt.SyncLeader.value, "^(%u%l+)")
		if Name == tempName then
			if not Recap:SendCommMessage("GROUP", recap_temp.Reply, Recap_Sync_Version, recap.Opt.SyncTimestamp.value, recap.Opt.SyncLeader.value, recap.Opt.SyncMergePets.value,
										 Name, false, false, false, false, false, false, false, false, false) then
				-- not sent (could be a lot of these messages)
				DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Recap..": "..recap_temp.Localize.FailSendReply)
			end
		end
	end

	-- derivative calculated values
	if recap.Combatant[Name].TotalTime > recap_temp.MinTime then
		recap.Combatant[Name].TotalDPS = Recap_Div1(recap.Combatant[Name].TotalDmgOut,recap.Combatant[Name].TotalTime)
	else
		recap.Combatant[Name].TotalDPS = 0
	end
	if recap.Combatant[Name].TotalTimeIn > recap_temp.MinTime then
		recap.Combatant[Name].TotalDPSIn = Recap_Div1(recap.Combatant[Name].TotalDmgIn,recap.Combatant[Name].TotalTimeIn)
	else
		recap.Combatant[Name].TotalDPSIn = 0
	end
	if recap.Combatant[Name].TotalTimeHeal > recap_temp.MinTime then
		recap.Combatant[Name].TotalHPS = Recap_Div1((recap.Combatant[Name].TotalRawHeal - recap.Combatant[Name].TotalOverHeal),recap.Combatant[Name].TotalTimeHeal)
	else
		recap.Combatant[Name].TotalHPS = 0
	end

	-- and display
	Recap_ConstructList()

	-- update our own time as member
	Recap_UpdateSyncMember(recap_temp.p, Recap_Sync_Version)

	-- remember the local time of the latest update activity
	recap.Opt.SyncUpdateWhen.value = math.floor(1000*GetTime()) -- integer thousandths
	Recap_SetSyncDisplay()

	-- turn on sync status light blink
	if recap_temp.SyncStatusLightBlink == 0 then
		recap_temp.SyncStatusLightBlink = 1
	end
end

function Recap_ProcessReply(Name, Time, TimeIn, TimeHeal, DmgOut, DmgIn, RawHeal, OverHeal, MaxHit, Kills)

	if not recap.Combatant[Name] then
		-- new combatant
		Recap_CreateBlankCombatant(Name)
	end

	-- if an incoming value is false, it is ignored
	-- if it is larger than our value, it replaces our value
	-- if our value is larger, we don't reply (we would have done so in response to the initial Summary)
	-- the times come in as integer thousandths for more compact transmission
	if Time then
		if Time >= math.floor(1000*recap.Combatant[Name].TotalTime) then
			recap.Combatant[Name].TotalTime = Time/1000
		end
	end
	if TimeIn then
		if TimeIn >= math.floor(1000*recap.Combatant[Name].TotalTimeIn) then
			recap.Combatant[Name].TotalTimeIn = TimeIn/1000
		end
	end
	if TimeHeal then
		if TimeHeal >= math.floor(1000*recap.Combatant[Name].TotalTimeHeal) then
			recap.Combatant[Name].TotalTimeHeal = TimeHeal/1000
		end
	end
	if DmgOut then
		if DmgOut >= recap.Combatant[Name].TotalDmgOut then
			recap.Combatant[Name].TotalDmgOut = DmgOut
		end
	end
	if DmgIn then
		if DmgIn >= recap.Combatant[Name].TotalDmgIn then
			recap.Combatant[Name].TotalDmgIn = DmgIn
		end
	end
	if RawHeal then
		if RawHeal >= recap.Combatant[Name].TotalRawHeal then
			recap.Combatant[Name].TotalRawHeal = RawHeal
		end
	end
	if OverHeal then
		if OverHeal >= recap.Combatant[Name].TotalOverHeal then
			recap.Combatant[Name].TotalOverHeal = OverHeal
		end
	end
	if MaxHit then
		if MaxHit >= recap.Combatant[Name].TotalMaxHit then
			recap.Combatant[Name].TotalMaxHit = MaxHit
		end
	end
	if Kills then
		if Kills >= recap.Combatant[Name].TotalKills then
			recap.Combatant[Name].TotalKills = Kills
		end
	end

	-- silently adjust overhealing number if necessary (could leave us out of sync)
	recap.Combatant[Name].TotalOverHeal = math.min(recap.Combatant[Name].TotalOverHeal,recap.Combatant[Name].TotalRawHeal)

	-- derivative calculated values
	if recap.Combatant[Name].TotalTime > recap_temp.MinTime then
		recap.Combatant[Name].TotalDPS = Recap_Div1(recap.Combatant[Name].TotalDmgOut,recap.Combatant[Name].TotalTime)
	else
		recap.Combatant[Name].TotalDPS = 0
	end
	if recap.Combatant[Name].TotalTimeIn > recap_temp.MinTime then
		recap.Combatant[Name].TotalDPSIn = Recap_Div1(recap.Combatant[Name].TotalDmgIn,recap.Combatant[Name].TotalTimeIn)
	else
		recap.Combatant[Name].TotalDPSIn = 0
	end
	if recap.Combatant[Name].TotalTimeHeal > recap_temp.MinTime then
		recap.Combatant[Name].TotalHPS = Recap_Div1((recap.Combatant[Name].TotalRawHeal - recap.Combatant[Name].TotalOverHeal),recap.Combatant[Name].TotalTimeHeal)
	else
		recap.Combatant[Name].TotalHPS = 0
	end

	-- and display
	Recap_ConstructList()

	-- remember the local time of the latest update activity
	recap.Opt.SyncUpdateWhen.value = math.floor(1000*GetTime()) -- integer thousandths
	Recap_SetSyncDisplay()

	-- turn on sync status light blink
	if recap_temp.SyncStatusLightBlink == 0 then
		recap_temp.SyncStatusLightBlink = 1
	end
end

function Recap_WarnNotAllowedDuringSync()
	StaticPopupDialogs["recap_temp.CONFIRMDATAMODE"] = {
		text = TEXT(recap_temp.Localize.NotAllowedDuringSync),
		button1 = TEXT(OKAY),
		OnAccept = function()
		end,
		timeout = 0,
		showAlert = 1,
		whileDead = 1
	}
	StaticPopup_Show("recap_temp.CONFIRMDATAMODE")
end

function Recap_UpdateSyncMember(name, version)
	if not recap_temp.SyncMembers[name] then
		-- new
		recap_temp.SyncMembers[name] = {}
	end
	local iMember = recap_temp.SyncMembers[name]
	iMember.Version = version
	iMember.When = math.floor(1000*GetTime())
end

function Recap_ListSyncMembers()

	local list_time, i, index, iMember

	list_time = GetTime()
	index = 0
	DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Recap..": "..recap_temp.Localize.SyncMembers)
	for i in pairs(recap_temp.SyncMembers) do
		index = index + 1
		iMember = recap_temp.SyncMembers[i]
		-- Hawksy: version not currently displayed
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Recap..": "..tostring(index)..": "..Recap_SyncParseMemberRealm(i).." "..recap_temp.Localize.LastUpdateWas.." "..Recap_FormatTime(list_time-(iMember.When/1000)).." "..recap_temp.Localize.Ago)
	end
end

RecapSync_lua_372 = true
